///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//  Copyright  NetworkDLS 2002, All rights reserved
//
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF 
// ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO 
// THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A 
// PARTICULAR PURPOSE.
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#ifndef _Config_CompressibleFiles_CPP
#define _Config_CompressibleFiles_CPP
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include <Windows.H>
#include <ShlObj.H>
#include <Stdio.H>
#include <ShlOBJ.H>
#include <Stdlib.H>

#include "../Source/Entry.H"
#include "../Source/CWebSites.H"
#include "../Source/ConfigTree.H"

#include "NetLogoDlg.H"

#include "Config_CompressibleFiles.H"
#include "Config_CompressibleFiles_Item.H"

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

using namespace NSWFL::Windows;
using namespace NSWFL::ListView;
using namespace NSWFL::Conversion;

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

INT_PTR CALLBACK Config_CompressibleFiles(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
    //--------------------------------------------------------------------------

    static HWND hList = NULL;
    static HWND hEnabled = NULL;
    static HWND hMinFileSize = NULL;
    static HWND hMaxFileSize = NULL;
    static HWND hCompressLevel = NULL;
    static HWND hMemoryLevel = NULL;
    static HWND hCachePath = NULL;
    static HWND hCacheSize = NULL;
	static CCompression *pCI; //Local config item.

    //--------------------------------------------------------------------------

    if(uMsg == WM_INITDIALOG)
    {
		CenterOverOwner(hWnd);

        HMENU hMenu = GetSystemMenu(hWnd, FALSE);
        AppendMenu(hMenu, MF_SEPARATOR, 0, 0);
        AppendMenu(hMenu, MF_STRING, GENERIC_MENU_ABOUT, "About");

        SendMessage(hWnd, WM_SETICON, TRUE, (LPARAM) LoadIcon(ghAppInstance, MAKEINTRESOURCE(IDI_MAIN)));

        hList = GetDlgItem(hWnd, IDC_LIST);
        hEnabled = GetDlgItem(hWnd, IDC_ENABLED);

        hMinFileSize = GetDlgItem(hWnd, IDC_MINFILESIZE);
        hMaxFileSize = GetDlgItem(hWnd, IDC_MAXFILESIZE);
        hCompressLevel = GetDlgItem(hWnd, IDC_COMPRESSLEVEL);
        hMemoryLevel = GetDlgItem(hWnd, IDC_MEMORYLEVEL);
        hCachePath = GetDlgItem(hWnd, IDC_CACHEPATH);
        hCacheSize = GetDlgItem(hWnd, IDC_CACHESIZE);

		pCI = (CCompression *)gCI.pClass;
		gCI.hList = hList;
		gCI.hDialog = hWnd;

		pCI->PopConfigList(hWnd, IDC_LIST, IDC_ENABLED);

		char sInt[64];
		Set_Text(hCachePath, pCI->CachePath());
		Set_Text(hMinFileSize, FormatInteger(sInt, sizeof(sInt), pCI->MinCompressionSize()));
		Set_Text(hMaxFileSize, FormatInteger(sInt, sizeof(sInt), pCI->MaxCompressionSize()));
		Set_Text(hCompressLevel, FormatInteger(sInt, sizeof(sInt), pCI->CompressionLevel()));
		Set_Text(hMemoryLevel, FormatInteger(sInt, sizeof(sInt), pCI->MemoryLevel()));
		Set_Text(hCacheSize, FormatInteger(sInt, sizeof(sInt), pCI->MaxCachedFiles()));
		CheckDlgButton(hWnd, IDC_COMPRESSDYNAMIC, pCI->CompressDynamicContent());

		char sText[255];
		sprintf_s(sText, sizeof(sText), "%s%s", gCI.iSite == INVALID_SITE_INDEX ? "Global " : "", "Compressible Files");
		Set_Text(GetDlgItem(hWnd, IDC_CONFIGTITLE), sText);
        SendMessage(hWnd, (UINT)WM_SETTEXT, (WPARAM)0, (LPARAM)sText);

		if(gCI.iSite == INVALID_SITE_INDEX)
		{
			ShowWindow(GetDlgItem(hWnd, IDC_USEGLOBAL), SW_HIDE);
			CheckDlgButton(hWnd, IDC_USEGLOBAL, BST_UNCHECKED);
		}
		else{
			if(gWebSites->Site(gCI.iSite)->pCompression == gWebSites->DfltCompression)
			{
				CheckDlgButton(hWnd, IDC_USEGLOBAL, BST_CHECKED);
			}
			else{
				CheckDlgButton(hWnd, IDC_USEGLOBAL, BST_UNCHECKED);
			}
			SendMessage(hWnd, WM_COMMAND, IDC_USEGLOBAL, 0);
		}
		
        return TRUE; // Return TRUE to set the keyboard focus, Otherwise return FALSE.
    }
    //--------------------------------------------------------------------------
    else if(uMsg == WM_COMMAND)
    {
		if(wParam == IDC_BROWSE)
		{
			char sFolder[MAX_PATH];
			Get_Text(hCachePath, sFolder, sizeof(sFolder));
			if(BrowseDirectory(hWnd, "Select a compression cache directory.",
				BIF_RETURNONLYFSDIRS | BIF_NONEWFOLDERBUTTON, sFolder, sizeof(sFolder), sFolder))
			{
				Set_Text(hCachePath, sFolder);
			}
		}
		else if(wParam == IDC_USEGLOBAL)
		{
			bool bChecked = IsDlgButtonChecked(hWnd, IDC_USEGLOBAL) > 0;
			EnableWindow(hList, !bChecked);
			EnableWindow(GetDlgItem(hWnd, IDC_ENABLED), !bChecked);
			EnableWindow(GetDlgItem(hWnd, IDC_ADD), !bChecked);
			EnableWindow(GetDlgItem(hWnd, IDC_EDIT), !bChecked);
			EnableWindow(GetDlgItem(hWnd, IDC_DELETE), !bChecked);
			EnableWindow(GetDlgItem(hWnd, IDC_MOVEUP), !bChecked);
			EnableWindow(GetDlgItem(hWnd, IDC_MOVEDOWN), !bChecked);
			EnableWindow(GetDlgItem(hWnd, IDC_COMPRESSDYNAMIC), !bChecked);
			EnableWindow(GetDlgItem(hWnd, IDC_BROWSE), !bChecked);

			EnableWindow(hMinFileSize, !bChecked);
			EnableWindow(hMaxFileSize, !bChecked);
			EnableWindow(hCompressLevel, !bChecked);
			EnableWindow(hMemoryLevel, !bChecked);
			EnableWindow(hCachePath, !bChecked);
			EnableWindow(hCacheSize, !bChecked);
		}
        else if(wParam == IDC_ADD)
        {
			gCI.iListIndex = INVALID_LIST_INDEX;

			DialogBox(ghAppInstance,
				MAKEINTRESOURCE(IDD_CONFIG_COMPRESSIBLEFILES_ITEM),
				hWnd, Config_CompressibleFiles_Item);
		}
        else if(wParam == IDC_EDIT)
        {
			if((gCI.iListIndex = LVGetFirstSelectedItemIndex(hList)) >= 0)
			{
				DialogBox(ghAppInstance,
					MAKEINTRESOURCE(IDD_CONFIG_COMPRESSIBLEFILES_ITEM),
					hWnd, Config_CompressibleFiles_Item);
			}
        }
        else if(wParam == IDC_DELETE)
        {
			LVDeleteItem(hList);
        }
        else if(wParam == IDC_MOVEUP)
        {
			LVMoveItemUp(hList);
        }
        else if(wParam == IDC_MOVEDOWN)
        {
			LVMoveItemDown(hList);
        }
		else if(wParam == IDC_OK)
        {
	
			int iMax = Get_Long(GetDlgItem(hWnd, IDC_MAXFILESIZE));
			if(iMax <= 0){
				MessageBox(hWnd, "You must specify a maximum file size (over 0 MB).", gsTitleCaption, MB_ICONINFORMATION);
				return FALSE;
			}
			int iMin = Get_Long(GetDlgItem(hWnd, IDC_MINFILESIZE));
			if(iMin >= iMax){
				MessageBox(hWnd, "You must specify a minimum file size (lower then maximum size).", gsTitleCaption, MB_ICONINFORMATION);
				return FALSE;
			}
			int iCacheSize = Get_Long(GetDlgItem(hWnd, IDC_CACHESIZE));
			if(iCacheSize <= 0){
				MessageBox(hWnd, "You must specify a maximum cahce size (greater than 1).", gsTitleCaption, MB_ICONINFORMATION);
				return FALSE;
			}
			int iCompressLevel = Get_Long(GetDlgItem(hWnd, IDC_COMPRESSLEVEL));
			if(iCompressLevel < 1 || iCompressLevel > 9){
				MessageBox(hWnd, "You must specify a compression level (between 1 and 9).", gsTitleCaption, MB_ICONINFORMATION);
				return FALSE;
			}
			int iMemLevel = Get_Long(GetDlgItem(hWnd, IDC_MEMORYLEVEL));
			if(iMemLevel < 1 || iMemLevel > 9){
				MessageBox(hWnd, "You must specify a memory level (between 1 and 9).", gsTitleCaption, MB_ICONINFORMATION);
				return FALSE;
			}

			if(gCI.iSite == INVALID_SITE_INDEX)
			{
				//---------------------------------------------------------------------------
				//Editing globally defined values. ------------------------------------------
				//---------------------------------------------------------------------------
				if(!gWebSites->DfltCompression->LoadNewConfiguration(hWnd, gCI.hList))
				{
					MessageBox(hWnd, "Failed to load the new configuration!", gsTitleCaption, MB_ICONERROR);
					return FALSE;
				}
			}
			else{
				//---------------------------------------------------------------------------
				//Editing site level defined values. ----------------------------------------
				//---------------------------------------------------------------------------
				if(gWebSites->Site(gCI.iSite)->pCompression == gWebSites->DfltCompression)
				{
					//---------------------------------------------------------------------------
					//This site inherits its settings from the globally defined values. ---------
					//---------------------------------------------------------------------------
					if(IsDlgButtonChecked(hWnd, IDC_USEGLOBAL))
					{
						//This site uses global settings and the global check box is checked - Nothing to do.
					}
					else{
						bool bIsSiteActive = gWebSites->Site(gCI.iSite)->IsActive();

						if(bIsSiteActive)
						{
							gWebSites->Site(gCI.iSite)->Stop();
						}

						gWebSites->Site(gCI.iSite)->pCompression = new CCompression(gWebSites);

						gWebSites->Site(gCI.iSite)->pCompression->LoadNewConfiguration(hWnd, gCI.hList);

						if(bIsSiteActive)
						{
							gWebSites->Site(gCI.iSite)->Start();
						}
					}
				}
				else{
					//---------------------------------------------------------------------------
					//This site has its own settings explicitly defined. ------------------------
					//---------------------------------------------------------------------------

					if(IsDlgButtonChecked(hWnd, IDC_USEGLOBAL))
					{
						bool bIsSiteActive = gWebSites->Site(gCI.iSite)->IsActive();

						if(bIsSiteActive)
						{
							gWebSites->Site(gCI.iSite)->Stop();
						}

						delete gWebSites->Site(gCI.iSite)->pCompression;
						gWebSites->Site(gCI.iSite)->pCompression = gWebSites->DfltCompression;

						if(bIsSiteActive)
						{
							gWebSites->Site(gCI.iSite)->Start();
						}
					}
					else{
						gWebSites->Site(gCI.iSite)->pCompression->LoadNewConfiguration(hWnd, gCI.hList);
					}
				}
			}
	
			EndDialog(hWnd, TRUE);
        }
        else if(wParam == IDC_CANCEL || wParam == IDCANCEL)
        {
            EndDialog(hWnd, FALSE);
        }
		return FALSE;
    }
	//--------------------------------------------------------------------------
	else if(uMsg == WM_NOTIFY)
    {
        LPNMHDR pHdr = (LPNMHDR)lParam;

		if(pHdr->hwndFrom == gCI.hList)
        {
			if(pHdr->code == NM_DBLCLK)
			{
				if(ListView_GetItemCount(gCI.hList) > 0 && ListView_GetSelectedCount(gCI.hList) == 1)
				{
					if((gCI.iListIndex = LVGetFirstSelectedItemIndex(hList)) >= 0)
					{
						DialogBox(ghAppInstance,
							MAKEINTRESOURCE(IDD_CONFIG_COMPRESSIBLEFILES_ITEM),
							hWnd, Config_CompressibleFiles_Item);
					}
				}
			}
		}
	}
	//--------------------------------------------------------------------------
	else if(uMsg == WM_PAINT)
    {
        HDC ThisHDC;
        PAINTSTRUCT ThisPS;

        ThisHDC = BeginPaint(hWnd, &ThisPS);

        //- Any painting should be done here.

        EndPaint(hWnd, &ThisPS);
        return TRUE;
    }
    //--------------------------------------------------------------------------
    else if(uMsg == WM_SYSCOMMAND) //- Received a system menu message.
    {
        if(LOWORD(wParam) == GENERIC_MENU_ABOUT) //- About.
        {
            _AboutDialogInfo ADI;
    
            ADI.DisplayIcon  = LoadIcon(ghAppInstance, MAKEINTRESOURCE(IDI_MAIN));
            ADI.TitleCaption = gsTitleCaption;
            ADI.FileVersion  = gsFileVersion;
            ADI.BuildDate    = __DATE__;
            ADI.BuildTime    = __TIME__;
            ADI.CopyRight    = gsAppCopyRight;
            ADI.OwnerHandle  = hWnd;

            NetLogo(&ADI);

            return TRUE;
        }

        return FALSE;
    }
    //--------------------------------------------------------------------------
    else if(uMsg == WM_CLOSE) //- Received close message.
    {
        EndDialog(hWnd, FALSE);

        return TRUE;
    }

    //--------------------------------------------------------------------------

    return FALSE;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#endif
