///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//  Copyright  NetworkDLS 2002, All rights reserved
//
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF 
// ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO 
// THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A 
// PARTICULAR PURPOSE.
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#ifndef _Config_ScriptingEngines_CPP
#define _Config_ScriptingEngines_CPP
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include <Windows.H>
#include <ShlObj.H>
#include <Stdio.H>
#include <ShlOBJ.H>
#include <Stdlib.H>

#include "../Source/Entry.H"
#include "../Source/CWebSites.H"
#include "../Source/ConfigTree.H"

#include "NetLogoDlg.H"

#include "Config_ScriptingEngines.H"
#include "Config_ScriptingEngines_Item.H"

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

using namespace NSWFL::Windows;
using namespace NSWFL::ListView;

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

INT_PTR CALLBACK Config_ScriptingEngines(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
    //--------------------------------------------------------------------------

    static HWND hList = NULL;
    static HWND hEnabled = NULL;
	static CScriptingEngines *pCI; //Local config item.

    //--------------------------------------------------------------------------

    if(uMsg == WM_INITDIALOG)
    {
		CenterOverOwner(hWnd);

        HMENU hMenu = GetSystemMenu(hWnd, FALSE);
        AppendMenu(hMenu, MF_SEPARATOR, 0, 0);
        AppendMenu(hMenu, MF_STRING, GENERIC_MENU_ABOUT, "About");

        SendMessage(hWnd, WM_SETICON, TRUE, (LPARAM) LoadIcon(ghAppInstance, MAKEINTRESOURCE(IDI_MAIN)));

        hList = GetDlgItem(hWnd, IDC_LIST);
        hEnabled = GetDlgItem(hWnd, IDC_ENABLED);

		pCI = (CScriptingEngines *)gCI.pClass;
		gCI.hList = hList;
		gCI.hDialog = hWnd;

		pCI->PopConfigList(hWnd, IDC_LIST, IDC_ENABLED);

		char sText[255];
		sprintf_s(sText, sizeof(sText), "%s%s", gCI.iSite == INVALID_SITE_INDEX ? "Global " : "", "Scripting Engines");
		Set_Text(GetDlgItem(hWnd, IDC_CONFIGTITLE), sText);
        SendMessage(hWnd, (UINT)WM_SETTEXT, (WPARAM)0, (LPARAM)sText);

		if(gCI.iSite == INVALID_SITE_INDEX)
		{
			ShowWindow(GetDlgItem(hWnd, IDC_USEGLOBAL), SW_HIDE);
			CheckDlgButton(hWnd, IDC_USEGLOBAL, BST_UNCHECKED);
		}
		else{
			if(gWebSites->Site(gCI.iSite)->pScriptingEngines == gWebSites->DfltScriptingEngines)
			{
				CheckDlgButton(hWnd, IDC_USEGLOBAL, BST_CHECKED);
			}
			else{
				CheckDlgButton(hWnd, IDC_USEGLOBAL, BST_UNCHECKED);
			}
			SendMessage(hWnd, WM_COMMAND, IDC_USEGLOBAL, 0);
		}
		
        return TRUE; // Return TRUE to set the keyboard focus, Otherwise return FALSE.
    }
    //--------------------------------------------------------------------------
    else if(uMsg == WM_COMMAND)
    {
		if(wParam == IDC_USEGLOBAL)
		{
			bool bChecked = IsDlgButtonChecked(hWnd, IDC_USEGLOBAL) > 0;
			EnableWindow(hList, !bChecked);
			EnableWindow(GetDlgItem(hWnd, IDC_ENABLED), !bChecked);
			EnableWindow(GetDlgItem(hWnd, IDC_ADD), !bChecked);
			EnableWindow(GetDlgItem(hWnd, IDC_EDIT), !bChecked);
			EnableWindow(GetDlgItem(hWnd, IDC_DELETE), !bChecked);
			EnableWindow(GetDlgItem(hWnd, IDC_MOVEUP), !bChecked);
			EnableWindow(GetDlgItem(hWnd, IDC_MOVEDOWN), !bChecked);
		}
        else if(wParam == IDC_ADD)
        {
			gCI.iListIndex = INVALID_LIST_INDEX;

			DialogBox(ghAppInstance,
				MAKEINTRESOURCE(IDD_CONFIG_SCRIPTINGENGINES_ITEM),
				hWnd, Config_ScriptingEngines_Item);
		}
        else if(wParam == IDC_EDIT)
        {
			if((gCI.iListIndex = LVGetFirstSelectedItemIndex(hList)) >= 0)
			{
				DialogBox(ghAppInstance,
					MAKEINTRESOURCE(IDD_CONFIG_SCRIPTINGENGINES_ITEM),
					hWnd, Config_ScriptingEngines_Item);
			}
        }
        else if(wParam == IDC_DELETE)
        {
			LVDeleteItem(hList);
        }
        else if(wParam == IDC_MOVEUP)
        {
			LVMoveItemUp(hList);
        }
        else if(wParam == IDC_MOVEDOWN)
        {
			LVMoveItemDown(hList);
        }
		else if(wParam == IDC_OK)
        {
			if(gCI.iSite == INVALID_SITE_INDEX)
			{
				//---------------------------------------------------------------------------
				//Editing globally defined values. ------------------------------------------
				//---------------------------------------------------------------------------
				if(!gWebSites->DfltScriptingEngines->LoadNewConfiguration(hWnd, gCI.hList))
				{
					MessageBox(hWnd, "Failed to load the new configuration!", gsTitleCaption, MB_ICONERROR);
					return FALSE;
				}
			}
			else{
				//---------------------------------------------------------------------------
				//Editing site level defined values. ----------------------------------------
				//---------------------------------------------------------------------------
				if(gWebSites->Site(gCI.iSite)->pScriptingEngines == gWebSites->DfltScriptingEngines)
				{
					//---------------------------------------------------------------------------
					//This site inherits its settings from the globally defined values. ---------
					//---------------------------------------------------------------------------
					if(IsDlgButtonChecked(hWnd, IDC_USEGLOBAL))
					{
						//This site uses global settings and the global check box is checked - Nothing to do.
					}
					else{
						bool bIsSiteActive = gWebSites->Site(gCI.iSite)->IsActive();

						if(bIsSiteActive)
						{
							gWebSites->Site(gCI.iSite)->Stop();
						}

						gWebSites->Site(gCI.iSite)->pScriptingEngines = new CScriptingEngines(gWebSites);

						gWebSites->Site(gCI.iSite)->pScriptingEngines->LoadNewConfiguration(hWnd, gCI.hList);

						if(bIsSiteActive)
						{
							gWebSites->Site(gCI.iSite)->Start();
						}
					}
				}
				else{
					//---------------------------------------------------------------------------
					//This site has its own settings explicitly defined. ------------------------
					//---------------------------------------------------------------------------

					if(IsDlgButtonChecked(hWnd, IDC_USEGLOBAL))
					{
						bool bIsSiteActive = gWebSites->Site(gCI.iSite)->IsActive();

						if(bIsSiteActive)
						{
							gWebSites->Site(gCI.iSite)->Stop();
						}

						delete gWebSites->Site(gCI.iSite)->pScriptingEngines;
						gWebSites->Site(gCI.iSite)->pScriptingEngines = gWebSites->DfltScriptingEngines;

						if(bIsSiteActive)
						{
							gWebSites->Site(gCI.iSite)->Start();
						}
					}
					else{
						gWebSites->Site(gCI.iSite)->pScriptingEngines->LoadNewConfiguration(hWnd, gCI.hList);
					}
				}
			}
	
			EndDialog(hWnd, TRUE);
        }
        else if(wParam == IDC_CANCEL || wParam == IDCANCEL)
        {
            EndDialog(hWnd, FALSE);
        }
		return FALSE;
    }
	//--------------------------------------------------------------------------
	else if(uMsg == WM_NOTIFY)
    {
        LPNMHDR pHdr = (LPNMHDR)lParam;

		if(pHdr->hwndFrom == gCI.hList)
        {
			if(pHdr->code == NM_DBLCLK)
			{
				if(ListView_GetItemCount(gCI.hList) > 0 && ListView_GetSelectedCount(gCI.hList) == 1)
				{
					if((gCI.iListIndex = LVGetFirstSelectedItemIndex(hList)) >= 0)
					{
						DialogBox(ghAppInstance,
							MAKEINTRESOURCE(IDD_CONFIG_SCRIPTINGENGINES_ITEM),
							hWnd, Config_ScriptingEngines_Item);
					}
				}
			}
		}
	}
	//--------------------------------------------------------------------------
	else if(uMsg == WM_PAINT)
    {
        HDC ThisHDC;
        PAINTSTRUCT ThisPS;

        ThisHDC = BeginPaint(hWnd, &ThisPS);

        //- Any painting should be done here.

        EndPaint(hWnd, &ThisPS);
        return TRUE;
    }
    //--------------------------------------------------------------------------
    else if(uMsg == WM_SYSCOMMAND) //- Received a system menu message.
    {
        if(LOWORD(wParam) == GENERIC_MENU_ABOUT) //- About.
        {
            _AboutDialogInfo ADI;
    
            ADI.DisplayIcon  = LoadIcon(ghAppInstance, MAKEINTRESOURCE(IDI_MAIN));
            ADI.TitleCaption = gsTitleCaption;
            ADI.FileVersion  = gsFileVersion;
            ADI.BuildDate    = __DATE__;
            ADI.BuildTime    = __TIME__;
            ADI.CopyRight    = gsAppCopyRight;
            ADI.OwnerHandle  = hWnd;

            NetLogo(&ADI);

            return TRUE;
        }

        return FALSE;
    }
    //--------------------------------------------------------------------------
    else if(uMsg == WM_CLOSE) //- Received close message.
    {
        EndDialog(hWnd, FALSE);

        return TRUE;
    }

    //--------------------------------------------------------------------------

    return FALSE;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#endif
