///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//  Copyright  NetworkDLS 2002, All rights reserved
//
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF 
// ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO 
// THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A 
// PARTICULAR PURPOSE.
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#ifndef _Config_Settings_CPP
#define _Config_Settings_CPP
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include <Windows.H>
#include <ShlObj.H>
#include <Stdio.H>
#include <ShlOBJ.H>
#include <Stdlib.H>

#include "../Source/Entry.H"
#include "../Source/CWebSites.H"
#include "../Source/ConfigTree.H"

#include "NetLogoDlg.H"

#include "Config_Settings.H"

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

using namespace NSWFL::Windows;
using namespace NSWFL::ListView;
using namespace NSWFL::Conversion;
using namespace NSWFL::File;

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

INT_PTR CALLBACK Config_Settings(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
    //--------------------------------------------------------------------------

    static HWND hMaxConnections = NULL;
    static HWND hBufferSize = NULL;
    static HWND hScriptTimeout = NULL;
    static HWND hAdminEmail = NULL;
    static HWND hKeepAliveTiemout = NULL;
    static HWND hLogPath = NULL;
    static HWND hNTUser = NULL;
    static HWND hNTDomain = NULL;
    static HWND hNTPass = NULL;
	static CSettings *pCI; //Local config item.

    //--------------------------------------------------------------------------

    if(uMsg == WM_INITDIALOG)
    {
		CenterOverOwner(hWnd);

        HMENU hMenu = GetSystemMenu(hWnd, FALSE);
        AppendMenu(hMenu, MF_SEPARATOR, 0, 0);
        AppendMenu(hMenu, MF_STRING, GENERIC_MENU_ABOUT, "About");

        SendMessage(hWnd, WM_SETICON, TRUE, (LPARAM) LoadIcon(ghAppInstance, MAKEINTRESOURCE(IDI_MAIN)));

		hMaxConnections = GetDlgItem(hWnd, IDC_MAXCONNECTIONS);
		hBufferSize = GetDlgItem(hWnd, IDC_BUFFERSIZE);
		hScriptTimeout = GetDlgItem(hWnd, IDC_SCRIPTTIMEOUT);
		hAdminEmail = GetDlgItem(hWnd, IDC_ADMINEMAIL);
		hKeepAliveTiemout = GetDlgItem(hWnd, IDC_KEEPALIVETIMEOUT);
		//IDC_DIRINDEXING
		//IDC_AUTOSTART
		hLogPath = GetDlgItem(hWnd, IDC_LOGPATH);
		//IDC_BROWSE
		//IDC_CACHELOGFILE
		//IDC_USEGLOBAL
		//IDC_USEKEEPALIVES
        hNTUser = GetDlgItem(hWnd, IDC_NTUSERNAME);
        hNTDomain = GetDlgItem(hWnd, IDC_NTDOMAIN);
        hNTPass = GetDlgItem(hWnd, IDC_NTPASSWORD);

		pCI = (CSettings *)gCI.pClass;
		gCI.hDialog = hWnd;

		char sInt[64];
		Set_Text(hMaxConnections, FormatInteger(sInt, sizeof(sInt), pCI->MaxConnections()));
		CheckDlgButton(hWnd, IDC_DIRINDEXING, pCI->AllowDirectoryIndexing());
		CheckDlgButton(hWnd, IDC_AUTOSTART, pCI->AutoStart());
		Set_Text(hLogPath, pCI->LogPath());
		Set_Text(hBufferSize, FormatInteger(sInt, sizeof(sInt), pCI->BufferSize()));
		CheckDlgButton(hWnd, IDC_CACHELOGFILE, pCI->CacheLogFile());
		Set_Text(hScriptTimeout, FormatInteger(sInt, sizeof(sInt), pCI->ScriptTimeout()));
		CheckDlgButton(hWnd, IDC_USEKEEPALIVES, pCI->UseKeepAlives());
		Set_Text(hKeepAliveTiemout, FormatInteger(sInt, sizeof(sInt), pCI->KeepAliveTimeout()));
		Set_Text(hAdminEmail, pCI->ServerAdminEmail());
		Set_Text(hNTUser, pCI->Username());
		Set_Text(hNTDomain, pCI->Domain());
		Set_Text(hNTPass, pCI->Password());

		if(strlen(pCI->Username()) > 0)
		{
			CheckDlgButton(hWnd, IDC_NTAUTHENABLED, TRUE);
			SendMessage(hWnd, WM_COMMAND, IDC_NTAUTHENABLED, 0);
		}

		char sText[255];
		sprintf_s(sText, sizeof(sText), "%s%s", gCI.iSite == INVALID_SITE_INDEX ? "Global " : "", "Settings");
		Set_Text(GetDlgItem(hWnd, IDC_CONFIGTITLE), sText);
        SendMessage(hWnd, (UINT)WM_SETTEXT, (WPARAM)0, (LPARAM)sText);

		if(gCI.iSite == INVALID_SITE_INDEX)
		{
			ShowWindow(GetDlgItem(hWnd, IDC_USEGLOBAL), SW_HIDE);
			CheckDlgButton(hWnd, IDC_USEGLOBAL, BST_UNCHECKED);
		}
		else{
			if(gWebSites->Site(gCI.iSite)->pSettings == gWebSites->DfltSettings)
			{
				CheckDlgButton(hWnd, IDC_USEGLOBAL, BST_CHECKED);
			}
			else{
				CheckDlgButton(hWnd, IDC_USEGLOBAL, BST_UNCHECKED);
			}
			SendMessage(hWnd, WM_COMMAND, IDC_USEGLOBAL, 0);
		}
		
        return TRUE; // Return TRUE to set the keyboard focus, Otherwise return FALSE.
    }
    //--------------------------------------------------------------------------
    else if(uMsg == WM_COMMAND)
    {
		if(wParam == IDC_NTAUTHENABLED)
        {
			BOOL bEnable = IsDlgButtonChecked(hWnd, IDC_NTAUTHENABLED);
			EnableWindow(hNTUser, bEnable);
			EnableWindow(hNTDomain, bEnable);
			EnableWindow(hNTPass, bEnable);
		}
		else if(wParam == IDC_USEGLOBAL)
		{
			bool bChecked = IsDlgButtonChecked(hWnd, IDC_USEGLOBAL) > 0;
			EnableWindow(GetDlgItem(hWnd, IDC_USEKEEPALIVES), !bChecked);
			EnableWindow(GetDlgItem(hWnd, IDC_DIRINDEXING), !bChecked);
			EnableWindow(GetDlgItem(hWnd, IDC_AUTOSTART), !bChecked);
			EnableWindow(GetDlgItem(hWnd, IDC_CACHELOGFILE), !bChecked);
			EnableWindow(GetDlgItem(hWnd, IDC_BROWSE), !bChecked);

			EnableWindow(hMaxConnections, !bChecked);
			EnableWindow(hBufferSize, !bChecked);
			EnableWindow(hScriptTimeout, !bChecked);
			EnableWindow(hAdminEmail, !bChecked);
			EnableWindow(hKeepAliveTiemout, !bChecked);
			EnableWindow(hLogPath, !bChecked);

			EnableWindow(hNTUser, !bChecked);
			EnableWindow(hNTDomain, !bChecked);
			EnableWindow(hNTPass, !bChecked);
			EnableWindow(GetDlgItem(hWnd, IDC_NTAUTHENABLED), !bChecked);
		}
		else if (wParam == IDC_BROWSE)
		{
			char sFolder[MAX_PATH];
			Get_Text(hLogPath, sFolder, sizeof(sFolder));
			if(BrowseDirectory(hWnd, "Select a directory to be used for log file storage.",
				BIF_RETURNONLYFSDIRS | BIF_NONEWFOLDERBUTTON, sFolder, sizeof(sFolder), sFolder))
			{
				Set_Text(hLogPath, sFolder);
			}
		}
		else if(wParam == IDC_OK)
        {
			//---(Begin Validation)------------------------------------------------------------------
			if(!IsDlgButtonChecked(hWnd, IDC_USEGLOBAL))
			{
				int iValue = 0;
				char sText[MAX_TEXT_SIZE];

				iValue = Get_Long(hMaxConnections);
				if(iValue <= 0 || iValue > FD_SETSIZE)
				{
					sprintf_s(sText, sizeof(sText), "Max connections must be between 1 and %d", FD_SETSIZE);
					MessageBox(hWnd, sText, gsTitleCaption, MB_ICONINFORMATION);
					return TRUE;
				}

				iValue = Get_Long(hBufferSize);
				if(iValue < 1024 || iValue > 8388608)
				{
					MessageBox(hWnd, "Buffer size must be between 1,024 and 8,388,608", gsTitleCaption, MB_ICONINFORMATION);
					return TRUE;
				}

				iValue = Get_Long(hKeepAliveTiemout);
				if(iValue < 5 || iValue > 86400)
				{
					MessageBox(hWnd, "Keep-alive timeout must be between 5 and 86,400", gsTitleCaption, MB_ICONINFORMATION);
					return TRUE;
				}

				iValue = Get_Long(hScriptTimeout);
				if(iValue < 5 || iValue > 86400)
				{
					MessageBox(hWnd, "Script timeout must be between 5 and 86,400", gsTitleCaption, MB_ICONINFORMATION);
					return TRUE;
				}

				iValue = Get_Text(hLogPath, sText, sizeof(sText));
				if(iValue <= 0 || !FileAccess(sText, FExist) || !IsDirectory(sText))
				{
					MessageBox(hWnd, "You must specify a valid log file path.", gsTitleCaption, MB_ICONINFORMATION);
					return TRUE;
				}
			}
			//---(End Validation)------------------------------------------------------------------

			if(gCI.iSite == INVALID_SITE_INDEX)
			{
				//---------------------------------------------------------------------------
				//Editing globally defined values. ------------------------------------------
				//---------------------------------------------------------------------------
				if(!gWebSites->DfltSettings->LoadNewConfiguration(hWnd, gCI.hList))
				{
					MessageBox(hWnd, "Failed to load the new configuration!", gsTitleCaption, MB_ICONERROR);
					return FALSE;
				}
			}
			else{
				//---------------------------------------------------------------------------
				//Editing site level defined values. ----------------------------------------
				//---------------------------------------------------------------------------
				if(gWebSites->Site(gCI.iSite)->pSettings == gWebSites->DfltSettings)
				{
					//---------------------------------------------------------------------------
					//This site inherits its settings from the globally defined values. ---------
					//---------------------------------------------------------------------------
					if(IsDlgButtonChecked(hWnd, IDC_USEGLOBAL))
					{
						//This site uses global settings and the global check box is checked - Nothing to do.
					}
					else{
						bool bIsSiteActive = gWebSites->Site(gCI.iSite)->IsActive();

						if(bIsSiteActive)
						{
							gWebSites->Site(gCI.iSite)->Stop();
						}

						gWebSites->Site(gCI.iSite)->pSettings = new CSettings(gWebSites);

						gWebSites->Site(gCI.iSite)->pSettings->LoadNewConfiguration(hWnd, gCI.hList);

						if(bIsSiteActive)
						{
							gWebSites->Site(gCI.iSite)->Start();
						}
					}
				}
				else{
					//---------------------------------------------------------------------------
					//This site has its own settings explicitly defined. ------------------------
					//---------------------------------------------------------------------------

					if(IsDlgButtonChecked(hWnd, IDC_USEGLOBAL))
					{
						bool bIsSiteActive = gWebSites->Site(gCI.iSite)->IsActive();

						if(bIsSiteActive)
						{
							gWebSites->Site(gCI.iSite)->Stop();
						}

						delete gWebSites->Site(gCI.iSite)->pSettings;
						gWebSites->Site(gCI.iSite)->pSettings = gWebSites->DfltSettings;

						if(bIsSiteActive)
						{
							gWebSites->Site(gCI.iSite)->Start();
						}
					}
					else{
						gWebSites->Site(gCI.iSite)->pSettings->LoadNewConfiguration(hWnd, gCI.hList);
					}
				}
			}

			gWebSites->Save();
	
			EndDialog(hWnd, TRUE);
        }
        else if(wParam == IDC_CANCEL || wParam == IDCANCEL)
        {
            EndDialog(hWnd, FALSE);
        }
		return FALSE;
    }
	//--------------------------------------------------------------------------
	else if(uMsg == WM_PAINT)
    {
        HDC ThisHDC;
        PAINTSTRUCT ThisPS;

        ThisHDC = BeginPaint(hWnd, &ThisPS);

        //- Any painting should be done here.

        EndPaint(hWnd, &ThisPS);
        return TRUE;
    }
    //--------------------------------------------------------------------------
    else if(uMsg == WM_SYSCOMMAND) //- Received a system menu message.
    {
        if(LOWORD(wParam) == GENERIC_MENU_ABOUT) //- About.
        {
            _AboutDialogInfo ADI;
    
            ADI.DisplayIcon  = LoadIcon(ghAppInstance, MAKEINTRESOURCE(IDI_MAIN));
            ADI.TitleCaption = gsTitleCaption;
            ADI.FileVersion  = gsFileVersion;
            ADI.BuildDate    = __DATE__;
            ADI.BuildTime    = __TIME__;
            ADI.CopyRight    = gsAppCopyRight;
            ADI.OwnerHandle  = hWnd;

            NetLogo(&ADI);

            return TRUE;
        }

        return FALSE;
    }
    //--------------------------------------------------------------------------
    else if(uMsg == WM_CLOSE) //- Received close message.
    {
        EndDialog(hWnd, FALSE);

        return TRUE;
    }

    //--------------------------------------------------------------------------

    return FALSE;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#endif
