///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//  Copyright  NetworkDLS 2002, All rights reserved
//
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF 
// ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO 
// THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A 
// PARTICULAR PURPOSE.
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#ifndef _Config_VirtualRoots_Item_CPP
#define _Config_VirtualRoots_Item_CPP
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include <Windows.H>
#include <Stdio.H>
#include <Stdlib.H>
#include <ShlObj.H>

#include "../Source/Entry.H"
#include "../Source/CWebSites.H"
#include "../Source/ConfigTree.H"

#include "../Dialogs/Config_VirtualRoots_Item.H"
#include "../Dialogs/NetLogoDlg.H"

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

using namespace NSWFL::Windows;
using namespace NSWFL::ListView;
using namespace NSWFL::File;

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

INT_PTR CALLBACK Config_VirtualRoots_Item(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
    //--------------------------------------------------------------------------

    static HWND hRoot = NULL;
    static HWND hPath = NULL;
    static HWND hDescr = NULL;
    static HWND hNTUser = NULL;
    static HWND hNTDomain = NULL;
    static HWND hNTPass = NULL;
	static CVirtualRoots *pCI; //Local config item.

    //--------------------------------------------------------------------------

    if(uMsg == WM_INITDIALOG)
    {
		char sText[MAX_TEXT_SIZE];
		CenterOverOwner(hWnd);

        HMENU hMenu = GetSystemMenu(hWnd, FALSE);
        AppendMenu(hMenu, MF_SEPARATOR, 0, 0);
        AppendMenu(hMenu, MF_STRING, GENERIC_MENU_ABOUT, "About");

        SendMessage(hWnd, WM_SETICON, TRUE, (LPARAM) LoadIcon(ghAppInstance, MAKEINTRESOURCE(IDI_MAIN)));

        hRoot		= GetDlgItem(hWnd, IDC_ROOT);
        hPath		= GetDlgItem(hWnd, IDC_PATH);
        hDescr		= GetDlgItem(hWnd, IDC_DESCR);
        hNTUser		= GetDlgItem(hWnd, IDC_NTUSERNAME);
        hNTDomain	= GetDlgItem(hWnd, IDC_NTDOMAIN);
        hNTPass 	= GetDlgItem(hWnd, IDC_NTPASSWORD);

		pCI = (CVirtualRoots *)gCI.pClass;

		if(gCI.iListIndex != INVALID_LIST_INDEX) //Editing an item.
		{
			//We have a list index, so we must be editing.

			LVGetItemText(gCI.hList, gCI.iListIndex, VIRTUALROOTS_LIST_POS_ROOT, sText, sizeof(sText));
			Set_Text(hRoot, sText);

			LVGetItemText(gCI.hList, gCI.iListIndex, VIRTUALROOTS_LIST_POS_PATH, sText, sizeof(sText));
			Set_Text(hPath, sText);

			LVGetItemText(gCI.hList, gCI.iListIndex, VIRTUALROOTS_LIST_POS_DESCR, sText, sizeof(sText));
			Set_Text(hDescr, sText);

			LVGetItemText(gCI.hList, gCI.iListIndex, VIRTUALROOTS_LIST_POS_USER, sText, sizeof(sText));
			Set_Text(hNTUser, sText);
			bool bEnableAuth = (strlen(sText) > 0);

			LVGetItemText(gCI.hList, gCI.iListIndex, VIRTUALROOTS_LIST_POS_DOMAIN, sText, sizeof(sText));
			Set_Text(hNTDomain, sText);

			LVGetItemText(gCI.hList, gCI.iListIndex, VIRTUALROOTS_LIST_POS_PASS, sText, sizeof(sText));
			Set_Text(hNTPass, sText);

			if(bEnableAuth)
			{
				CheckDlgButton(hWnd, IDC_NTAUTHENABLED, TRUE);
				SendMessage(hWnd, WM_COMMAND, IDC_NTAUTHENABLED, 0);
			}

			CheckDlgButton(hWnd, IDC_ENABLED, LVChecked(gCI.hList, gCI.iListIndex));
		}
		else{
			//Adding an item.
			CheckDlgButton(hWnd, IDC_ENABLED, BST_CHECKED);
		}

		sprintf_s(sText, sizeof(sText), "%s%s", gCI.iSite == INVALID_SITE_INDEX ? "Global " : "", "Virtual Root");
		Set_Text(GetDlgItem(hWnd, IDC_CONFIGTITLE), sText);
        SendMessage(hWnd, (UINT)WM_SETTEXT, (WPARAM)0, (LPARAM)sText);
		
		return TRUE; // Return TRUE to set the keyboard focus, Otherwise return FALSE.
    }
    //--------------------------------------------------------------------------
    else if(uMsg == WM_COMMAND)
    {
		if(wParam == IDC_NTAUTHENABLED)
        {
			BOOL bEnable = IsDlgButtonChecked(hWnd, IDC_NTAUTHENABLED);
			EnableWindow(hNTUser, bEnable);
			EnableWindow(hNTDomain, bEnable);
			EnableWindow(hNTPass, bEnable);
		}
		else if(wParam == IDC_BROWSE)
        {
			char sFolder[MAX_PATH];
			Get_Text(hPath, sFolder, sizeof(sFolder));
			if(BrowseDirectory(hWnd, "Select a Local Directory for the Virtual Root",
				BIF_RETURNONLYFSDIRS | BIF_NONEWFOLDERBUTTON, sFolder, sizeof(sFolder), sFolder))
			{
				Set_Text(hPath, sFolder);
			}
		}
		else if(wParam == IDC_OK)
        {
			ROOT Item;

			char sURI[VIRTUALROOTS_MAX_ROOT_LENGTH];
			char sPath[VIRTUALROOTS_MAX_PATH_LENGTH];
			char sDescr[VIRTUALROOTS_MAX_DESCR_LENGTH];

			Get_Text(hRoot, sURI, sizeof(sURI));
			Get_Text(hPath, sPath, sizeof(sPath));
			Get_Text(hDescr, sDescr, sizeof(sDescr));

			if(strlen(sURI) == 0)
			{
				MessageBox(hWnd, "You must specify a web path.", gsTitleCaption, MB_ICONINFORMATION);
				return FALSE;
			}
			if(strlen(sPath) == 0)
			{
				MessageBox(hWnd, "You must specify a file path.", gsTitleCaption, MB_ICONINFORMATION);
				return FALSE;
			}

			CorrectForwardPath(sURI, sizeof(sURI));
			if(sURI[0] != '/')
			{
				char sTmp[sizeof(sURI)];
				sprintf_s(sTmp, sizeof(sTmp), "/%s", sURI);
				strcpy_s(sURI, sizeof(sURI), sTmp);
			}

			if(IsDlgButtonChecked(hWnd, IDC_NTAUTHENABLED))
			{
				char sUser[VIRTUALROOTS_MAX_USER_LENGTH];
				char sDomain[VIRTUALROOTS_MAX_DOMAIN_LENGTH];
				char sPass[VIRTUALROOTS_MAX_PASS_LENGTH];

				Get_Text(hNTUser, sUser, sizeof(sUser));
				Get_Text(hNTDomain, sDomain, sizeof(sDomain));
				Get_Text(hNTPass, sPass, sizeof(sPass));

				Item.Username = sUser;
				Item.Domain = sDomain;
				Item.Password = sPass;
			}
			else{
				Item.Username = NULL;
				Item.Domain = NULL;
				Item.Password = NULL;
			}

			Item.Root = sURI;
			Item.RootLength = (int)strlen(Item.Root);
			Item.Path = sPath;
			Item.PathLength = (int)strlen(Item.Path);
			Item.Description = sDescr;
			Item.Enabled = (IsDlgButtonChecked(hWnd, IDC_ENABLED) > 0);

			if(gCI.iListIndex != INVALID_LIST_INDEX) //Editing / Adding
			{
				//We have a list index, so we must be editing.
				LVDeleteItem(gCI.hList, gCI.iListIndex);
				gWebSites->DfltVirtualRoots->AddConfigItem(gCI.hList, &Item, gCI.iListIndex);
			}
			else {
				//We DO NOT have a list index, so we must be adding.
				gWebSites->DfltVirtualRoots->AddConfigItem(gCI.hList, &Item);
			}

            EndDialog(hWnd, TRUE);
            DestroyWindow(hWnd);
        }
        else if(wParam == IDC_CANCEL || wParam == IDCANCEL)
        {
            EndDialog(hWnd, FALSE);
            DestroyWindow(hWnd);
        }
        return FALSE;
    }
    //--------------------------------------------------------------------------
	else if(uMsg == WM_PAINT)
    {
        HDC ThisHDC;
        PAINTSTRUCT ThisPS;

        ThisHDC = BeginPaint(hWnd, &ThisPS);

        //- Any painting should be done here.

        EndPaint(hWnd, &ThisPS);
        return TRUE;
    }
    //--------------------------------------------------------------------------
    else if(uMsg == WM_SYSCOMMAND) //- Received a system menu message.
    {
        if(LOWORD(wParam) == GENERIC_MENU_ABOUT) //- About.
        {
            _AboutDialogInfo ADI;
    
            ADI.DisplayIcon  = LoadIcon(ghAppInstance, MAKEINTRESOURCE(IDI_MAIN));
            ADI.TitleCaption = gsTitleCaption;
            ADI.FileVersion  = gsFileVersion;
            ADI.BuildDate    = __DATE__;
            ADI.BuildTime    = __TIME__;
            ADI.CopyRight    = gsAppCopyRight;
            ADI.OwnerHandle  = hWnd;

            NetLogo(&ADI);

            return TRUE;
        }

        return FALSE;
    }
    //--------------------------------------------------------------------------
    else if(uMsg == WM_CLOSE) //- Received close message.
    {
        EndDialog(hWnd, 0);
        DestroyWindow(hWnd);
        return TRUE;
    }
    //--------------------------------------------------------------------------

    return FALSE;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#endif
