///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//  Copyright  NetworkDLS 2002, All rights reserved
//
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF 
// ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO 
// THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A 
// PARTICULAR PURPOSE.
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#ifndef _Config_WebSites_CPP
#define _Config_WebSites_CPP
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include <Windows.H>
#include <ShlObj.H>
#include <Stdio.H>
#include <ShlOBJ.H>
#include <Stdlib.H>

#include "../Source/Entry.H"
#include "../Source/CWebSites.H"
#include "../Source/ConfigTree.H"

#include "NetLogoDlg.H"

#include "Config_WebSites.H"
#include "Config_WebSites_Item.H"

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

using namespace NSWFL::Windows;
using namespace NSWFL::ListView;

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

INT_PTR CALLBACK Config_WebSites(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
    //--------------------------------------------------------------------------

    static HWND hList = NULL;
    static HWND hEnabled = NULL;
	static CWebSites *pCI; //Local config item.

    //--------------------------------------------------------------------------

    if(uMsg == WM_INITDIALOG)
    {
		CenterOverOwner(hWnd);

        HMENU hMenu = GetSystemMenu(hWnd, FALSE);
        AppendMenu(hMenu, MF_SEPARATOR, 0, 0);
        AppendMenu(hMenu, MF_STRING, GENERIC_MENU_ABOUT, "About");

        SendMessage(hWnd, WM_SETICON, TRUE, (LPARAM) LoadIcon(ghAppInstance, MAKEINTRESOURCE(IDI_MAIN)));

        hList = GetDlgItem(hWnd, IDC_LIST);
        hEnabled = GetDlgItem(hWnd, IDC_ENABLED);

		pCI = (CWebSites *)gCI.pClass;
		gCI.hList = hList;
		gCI.hDialog = hWnd;

		pCI->PopConfigList(hWnd, IDC_LIST);

		Set_Text(GetDlgItem(hWnd, IDC_CONFIGTITLE), "Web Sites");
        SendMessage(hWnd, (UINT)WM_SETTEXT, (WPARAM)0, (LPARAM)"Web Sites");
		ShowWindow(GetDlgItem(hWnd, IDC_USEGLOBAL), SW_HIDE);
		ShowWindow(GetDlgItem(hWnd, IDC_ENABLED), SW_HIDE);
	}
    //--------------------------------------------------------------------------
    else if(uMsg == WM_COMMAND)
    {
        if(wParam == IDC_ADD)
        {
			gCI.iListIndex = INVALID_LIST_INDEX;
			gCI.iSite = INVALID_SITE_INDEX;

			DialogBox(ghAppInstance,
				MAKEINTRESOURCE(IDD_CONFIG_WEBSITES_ITEM),
				hWnd, Config_WebSites_Item);
		}
        else if(wParam == IDC_EDIT)
        {
			if((gCI.iListIndex = LVGetFirstSelectedItemIndex(hList)) >= 0)
			{
				char sIndex[64];
				LVGetText(hList, gCI.iListIndex, WEBSITES_LIST_POS_INDEX, sIndex, sizeof(sIndex));
				gCI.iSite = atol(sIndex);

				DialogBox(ghAppInstance,
					MAKEINTRESOURCE(IDD_CONFIG_WEBSITES_ITEM),
					hWnd, Config_WebSites_Item);
			}
        }
        else if(wParam == IDC_DELETE)
        {
			LVDeleteItem(hList);
        }
        else if(wParam == IDC_MOVEUP)
        {
			LVMoveItemUp(hList);
        }
        else if(wParam == IDC_MOVEDOWN)
        {
			LVMoveItemDown(hList);
        }
		else if(wParam == IDC_OK)
        {
			if(!pCI->SaveNewConfiguration(hWnd, gCI.hList))
			{
				MessageBox(hWnd, "Failed to save the new configuration!", gsTitleCaption, MB_ICONERROR);
				return FALSE;
			}

			if(!pCI->ReloadSites())
			{
				MessageBox(hWnd, "Failed to load the new configuration!", gsTitleCaption, MB_ICONERROR);
				return FALSE;
			}

			EndDialog(hWnd, TRUE);
        }
        else if(wParam == IDC_CANCEL || wParam == IDCANCEL)
        {
            EndDialog(hWnd, FALSE);
        }
		return FALSE;
    }
	//--------------------------------------------------------------------------
	else if(uMsg == WM_NOTIFY)
    {
        LPNMHDR pHdr = (LPNMHDR)lParam;

		if(pHdr->hwndFrom == gCI.hList)
        {
			if(pHdr->code == NM_DBLCLK)
			{
				if(ListView_GetItemCount(gCI.hList) > 0 && ListView_GetSelectedCount(gCI.hList) == 1)
				{
					if((gCI.iListIndex = LVGetFirstSelectedItemIndex(hList)) >= 0)
					{
						char sIndex[64];
						LVGetText(hList, gCI.iListIndex, WEBSITES_LIST_POS_INDEX, sIndex, sizeof(sIndex));
						gCI.iSite = atol(sIndex);

						DialogBox(ghAppInstance,
							MAKEINTRESOURCE(IDD_CONFIG_WEBSITES_ITEM),
							hWnd, Config_WebSites_Item);
					}
				}
			}
		}
	}
	//--------------------------------------------------------------------------
	else if(uMsg == WM_PAINT)
    {
        HDC ThisHDC;
        PAINTSTRUCT ThisPS;

        ThisHDC = BeginPaint(hWnd, &ThisPS);

        //- Any painting should be done here.

        EndPaint(hWnd, &ThisPS);
        return TRUE;
    }
    //--------------------------------------------------------------------------
    else if(uMsg == WM_SYSCOMMAND) //- Received a system menu message.
    {
        if(LOWORD(wParam) == GENERIC_MENU_ABOUT) //- About.
        {
            _AboutDialogInfo ADI;
    
            ADI.DisplayIcon  = LoadIcon(ghAppInstance, MAKEINTRESOURCE(IDI_MAIN));
            ADI.TitleCaption = gsTitleCaption;
            ADI.FileVersion  = gsFileVersion;
            ADI.BuildDate    = __DATE__;
            ADI.BuildTime    = __TIME__;
            ADI.CopyRight    = gsAppCopyRight;
            ADI.OwnerHandle  = hWnd;

            NetLogo(&ADI);

            return TRUE;
        }

        return FALSE;
    }
    //--------------------------------------------------------------------------
    else if(uMsg == WM_CLOSE) //- Received close message.
    {
        EndDialog(hWnd, FALSE);

        return TRUE;
    }

    //--------------------------------------------------------------------------

    return FALSE;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#endif
