///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//  Copyright  NetworkDLS 2002, All rights reserved
//
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF 
// ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO 
// THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A 
// PARTICULAR PURPOSE.
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#ifndef _Config_WebSites_Item_CPP
#define _Config_WebSites_Item_CPP
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include <Windows.H>
#include <Stdio.H>
#include <Stdlib.H>
#include <ShlObj.H>

#include "../Source/Entry.H"
#include "../Source/CWebSites.H"
#include "../Source/ConfigTree.H"

#include "../Dialogs/Config_WebSites_Item.H"
#include "../Dialogs/NetLogoDlg.H"

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

using namespace NSWFL::Windows;
using namespace NSWFL::ListView;

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

INT_PTR CALLBACK Config_WebSites_Item(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
    //--------------------------------------------------------------------------

    static HWND hPort = NULL;
    static HWND hRoot = NULL;
    static HWND hName = NULL;
    static HWND hDescr = NULL;
	static CWebSites *pCI; //Local config item.

    //--------------------------------------------------------------------------

    if(uMsg == WM_INITDIALOG)
    {
		char sText[MAX_TEXT_SIZE];
		CenterOverOwner(hWnd);

        HMENU hMenu = GetSystemMenu(hWnd, FALSE);
        AppendMenu(hMenu, MF_SEPARATOR, 0, 0);
        AppendMenu(hMenu, MF_STRING, GENERIC_MENU_ABOUT, "About");

        SendMessage(hWnd, WM_SETICON, TRUE, (LPARAM) LoadIcon(ghAppInstance, MAKEINTRESOURCE(IDI_MAIN)));

        hName	= GetDlgItem(hWnd, IDC_NAME);
        hPort	= GetDlgItem(hWnd, IDC_PORT);
        hRoot	= GetDlgItem(hWnd, IDC_ROOT);
        hDescr	= GetDlgItem(hWnd, IDC_DESCR);

		pCI = (CWebSites *)gCI.pClass;

		if(gCI.iListIndex != INVALID_LIST_INDEX)
		{
			//Editing site from list.
			LVGetItemText(gCI.hList, gCI.iListIndex, WEBSITES_LIST_POS_NAME, sText, sizeof(sText));
			Set_Text(hName, sText);

			LVGetItemText(gCI.hList, gCI.iListIndex, WEBSITES_LIST_POS_PORT, sText, sizeof(sText));
			Set_Text(hPort, sText);

			LVGetItemText(gCI.hList, gCI.iListIndex, WEBSITES_LIST_POS_DESCR, sText, sizeof(sText));
			Set_Text(hDescr, sText);

			LVGetItemText(gCI.hList, gCI.iListIndex, WEBSITES_LIST_POS_ROOT, sText, sizeof(sText));
			Set_Text(hRoot, sText);
		}
		else if(gCI.iSite != INVALID_SITE_INDEX) {
			//Editing item from treeview.
			Set_Text(hName, pCI->Site(gCI.iSite)->Name);
			Set_Long(hPort, pCI->Site(gCI.iSite)->Port);
			Set_Text(hDescr, pCI->Site(gCI.iSite)->Description);
			Set_Text(hRoot, pCI->Site(gCI.iSite)->Root);
		}

		Set_Text(GetDlgItem(hWnd, IDC_CONFIGTITLE), "Web Site");
        SendMessage(hWnd, (UINT)WM_SETTEXT, (WPARAM)0, (LPARAM)"Web Site");

		return TRUE; // Return TRUE to set the keyboard focus, Otherwise return FALSE.
    }
    //--------------------------------------------------------------------------
    else if(uMsg == WM_COMMAND)
    {
		if(wParam == IDC_BROWSE)
        {
			char sFolder[MAX_PATH];
			Get_Text(hRoot, sFolder, sizeof(sFolder));
			if(BrowseDirectory(hWnd, "Select a Local Directory for the website Root",
				BIF_RETURNONLYFSDIRS | BIF_NONEWFOLDERBUTTON, sFolder, sizeof(sFolder), sFolder))
			{
				Set_Text(hRoot, sFolder);
			}
		}
		else if(wParam == IDC_OK)
        {
			CWebSite Item;

			char sName[MAX_SITE_NAME];
			char sRoot[MAX_PATH];
			char sDescr[MAX_SITE_DESCR];

			Get_Text(hName, sName, sizeof(sName));
			Get_Text(hRoot, sRoot, sizeof(sRoot));
			Get_Text(hDescr, sDescr, sizeof(sDescr));
			int iPort = Get_Long(hPort);

			if(strlen(sName) <= 0)
			{
				MessageBox(hWnd, "You must specify site name.", gsTitleCaption, MB_ICONINFORMATION);
				return TRUE;
			}
			if(iPort <= 0)
			{
				MessageBox(hWnd, "You must specify a valid server port.", gsTitleCaption, MB_ICONINFORMATION);
				return TRUE;
			}
			if(strlen(sRoot) <= 0)
			{
				MessageBox(hWnd, "You must specify a root folder.", gsTitleCaption, MB_ICONINFORMATION);
				return TRUE;
			}

			strcpy_s(Item.Name, sizeof(Item.Name), sName);
			strcpy_s(Item.Root, sizeof(Item.Root), sRoot);
			strcpy_s(Item.Description, sizeof(Item.Description), sDescr);
			Item.Port = iPort;

			if(gCI.iListIndex != INVALID_LIST_INDEX)
			{
				//We have a list index, so we must be editing.
				LVDeleteItem(gCI.hList, gCI.iListIndex);
				((CWebSites *)gCI.pClass)->AddConfigItem(gCI.hList, &Item, gCI.iSite, gCI.iListIndex);
			}
			else {
				if(gCI.iSite == INVALID_SITE_INDEX)
				{
					//We DO NOT have a list index, so we must be adding.
					((CWebSites *)gCI.pClass)->AddConfigItem(gCI.hList, &Item, gCI.iSite);
				}
				else{
					//Editing item from the treeview.
					bool bRunning = pCI->Site(gCI.iSite)->IsActive();

					if(bRunning)
					{
						pCI->Site(gCI.iSite)->Stop();
					}

					Get_Text(hName, pCI->Collection.WebSite[gCI.iSite]->Name,
						sizeof(pCI->Collection.WebSite[gCI.iSite]->Name));
					Get_Text(hDescr, pCI->Collection.WebSite[gCI.iSite]->Description,
						sizeof(pCI->Collection.WebSite[gCI.iSite]->Description));
					Get_Text(hRoot, pCI->Collection.WebSite[gCI.iSite]->Root,
						sizeof(pCI->Collection.WebSite[gCI.iSite]->Root));
					pCI->Collection.WebSite[gCI.iSite]->Port = Get_Long(hPort);

					if(bRunning)
					{
						pCI->Site(gCI.iSite)->Start();
					}

					pCI->Save();
				}
			}

            EndDialog(hWnd, TRUE);
        }
        else if(wParam == IDC_CANCEL || wParam == IDCANCEL)
        {
            EndDialog(hWnd, FALSE);
            DestroyWindow(hWnd);
        }
        return FALSE;
    }
    //--------------------------------------------------------------------------
	else if(uMsg == WM_PAINT)
    {
        HDC ThisHDC;
        PAINTSTRUCT ThisPS;

        ThisHDC = BeginPaint(hWnd, &ThisPS);

        //- Any painting should be done here.

        EndPaint(hWnd, &ThisPS);
        return TRUE;
    }
    //--------------------------------------------------------------------------
    else if(uMsg == WM_SYSCOMMAND) //- Received a system menu message.
    {
        if(LOWORD(wParam) == GENERIC_MENU_ABOUT) //- About.
        {
            _AboutDialogInfo ADI;
    
            ADI.DisplayIcon  = LoadIcon(ghAppInstance, MAKEINTRESOURCE(IDI_MAIN));
            ADI.TitleCaption = gsTitleCaption;
            ADI.FileVersion  = gsFileVersion;
            ADI.BuildDate    = __DATE__;
            ADI.BuildTime    = __TIME__;
            ADI.CopyRight    = gsAppCopyRight;
            ADI.OwnerHandle  = hWnd;

            NetLogo(&ADI);

            return TRUE;
        }

        return FALSE;
    }
    //--------------------------------------------------------------------------
    else if(uMsg == WM_CLOSE) //- Received close message.
    {
        EndDialog(hWnd, 0);
        DestroyWindow(hWnd);
        return TRUE;
    }
    //--------------------------------------------------------------------------

    return FALSE;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#endif
