///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//  Copyright  NetworkDLS 2002, All rights reserved
//
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF 
// ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO 
// THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A 
// PARTICULAR PURPOSE.
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#ifndef _MAINDLG_CPP
#define _MAINDLG_CPP
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include <Windows.H>
#include <ShlObj.H>
#include <Stdio.H>
#include <ShlOBJ.H>
#include <Stdlib.H>

#include "../Source/Entry.H"
#include "../Source/CWebSites.H"
#include "../Source/ConfigTree.H"

#include "../Dialogs/MainDlg.H"
#include "../Dialogs/NetLogoDlg.H"

#include "../../@Classes/CTrayIcon/CTrayIcon.H"

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

using namespace NSWFL::Windows;
using namespace NSWFL::ListView;

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

INT_PTR CALLBACK MainDialog(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);

#define ID_TRAY_CONFIGURE	WM_USER + 1
#define ID_TRAY_SHUTDOWN	WM_USER + 2

//---------------------(Variable Declarations)
HWND hMainDialog = NULL;
HIMAGELIST hOnePixilImageList = NULL;
HIMAGELIST hEnableDisableImageList = NULL;

int iLastPageType = 0;
int iLastSiteIndex = 0;

CTrayIcon gTrayIcon;

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

DWORD WINAPI MainDialogProcThread(LPVOID pVoid)
{
	return (DWORD)DialogBox(ghAppInstance, MAKEINTRESOURCE(IDD_MAIN), NULL, MainDialog);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void LoadMainDialog(void)
{
	if(hMainDialog)
	{
		ShowWindow(hMainDialog, SW_SHOW);
		SetForegroundWindow(hMainDialog);
		SetFocus(hMainDialog);
	}
	else {
		if(gService.bRunningAsService)
		{
			CreateThread(NULL, 0, MainDialogProcThread, NULL, 0, NULL);
		}
		else{
			DialogBox(ghAppInstance, MAKEINTRESOURCE(IDD_MAIN), NULL, MainDialog);
		}
	}
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void CloseMainDialog(void)
{
	if(hMainDialog)
	{
		EndDialog(hMainDialog, 0);
		hMainDialog = NULL;
	}
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void ProgressTrayIconProc(CTrayIcon *lpTI, HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	static int iLastIcon = 0;

	if(uMsg == WM_CTRAYICON_INITIALIZE)
	{

	}
	else if(uMsg == WM_COMMAND)
    {
		if(LOWORD(wParam) == ID_TRAY_SHUTDOWN)
        {
			if(MessageBox(GetActiveWindow(), "Are you sure you want to shutdown the Web Server?",
				gsTitleCaption, MB_YESNO|MB_DEFBUTTON2|MB_ICONQUESTION) == IDYES)
			{
				CloseMainDialog();
				gTrayIcon.Destroy();
			}
		}
		else if(LOWORD(wParam) == ID_TRAY_CONFIGURE)
        {
			gTrayIcon.Destroy();
		}
    }
    else if(uMsg == WM_SHELLNOTIFY)
    {
		if(lParam == WM_LBUTTONDBLCLK)
		{
			gTrayIcon.Destroy();
		}
    }
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool PopDetailPane(HWND hTreeView, HWND hListView, int iPageType, int iSiteIndex)
{
	iLastPageType = iPageType;
	iLastSiteIndex = iSiteIndex;

	LVClear(hListView);

	if(!hEnableDisableImageList)
	{
		HICON hIcon = NULL;

		hEnableDisableImageList = ImageList_Create(
			GetSystemMetrics(SM_CXSMICON), GetSystemMetrics(SM_CYSMICON), ILC_MASK|ILC_COLOR32, 1, 1);

		hIcon = LoadIcon(ghAppInstance, MAKEINTRESOURCE(IDI_DISABLED));
		ImageList_AddIcon(hEnableDisableImageList, hIcon);

		hIcon = LoadIcon(ghAppInstance, MAKEINTRESOURCE(IDI_ENABLED));
		ImageList_AddIcon(hEnableDisableImageList, hIcon);
	}

	if(!hOnePixilImageList)
	{
		hOnePixilImageList = ImageList_Create(1, 1, ILC_MASK|ILC_COLOR32, 1, 1);
	}

	if(iPageType == PAGE_TYPE_INVALID)
	{
		return TRUE;
	}
	else if(iSiteIndex == INVALID_SITE_INDEX)
	{
		//Global settings...
		if(iPageType == PAGE_TYPE_SERVER)
			gWebSites->PopPerformanceList(hListView);
		else if(iPageType == PAGE_TYPE_WEBSITES)
			gWebSites->PopInfoList(hListView);
		else if(iPageType == PAGE_TYPE_SETTINGS)
			gWebSites->DfltSettings->PopInfoList(hListView);
		else if(iPageType == PAGE_TYPE_DEFAULTPAGES)
			gWebSites->DfltDefaultPages->PopInfoList(hListView);
		else if(iPageType == PAGE_TYPE_VIRTUAL_ROOTS)
			gWebSites->DfltVirtualRoots->PopInfoList(hListView);
		else if(iPageType == PAGE_TYPE_COMPRESSION)
			gWebSites->DfltCompression->PopInfoList(hListView);
		else if(iPageType == PAGE_TYPE_MIMETYPES)
			gWebSites->DfltMimeTypes->PopInfoList(hListView);
		else if(iPageType == PAGE_TYPE_IPFILTERS)
			gWebSites->DfltIPFilters->PopInfoList(hListView);
		else if(iPageType == PAGE_TYPE_URLFILTERS)
			gWebSites->DfltURLFilters->PopInfoList(hListView);
		else if(iPageType == PAGE_TYPE_EXTFILTERS)
			gWebSites->DfltExtensionFilters->PopInfoList(hListView);
		else if(iPageType == PAGE_TYPE_CUSTOMFILTERS)
			gWebSites->DfltCustomFilters->PopInfoList(hListView);
		else if(iPageType == PAGE_TYPE_SCRIPTINGENGINES)
			gWebSites->DfltScriptingEngines->PopInfoList(hListView);
		else if(iPageType == PAGE_TYPE_REALMS)
			gWebSites->DfltRealms->PopInfoList(hListView);
		else if(iPageType == PAGE_TYPE_AUTHENTICATION)
			gWebSites->DfltAuthentication->PopInfoList(hListView);
		else if(iPageType == PAGE_TYPE_SSIFILES)
			gWebSites->DfltSSIFiles->PopInfoList(hListView);
		else if(iPageType == PAGE_TYPE_CGIFOLDERS)
			gWebSites->DfltCGIFolders->PopInfoList(hListView);
		else if(iPageType == PAGE_TYPE_ERRORPAGES)
			gWebSites->DfltErrorPages->PopInfoList(hListView);
	}
	else{
		//Site settings.
		if(iPageType == PAGE_TYPE_WEBSITE)
			gWebSites->Site(iSiteIndex)->PopInfoList(hListView);
		if(iPageType == PAGE_TYPE_SETTINGS)
			gWebSites->Site(iSiteIndex)->pSettings->PopInfoList(hListView);
		else if(iPageType == PAGE_TYPE_DEFAULTPAGES)
			gWebSites->Site(iSiteIndex)->pDefaultPages->PopInfoList(hListView);
		else if(iPageType == PAGE_TYPE_VIRTUAL_ROOTS)
			gWebSites->Site(iSiteIndex)->pVirtualRoots->PopInfoList(hListView);
		else if(iPageType == PAGE_TYPE_CACHE)
			gWebSites->Site(iSiteIndex)->pCompression->PopCacheList(hListView);
		else if(iPageType == PAGE_TYPE_COMPRESSION)
			gWebSites->Site(iSiteIndex)->pCompression->PopInfoList(hListView);
		else if(iPageType == PAGE_TYPE_MIMETYPES)
			gWebSites->Site(iSiteIndex)->pMimeTypes->PopInfoList(hListView);
		else if(iPageType == PAGE_TYPE_IPFILTERS)
			gWebSites->Site(iSiteIndex)->pIPFilters->PopInfoList(hListView);
		else if(iPageType == PAGE_TYPE_URLFILTERS)
			gWebSites->Site(iSiteIndex)->pURLFilters->PopInfoList(hListView);
		else if(iPageType == PAGE_TYPE_EXTFILTERS)
			gWebSites->Site(iSiteIndex)->pExtensionFilters->PopInfoList(hListView);
		else if(iPageType == PAGE_TYPE_CUSTOMFILTERS)
			gWebSites->Site(iSiteIndex)->pCustomFilters->PopInfoList(hListView);
		else if(iPageType == PAGE_TYPE_SCRIPTINGENGINES)
			gWebSites->Site(iSiteIndex)->pScriptingEngines->PopInfoList(hListView);
		else if(iPageType == PAGE_TYPE_REALMS)
			gWebSites->Site(iSiteIndex)->pRealms->PopInfoList(hListView);
		else if(iPageType == PAGE_TYPE_AUTHENTICATION)
			gWebSites->Site(iSiteIndex)->pAuthentication->PopInfoList(hListView);
		else if(iPageType == PAGE_TYPE_SSIFILES)
			gWebSites->Site(iSiteIndex)->pSSIFiles->PopInfoList(hListView);
		else if(iPageType == PAGE_TYPE_CGIFOLDERS)
			gWebSites->Site(iSiteIndex)->pCGIFolders->PopInfoList(hListView);
		else if(iPageType == PAGE_TYPE_ERRORPAGES)
			gWebSites->Site(iSiteIndex)->pErrorPages->PopInfoList(hListView);
	}

	return true;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

INT_PTR CALLBACK MainDialog(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
    //--------------------------------------------------------------------------

    static HWND hTreeView = NULL;
    static HWND hListView = NULL;
    static HWND hStatusBar = NULL;

    //--------------------------------------------------------------------------

    if(uMsg == WM_INITDIALOG)
    {
        hMainDialog = hWnd;

        HMENU hMenu = GetSystemMenu(hWnd, FALSE);
        AppendMenu(hMenu, MF_SEPARATOR, 0, 0);
        AppendMenu(hMenu, MF_STRING, GENERIC_MENU_ABOUT, "About");

        SendMessage(hWnd, (UINT)WM_SETTEXT, (WPARAM)0, (LPARAM)gsTitleCaption);
        SendMessage(hWnd, WM_SETICON, TRUE, (LPARAM) LoadIcon(ghAppInstance, MAKEINTRESOURCE(IDI_MAIN)));

        hTreeView = GetDlgItem(hWnd, IDC_SITES);
        hListView = GetDlgItem(hWnd, IDC_DETAIL);
		hStatusBar = CreateStatusWindow(WS_CHILD|WS_VISIBLE, "", hWnd, 1);

        int Widths[] = {200, 400, 500, -1};
        SendMessage(hStatusBar, (UINT)SB_SETPARTS, (WPARAM)4, (LPARAM)&Widths);

		RECT rcWindow;
		memset(&rcWindow, 0, sizeof(rcWindow));
        GetClientRect(hWnd, &rcWindow);
		SendMessage(hWnd, WM_SIZE, 0, MAKELONG(rcWindow.right, rcWindow.bottom));

		CenterOverOwner(hWnd);

		iLastPageType = -1;
		iLastSiteIndex = -1;

		InitializeConfigTree(hTreeView);

		SetTimer(hWnd, 0, 2500, NULL);

        return TRUE; // Return TRUE to set the keyboard focus, Otherwise return FALSE.
    }
    //--------------------------------------------------------------------------
    else if(uMsg == WM_COMMAND)
    {
		if(wParam == ID_HELP_ABOUT)
		{
            _AboutDialogInfo ADI;
    
            ADI.DisplayIcon  = LoadIcon(ghAppInstance, MAKEINTRESOURCE(IDI_MAIN));
            ADI.TitleCaption = gsTitleCaption;
            ADI.FileVersion  = gsFileVersion;
            ADI.BuildDate    = __DATE__;
            ADI.BuildTime    = __TIME__;
            ADI.CopyRight    = gsAppCopyRight;
            ADI.OwnerHandle  = hWnd;

            NetLogo(&ADI);
		}
		else if(wParam == ID_FILE_EXIT)
		{
			if(gService.bRunningAsService)
			{
				ShowWindow(hWnd, SW_HIDE);
			}
			else{
				if(MessageBox(hWnd,
					"Are you sure you want to shutdown the Web Server?",
					gsTitleCaption, MB_YESNO|MB_DEFBUTTON2|MB_ICONQUESTION) == IDYES)
				{
					hMainDialog = NULL;
					EndDialog(hWnd, 0);
				}
			}
		}
        return FALSE;
    }
	//--------------------------------------------------------------------------
    else if(uMsg == WM_TIMER)
    {
		if(iLastPageType == PAGE_TYPE_SERVER && iLastSiteIndex == INVALID_SITE_INDEX)
		{
			gWebSites->RefreshPerformanceList(hListView);
		}
        return FALSE;
    }
    //--------------------------------------------------------------------------
	else if(uMsg == WM_SIZE)
    {
		int nWidth = LOWORD(lParam);
        int nHeight = HIWORD(lParam);

		MoveWindow(hTreeView, 5, 5, 250, nHeight - 30, TRUE);
		MoveWindow(hListView, 260, 5, nWidth - 265, nHeight - 30, TRUE);
		MoveWindow(hStatusBar, nWidth-20, nHeight-20, nWidth, nHeight, TRUE);

        return FALSE;
    }
	//---------------------------------------------------------------------------------------------------------------------
	else if(uMsg == WM_GETMINMAXINFO)
	{
		MINMAXINFO *pMMI = (MINMAXINFO *)lParam;
		pMMI->ptMinTrackSize.x = 550;
		pMMI->ptMinTrackSize.y = 350;
		return 0;
	}
	//--------------------------------------------------------------------------
	else if(uMsg == WM_REFRESH_SELECTED_CONFIG_ITEM)
	{
		int iPageType = -1;
		int iSiteIndex = -1;

		if(GetSelectedConfigItem(&iPageType, &iSiteIndex))
		{
			PopDetailPane(hTreeView, hListView, iPageType, iSiteIndex);
		}
	}
	//--------------------------------------------------------------------------
	else if(uMsg == WM_REFRESH_CONFIG_TREE_ITEM)
	{
		RefreshConfigTree();
	}
	//--------------------------------------------------------------------------
	else if(uMsg == WM_NOTIFY)
    {
        LPNMHDR lpnmhdr = (LPNMHDR)lParam;
		if(lpnmhdr->code == TVN_SELCHANGED && lpnmhdr->hwndFrom == hTreeView)
		{
			SendMessage(hWnd, WM_REFRESH_SELECTED_CONFIG_ITEM, 0, 0);
		}
		else if(lpnmhdr->code == NM_DBLCLK && lpnmhdr->hwndFrom == hListView)
		{
			ConfigureTreeItem(hListView, iLastSiteIndex, iLastPageType);
		}
	}
    //--------------------------------------------------------------------------
	else if(uMsg == WM_PAINT)
    {
        HDC ThisHDC;
        PAINTSTRUCT ThisPS;

        ThisHDC = BeginPaint(hWnd, &ThisPS);

        //- Any painting should be done here.

        EndPaint(hWnd, &ThisPS);
        return TRUE;
    }
    //--------------------------------------------------------------------------
    else if(uMsg == WM_SYSCOMMAND) //- Received a system menu message.
    {
        if(LOWORD(wParam) == GENERIC_MENU_ABOUT) //- About.
        {
            _AboutDialogInfo ADI;
    
            ADI.DisplayIcon  = LoadIcon(ghAppInstance, MAKEINTRESOURCE(IDI_MAIN));
            ADI.TitleCaption = gsTitleCaption;
            ADI.FileVersion  = gsFileVersion;
            ADI.BuildDate    = __DATE__;
            ADI.BuildTime    = __TIME__;
            ADI.CopyRight    = gsAppCopyRight;
            ADI.OwnerHandle  = hWnd;

            NetLogo(&ADI);

            return TRUE;
        }

        return FALSE;
    }
    //--------------------------------------------------------------------------
    else if(uMsg == WM_CLOSE) //- Received close message.
    {
		if(gService.bRunningAsService)
		{
			ShowWindow(hWnd, SW_HIDE);
		}
		else {
			HMENU hPopupMenu = CreateMenu();
			AppendMenu(hPopupMenu, MF_STRING, ID_TRAY_CONFIGURE, "&Configure");
			AppendMenu(hPopupMenu, MF_SEPARATOR, 0, "");
			AppendMenu(hPopupMenu, MF_STRING, ID_TRAY_SHUTDOWN, "&Shutdown");

			ShowWindow(hWnd, SW_HIDE);
			gTrayIcon.Create(IDI_MAIN, gsTitleCaption, hPopupMenu, &ProgressTrayIconProc);
			ShowWindow(hWnd, SW_SHOW);
		}
        return TRUE;
    }
    //--------------------------------------------------------------------------

    return FALSE;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#endif
