///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//  Copyright  NetworkDLS 2002, All rights reserved
//
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF 
// ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO 
// THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A 
// PARTICULAR PURPOSE.
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#ifndef _CAuthentication_CPP
#define _CAuthentication_CPP
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include <Windows.H>
#include <ShlObj.H>
#include <Stdio.H>
#include <ShlOBJ.H>
#include <Stdlib.H>
#include <ShellAPI.H>

extern HIMAGELIST hEnableDisableImageList; //Declared in MainDialog.cpp
extern HIMAGELIST hOnePixilImageList; //Declared in MainDialog.cpp

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include "CAuthentication.H"
#include "Entry.H"
#include "CWebSites.H"

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

using namespace NSWFL::Windows;
using namespace NSWFL::ListView;
using namespace NSWFL::String;

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void CAuthentication::AddConfigItem(HWND hListView, PROTECTEDPAGE *pPage)
{
	this->Locks.Lock();
	AddConfigItem(hListView, pPage, ListView_GetItemCount(hListView));
	this->Locks.UnLock();
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void CAuthentication::AddConfigItem(HWND hListView, PROTECTEDPAGE *pPage, int iInsertionIndex)
{
	this->Locks.Lock();
	LV_ITEM ListItem;
    memset(&ListItem, 0, sizeof(ListItem));

	int iImage = pPage->Enabled && this->Collection.Enabled;

    ListItem.pszText = (char *) pPage->Page;
    ListItem.mask = LVIF_TEXT|LVIF_IMAGE;
	ListItem.iImage = iImage;
    ListItem.iSubItem = AUTHENTICATION_LIST_POS_PAGE;
	ListItem.iItem = iInsertionIndex;
    int iItem = ListView_InsertItem(hListView, &ListItem);

    ListItem.pszText = (char *) pPage->Description;
    ListItem.mask = LVIF_TEXT;
    ListItem.iSubItem = AUTHENTICATION_LIST_POS_DESCR;
	ListItem.iItem = iInsertionIndex;
    ListView_SetItem(hListView, &ListItem);

	ListView_SetCheckState(hListView, iItem, pPage->Enabled);
	this->Locks.UnLock();
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void CAuthentication::PopConfigList(HWND hWnd, int iListView, int iEnabled)
{
	this->Locks.Lock();
	CheckDlgButton(hWnd, iEnabled, this->Collection.Enabled);
	this->PopInfoList(GetDlgItem(hWnd, iListView), false);
	this->Locks.UnLock();
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void CAuthentication::PopInfoList(HWND hListView)
{
	this->Locks.Lock();
	this->PopInfoList(hListView, true);
	this->Locks.UnLock();
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void CAuthentication::PopInfoList(HWND hListView, bool bUseIcons)
{
	this->Locks.Lock();
	SendMessage(hListView, (UINT)WM_SETREDRAW , (WPARAM)FALSE, (LPARAM)0);

    LV_COLUMN MyLVColumn;
    memset(&MyLVColumn, 0, sizeof(LV_COLUMN));

	MyLVColumn.cx        = 100;
    MyLVColumn.pszText   = "Page";
    MyLVColumn.iSubItem  = AUTHENTICATION_LIST_POS_PAGE;
    MyLVColumn.mask      = LVCF_WIDTH | LVCF_TEXT | LVCF_SUBITEM | LVCF_FMT;
    MyLVColumn.fmt       = LVCFMT_LEFT;
    ListView_InsertColumn(hListView, AUTHENTICATION_LIST_POS_PAGE, &MyLVColumn);

	MyLVColumn.cx        = 300;
    MyLVColumn.pszText   = "Description";
    MyLVColumn.iSubItem  = AUTHENTICATION_LIST_POS_DESCR;
    MyLVColumn.mask      = LVCF_WIDTH | LVCF_TEXT | LVCF_SUBITEM | LVCF_FMT;
    MyLVColumn.fmt       = LVCFMT_LEFT;
    ListView_InsertColumn(hListView, AUTHENTICATION_LIST_POS_DESCR, &MyLVColumn);

	DWORD dwFlags = LVS_EX_FULLROWSELECT|LVS_EX_GRIDLINES|LVS_EX_HEADERDRAGDROP;

	if(!bUseIcons)
	{
		dwFlags |= LVS_EX_CHECKBOXES;
	}

    SendMessage(hListView, (UINT)LVM_SETEXTENDEDLISTVIEWSTYLE, (WPARAM)0, (LPARAM)dwFlags);

	if(bUseIcons)
	{
		ListView_SetImageList(hListView, hEnableDisableImageList, LVSIL_SMALL);
	}

	ListView_SetTextColor(hListView, RGB(0,0,100));

	//Pop the list-view.
	for(int x = 0; x < this->Collection.Count; x++)
	{
		this->AddConfigItem(hListView, &this->Collection.Items[x]);
	}
	LVAutoSizeColumns(hListView, 500);
	SendMessage(hListView, (UINT)WM_SETREDRAW , (WPARAM)TRUE, (LPARAM)0);

	this->Locks.UnLock();
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

CAuthentication::~CAuthentication(void)
{
	this->Destroy();
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

CAuthentication::CAuthentication(void *lpWebSites)
{
	this->Initialized = false;
	this->pWebSites = lpWebSites;
	strcpy_s(sFileName, sizeof(sFileName), "");
	memset(&this->Collection, 0, sizeof(this->Collection));
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

CAuthentication::CAuthentication(void *lpWebSites, CXMLReader *xmlConfig, CAuthentication *pDefaults)
{
	this->Initialized = true;
	this->pWebSites = lpWebSites;
	strcpy_s(sFileName, sizeof(sFileName), "");
	memset(&this->Collection, 0, sizeof(this->Collection));
	this->Load(xmlConfig, pDefaults);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CAuthentication::Save(void)
{
	this->Locks.Lock();

	CXMLReader xmlConfig;
	if(this->ToXML(&xmlConfig))
	{
		bool bResult = xmlConfig.ToFile(this->sFileName);
		if(!bResult)
		{
			char sErrorMsg[2048];
			GetLastError(sErrorMsg, sizeof(sErrorMsg),
				"Failed to save websites configuration file (", ")");
			strcat_s(sErrorMsg, sizeof(sErrorMsg), "\r\n\r\n\"");
			strcat_s(sErrorMsg, sizeof(sErrorMsg), this->sFileName);
			strcat_s(sErrorMsg, sizeof(sErrorMsg), "\"");
			((CWebSites *)this->pWebSites)->Trace(sErrorMsg);
		}
		return this->Locks.UnLock(bResult);
	}
	return this->Locks.UnLock(false);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CAuthentication::ToXML(CXMLReader *lpXML)
{
	this->Locks.Lock();

	CXMLWriter xmlConfig("Authentication");

	xmlConfig.AddBool("Enable", this->Collection.Enabled);

	for(int iItem = 0; iItem < this->Collection.Count; iItem++)
	{
		CXMLWriter Item("Page");
		Item.Add("File", this->Collection.Items[iItem].Page);
		Item.Add("Description", this->Collection.Items[iItem].Description);
		Item.AddBool("Enable", this->Collection.Items[iItem].Enabled);
		xmlConfig.Add(&Item);
	}

	xmlConfig.ToReader(lpXML);

	xmlConfig.Destroy();

	return this->Locks.UnLock(true);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
	Loads a modified configuration from the configuration dialog (also saves to original file).
*/
bool CAuthentication::LoadNewConfiguration(HWND hWnd, HWND hGrid)
{
	this->Locks.LockExclusive();

	if(this->Initialized)
	{
		this->Destroy();
	}

	int iGridItems = LVItemCount(hGrid);

	char sPage[AUTHENTICATION_MAX_PAGE_LENGTH];
	char sDescr[AUTHENTICATION_MAX_DESCR_LENGTH];

	CXMLWriter xmlConfig("Authentication");

	xmlConfig.AddBool("Enable", IsDlgButtonChecked(hWnd, IDC_ENABLED) == BST_CHECKED);

	for(int iItem = 0; iItem < iGridItems; iItem++)
	{
		CXMLWriter Item("Page");

		LVGetText(hGrid, iItem, AUTHENTICATION_LIST_POS_PAGE, sPage, sizeof(sPage));
		Item.Add("File", sPage);

		LVGetText(hGrid, iItem, AUTHENTICATION_LIST_POS_DESCR, sDescr, sizeof(sDescr));
		Item.Add("Description", sDescr);

		Item.AddBool("Enable", LVChecked(hGrid, iItem));
		xmlConfig.Add(&Item);
	}

	if(strlen(this->sFileName) > 0)
	{
		if(xmlConfig.ToFile(this->sFileName))
		{
			return this->Locks.UnLockExclusive(this->Reload());
		}
	}
	else{
		CXMLReader XMLRConfig;
		return this->Locks.UnLockExclusive(Load(xmlConfig.ToReader(&XMLRConfig), NULL));
	}

	return this->Locks.UnLockExclusive(false);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
	Reloads the configuration from the file it was originally loaded from.
*/
bool CAuthentication::Reload(void)
{
	this->Locks.LockExclusive();

	if(this->Initialized)
	{
		this->Destroy();
	}

	this->Load(this->sFileName);

	return this->Locks.UnLockExclusive(this->Initialized);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CAuthentication::Load(const char *sXMLFileName)
{
	this->Locks.LockExclusive();
	this->Initialized = false;

	strcpy_s(this->sFileName, sizeof(this->sFileName), sXMLFileName);

	this->Initialized = false;

	strcpy_s(this->sFileName, sizeof(this->sFileName), sXMLFileName);

	CXMLReader xmlConfig;
	if(xmlConfig.FromFile(sXMLFileName))
	{
		CXMLReader xmlEntity;
		if(xmlConfig.ToReader("Authentication", &xmlEntity))
		{
			this->Initialized = this->Load(&xmlEntity, NULL);
		}
		xmlEntity.Destroy();
		xmlConfig.Destroy();
	}

	return this->Locks.UnLockExclusive(this->Initialized);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CAuthentication::Load(CXMLReader *xmlConfig, CAuthentication *pDefaults)
{
	this->Locks.LockExclusive();

	int iLength = 0;
	memset(&this->Collection, 0, sizeof(this->Collection));

	this->Collection.Enabled = xmlConfig->ToBoolean("Enable", true);

	xmlConfig->ProgressiveScan(true);
	CXMLReader XPDefaultPage;

	while(xmlConfig->ToReader("Page", &XPDefaultPage))
	{
		char sPage[AUTHENTICATION_MAX_PAGE_LENGTH];
		char sDescr[AUTHENTICATION_MAX_DESCR_LENGTH];

		this->Collection.Items = (PROTECTEDPAGE *)
			pMem->ReAllocate(this->Collection.Items, sizeof(PROTECTEDPAGE), this->Collection.Count + 1);

		PROTECTEDPAGE *p = &this->Collection.Items[this->Collection.Count++];

		XPDefaultPage.ToString("File", sPage, sizeof(sPage), &iLength);
		p->Page = (char *) pMem->Allocate(sizeof(char), iLength + 1);
		strcpy_s(p->Page, iLength + 1, LCase(sPage, iLength));
		p->PageLength = iLength;

		XPDefaultPage.ToString("Description", sDescr, sizeof(sDescr), &iLength);
		p->Description = (char *) pMem->Allocate(sizeof(char), iLength + 1);
		strcpy_s(p->Description, iLength + 1, sDescr);

		p->Enabled = XPDefaultPage.ToBoolean("Enable", true);

		XPDefaultPage.Destroy();
	}

	return this->Locks.UnLockExclusive(this->Initialized = true);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CAuthentication::Destroy(void)
{
	this->Locks.LockExclusive();

	if(this->Initialized)
	{
		this->Initialized = false;
		if(this->Collection.Count > 0)
		{
			for(int iPos = 0; iPos < this->Collection.Count; iPos++)
			{
				pMem->Free(this->Collection.Items[iPos].Page);
				pMem->Free(this->Collection.Items[iPos].Description);
				this->Collection.Items[iPos].PageLength = 0;
			}

			pMem->Free(this->Collection.Items);

			this->Collection.Count = 0;
		}
	}
	return this->Locks.UnLockExclusive(true);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CAuthentication::Authenticate(VOID *pClient)
{
	CLIENT *pC = (CLIENT *)pClient;

	this->Locks.Lock();

	//FIXFIX: Implement!

	return this->Locks.UnLock(false);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CAuthentication::IsAuthenticated(VOID *pClient)
{
	CLIENT *pC = (CLIENT *)pClient;

	this->Locks.Lock();

	//FIXFIX: Implement!

	return this->Locks.UnLock(false);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
	Returns true if a default page is found, otherwise false.
*/
bool CAuthentication::IsAuthorizationRequired(VOID *pClient)
{
	CLIENT *pC = (CLIENT *)pClient;
	CWebSite *pWebSite = (CWebSite *)pC->pWebSite;

	this->Locks.Lock();

	if(this->Collection.Enabled && (this->Collection.Count > 0))
	{
		for(int iPos = 0; iPos < this->Collection.Count; iPos++)
		{
			if(this->Collection.Items[iPos].Enabled)
			{
				if(_strcmpi(this->Collection.Items[iPos].Page, pC->Header.Request) == 0)
				{
					return true;
				}
			}
		}
	}

	return this->Locks.UnLock(false);
}


///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#endif
