///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//  Copyright  NetworkDLS 2002, All rights reserved
//
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF 
// ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO 
// THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A 
// PARTICULAR PURPOSE.
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#ifndef _CCompression_H
#define _CCompression_H
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include "Entry.H"
#include "../../@Classes/CXML/CXMLReader.H"
#include "../../@Classes/CStringBuilder/CStringBuilder.H"
#include "../../@Classes/CLocks/CLocks.H"
#include "../../@Classes/CCRC32/CCRC32.H"
#include "../../@Classes/CStack/CStack.H"

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#define COMPRESSION_LIST_POS_EXT		0
#define COMPRESSION_LIST_POS_LEVEL		1
#define COMPRESSION_LIST_POS_DESCR		2

#define CACHE_LIST_POS_FILENAME			0
#define CACHE_LIST_POS_ORIGINALSIZE		1
#define CACHE_LIST_POS_CACHEDSIZE		2
#define CACHE_LIST_POS_HITS				3

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#define GZIP_CHUNK 16384
#define CRC32C(c,d) (c=(c>>8)^crc_c[(c^(d))&0xFF])

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#define COMPRESSION_MAX_EXT_LENGTH	MAX_PATH
#define COMPRESSION_MAX_DESCR_LENGTH		1024

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

typedef struct _tag_Compressible_File{
	bool Enabled;
	char *Extension;
	char *Description;
	int CompressionLevel;
} COMPRESSIBLEFILE, *LPCOMPRESSIBLEFILE;

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

typedef struct _tag__tag_Compressible_Files{
	int Count;

	bool Enabled;

    bool CompressDynamicContent;
    int MaxCompressionSize;
    int MinCompressionSize;
	int CompressionLevel;
	int MemoryLevel;

	COMPRESSIBLEFILE *Items;

} COMPRESSIBLEFILES, *LPCOMPRESSIBLEFILES;

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

typedef struct _tag_Compression_Cache_Item{
	char *OriginalFile;
	char *CachedFile;
	__int64 OriginalSize;
	__int64 CachedSize;
	int Hits;
} COMPRESSIONCACHEITEM, *LPCOMPRESSIONCACHEITEM;

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

typedef struct _tag_Compression_Cache{
	int Count;
	int MaxCachedFiles;
	char *CachePath;
	COMPRESSIONCACHEITEM *Items;
} COMPRESSIONCACHE, *LPCOMPRESSIONCACHE;

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

class CCompression {

private:
	bool Initialized;
	void *pWebSites;
	CCRC32 Crc32;
	COMPRESSIBLEFILES Collection;
	COMPRESSIONCACHE Cache;
	char sFileName[MAX_PATH];

	CLocks Locks;

	bool ResetCacheFileTime(COMPRESSIONCACHEITEM *lpItem);
	bool HasCacheFileChanged(COMPRESSIONCACHEITEM *lpItem);

	int GetLeastHitItem(void);
	bool FreeCacheItem(COMPRESSIONCACHEITEM *lpItem);
	int GetFileCompressionLevel(const char *sFileName);

	CRITICAL_SECTION CS;

	void AddConfigItem(HWND hListView, COMPRESSIONCACHEITEM *pCacheItem);

public:
	~CCompression(void);
	CCompression(void *lpWebSites);
	CCompression(void *lpWebSites, CXMLReader *xmlConfig, CCompression *pDefaults);
	bool Save(void);

	bool ToXML(CXMLReader *lpXML);

	void PopInfoList(HWND hListView);
	void PopInfoList(HWND hListView, bool bUseIcons);
	void PopConfigList(HWND hWnd, int iListView, int iEnabled);
	void PopCacheList(HWND hListView);

	__int64 GetCompressedCacheSize(void);
	__int64 GetNativeCacheSize(void);
	double GetCacheCompressionRatio(void);
	int CachedFiles(void);
	int MaxCachedFiles(void);
	int MaxCompressionSize(void);
	int MinCompressionSize(void);
	int CompressionLevel(void);
	int MemoryLevel(void);

	bool Deflate(const char *sSourceName, const char *sTargetName);
	bool Inflate(const char *sSourceName, const char *sTargetName);

	char *CachePath(void);
	bool CompressDynamicContent(void);
	bool Enabled(void);
	bool GetUniqueFileName(char *sTarget, int iMaxTargetSz);
	bool GetUniqueFileName(char *sTarget, int iMaxTargetSz, const char *sOriginalFileName);

	bool RemoveFileFromCache(const char *sOriginalFile);
	bool CompressFile(const char *sSource, char *sTarget, int iMaxTargetSz);
	bool IsFileCompressible(const char *sFileName);
	bool IsCompressibleType(const char *sFileName);
	bool FlushCache(void);
	bool Load(const char *sXMLFileName);
	bool Load(CXMLReader *xmlConfig, CCompression *pDefaults);
	bool LoadNewConfiguration(HWND hWnd, HWND hGrid);
	bool Reload(void);
	bool Destroy(void);

	void AddConfigItem(HWND hListView, COMPRESSIBLEFILE *pCompressibleFile);
	void AddConfigItem(HWND hListView, COMPRESSIBLEFILE *pCompressibleFile, int iInsertionIndex);
};

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#endif
