///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//  Copyright  NetworkDLS 2002, All rights reserved
//
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF 
// ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO 
// THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A 
// PARTICULAR PURPOSE.
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#ifndef _CCustomFilters_H
#define _CCustomFilters_H
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include "../../@Classes/CXML/CXMLReader.H"
#include "../../@Classes/CStringBuilder/CStringBuilder.H"
#include "../../@Classes/CLocks/CLocks.H"
#include "../../@Classes/CStack/CStack.H"

#include "HTTPFilter.H"

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#define CUSTOMFILTERS_LIST_POS_DESCR	0
#define CUSTOMFILTERS_LIST_POS_DLL		1

#define CUSTOMFILTERS_MAX_DLL_LENGTH	MAX_PATH
#define CUSTOMFILTERS_MAX_DESCR_LENGTH	1024

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

typedef struct _tag_Custom_Filter{
	char *DLL;
	char *Description;
	HMODULE ModuleHandle;
	bool Enabled;

	LPOnPreProcHeaders OnPreProcHeaders;
	LPOnUrlMap OnUrlMap;
	LPOnSendRawData OnSendRawData;
	LPOnRecvRawData OnRecvRawData;
	LPOnLog OnLog;
	LPOnConnect OnConnect;
	LPOnDisconnect OnDisconnect;

} CUSTOMFILTER, *LPCUSTOMFILTER;

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

typedef struct _tag_Custom_Filters{
	int Count;
	CUSTOMFILTER *Items;
	bool Enabled;
} CUSTOMFILTERS, *LPCUSTOMFILTERS;

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

class CCustomFilters{

private:
	bool Initialized;
	void *pWebSites;
	CUSTOMFILTERS Collection;
	char sFileName[MAX_PATH];

	CLocks Locks;

public:
	~CCustomFilters(void);
	CCustomFilters(void *lpWebSites);
	CCustomFilters(void *lpWebSites, CXMLReader *xmlConfig, CCustomFilters *pDefaults);
	bool Save(void);
	
	bool ToXML(CXMLReader *lpXML);

	void AddConfigItem(HWND hListView, CUSTOMFILTER *pFilter);
	void AddConfigItem(HWND hListView, CUSTOMFILTER *pFilter, int iInsertionIndex);
	bool LoadNewConfiguration(HWND hWnd, HWND hGrid);

	void PopInfoList(HWND hListView, bool bUseIcons);
	void PopConfigList(HWND hWnd, int iListView, int iEnabled);
	void PopInfoList(HWND hListView);

	void ProcessDisconnectFilters(void *pvC /*CLIENT *pC*/);
	bool ProcessConnectFilters(void *pvC /*CLIENT *pC*/, const char *sIPAddress);
	bool ProcessLogFilters(void *pvC /*CLIENT *pC*/, FILTERLOGENTRY *pLFE);
	bool ProcessHeadFilters(void *pvC /*CLIENT *pC*/, FILTERHTTPHEADER *pHead);
	bool ProcessSendFilters(void *pvC /*CLIENT *pC*/, const char *sBuf, unsigned int uiBufSz);
	bool ProcessRecvFilters(void *pvC /*CLIENT *pC*/, const char *sBuf, unsigned int uiBufSz);
	bool ProcessUrlMap(void *pvC /*CLIENT *pC*/, const char *sURIStem, const char *sFileName);

	bool Load(const char *sXMLFileName);
	bool Load(CXMLReader *xmlConfig, CCustomFilters *pDefaults);
	bool Reload(void);
	bool Destroy();
};

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#endif
