///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//  Copyright  NetworkDLS 2002, All rights reserved
//
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF 
// ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO 
// THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A 
// PARTICULAR PURPOSE.
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#ifndef _CCustomFilters_CPP
#define _CCustomFilters_CPP
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include <Windows.H>
#include <ShlObj.H>
#include <Stdio.H>
#include <ShlOBJ.H>
#include <Stdlib.H>
#include <ShellAPI.H>

extern HIMAGELIST hEnableDisableImageList; //Declared in MainDialog.cpp
extern HIMAGELIST hOnePixilImageList; //Declared in MainDialog.cpp

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include "CCustomFilters.H"
#include "Entry.H"
#include "CWebSites.H"

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

using namespace NSWFL::Windows;
using namespace NSWFL::ListView;
using namespace NSWFL::Conversion;

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void CCustomFilters::AddConfigItem(HWND hListView, CUSTOMFILTER *pFilter)
{
	this->Locks.Lock();
	AddConfigItem(hListView, pFilter, ListView_GetItemCount(hListView));
	this->Locks.UnLock();
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void CCustomFilters::AddConfigItem(HWND hListView, CUSTOMFILTER *pFilter, int iInsertionIndex)
{
	this->Locks.Lock();
	//char sFile[MAX_PATH];
	LV_ITEM ListItem;
    memset(&ListItem, 0, sizeof(ListItem));

	int iImage = pFilter->Enabled && this->Collection.Enabled;

    ListItem.pszText = (char *) pFilter->Description;
	//GetFileName(pFilter->DLL, sFile, sizeof(sFile));
    ListItem.mask = LVIF_TEXT|LVIF_IMAGE;
	ListItem.iImage = iImage;
    ListItem.iSubItem = CUSTOMFILTERS_LIST_POS_DESCR;
	ListItem.iItem = iInsertionIndex;
    int iItem = ListView_InsertItem(hListView, &ListItem);

    ListItem.pszText = (char *) pFilter->DLL;
    ListItem.mask = LVIF_TEXT;
    ListItem.iSubItem = CUSTOMFILTERS_LIST_POS_DLL;
	ListItem.iItem = iInsertionIndex;
    ListView_SetItem(hListView, &ListItem);

	ListView_SetCheckState(hListView, iItem, pFilter->Enabled);
	this->Locks.UnLock();
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void CCustomFilters::PopConfigList(HWND hWnd, int iListView, int iEnabled)
{
	this->Locks.Lock();
	CheckDlgButton(hWnd, iEnabled, this->Collection.Enabled);
	this->PopInfoList(GetDlgItem(hWnd, iListView), false);
	this->Locks.UnLock();
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void CCustomFilters::PopInfoList(HWND hListView)
{
	this->Locks.Lock();
	this->PopInfoList(hListView, true);
	this->Locks.UnLock();
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void CCustomFilters::PopInfoList(HWND hListView, bool bUseIcons)
{
	this->Locks.Lock();
	SendMessage(hListView, (UINT)WM_SETREDRAW , (WPARAM)FALSE, (LPARAM)0);

	LV_COLUMN MyLVColumn;
    memset(&MyLVColumn, 0, sizeof(LV_COLUMN));

	MyLVColumn.cx        = 300;
    MyLVColumn.pszText   = "Description";
    MyLVColumn.iSubItem  = CUSTOMFILTERS_LIST_POS_DESCR;
    MyLVColumn.mask      = LVCF_WIDTH | LVCF_TEXT | LVCF_SUBITEM | LVCF_FMT;
    MyLVColumn.fmt       = LVCFMT_LEFT;
    ListView_InsertColumn(hListView, CUSTOMFILTERS_LIST_POS_DESCR, &MyLVColumn);

	MyLVColumn.cx        = 100;
    MyLVColumn.pszText   = "DLL";
    MyLVColumn.iSubItem  = CUSTOMFILTERS_LIST_POS_DLL;
    MyLVColumn.mask      = LVCF_WIDTH | LVCF_TEXT | LVCF_SUBITEM | LVCF_FMT;
    MyLVColumn.fmt       = LVCFMT_LEFT;
    ListView_InsertColumn(hListView, CUSTOMFILTERS_LIST_POS_DLL, &MyLVColumn);

	DWORD dwFlags = LVS_EX_FULLROWSELECT|LVS_EX_GRIDLINES|LVS_EX_HEADERDRAGDROP;

	if(!bUseIcons)
	{
		dwFlags |= LVS_EX_CHECKBOXES;
	}

    SendMessage(hListView, (UINT)LVM_SETEXTENDEDLISTVIEWSTYLE, (WPARAM)0, (LPARAM)dwFlags);

	if(bUseIcons)
	{
		ListView_SetImageList(hListView, hEnableDisableImageList, LVSIL_SMALL);
	}

	ListView_SetTextColor(hListView, RGB(0,0,100));

	//Pop the list-view.
	for(int x = 0; x < this->Collection.Count; x++)
	{
		this->AddConfigItem(hListView, &this->Collection.Items[x]);
	}
	LVAutoSizeColumns(hListView, 500);
	SendMessage(hListView, (UINT)WM_SETREDRAW , (WPARAM)TRUE, (LPARAM)0);
	this->Locks.UnLock();
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

CCustomFilters::~CCustomFilters(void)
{
	this->Destroy();
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

CCustomFilters::CCustomFilters(void *lpWebSites)
{
	this->Initialized = false;
	this->pWebSites = lpWebSites;
	strcpy_s(sFileName, sizeof(sFileName), "");
	memset(&this->Collection, 0, sizeof(this->Collection));
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

CCustomFilters::CCustomFilters(void *lpWebSites, CXMLReader *xmlConfig, CCustomFilters *pDefaults)
{
	this->Initialized = true;
	this->pWebSites = lpWebSites;
	strcpy_s(sFileName, sizeof(sFileName), "");
	memset(&this->Collection, 0, sizeof(this->Collection));
	this->Load(xmlConfig, pDefaults);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CCustomFilters::Save(void)
{
	this->Locks.Lock();

	CXMLReader xmlConfig;
	if(this->ToXML(&xmlConfig))
	{
		bool bResult = xmlConfig.ToFile(this->sFileName);
		if(!bResult)
		{
			char sErrorMsg[2048];
			GetLastError(sErrorMsg, sizeof(sErrorMsg),
				"Failed to save websites configuration file (", ")");
			strcat_s(sErrorMsg, sizeof(sErrorMsg), "\r\n\r\n\"");
			strcat_s(sErrorMsg, sizeof(sErrorMsg), this->sFileName);
			strcat_s(sErrorMsg, sizeof(sErrorMsg), "\"");
			((CWebSites *)this->pWebSites)->Trace(sErrorMsg);
		}
		return this->Locks.UnLock(bResult);
	}
	return this->Locks.UnLock(false);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CCustomFilters::ToXML(CXMLReader *lpXML)
{
	this->Locks.Lock();

	CXMLWriter xmlConfig("CustomFilters");

	xmlConfig.AddBool("Enable", this->Collection.Enabled);

	for(int iItem = 0; iItem < this->Collection.Count; iItem++)
	{
		CXMLWriter Item("Filter");
		Item.Add("DLL", this->Collection.Items[iItem].DLL);
		Item.Add("Description", this->Collection.Items[iItem].Description);
		Item.AddBool("Enable", this->Collection.Items[iItem].Enabled);

		xmlConfig.Add(&Item);
	}

	xmlConfig.ToReader(lpXML);

	xmlConfig.Destroy();

	return this->Locks.UnLock(true);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
	Loads a modified configuration from the configuration dialog (also saves to original file).
*/
bool CCustomFilters::LoadNewConfiguration(HWND hWnd, HWND hGrid)
{
	this->Locks.LockExclusive();

	if(this->Initialized)
	{
		this->Destroy();
	}

	int iGridItems = LVItemCount(hGrid);

	char sDLL[CUSTOMFILTERS_MAX_DLL_LENGTH];
	char sDescr[CUSTOMFILTERS_MAX_DESCR_LENGTH];

	CXMLWriter xmlConfig("CustomFilters");

	xmlConfig.AddBool("Enable", ParseBoolean(IsDlgButtonChecked(hWnd, IDC_ENABLED) == BST_CHECKED));

	for(int iItem = 0; iItem < iGridItems; iItem++)
	{
		CXMLWriter Item("Filter");

		LVGetText(hGrid, iItem, CUSTOMFILTERS_LIST_POS_DLL, sDLL, sizeof(sDLL));
		Item.Add("DLL", sDLL);

		LVGetText(hGrid, iItem, CUSTOMFILTERS_LIST_POS_DESCR, sDescr, sizeof(sDescr));
		Item.Add("Description", sDescr);

		Item.AddBool("Enable", ParseBoolean(LVChecked(hGrid, iItem)));
		xmlConfig.Add(&Item);
	}

	if(strlen(this->sFileName) > 0)
	{
		if(xmlConfig.ToFile(this->sFileName))
		{
			return this->Locks.UnLockExclusive(this->Reload());
		}
	}
	else{
		CXMLReader XMLRConfig;
		return this->Locks.UnLockExclusive(Load(xmlConfig.ToReader(&XMLRConfig), NULL));
	}

	return this->Locks.UnLockExclusive(false);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
	Reloads the configuration from the file it was originally loaded from.
*/
bool CCustomFilters::Reload(void)
{
	this->Locks.LockExclusive();

	if(this->Initialized)
	{
		this->Destroy();
	}

	this->Load(this->sFileName);

	return this->Locks.UnLockExclusive(this->Initialized);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CCustomFilters::Load(const char *sXMLFileName)
{
	this->Locks.LockExclusive();
	this->Initialized = false;

	strcpy_s(this->sFileName, sizeof(this->sFileName), sXMLFileName);

	CXMLReader xmlConfig;

	if(xmlConfig.FromFile(sXMLFileName))
	{
		CXMLReader xmlEntity;
		if(xmlConfig.ToReader("CustomFilters", &xmlEntity))
		{
			this->Initialized = this->Load(&xmlEntity, NULL);
		}
		xmlEntity.Destroy();
		xmlConfig.Destroy();
	}

	return this->Locks.UnLockExclusive(this->Initialized);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CCustomFilters::Load(CXMLReader *xmlConfig, CCustomFilters *pDefaults)
{
	this->Locks.LockExclusive();

	int iLength = 0;

	memset(&this->Collection, 0, sizeof(this->Collection));

	this->Collection.Enabled = xmlConfig->ToBoolean("Enable", true);

	xmlConfig->ProgressiveScan(true);
	CXMLReader XPCustomFilter;

	while(xmlConfig->ToReader("Filter", &XPCustomFilter))
	{
		char sDLL[CUSTOMFILTERS_MAX_DLL_LENGTH];
		char sDescr[CUSTOMFILTERS_MAX_DESCR_LENGTH];

		this->Collection.Items = (CUSTOMFILTER *)
			pMem->ReAllocate(this->Collection.Items, sizeof(CUSTOMFILTER), this->Collection.Count + 1);

		CUSTOMFILTER *p = &this->Collection.Items[this->Collection.Count++];
		memset(p, 0, sizeof(CUSTOMFILTER));

		XPCustomFilter.ToString("DLL", sDLL, sizeof(sDLL), &iLength);
		p->DLL = (char *) pMem->Allocate(sizeof(char), iLength + 1);
		strcpy_s(p->DLL, iLength + 1, sDLL);

		XPCustomFilter.ToString("Description", sDescr, sizeof(sDescr), &iLength);
		p->Description = (char *) pMem->Allocate(sizeof(char), iLength + 1);
		strcpy_s(p->Description, iLength + 1, sDescr);

		p->Enabled = XPCustomFilter.ToBoolean("Enable", true);

		if((p->ModuleHandle = LoadLibrary(p->DLL)))
		{
			p->OnConnect = (LPOnConnect)GetProcAddress(p->ModuleHandle, "OnConnect");
			p->OnDisconnect   = (LPOnDisconnect)GetProcAddress(p->ModuleHandle, "OnDisconnect");
			p->OnLog = (LPOnLog)GetProcAddress(p->ModuleHandle, "OnLog");
			p->OnPreProcHeaders = (LPOnPreProcHeaders)GetProcAddress(p->ModuleHandle, "OnPreProcHeaders");
			p->OnRecvRawData = (LPOnRecvRawData)GetProcAddress(p->ModuleHandle, "OnRecvRawData");
			p->OnSendRawData = (LPOnSendRawData)GetProcAddress(p->ModuleHandle, "OnSendRawData");
			p->OnUrlMap = (LPOnUrlMap)GetProcAddress(p->ModuleHandle, "OnUrlMap");
		}
		else{
			p->Enabled = false;
		}

		XPCustomFilter.Destroy();
	}

	return this->Locks.UnLockExclusive((this->Initialized = true));
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CCustomFilters::Destroy(void)
{
	this->Locks.LockExclusive();

	if(this->Initialized)
	{
		this->Initialized = false;
		if(this->Collection.Count > 0)
		{
			for(int iPos = 0; iPos < this->Collection.Count; iPos++)
			{
				pMem->Free(this->Collection.Items[iPos].DLL);
				pMem->Free(this->Collection.Items[iPos].Description);

				if(this->Collection.Items[iPos].ModuleHandle)
				{
					FreeLibrary(this->Collection.Items[iPos].ModuleHandle);
				}
			}

			pMem->Free(this->Collection.Items);

			this->Collection.Count = 0;
		}
	}

	return this->Locks.UnLockExclusive(true);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
	Returns true if the filters passed, otherwise false.
*/
bool CCustomFilters::ProcessHeadFilters(void *pvC /*CLIENT *pC*/, FILTERHTTPHEADER *pHead)
{
	this->Locks.Lock();

	if(this->Collection.Enabled && (this->Collection.Count > 0))
	{
		CLIENT *pC = (CLIENT*)pvC;

		for(int iPos = 0; iPos < this->Collection.Count; iPos++)
		{
			if(this->Collection.Items[iPos].Enabled && this->Collection.Items[iPos].OnPreProcHeaders)
			{
				if(this->Collection.Items[iPos].OnPreProcHeaders(pC->pClient->PeerID(), pHead) != FILTER_EVENT_RESULT_OK)
				{
					return this->Locks.UnLock(false);
				}
			}
		}
	}

	return this->Locks.UnLock(true);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
	Returns true if the filters passed, otherwise false.
*/
bool CCustomFilters::ProcessLogFilters(void *pvC /*CLIENT *pC*/, FILTERLOGENTRY *pLFE)
{
	this->Locks.Lock();

	if(this->Collection.Enabled && (this->Collection.Count > 0))
	{
		CLIENT *pC = (CLIENT*)pvC;

		for(int iPos = 0; iPos < this->Collection.Count; iPos++)
		{
			if(this->Collection.Items[iPos].Enabled && this->Collection.Items[iPos].OnPreProcHeaders)
			{
				if(this->Collection.Items[iPos].OnLog(pC->pClient->PeerID(), pLFE) != FILTER_EVENT_RESULT_OK)
				{
					return this->Locks.UnLock(false);
				}
			}
		}
	}

	return this->Locks.UnLock(true);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
	Returns true if the filters passed, otherwise false.
*/
bool CCustomFilters::ProcessConnectFilters(void *pvC /*CLIENT *pC*/, const char *sIPAddress)
{
	this->Locks.Lock();

	if(this->Collection.Enabled && (this->Collection.Count > 0))
	{
		CLIENT *pC = (CLIENT*)pvC;

		for(int iPos = 0; iPos < this->Collection.Count; iPos++)
		{
			if(this->Collection.Items[iPos].Enabled && this->Collection.Items[iPos].OnConnect)
			{
				if(this->Collection.Items[iPos].OnConnect(pC->pClient->PeerID(), sIPAddress) != FILTER_EVENT_RESULT_OK)
				{
					return this->Locks.UnLock(false);
				}
			}
		}
	}

	return this->Locks.UnLock(true);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void CCustomFilters::ProcessDisconnectFilters(void *pvC /*CLIENT *pC*/)
{
	this->Locks.Lock();

	if(this->Collection.Enabled && (this->Collection.Count > 0))
	{
		CLIENT *pC = (CLIENT*)pvC;

		for(int iPos = 0; iPos < this->Collection.Count; iPos++)
		{
			if(this->Collection.Items[iPos].Enabled && this->Collection.Items[iPos].OnDisconnect)
			{
				if(this->Collection.Items[iPos].OnDisconnect(pC->pClient->PeerID()) != FILTER_EVENT_RESULT_OK)
				{
					break;
				}
			}
		}
	}

	this->Locks.UnLock();
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CCustomFilters::ProcessSendFilters(void *pvC /*CLIENT *pC*/, const char *sBuf, unsigned int uiBufSz)
{
	this->Locks.Lock();

	if(this->Collection.Enabled && (this->Collection.Count > 0))
	{
		CLIENT *pC = (CLIENT*)pvC;

		for(int iPos = 0; iPos < this->Collection.Count; iPos++)
		{
			if(this->Collection.Items[iPos].Enabled && this->Collection.Items[iPos].OnSendRawData)
			{
				if(this->Collection.Items[iPos].OnSendRawData(pC->pClient->PeerID(), sBuf, uiBufSz) != FILTER_EVENT_RESULT_OK)
				{
					return this->Locks.UnLock(false);
				}
			}
		}
	}

	return this->Locks.UnLock(true);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CCustomFilters::ProcessRecvFilters(void *pvC /*CLIENT *pC*/, const char *sBuf, unsigned int uiBufSz)
{
	this->Locks.Lock();

	if(this->Collection.Enabled && (this->Collection.Count > 0))
	{
		CLIENT *pC = (CLIENT*)pvC;

		for(int iPos = 0; iPos < this->Collection.Count; iPos++)
		{
			if(this->Collection.Items[iPos].Enabled && this->Collection.Items[iPos].OnRecvRawData)
			{
				if(this->Collection.Items[iPos].OnRecvRawData(pC->pClient->PeerID(), sBuf, uiBufSz) != FILTER_EVENT_RESULT_OK)
				{
					return this->Locks.UnLock(false);
				}
			}
		}
	}

	return this->Locks.UnLock(true);
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CCustomFilters::ProcessUrlMap(void *pvC /*CLIENT *pC*/, const char *sURIStem, const char *sFileName)
{
	this->Locks.Lock();

	if(this->Collection.Enabled && (this->Collection.Count > 0))
	{
		CLIENT *pC = (CLIENT*)pvC;

		for(int iPos = 0; iPos < this->Collection.Count; iPos++)
		{
			if(this->Collection.Items[iPos].Enabled && this->Collection.Items[iPos].OnUrlMap)
			{
				if(this->Collection.Items[iPos].OnUrlMap(pC->pClient->PeerID(), sURIStem, sFileName) != FILTER_EVENT_RESULT_OK)
				{
					return this->Locks.UnLock(false);
				}
			}
		}
	}

	return this->Locks.UnLock(true);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#endif
