#ifndef CErrorCodes_CPP
#define CErrorCodes_CPP
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include <Windows.H>
#include <Stdio.H>
#include <Stdlib.H>

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include "Entry.H"
#include "Init.H"

#include "CWebSites.H"
#include "Routines.H"

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

char *HTTP_Error_Codes[] = {
	"403:14", // ERROR: 403:14  | Forbidden - Directory Listing Denied.
	"400",    // Error: 400     | Bad Request.
	"501",    // Error: 501     | Not Implemented.
	"404",    // Error: 404     | File Not Found.
	"405",    // Error: 405     | Method Not Allowed
	"401:4",  // Error: 401:4   | Unauthorized - Authorization Failed By Filter.
	"403:1",  // Error: 403:1   | Forbidden - Execute Access Forbidden.
	"403:6",  // Error: 403:6   | Forbidden - IP Address Rejected.
	"403:8",  // Error: 403:8   | Forbidden - Site Access Denied 
	"403:9",  // Error: 403:9   | Forbidden - To Many Users.
	"408",    // Error: 408     | Request Timed Out.
	"414",    // Error: 414     | Request -URI Too Long.
	"500",    // Error: 500     | Internal Server Error.
	"500:12", // Error: 500:12  | Internal Server Error - Application Restarting.
	"500:13", // Error: 500:13  | Internal Server Error - Server Too Buisy.
	"200",	  // Error: 200		| O.K.
	NULL
};

char *HTTP_Error_Pages[] = {
	"403_14.Html",
	"400.Html",
	"501.Html",
	"404.Html",
	"405.Html",
	"401_4.Html",
	"403_1.Html",
	"403_6.Html",
	"403_8.Html",
	"403_9.Html",
	"408.Html",
	"414.Html",
	"500.Html",
	"500_12.Html",
	"500_13.Html",
	"",
	NULL
};

char *HTTP_Error_Desc[] = {
	"Directory Listing Denied",
	"Bad Request",
	"Not Implemented",
	"Object Not Found",
	"Method Not Allowed",
	"Unauthorized - Authorization Failed By Filter",
	"Forbidden - Execute Access Forbidden",
	"Forbidden - IP Address Rejected",
	"Forbidden - Site Access Denied",
	"Forbidden - To Many Users",
	"Request Timed Out",
	"Request-URI Too Long",
	"Internal Server Error",
	"Internal Server Error - Application Restarting",
	"Internal Server Error - Server Too Buisy",
	"OK",
	NULL
};

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CErrorCodes::Load(LPVOID lpHttp)
{
	this->pHttp = lpHttp;
	this->dwErrorCount = 0;
	this->dwWarningCount = 0;
	return false;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

DWORD CErrorCodes::WarningCount(void)
{
	return this->dwWarningCount;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

DWORD CErrorCodes::ErrorCount(void)
{
	return this->dwErrorCount;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CErrorCodes::Destroy(void)
{
	this->pHttp = NULL;
	this->dwErrorCount = 0;
	this->dwWarningCount = 0;
	return false;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CErrorCodes::SendErrorCode(CLIENT *pC, char *sErrorCode)
{
	int iRPos = 0;

	while(HTTP_Error_Codes[iRPos])
	{
		if(_strcmpi(sErrorCode, HTTP_Error_Codes[iRPos]) == 0)
		{
			return SendErrorPage(pC, sErrorCode, HTTP_Error_Desc[iRPos], HTTP_Error_Pages[iRPos]);
		}
		iRPos++;
	}

	return false;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CErrorCodes::SendErrorPage(CLIENT *pC, char *sErrCode,
								char *sErrDesc, char *sErrorPage)
{
	char sRequest[MAX_PATH];
	sprintf_s(sRequest, sizeof(sRequest), "%s\\Errors\\%s", gsMessagePath, sErrorPage);
	CorrectReversePath(sRequest, sizeof(sRequest), true);

	//Keep track of warnings and errors. Errors are all 500 (Internal Server) errors
	//	and warnings are all other errors except 404 (file not found) errors.
	if(strncmp(sErrCode, "404", 3) != 0)
	{
		if(strncmp(sErrCode, "500", 3) == 0)
		{
			this->dwErrorCount++;
		}
		else {
			this->dwWarningCount++;
		}
	}

	__int64 i64FileSize = 0;
	if(FileSize(sRequest, &i64FileSize))
	{
		if(((CHTTP *)this->pHttp)->SendHTTPHeader
			(pC, sErrCode, sErrDesc, NULL, "text/html", i64FileSize, true))
		{
			return ((CHTTP *)this->pHttp)->SendRawFile(pC, sRequest);
		}
	}

	return false;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#endif
