///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//  Copyright  NetworkDLS 2002, All rights reserved
//
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF 
// ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO 
// THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A 
// PARTICULAR PURPOSE.
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#ifndef _CErrorPages_CPP
#define _CErrorPages_CPP
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include <Windows.H>
#include <ShlObj.H>
#include <Stdio.H>
#include <ShlOBJ.H>
#include <Stdlib.H>

extern HIMAGELIST hEnableDisableImageList; //Declared in MainDialog.cpp
extern HIMAGELIST hOnePixilImageList; //Declared in MainDialog.cpp

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include "CErrorPages.H"
#include "Entry.H"
#include "CWebSites.H"

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

using namespace NSWFL::Windows;
using namespace NSWFL::ListView;

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void CErrorPages::AddConfigItem(HWND hListView, ERRORPAGE *pPage)
{
	this->Locks.Lock();
	this->AddConfigItem(hListView, pPage, ListView_GetItemCount(hListView));
	this->Locks.UnLock();
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void CErrorPages::AddConfigItem(HWND hListView, ERRORPAGE *pPage, int iInsertionIndex)
{
	this->Locks.Lock();
	LV_ITEM ListItem;
    memset(&ListItem, 0, sizeof(ListItem));

	int iImage = pPage->Enabled;

    ListItem.pszText = (char *) pPage->Code;
    ListItem.mask = LVIF_TEXT|LVIF_IMAGE;
	ListItem.iImage = iImage;
    ListItem.iSubItem = ERRORPAGES_LIST_POS_CODE;
	ListItem.iItem = iInsertionIndex;
    int iItem = ListView_InsertItem(hListView, &ListItem);

    ListItem.pszText = (char *) pPage->Description;
    ListItem.mask = LVIF_TEXT;
    ListItem.iSubItem = ERRORPAGES_LIST_POS_DESCR;
	ListItem.iItem = iInsertionIndex;
    ListView_SetItem(hListView, &ListItem);

    ListItem.pszText = (char *) pPage->File;
    ListItem.mask = LVIF_TEXT;
    ListItem.iSubItem = ERRORPAGES_LIST_POS_FILE;
	ListItem.iItem = iInsertionIndex;
    ListView_SetItem(hListView, &ListItem);

	ListView_SetCheckState(hListView, iItem, pPage->Enabled);
	this->Locks.UnLock();
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void CErrorPages::PopConfigList(HWND hWnd, int iListView, int iDefaultFile)
{
	this->Locks.Lock();
	Set_Text(GetDlgItem(hWnd, iDefaultFile), this->Collection.DefaultFile);
	this->PopInfoList(GetDlgItem(hWnd, iListView), false);
	this->Locks.UnLock();
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void CErrorPages::PopInfoList(HWND hListView)
{
	this->Locks.Lock();
	this->PopInfoList(hListView, true);
	this->Locks.UnLock();
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void CErrorPages::PopInfoList(HWND hListView, bool bUseIcons)
{
	this->Locks.Lock();
	SendMessage(hListView, (UINT)WM_SETREDRAW , (WPARAM)FALSE, (LPARAM)0);

	LV_COLUMN MyLVColumn;
    memset(&MyLVColumn, 0, sizeof(LV_COLUMN));

	MyLVColumn.cx        = 100;
    MyLVColumn.pszText   = "Code";
    MyLVColumn.iSubItem  = ERRORPAGES_LIST_POS_CODE;
    MyLVColumn.mask      = LVCF_WIDTH | LVCF_TEXT | LVCF_SUBITEM | LVCF_FMT;
    MyLVColumn.fmt       = LVCFMT_LEFT;
    ListView_InsertColumn(hListView, ERRORPAGES_LIST_POS_CODE, &MyLVColumn);

	MyLVColumn.cx        = 300;
    MyLVColumn.pszText   = "Description";
    MyLVColumn.iSubItem  = ERRORPAGES_LIST_POS_DESCR;
    MyLVColumn.mask      = LVCF_WIDTH | LVCF_TEXT | LVCF_SUBITEM | LVCF_FMT;
    MyLVColumn.fmt       = LVCFMT_LEFT;
    ListView_InsertColumn(hListView, ERRORPAGES_LIST_POS_DESCR, &MyLVColumn);

	MyLVColumn.cx        = 300;
    MyLVColumn.pszText   = "File";
    MyLVColumn.iSubItem  = ERRORPAGES_LIST_POS_FILE;
    MyLVColumn.mask      = LVCF_WIDTH | LVCF_TEXT | LVCF_SUBITEM | LVCF_FMT;
    MyLVColumn.fmt       = LVCFMT_LEFT;
    ListView_InsertColumn(hListView, ERRORPAGES_LIST_POS_FILE, &MyLVColumn);

	DWORD dwFlags = LVS_EX_FULLROWSELECT|LVS_EX_GRIDLINES|LVS_EX_HEADERDRAGDROP;

	if(!bUseIcons)
	{
		dwFlags |= LVS_EX_CHECKBOXES;
	}

    SendMessage(hListView, (UINT)LVM_SETEXTENDEDLISTVIEWSTYLE, (WPARAM)0, (LPARAM)dwFlags);

	if(bUseIcons)
	{
		ListView_SetImageList(hListView, hEnableDisableImageList, LVSIL_SMALL);
	}

	ListView_SetTextColor(hListView, RGB(0,0,100));

	//Pop the list-view.
	for(int x = 0; x < this->Collection.Count; x++)
	{
		this->AddConfigItem(hListView, &this->Collection.Items[x]);
	}
	LVAutoSizeColumns(hListView, 500);
	SendMessage(hListView, (UINT)WM_SETREDRAW , (WPARAM)TRUE, (LPARAM)0);
	this->Locks.UnLock();
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

CErrorPages::~CErrorPages(void)
{
	this->Destroy();
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

CErrorPages::CErrorPages(void *lpWebSites)
{
	this->Initialized = false;
	this->pWebSites = lpWebSites;
	strcpy_s(sFileName, sizeof(sFileName), "");
	memset(&this->Collection, 0, sizeof(this->Collection));
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

CErrorPages::CErrorPages(void *lpWebSites, CXMLReader *xmlConfig, CErrorPages *pDefaults)
{
	this->Initialized = true;
	this->pWebSites = lpWebSites;
	strcpy_s(sFileName, sizeof(sFileName), "");
	this->Load(xmlConfig, pDefaults);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CErrorPages::Save(void)
{
	this->Locks.Lock();

	CXMLReader xmlConfig;
	if(this->ToXML(&xmlConfig))
	{
		bool bResult = xmlConfig.ToFile(this->sFileName);
		if(!bResult)
		{
			char sErrorMsg[2048];
			GetLastError(sErrorMsg, sizeof(sErrorMsg),
				"Failed to save websites configuration file (", ")");
			strcat_s(sErrorMsg, sizeof(sErrorMsg), "\r\n\r\n\"");
			strcat_s(sErrorMsg, sizeof(sErrorMsg), this->sFileName);
			strcat_s(sErrorMsg, sizeof(sErrorMsg), "\"");
			((CWebSites *)this->pWebSites)->Trace(sErrorMsg);
		}
		return this->Locks.UnLock(bResult);
	}
	return this->Locks.UnLock(false);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CErrorPages::ToXML(CXMLReader *lpXML)
{
	this->Locks.Lock();

	CXMLWriter xmlConfig("ErrorPages");

	if(strlen(this->Collection.DefaultFile) == 0)
	{
		strcpy_s(this->Collection.DefaultFile,
			sizeof(this->Collection.DefaultFile), "GenericError.html");
	}
	xmlConfig.Add("DefaultFile", this->Collection.DefaultFile);

	for(int iItem = 0; iItem < this->Collection.Count; iItem++)
	{
		CXMLWriter Item("Page");
		Item.Add("Code", this->Collection.Items[iItem].Code);
		Item.Add("File", this->Collection.Items[iItem].File);
		Item.Add("Description", this->Collection.Items[iItem].Description);
		Item.AddBool("Enable", this->Collection.Items[iItem].Enabled);
		xmlConfig.Add(&Item);
	}

	xmlConfig.ToReader(lpXML);

	xmlConfig.Destroy();

	return this->Locks.UnLock(true);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
	Loads a modified configuration from the configuration dialog (also saves to original file).
*/
bool CErrorPages::LoadNewConfiguration(HWND hWnd, HWND hGrid, HWND hDefaultFile)
{
	this->Locks.LockExclusive();

	if(this->Initialized)
	{
		this->Destroy();
	}

	int iGridItems = LVItemCount(hGrid);

	char sCode[ERRORPAGES_MAX_CODE_LENGTH];
	char sFile[ERRORPAGES_MAX_FILE_LENGTH];
	char sDescr[ERRORPAGES_MAX_DESCR_LENGTH];

	CXMLWriter xmlConfig("ErrorPages");

	Get_Text(hDefaultFile, sFile, sizeof(sFile));
	if(strlen(sCode) == 0)
	{
		strcpy_s(sFile, sizeof(sFile), "GenericError.html");
	}
	xmlConfig.Add("DefaultFile", sFile);

	for(int iItem = 0; iItem < iGridItems; iItem++)
	{
		CXMLWriter Item("Page");

		LVGetText(hGrid, iItem, ERRORPAGES_LIST_POS_CODE, sCode, sizeof(sCode));
		Item.Add("Code", sCode);

		LVGetText(hGrid, iItem, ERRORPAGES_LIST_POS_DESCR, sDescr, sizeof(sDescr));
		Item.Add("Description", sDescr);

		LVGetText(hGrid, iItem, ERRORPAGES_LIST_POS_FILE, sFile, sizeof(sFile));
		Item.Add("File", sFile);

		Item.AddBool("Enable", LVChecked(hGrid, iItem));
		xmlConfig.Add(&Item);
	}

	if(strlen(this->sFileName) > 0)
	{
		if(xmlConfig.ToFile(this->sFileName))
		{
			return this->Locks.UnLockExclusive(this->Reload());
		}
	}
	else{
		CXMLReader XMLRConfig;
		return this->Locks.UnLockExclusive(Load(xmlConfig.ToReader(&XMLRConfig), NULL));
	}

	return this->Locks.UnLockExclusive(false);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
	Reloads the configuration from the file it was originally loaded from.
*/
bool CErrorPages::Reload(void)
{
	this->Locks.LockExclusive();

	if(this->Initialized)
	{
		this->Destroy();
	}

	this->Load(this->sFileName);

	return this->Locks.UnLockExclusive(this->Initialized);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CErrorPages::Load(const char *sXMLFileName)
{
	this->Locks.LockExclusive();
	this->Initialized = false;

	strcpy_s(this->sFileName, sizeof(this->sFileName), sXMLFileName);
	strcpy_s(this->Collection.DefaultFile, sizeof(this->Collection.DefaultFile), "");

	CXMLReader xmlConfig;

	if(xmlConfig.FromFile(sXMLFileName))
	{
		CXMLReader xmlEntity;
		if(xmlConfig.ToReader("ErrorPages", &xmlEntity))
		{
			this->Initialized = this->Load(&xmlEntity, NULL);
		}
		xmlEntity.Destroy();
		xmlConfig.Destroy();
	}

	return this->Locks.UnLockExclusive(this->Initialized);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CErrorPages::Load(CXMLReader *xmlConfig, CErrorPages *pDefaults)
{
	this->dwErrorCount = 0;
	this->dwWarningCount = 0;

	this->Locks.LockExclusive();

	int iLength = 0;
	memset(&this->Collection, 0, sizeof(this->Collection));

	xmlConfig->ToString("DefaultFile", this->Collection.DefaultFile, sizeof(this->Collection.DefaultFile), &iLength);
	if(iLength == 0)
	{
		strcpy_s(this->Collection.DefaultFile,
			sizeof(this->Collection.DefaultFile), "GenericError.html");
	}

	xmlConfig->ProgressiveScan(true);
	CXMLReader XPVirtualPage;

	while(xmlConfig->ToReader("Page", &XPVirtualPage))
	{
		char sCode[ERRORPAGES_MAX_CODE_LENGTH];
		char sFile[ERRORPAGES_MAX_FILE_LENGTH];
		char sDescr[ERRORPAGES_MAX_DESCR_LENGTH];

		this->Collection.Items = (ERRORPAGE *) pMem->ReAllocate(
			this->Collection.Items, sizeof(ERRORPAGE), this->Collection.Count + 1);

		ERRORPAGE *p = &this->Collection.Items[this->Collection.Count++];

		XPVirtualPage.ToString("Code", sCode, sizeof(sCode), &iLength);
		p->Code = (char *) pMem->Allocate(sizeof(char), iLength + 1);
		strcpy_s(p->Code, iLength + 1, sCode);

		XPVirtualPage.ToString("File", sFile, sizeof(sFile), &iLength);
		p->File = (char *) pMem->Allocate(sizeof(char), iLength + 1);
		strcpy_s(p->File, iLength + 1, sFile);

		XPVirtualPage.ToString("Description", sDescr, sizeof(sDescr), &iLength);
		p->Description = (char *) pMem->Allocate(sizeof(char), iLength + 1);
		strcpy_s(p->Description, iLength + 1, sDescr);

		p->Enabled = XPVirtualPage.ToBoolean("Enable", false);

		XPVirtualPage.Destroy();
	}

	return this->Locks.UnLockExclusive(this->Initialized = true);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CErrorPages::Destroy(void)
{
	this->dwErrorCount = 0;
	this->dwWarningCount = 0;

	this->Locks.LockExclusive();

	if(this->Initialized)
	{
		this->Initialized = false;
		if(this->Collection.Count > 0)
		{
			for(int iPos = 0; iPos < this->Collection.Count; iPos++)
			{
				pMem->Free(this->Collection.Items[iPos].Code);
				pMem->Free(this->Collection.Items[iPos].File);
				pMem->Free(this->Collection.Items[iPos].Description);
			}

			pMem->Free(this->Collection.Items);

			this->Collection.Count = 0;
		}
	}
	return this->Locks.UnLockExclusive(true);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

DWORD CErrorPages::WarningCount(void)
{
	return this->dwWarningCount;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

DWORD CErrorPages::ErrorCount(void)
{
	return this->dwErrorCount;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CErrorPages::AuthorizationRequired(VOID *pClient)
{
	CStringBuilder XtraHdrs;

	XtraHdrs.Append("WWW-Authenticate: Basic");
	XtraHdrs.Append(" realm=Secure Area #1");
	XtraHdrs.Append("\r\n");

	for(int iPos = 0; iPos < this->Collection.Count; iPos++)
	{
		if(_strcmpi(this->Collection.Items[iPos].Code, "401") == 0)
		{
			return this->SendAdvancedError(pClient, "401",
				this->Collection.Items[iPos].Description,
				"", this->Collection.Items[iPos].File, XtraHdrs.Buffer);
		}
	}

	return this->SendAdvancedError(pClient, "401",
		"</i>Unauthorised</i>", "", this->Collection.DefaultFile, XtraHdrs.Buffer);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CErrorPages::SendError(VOID *pClient, const char *sErrorCode)
{
	return this->SendError(pClient, sErrorCode, "");
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CErrorPages::SendError(VOID *pClient, const char *sErrorCode, const char *sDetails)
{
	for(int iPos = 0; iPos < this->Collection.Count; iPos++)
	{
		if(_strcmpi(sErrorCode, this->Collection.Items[iPos].Code) == 0)
		{
			return this->SendAdvancedError(pClient, sErrorCode,
				this->Collection.Items[iPos].Description,
				sDetails, this->Collection.Items[iPos].File, NULL);
		}
	}

	return this->SendAdvancedError(pClient, sErrorCode,
		"</i>Unknown Error</i>", sDetails, this->Collection.DefaultFile, NULL);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CErrorPages::SendAdvancedError(VOID *pClient, const char *sErrCode,
								const char *sErrDesc, const char *sDetails, const char *sErrorPage, const char *sExtraHeaders)
{
	CLIENT *pC = (CLIENT *)pClient;
	CWebSite *pWebSite = (CWebSite *)pC->pWebSite;
	CHTTP *pHTTP = (CHTTP *)pWebSite->pHttp;

	//Keep track of warnings and errors. Errors are all 500 (Internal Server) errors
	//	and warnings are all other errors except 404 (file not found) errors.
	if(strncmp(sErrCode, "404", 3) != 0)
	{
		if(strncmp(sErrCode, "500", 3) == 0)
		{
			this->dwErrorCount++;
		}
		else {
			this->dwWarningCount++;
		}
	}

	CStringBuilder sError;
	sError.AppendFromFile(sErrorPage);
	sError.Replace("###Code###", sErrCode);
	sError.Replace("###Description###", sErrDesc);
	sError.Replace("###Details###", sDetails);

	if(pHTTP->SendHTTPHeader(pC, sErrCode, sErrDesc, NULL, "text/html", sError.Length, sExtraHeaders, true))
	{
		if(_strcmpi(pC->Header.Method, "HEAD") != 0)
		{
			return pHTTP->BufferDataToClient(pC, &sError);
		}
		else{
			return true;
		}
	}

	return false;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#endif
