///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//  Copyright  NetworkDLS 2002, All rights reserved
//
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF 
// ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO 
// THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A 
// PARTICULAR PURPOSE.
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#ifndef _CHTTP_CPP
#define _CHTTP_CPP
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include <Windows.H>
#include <WindowsX.H>
#include <ShellAPI.H>
#include <Stdio.H>
#include <Stdlib.H>

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include "Entry.H"
#include "CHTTP.H"
#include "CWebSites.H"
#include "../../@Libraries/URLEncoding/URLEncoding.H"
#include "CDirectoryIndexing.H"
#include "HTTPFilter.H"

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

using namespace NSWFL::File;
using namespace NSWFL::String;

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

CHTTP::CHTTP(VOID *lpWebSites)
{
	pWebSites = lpWebSites;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

CHTTP::~CHTTP(void)
{
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

/*
	Return TRUE to keep the client connected (if applicable) otherwise FALSE.
*/
bool CHTTP::ProcessRequest(CWebSite *pWebSite, CLIENT *pC, const char *sHeader, const int iHeaderLength)
{
	if(!((CWebSite*)pC->pWebSite)->pURLFilters->ScanHeader(&pC->Header))
	{
		pWebSite->pErrorPages->SendError(pC, "401", "Authorization Failed By Filter");
		return true;
	}

	if(!pWebSite->pExtensionFilters->CheckFile(pC->Header.Request))
	{
		pWebSite->pErrorPages->SendError(pC, "401", "Authorization Failed By Filter");
		return true;
	}

	if(this->AssembleFullRequest(pWebSite, pC))
	{
		if(pWebSite->pAuthentication->IsAuthorizationRequired(pC))
		{
			return true; //Do not proccess the request at this time.
		}

		if(IsDirectory(pC->Header.FullRequest))
		{
			CDirectoryIndexing DI;
			if(!DI.SendContents(pC, pC->Header.FullRequest))
			{
				char sErrorMessage[1024];
				if(GetLastError(sErrorMessage, sizeof(sErrorMessage)))
				{
					pWebSite->pErrorPages->SendError(pC, "500", sErrorMessage);
				}
				else{
					pWebSite->pErrorPages->SendError(pC, "500", "Unknown logon error.");
				}
			}
		}
		else {
			if(pWebSite->pScriptingEngines->IsScriptFile(pC->Header.FullRequest))
			{
				int iResult = EXEC_RESULT_OK;
				if((iResult = pWebSite->pScriptingEngines->ProcessScript(pC)) != EXEC_RESULT_OK)
				{
					pWebSite->pErrorPages->SendError(pC, "500", pWebSite->pScriptingEngines->ErrorString(iResult));
				}
			}
			else if(pWebSite->pSSIFiles->IsSSIFile(pC->Header.FullRequest))
			{
				if(!pWebSite->pSSIFiles->ProcessServerSideInclude(pC, pC->Header.FullRequest))
				{
					pWebSite->pErrorPages->SendError(pC, "500");
				}
			}
			else if(pWebSite->pCGIFolders->IsInCGIFolder(pC))
			{
				if(pWebSite->pScriptingEngines->ProcessScript(pC) != EXEC_RESULT_OK)
				{
					pWebSite->pErrorPages->SendError(pC, "500");
				}
			}
			else{
				if(!this->SendFileWithHeader(pC, pC->Header.FullRequest))
				{
					pWebSite->pErrorPages->SendError(pC, "500");
				}
			}
		}
	}

	return true; //No reason to force a disconnect.
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CHTTP::SendRawFileEx(CLIENT *pC, const char *sFileName, char *sBuffer, unsigned int iBufferSz)
{
	bool bResult = true;
	DWORD dwBytesRead = 0;
    int iBytesRead = 0;
	__int64 i64FileSize = 0;
	__int64 i64BytesRead = 0;
	__int64 i64MaxSend = (pC->Header.ContentEnd - pC->Header.ContentBegin);

	HANDLE hFile = CreateFile(sFileName,
		GENERIC_READ, FILE_SHARE_READ | FILE_SHARE_WRITE,
		NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);

	if(hFile == INVALID_HANDLE_VALUE)
	{
		return false;
	}

	if(!FileSize(hFile, &i64FileSize))
	{
		return false;
	}

	if(pC->Header.ContentBegin > 0)
	{
		FileSeek(hFile, pC->Header.ContentBegin, FILE_BEGIN);
	}

	if(i64MaxSend > 0)
	{
		i64MaxSend++;
	}

	do{
		if(i64MaxSend > 0 && (i64BytesRead + iBufferSz) > i64MaxSend)
		{
			iBufferSz = (unsigned long)(i64MaxSend - i64BytesRead);
		}

		ReadFile(hFile, sBuffer, iBufferSz, &dwBytesRead, NULL);

		i64BytesRead += dwBytesRead;

		if(!this->BufferDataToClient(pC, sBuffer, dwBytesRead))
		{
			bResult = false;
			break;
		}

		if(i64MaxSend > 0 && i64BytesRead == i64MaxSend)
		{
			break;
		}

	} while(dwBytesRead == iBufferSz && iBufferSz > 0 && pC->pClient->IsConnected());

	CloseHandle(hFile);

	return bResult;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CHTTP::SendFileWithHeader(CLIENT *pC, const char *sFileName)
{
	CWebSite *pWebSite = (CWebSite *)pC->pWebSite;
	CHTTP *pHTTP = (CHTTP *)pWebSite->pHttp;
	CWebSites *pWebSites = (CWebSites *)pWebSite->pWebSites;

	if(!FileAccess(sFileName, FExist))
	{
		pWebSite->pErrorPages->SendError(pC, "404");
		return false;
	}

	char *sEncoding = NULL;
	char sSendFile[MAX_PATH];

	if(!this->IsValueEqual(pC->Header.CacheControl, "no-cache"))
	{
		if(this->DoesValueContain(pC->Header.AcceptEncoding, "gzip"))
		{
			if(((CWebSite*)pC->pWebSite)->pCompression->IsFileCompressible(sFileName))
			{
				if(((CWebSite*)pC->pWebSite)->pCompression->CompressFile(sFileName, sSendFile, sizeof(sSendFile)))
				{
					sEncoding = "gzip";
				}
			}
		}
	}

	if(!sEncoding) //If the file is not compressed.
	{
		strcpy_s(sSendFile, sizeof(sSendFile), sFileName);
	}

	__int64 i64FileSize = 0;

	if(FileSize(sSendFile, &i64FileSize))
	{
		char sMimeType[MIMETYPES_MAX_TYPE_LENGTH];

		if(!((CWebSite*)pC->pWebSite)->pMimeTypes->GetType(sFileName, sMimeType, sizeof(sMimeType)))
		{
			return false;
		}

		if(this->SendOKHeader(pC, sMimeType, sEncoding, i64FileSize))
		{
			if(_strcmpi(pC->Header.Method, "HEAD") != 0)
			{
				unsigned int iBufferSize = ((CWebSite*)pC->pWebSite)->pSettings->BufferSize();
				if(i64FileSize < iBufferSize)
				{
					iBufferSize = (int) i64FileSize;
				}
				
				if(iBufferSize > 0)
				{
					char *sBuffer = (char *) pMem->Allocate(sizeof(char), iBufferSize + 1);
					if(sBuffer)
					{
						bool bResult = this->SendRawFileEx(pC, sSendFile, sBuffer, iBufferSize);
						pMem->Free(sBuffer);
						return bResult;
					}
				}
				else{
					return true; //Zero-length file.
				}
			}
			else{
				return true; //Only sending header.
			}
		}
	}

	pWebSite->pErrorPages->SendError(pC, "500");
	return false;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CHTTP::SendRawFile(CLIENT *pC, const char *sFileName)
{
	__int64 i64FileSize = 0;

	if(FileSize(sFileName, &i64FileSize))
	{
		unsigned int iBufferSize = ((CWebSite*)pC->pWebSite)->pSettings->BufferSize();
		if(i64FileSize < iBufferSize)
		{
			iBufferSize = (int) i64FileSize;
		}
		
		char *sBuffer = (char *) pMem->Allocate(sizeof(char), iBufferSize + 1);
		if(sBuffer)
		{
			bool bResult = this->SendRawFileEx(pC, sFileName, sBuffer, iBufferSize);
			pMem->Free(sBuffer);
			return bResult;
		}
	}
	return false;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CHTTP::SendHTTPHeader(CLIENT *pC, const char *sErrCode,
						   const char *sErrDesc, const char *sEncoding,
						   const char *sMimeType, __int64 i64ContentLength,
						   const char *sExtraHeaders, bool bTerminateHeader)
{
	CStringBuilder sHDR;
	char sGMT[128];

	if(!GetGMTTime(sGMT, sizeof(sGMT)))
	{
		return false;
	}

	sHDR.AppendF("%s %s %s\n", HTTP_VERSION, sErrCode, sErrDesc);

	if(sMimeType && sMimeType[0] != 0) {
		sHDR.AppendF("Content-Type: %s\n", sMimeType);
	}
	if(sEncoding && sEncoding[0] != 0) {
		sHDR.AppendF("Content-Encoding: %s\n", sEncoding);
	}
	if(pC->Header.Cookie && pC->Header.Cookie[0] != 0) {
		sHDR.AppendF("Cookie: %s\n", pC->Header.Cookie);
	}

	sHDR.AppendF("Server: NetworkDLS - %s\n", gsTitleCaption);
	
	if(pC->KeepAlive)
	{
		sHDR.Append("Connection: Keep-Alive\n");
	}
	else{
		sHDR.Append("Connection: Close\n");
	}

	if(sExtraHeaders)
	{
		sHDR.Append(sExtraHeaders);
	}

	//We don't support content ranges on dynamic content.
	if(!pC->IsDynamicContent)
	{
		sHDR.Append("Accept-Ranges: bytes\n");

		if((pC->Header.ContentBegin != 0 || pC->Header.ContentEnd) && !pC->IsDynamicContent)
		{
			bool bAddOneByte = false;

			if(pC->Header.ContentEnd <= 0)
			{
				pC->Header.ContentEnd = i64ContentLength - 1;
				bAddOneByte = true;
			}

			sHDR.AppendF("Content-Range: bytes %I64d-%I64d/%I64d\n",
				pC->Header.ContentBegin, pC->Header.ContentEnd, i64ContentLength);

			i64ContentLength = (pC->Header.ContentEnd - pC->Header.ContentBegin);

			if(bAddOneByte)
			{
				i64ContentLength++;
			}
		}
	}

	if(i64ContentLength > 0)
	{
		sHDR.AppendF("Content-Length: %I64d\n", i64ContentLength);
	}

	sHDR.AppendF("Date: %s\n", sGMT);

	if(bTerminateHeader)
	{
		sHDR.Append("\n");
	}

	//strcat_s(sHdrStr, sizeof(sHdrStr), "Transfer-Encoding: chunked\n");

	//Save the clients last response information.
	strcpy_s(pC->LastResponse.ResponseCode, sizeof(pC->LastResponse.ResponseCode), sErrCode);
	pC->LastResponse.ContentLength = i64ContentLength;

	bool bResult = pC->pClient->SetNextSendData(sHDR.Buffer);

	sHDR.Destroy();

	return bResult;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CHTTP::SendOKHeader(CLIENT *pC, __int64 i64FileSize)
{
	return this->SendOKHeader(pC, "text/html", i64FileSize);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CHTTP::SendOKHeader(CLIENT *pC, const char *sMimeType, const char *sEncoding, __int64 i64FileSize)
{
	char *sCode = "200";
	char *sDesc = "OK";
	if((pC->Header.ContentBegin != 0 || pC->Header.ContentEnd) && !pC->IsDynamicContent)
	{
		sCode = "206";
		sDesc = "Partial content";
	}
	return this->SendHTTPHeader(pC, sCode, sDesc, sEncoding, sMimeType, i64FileSize, NULL, true);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CHTTP::SendOKHeader(CLIENT *pC, const char *sMimeType,
						 const char *sEncoding, __int64 i64FileSize, bool bTerminateHeader)
{
	char *sCode = "200";
	char *sDesc = "OK";
	if((pC->Header.ContentBegin != 0 || pC->Header.ContentEnd) && !pC->IsDynamicContent)
	{
		sCode = "206";
		sDesc = "Partial content";
	}
	return this->SendHTTPHeader(pC, sCode, sDesc, sEncoding, sMimeType, i64FileSize, NULL, bTerminateHeader);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CHTTP::SendOKHeader(CLIENT *pC, const char *sMimeType, __int64 i64FileSize)
{
	char *sCode = "200";
	char *sDesc = "OK";
	if((pC->Header.ContentBegin != 0 || pC->Header.ContentEnd) && !pC->IsDynamicContent)
	{
		sCode = "206";
		sDesc = "Partial content";
	}
	return this->SendHTTPHeader(pC, sCode, sDesc, NULL, sMimeType, i64FileSize, NULL, true);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CHTTP::BufferDataToClient(CLIENT *pC, CStringBuilder *sData)
{
	return this->BufferDataToClient(pC, sData->Buffer, sData->Length);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CHTTP::BufferDataToClient(CLIENT *pC, const char *sData, unsigned int iDataLength)
{
	int iBytesRemaining = iDataLength;
	int iBytes = 0;
	int iBufferSize = ((CWebSite*)pC->pWebSite)->pSettings->BufferSize();

	while(iBytesRemaining > 0 && pC->pClient->IsConnected())
	{
		if(iBytesRemaining > iBufferSize)
		{
			iBytes = iBufferSize;
		}
		else{
			iBytes = iBytesRemaining;
		}

		if(!pC->pClient->SetNextSendData(sData + iDataLength - iBytesRemaining, iBytes))
		{
			return false;
		}

		iBytesRemaining -= iBytes;
	}

	return true;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CHTTP::SendOKHeader(CLIENT *pC, const char *sFileName)
{
	return this->SendOKHeader(pC, sFileName, (char *)NULL);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CHTTP::SendOKHeader(CLIENT *pC, const char *sFileName, const char *sEncoding)
{
	char sMimeType[MIMETYPES_MAX_TYPE_LENGTH];

	if(!((CWebSite*)pC->pWebSite)->pMimeTypes->GetType(sFileName, sMimeType, sizeof(sMimeType)))
	{
		return false;
	}

	__int64 i64FileSize = 0;

	if(!FileSize(sFileName, &i64FileSize))
	{
		return false;
	}

	return this->SendOKHeader(pC, sMimeType, sEncoding, i64FileSize);
}


///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CHTTP::FreeClientHeader(CLIENT *pC)
{
	if(pC->Header.AuthenticationRealm) pMem->Free(pC->Header.AuthenticationRealm);
	if(pC->Header.Method) pMem->Free(pC->Header.Method);
	if(pC->Header.Request) pMem->Free(pC->Header.Request);
	if(pC->Header.Version) pMem->Free(pC->Header.Version);
	if(pC->Header.Accept) pMem->Free(pC->Header.Accept);
	if(pC->Header.AcceptLanguage) pMem->Free(pC->Header.AcceptLanguage);
	if(pC->Header.AcceptEncoding) pMem->Free(pC->Header.AcceptEncoding);
	if(pC->Header.UserAgent) pMem->Free(pC->Header.UserAgent);
	if(pC->Header.Host) pMem->Free(pC->Header.Host);
	if(pC->Header.Connection) pMem->Free(pC->Header.Connection);
	if(pC->Header.Query) pMem->Free(pC->Header.Query);
	if(pC->Header.CacheControl) pMem->Free(pC->Header.CacheControl);
	if(pC->Header.Referer) pMem->Free(pC->Header.Referer);
	if(pC->Header.Cookie) pMem->Free(pC->Header.Cookie);
	if(pC->Header.PostData) pMem->Free(pC->Header.PostData);
	if(pC->Header.FullRequest) pMem->Free(pC->Header.FullRequest);
	if(pC->Header.ContentRange) pMem->Free(pC->Header.ContentRange);
	if(pC->Header.ContentType) pMem->Free(pC->Header.ContentType);

	memset(&pC->Header, 0, sizeof(pC->Header));

	return true;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CHTTP::GetHeaderRequestRange(CWebSite *pWebSite, CLIENT *pC, const char *sBuf, const int iBufLen)
{
	bool bResult = false;

	if(iBufLen > 0)
	{
		char sTok[255];
		int iRPos = 0;
		int iTokSz = 0;

		const char *sTokSep = "`~!@#$%^&*()_-+=<,>.?/;:'\"[{]}\\|";
		const int iTokSep = (int)strlen(sTokSep);

		if((iTokSz = GetNextToken(sBuf, iBufLen, sTok, sizeof(sTok), &iRPos, sTokSep, iTokSep)) > 0)
		{
			if(sBuf[iRPos++] == '=')
			{
				if((iTokSz = GetNextToken(sBuf, iBufLen, sTok, sizeof(sTok), &iRPos, sTokSep, iTokSep)) > 0)
				{
					pC->Header.ContentBegin = _atoi64(sTok);

					if(sBuf[iRPos++] == '-')
					{
						bResult = true;
						if((iTokSz = GetNextToken(sBuf, iBufLen, sTok, sizeof(sTok), &iRPos, sTokSep, iTokSep)) > 0)
						{
							pC->Header.ContentEnd = _atoi64(sTok);
						}
					}
				}
			}
		}
	}
	
	return bResult;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CHTTP::ParseHTTPHeader(CWebSite *pWebSite, CLIENT *pC, const char *sBuf, const int iBufLen)
{
	pWebSite->Stats.Hits++;

	memset(&pC->Header, 0, sizeof(HTTPHEADER));

	// -- ANYTHING ALLOCATED HERE MUST BE FREED IN FreeClientHeader() --
	int iTokenSz = 0;
	int iRequestLength = 0;
	int iRPos = 0;

	iTokenSz = GetNextToken(sBuf, iBufLen, pC->Header.Method, &iRPos);

	if(_strcmpi(pC->Header.Method, "GET") != 0
		&& _strcmpi(pC->Header.Method, "PUT") != 0
		&& _strcmpi(pC->Header.Method, "POST") != 0
		&& _strcmpi(pC->Header.Method, "HEAD") != 0)
	{
		pWebSite->pErrorPages->SendError(pC, "400");
		return false;
	}

	iRequestLength = GetNextToken(sBuf, iBufLen, pC->Header.Request, &iRPos);
	iTokenSz = GetNextToken(sBuf, iBufLen, pC->Header.Version, &iRPos);

	iTokenSz = this->GetHTTPHeaderTag(sBuf, iBufLen, "Accept:", pC->Header.Accept);
	iTokenSz = this->GetHTTPHeaderTag(sBuf, iBufLen, "Accept-Language:", pC->Header.AcceptLanguage);
	iTokenSz = this->GetHTTPHeaderTag(sBuf, iBufLen, "Accept-Encoding:", pC->Header.AcceptEncoding);
	iTokenSz = this->GetHTTPHeaderTag(sBuf, iBufLen, "User-Agent:", pC->Header.UserAgent);
	iTokenSz = this->GetHTTPHeaderTag(sBuf, iBufLen, "Cache-Control:", pC->Header.CacheControl);
	iTokenSz = this->GetHTTPHeaderTag(sBuf, iBufLen, "Cookie:", pC->Header.Cookie);
	iTokenSz = this->GetHTTPHeaderTag(sBuf, iBufLen, "Host:", pC->Header.Host);
	iTokenSz = this->GetHTTPHeaderTag(sBuf, iBufLen, "Connection:", pC->Header.Connection);
	iTokenSz = this->GetHTTPHeaderTag(sBuf, iBufLen, "Referer:", pC->Header.Referer);
	iTokenSz = this->GetHTTPHeaderTag(sBuf, iBufLen, "Content-Type:", pC->Header.ContentType);
	iTokenSz = this->GetHTTPHeaderTag(sBuf, iBufLen, "Range:", pC->Header.ContentRange);

	GetHeaderRequestRange(pWebSite, pC, pC->Header.ContentRange, iTokenSz);

	//Parse query string.
	int iQueryIndex = CharIndex(pC->Header.Request, '?');
	if(iQueryIndex > 0)
	{
		iQueryIndex++; //Skip the question mark;

		pC->Header.Query = (char *) pMem->Allocate(sizeof(char), (iRequestLength - iQueryIndex) + 1);
		strcpy_s(pC->Header.Query, (iRequestLength - iQueryIndex) + 1,
			pC->Header.Request + iQueryIndex);

		pC->Header.Request[iQueryIndex - 1] = '\0';
	}
	else{
		pC->Header.Query = (char *) pMem->Allocate(sizeof(char), 1);
		strcpy_s(pC->Header.Query, 1, "");
	}

	URLDecode(pC->Header.Request);
	URLDecode(pC->Header.Query);

	//Parse post data.
	int iPostStartPos = -1;
	if((iPostStartPos = InStr("\n\n", 2, sBuf, iBufLen, 0)) <= 0)
	{
		if((iPostStartPos = InStr("\r\n\r\n", 4, sBuf, iBufLen, 0)) <= 0)
		{
			iPostStartPos = -1;
		}
		else{
			iPostStartPos += 4;
		}
	}
	else{
		iPostStartPos += 2;
	}

	if(iPostStartPos > 0 && iBufLen > iPostStartPos)
	{
		pC->Header.PostDataSize = (iBufLen - iPostStartPos);
		pC->Header.PostData = (char *) pMem->Allocate(sizeof(char), pC->Header.PostDataSize + 1);
		strcpy_s(pC->Header.PostData, pC->Header.PostDataSize + 1, sBuf + iPostStartPos);
	}
	else{
		pC->Header.PostDataSize = 0;
		pC->Header.PostData = (char *) pMem->Allocate(sizeof(char), 1);
		strcpy_s(pC->Header.PostData, 1, "");
	}

	//Will this connection be using keep-alives?
	if(((CWebSite*)pC->pWebSite)->pSettings->UseKeepAlives())
	{
		if(_strcmpi(pC->Header.Connection, "Keep-Alive") == 0) //Keep-Alive vs. Close
		{
			pC->KeepAlive = true;
		}
		else{
			pC->KeepAlive = false;
		}
	}

	FILTERHTTPHEADER FH;
	memset(&FH, 0, sizeof(FH));
	FH.Method = pC->Header.Method;
	FH.Request = pC->Header.Request;
	FH.Version = pC->Header.Version;
	FH.Query = pC->Header.Query;
	FH.Accept = pC->Header.Accept;
	FH.AcceptLanguage = pC->Header.AcceptLanguage;
	FH.AcceptEncoding = pC->Header.AcceptEncoding;
	FH.UserAgent = pC->Header.UserAgent;
	FH.Host = pC->Header.Host;
	FH.Connection = pC->Header.Connection;
	FH.CacheControl = pC->Header.CacheControl;
	FH.Referer = pC->Header.Referer;
	FH.Cookie = pC->Header.Cookie;
	FH.ContentRange = pC->Header.ContentRange;
	FH.ContentBegin = pC->Header.ContentBegin;
	FH.ContentEnd = pC->Header.ContentEnd;
	FH.PostData = pC->Header.PostData;
	FH.PostDataSize = pC->Header.PostDataSize;
	FH.ContentType = pC->Header.ContentType;

	return pWebSite->pCustomFilters->ProcessHeadFilters(pC, &FH);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
	Returns true if processing should continue, otherwise false.
*/
bool CHTTP::AssembleFullRequest(CWebSite *pWebSite, CLIENT *pC)
{
	int iAlloc = 0;

	ROOT *pVirtualRoot = pWebSite->pVirtualRoots->IsVirtualRoot(pC->Header.Request, pC->Header.FullRequest);

	if(pVirtualRoot)
	{
		if(pVirtualRoot->Username)
		{
			HANDLE hToken = NULL;
			char *sDomain = NULL;
			char sErrorMessage[1024];

			if(strlen(pVirtualRoot->Domain) > 0)
			{
				sDomain = pVirtualRoot->Domain;
			}

			if((hToken = NTLogin(pVirtualRoot->Username, sDomain, pVirtualRoot->Password, sErrorMessage, sizeof(sErrorMessage))))
			{
				if(!ImpersonateLoggedOnUser(hToken))
				{
					char sErrorMessage[1024];
					if(GetLastError(sErrorMessage, sizeof(sErrorMessage)))
					{
						pWebSite->pErrorPages->SendError(pC, "500", sErrorMessage);
					}
					else{
						pWebSite->pErrorPages->SendError(pC, "500", "Unknown logon error.");
					}

					CloseHandle(hToken);
					return false;
				}
				CloseHandle(hToken);
			}
			else{
				pWebSite->pErrorPages->SendError(pC, "500", sErrorMessage);
				return false;
			}
		}
	}
	else {
		iAlloc = (int)(strlen(pWebSite->Root) + strlen(pC->Header.Request) + 2);
		pC->Header.FullRequest = (char *) pMem->Allocate(sizeof(char), iAlloc);
		sprintf_s(pC->Header.FullRequest, iAlloc, "%s\\%s", pWebSite->Root, pC->Header.Request);
		CorrectReversePath(pC->Header.FullRequest, iAlloc, true);
	}

	DWORD dwAttributes = GetFileAttributes(pC->Header.FullRequest);
	if(dwAttributes == INVALID_FILE_ATTRIBUTES)
	{
		char sErrorMessage[1024];
		if(GetLastError(sErrorMessage, sizeof(sErrorMessage)))
		{
			if(InStrI("cannot find", sErrorMessage) > 0)
			{
				//If the file nolonger exists, remove it from the cache (if it exists there).
				((CWebSite*)pC->pWebSite)->pCompression->RemoveFileFromCache(pC->Header.FullRequest);

				pWebSite->pErrorPages->SendError(pC, "404", sErrorMessage);
				return true;
			}

			pWebSite->pErrorPages->SendError(pC, "500", sErrorMessage);
		}
		else{
			pWebSite->pErrorPages->SendError(pC, "500", "Unknown logon error.");
		}
		return false;
	}
	else{
		if(dwAttributes &FILE_ATTRIBUTE_DIRECTORY)
		{
			if(pWebSite->pDefaultPages->CheckPath(pC->Header.FullRequest))
			{
				//pC->Header.FullRequest now contains the full request.
				return pWebSite->pCustomFilters->ProcessUrlMap(pC, pC->Header.Request, pC->Header.FullRequest);
			}
			else {
				if(pWebSite->pSettings->AllowDirectoryIndexing() && !pWebSite->pCGIFolders->IsInCGIFolder(pC))
				{
					//Will be sending directory structure.
					return pWebSite->pCustomFilters->ProcessUrlMap(pC, pC->Header.Request, pC->Header.FullRequest);
				}
				else{
					pWebSite->pErrorPages->SendError(pC, "403", "Directory Listing Denied");
					return false;
				}
			}
		}
		else{
			//pC->Header.FullRequest now contains the full request.
			return pWebSite->pCustomFilters->ProcessUrlMap(pC, pC->Header.Request, pC->Header.FullRequest);
		}
	}

	return false;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

int CHTTP::GetHTTPHeaderTag(const char *sHeader, const int iHeaderLen, const char *sTag, char *&sBuf)
{
	int iTagLength = (int)strlen(sTag);
	int iRPos = 0;

	iRPos = InStrI(sTag, iTagLength, sHeader, iHeaderLen, 0);
	//The tag must either be on a line by its-self or the first line.
	while(iRPos > 0 && sHeader[iRPos - 1] != '\n')
	{
		//the tag may be further down the line...
		iRPos = InStrI(sTag, iTagLength, sHeader, iHeaderLen, iRPos + 1);
	}

	if(iRPos >= 0)
	{
		iRPos += iTagLength; //Skip the tag itsself;
		SkipWhiteSpaces(sHeader, iHeaderLen, &iRPos);

		int iEndPos = InStr("\n", 1, sHeader, iHeaderLen, iRPos);
		if(iEndPos > 0 && iEndPos > iRPos)
		{
			if(iEndPos > iRPos)
			{
				int iLength = iEndPos - iRPos;
				sBuf = (char *) pMem->Allocate(sizeof(char), iLength + 1);
				strncpy_s(sBuf, iLength + 1, sHeader + iRPos, iLength);
				return Trim(sBuf, iLength);
			}
		}
	}

	sBuf = (char *)pMem->Allocate(1,1);
	sBuf[0] = '\0';

	return 0;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

int CHTTP::GetHTTPHeaderTag(const char *sHeader, const int iHeaderLen,
								const char *sTag, char *sBuf, int iMaxBuf)
{
	int iTagLength = (int)strlen(sTag);
	int iRPos = 0;

	if((iRPos = InStrI(sTag, iTagLength, sHeader, iHeaderLen, 0)) >= 0)
	{
		iRPos += iTagLength; //Skip the tag itsself;
		SkipWhiteSpaces(sHeader, iHeaderLen, &iRPos);

		int iEndPos = InStr("\n", 1, sHeader, iHeaderLen, iRPos);
		if(iEndPos > 0 && iEndPos > iRPos)
		{
			if(iEndPos > iRPos)
			{
				int iLength = iEndPos - iRPos;
				strncpy_s(sBuf, iMaxBuf, sHeader + iRPos, iLength);
				return Trim(sBuf, iLength);
			}
		}
	}

	strcpy_s(sBuf, iMaxBuf, "");

	return 0;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CHTTP::DoesHeaderContainTag(const char *sBuf, const int iBufLen, const char *sTag)
{
	return this->DoesHeaderContainTag(sBuf, iBufLen, sTag, NULL);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CHTTP::DoesHeaderContainTag(const char *sBuf, const int iBufLen,
								 const char *sTag, int *iHeaderEndPos)
{
	if(iHeaderEndPos)
	{
		*iHeaderEndPos = 0;
	}

	int iTagPos = InStrI(sTag, sBuf, iBufLen, 0);

	//The tag must either be on a line by its-self or the first line.
	while(iTagPos > 0 && sBuf[iTagPos - 1] != '\n')
	{
		//the tag may be further down the line...
		iTagPos = InStrI(sTag, sBuf, iBufLen, iTagPos + 1);
	}

	if(iTagPos >= 0)
	{
		int iLineFeedLen = 0;
		int iLineFeedPos = 0;
		int iSingleLineFeedPos = InStr("\n\n", 2, sBuf, iBufLen, 0);
		int iDoubleLineFeedPos = InStr("\r\n\r\n", 4, sBuf, iBufLen, 0);

		if(iDoubleLineFeedPos > 0 && iSingleLineFeedPos > 0)
		{
			if(iDoubleLineFeedPos < iSingleLineFeedPos) //We want to use the first one.
			{
				iLineFeedLen = 4;
				iLineFeedPos = iDoubleLineFeedPos;
			}
			else {
				iLineFeedLen = 2;
				iLineFeedPos = iSingleLineFeedPos;
			}
		}
		else if(iDoubleLineFeedPos > 0) {
			iLineFeedLen = 4;
			iLineFeedPos = iDoubleLineFeedPos;
		}
		else if(iSingleLineFeedPos > 0) {
			iLineFeedLen = 2;
			iLineFeedPos = iSingleLineFeedPos;
		}

		if(iLineFeedPos > 0 && iTagPos < iLineFeedPos)
		{
			if(iHeaderEndPos)
			{
				*iHeaderEndPos = (iLineFeedPos + iLineFeedLen);
			}
			return true;
		}
	}

	return false;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CHTTP::IsValueEqual(const char *sValue1, const char *sValue2)
{
	if(sValue1 && sValue2)
	{
		return(_strcmpi(sValue1, sValue2) == 0);
	}
	return false;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CHTTP::DoesValueContain(const char *sIn, const char *sValue)
{
	if(sIn && sValue)
	{
		return(InStrI(sValue, sIn) >= 0);
	}
	return false;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#endif
