///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//  Copyright  NetworkDLS 2002, All rights reserved
//
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF 
// ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO 
// THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A 
// PARTICULAR PURPOSE.
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#ifndef _CMIMETYPES_CPP
#define _CMIMETYPES_CPP
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include <Windows.H>
#include <ShlObj.H>
#include <Stdio.H>
#include <ShlOBJ.H>
#include <Stdlib.H>

extern HIMAGELIST hEnableDisableImageList; //Declared in MainDialog.cpp
extern HIMAGELIST hOnePixilImageList; //Declared in MainDialog.cpp

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include "CMimeTypes.H"
#include "Entry.H"
#include "CWebSites.H"
#include "CWebSite.H"

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

using namespace NSWFL::Windows;
using namespace NSWFL::ListView;
using namespace NSWFL::String;
using namespace NSWFL::File;

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void CMimeTypes::AddConfigItem(HWND hListView, MIMETYPE *pType)
{
	this->Locks.Lock();
	this->AddConfigItem(hListView, pType, ListView_GetItemCount(hListView));
	this->Locks.UnLock();
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void CMimeTypes::AddConfigItem(HWND hListView, MIMETYPE *pType, int iInsertionIndex)
{
	this->Locks.Lock();
	LV_ITEM ListItem;
    memset(&ListItem, 0, sizeof(ListItem));

	int iImage = pType->Enabled && this->Collection.Enabled;

	ListItem.pszText = (char *) pType->Extension;
    ListItem.mask = LVIF_TEXT|LVIF_IMAGE;
	ListItem.iImage = iImage;
    ListItem.iSubItem = MIMETYPES_LIST_POS_EXTENSION;
	ListItem.iItem = iInsertionIndex;
    int iItem = ListView_InsertItem(hListView, &ListItem);

    ListItem.pszText = (char *) pType->Type;
    ListItem.mask = LVIF_TEXT;
    ListItem.iSubItem = MIMETYPES_LIST_POS_TYPE;
	ListItem.iItem = iInsertionIndex;
    ListView_SetItem(hListView, &ListItem);

    ListItem.pszText = (char *) pType->Description;
    ListItem.mask = LVIF_TEXT;
    ListItem.iSubItem = MIMETYPES_LIST_POS_DESCR;
	ListItem.iItem = iInsertionIndex;
    ListView_SetItem(hListView, &ListItem);

	ListView_SetCheckState(hListView, iItem, pType->Enabled);
	this->Locks.UnLock();
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void CMimeTypes::PopConfigList(HWND hWnd, int iListView, int iEnabled)
{
	this->Locks.Lock();
	CheckDlgButton(hWnd, iEnabled, this->Collection.Enabled);
	this->PopInfoList(GetDlgItem(hWnd, iListView), false);
	this->Locks.UnLock();
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void CMimeTypes::PopInfoList(HWND hListView)
{
	this->Locks.Lock();
	this->PopInfoList(hListView, true);
	this->Locks.UnLock();
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void CMimeTypes::PopInfoList(HWND hListView, bool bUseIcons)
{
	this->Locks.Lock();
	SendMessage(hListView, (UINT)WM_SETREDRAW , (WPARAM)FALSE, (LPARAM)0);

	LV_COLUMN MyLVColumn;
    memset(&MyLVColumn, 0, sizeof(LV_COLUMN));

	MyLVColumn.cx        = 100;
    MyLVColumn.pszText   = "Extension";
    MyLVColumn.iSubItem  = MIMETYPES_LIST_POS_EXTENSION;
    MyLVColumn.mask      = LVCF_WIDTH | LVCF_TEXT | LVCF_SUBITEM | LVCF_FMT;
    MyLVColumn.fmt       = LVCFMT_LEFT;
    ListView_InsertColumn(hListView, MIMETYPES_LIST_POS_EXTENSION, &MyLVColumn);

	MyLVColumn.cx        = 300;
    MyLVColumn.pszText   = "Mime Type";
    MyLVColumn.iSubItem  = MIMETYPES_LIST_POS_TYPE;
    MyLVColumn.mask      = LVCF_WIDTH | LVCF_TEXT | LVCF_SUBITEM | LVCF_FMT;
    MyLVColumn.fmt       = LVCFMT_LEFT;
    ListView_InsertColumn(hListView, MIMETYPES_LIST_POS_TYPE, &MyLVColumn);

	MyLVColumn.cx        = 300;
    MyLVColumn.pszText   = "Description";
    MyLVColumn.iSubItem  = MIMETYPES_LIST_POS_DESCR;
    MyLVColumn.mask      = LVCF_WIDTH | LVCF_TEXT | LVCF_SUBITEM | LVCF_FMT;
    MyLVColumn.fmt       = LVCFMT_LEFT;
    ListView_InsertColumn(hListView, MIMETYPES_LIST_POS_DESCR, &MyLVColumn);

	DWORD dwFlags = LVS_EX_FULLROWSELECT|LVS_EX_GRIDLINES|LVS_EX_HEADERDRAGDROP;

	if(!bUseIcons)
	{
		dwFlags |= LVS_EX_CHECKBOXES;
	}

    SendMessage(hListView, (UINT)LVM_SETEXTENDEDLISTVIEWSTYLE, (WPARAM)0, (LPARAM)dwFlags);

	if(bUseIcons)
	{
		ListView_SetImageList(hListView, hEnableDisableImageList, LVSIL_SMALL);
	}

	ListView_SetTextColor(hListView, RGB(0,0,100));

	//Pop the list-view.
	for(int x = 0; x < this->Collection.Count; x++)
	{
		this->AddConfigItem(hListView, &this->Collection.Items[x]);
	}
	LVAutoSizeColumns(hListView, 500);
	SendMessage(hListView, (UINT)WM_SETREDRAW , (WPARAM)TRUE, (LPARAM)0);
	this->Locks.UnLock();
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

CMimeTypes::~CMimeTypes(void)
{
	this->Destroy();
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

CMimeTypes::CMimeTypes(void *lpWebSites)
{
	this->Initialized = false;
	this->pWebSites = lpWebSites;
	strcpy_s(sFileName, sizeof(sFileName), "");
	memset(&this->Collection, 0, sizeof(this->Collection));
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

CMimeTypes::CMimeTypes(void *lpWebSites, CXMLReader *xmlConfig, CMimeTypes *pDefaults)
{
	this->Initialized = true;
	this->pWebSites = lpWebSites;
	strcpy_s(sFileName, sizeof(sFileName), "");
	memset(&this->Collection, 0, sizeof(this->Collection));
	this->Load(xmlConfig, pDefaults);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CMimeTypes::Save(void)
{
	this->Locks.Lock();

	CXMLReader xmlConfig;
	if(this->ToXML(&xmlConfig))
	{
		bool bResult = xmlConfig.ToFile(this->sFileName);
		if(!bResult)
		{
			char sErrorMsg[2048];
			GetLastError(sErrorMsg, sizeof(sErrorMsg),
				"Failed to save websites configuration file (", ")");
			strcat_s(sErrorMsg, sizeof(sErrorMsg), "\r\n\r\n\"");
			strcat_s(sErrorMsg, sizeof(sErrorMsg), this->sFileName);
			strcat_s(sErrorMsg, sizeof(sErrorMsg), "\"");
			((CWebSites *)this->pWebSites)->Trace(sErrorMsg);
		}
		return this->Locks.UnLock(bResult);
	}
	return this->Locks.UnLock(false);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CMimeTypes::ToXML(CXMLReader *lpXML)
{
	this->Locks.Lock();

	CXMLWriter xmlConfig("MimeTypes");

	xmlConfig.AddBool("Enable", this->Collection.Enabled);

	for(int iItem = 0; iItem < this->Collection.Count; iItem++)
	{
		CXMLWriter Item("Type");
		Item.Add("Extension", this->Collection.Items[iItem].Extension);
		Item.Add("Mime", this->Collection.Items[iItem].Type);
		Item.Add("Description", this->Collection.Items[iItem].Description);
		Item.AddBool("Enable", this->Collection.Items[iItem].Enabled);
		xmlConfig.Add(&Item);
	}

	xmlConfig.ToReader(lpXML);

	xmlConfig.Destroy();

	return this->Locks.UnLock(true);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

/*
	Loads a modified configuration from the configuration dialog (also saves to original file).
*/
bool CMimeTypes::LoadNewConfiguration(HWND hWnd, HWND hGrid)
{
	this->Locks.LockExclusive();

	if(this->Initialized)
	{
		this->Destroy();
	}

	int iGridItems = LVItemCount(hGrid);

	char sExt[MIMETYPES_MAX_EXT_LENGTH];
	char sType[MIMETYPES_MAX_TYPE_LENGTH];
	char sDescr[MIMETYPES_MAX_DESCR_LENGTH];

	CXMLWriter xmlConfig("MimeTypes");

	xmlConfig.AddBool("Enable", IsDlgButtonChecked(hWnd, IDC_ENABLED) == BST_CHECKED);

	for(int iItem = 0; iItem < iGridItems; iItem++)
	{
		CXMLWriter Item("Type");

		LVGetText(hGrid, iItem, MIMETYPES_LIST_POS_EXTENSION, sExt, sizeof(sExt));
		Item.Add("Extension", sExt);

		LVGetText(hGrid, iItem, MIMETYPES_LIST_POS_TYPE, sType, sizeof(sType));
		Item.Add("Mime", sType);

		LVGetText(hGrid, iItem, MIMETYPES_LIST_POS_DESCR, sDescr, sizeof(sDescr));
		Item.Add("Description", sDescr);

		Item.AddBool("Enable", LVChecked(hGrid, iItem));
		xmlConfig.Add(&Item);
	}

	if(strlen(this->sFileName) > 0)
	{
		if(xmlConfig.ToFile(this->sFileName))
		{
			return this->Locks.UnLockExclusive(this->Reload());
		}
	}
	else{
		CXMLReader XMLRConfig;
		return this->Locks.UnLockExclusive(Load(xmlConfig.ToReader(&XMLRConfig), NULL));
	}

	return this->Locks.UnLockExclusive(false);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
	Reloads the configuration from the file it was originally loaded from.
*/
bool CMimeTypes::Reload(void)
{
	this->Locks.LockExclusive();

	if(this->Initialized)
	{
		this->Destroy();
	}

	this->Load(this->sFileName);

	return this->Locks.UnLockExclusive(this->Initialized);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CMimeTypes::Load(const char *sXMLFileName)
{
	this->Locks.LockExclusive();

	this->Initialized = false;

	strcpy_s(this->sFileName, sizeof(this->sFileName), sXMLFileName);

	CXMLReader xmlConfig;

	if(xmlConfig.FromFile(sXMLFileName))
	{
		CXMLReader xmlEntity;
		if(xmlConfig.ToReader("MimeTypes", &xmlEntity))
		{
			this->Initialized = this->Load(&xmlEntity, NULL);
		}
		xmlEntity.Destroy();
		xmlConfig.Destroy();
	}

	return this->Locks.UnLockExclusive(this->Initialized);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CMimeTypes::Load(CXMLReader *xmlConfig, CMimeTypes *pDefaults)
{
	this->Locks.LockExclusive();

	int iLength = 0;
	memset(&this->Collection, 0, sizeof(this->Collection));

	this->Collection.Enabled = xmlConfig->ToBoolean("Enable", true);

	xmlConfig->ProgressiveScan(true);
	CXMLReader XPMimeType;

	while(xmlConfig->ToReader("Type", &XPMimeType))
	{
		char sExt[MIMETYPES_MAX_EXT_LENGTH];
		char sType[MIMETYPES_MAX_TYPE_LENGTH];
		char sDescr[MIMETYPES_MAX_DESCR_LENGTH];

		this->Collection.Items = (MIMETYPE *)
			pMem->ReAllocate(this->Collection.Items, sizeof(MIMETYPE), this->Collection.Count + 1);

		MIMETYPE *p = &this->Collection.Items[Collection.Count++];

		XPMimeType.ToString("Extension", sExt, sizeof(sExt), &iLength);
		p->Extension = (char *) pMem->Allocate(sizeof(char), iLength + 1);
		strcpy_s(p->Extension, iLength + 1, LCase(sExt, iLength));

		XPMimeType.ToString("Mime", sType, sizeof(sType), &iLength);
		p->Type = (char *) pMem->Allocate(sizeof(char), iLength + 1);
		strcpy_s(p->Type, iLength + 1, sType);

		XPMimeType.ToString("Description", sDescr, sizeof(sDescr), &iLength);
		p->Description = (char *) pMem->Allocate(sizeof(char), iLength + 1);
		strcpy_s(p->Description, iLength + 1, sDescr);

		p->Enabled = XPMimeType.ToBoolean("Enable", true);

		XPMimeType.Destroy();
	}

	return this->Locks.UnLockExclusive(this->Initialized = true);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CMimeTypes::Destroy(void)
{
	this->Locks.LockExclusive();

	if(this->Initialized)
	{
		this->Initialized = false;
		if(this->Collection.Count > 0)
		{
			for(int iPos = 0; iPos < this->Collection.Count; iPos++)
			{
				pMem->Free(this->Collection.Items[iPos].Extension);
				pMem->Free(this->Collection.Items[iPos].Type);
				pMem->Free(this->Collection.Items[iPos].Description);
			}

			pMem->Free(this->Collection.Items);

			this->Collection.Count = 0;
		}
	}
	return this->Locks.UnLockExclusive(true);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CMimeTypes::GetType(const char *sFileName, char *sMimeName, int iMaxMimeName)
{
	this->Locks.Lock();
	if(this->Collection.Enabled && (this->Collection.Count > 0))
	{
		char sExt[MAX_PATH];
		GetFileExtension(sFileName, sExt, sizeof(sExt));

		for(int iPos = 0; iPos < this->Collection.Count; iPos++)
		{
			if(this->Collection.Items[iPos].Enabled)
			{
				if(_strcmpi(sExt, this->Collection.Items[iPos].Extension) == 0)
				{
					strcpy_s(sMimeName, iMaxMimeName, this->Collection.Items[iPos].Type);
					return this->Locks.UnLock(true);
				}
			}
		}
	}

	strcpy_s(sMimeName, iMaxMimeName, "application/octet-stream");
	
	return this->Locks.UnLock(true);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CMimeTypes::Export(const char *sFileName)
{
	this->Locks.Lock();

	FILE *hTarget = NULL;
	if(fopen_s(&hTarget, sFileName, "wb") == 0)
	{
		for(int iPos = 0; iPos < this->Collection.Count; iPos++)
		{
			fprintf(hTarget, "%s\t%s\t%s\t%s\r\n",
				this->Collection.Items[iPos].Extension,
				this->Collection.Items[iPos].Type,
				this->Collection.Items[iPos].Enabled ? "Yes" : "No",
				this->Collection.Items[iPos].Description);
		}
		fclose(hTarget);
		return this->Locks.UnLock(true);
	}

	return this->Locks.UnLock(false);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#endif
