///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//  Copyright  NetworkDLS 2002, All rights reserved
//
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF 
// ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO 
// THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A 
// PARTICULAR PURPOSE.
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#ifndef _CSSIFiles_CPP
#define _CSSIFiles_CPP
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include <Windows.H>
#include <ShlObj.H>
#include <Stdio.H>
#include <ShlOBJ.H>
#include <Stdlib.H>
#include <ShellAPI.H>
#include <Psapi.h>

extern HIMAGELIST hEnableDisableImageList; //Declared in MainDialog.cpp
extern HIMAGELIST hOnePixilImageList; //Declared in MainDialog.cpp

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include "../../@Libraries/URLEncoding/URLEncoding.H"

#include "CSSIFiles.H"
#include "Entry.H"
#include "CWebSites.H"

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

using namespace NSWFL::Windows;
using namespace NSWFL::ListView;
using namespace NSWFL::String;
using namespace NSWFL::File;
using namespace NSWFL::System;
using namespace NSWFL::System;
using namespace NSWFL::Registry;

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void CSSIFiles::AddConfigItem(HWND hListView, SSIFILE *pSSIFile)
{
	this->Locks.Lock();
	this->AddConfigItem(hListView, pSSIFile, ListView_GetItemCount(hListView));
	this->Locks.UnLock();
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void CSSIFiles::AddConfigItem(HWND hListView, SSIFILE *pSSIFile, int iInsertionIndex)
{
	this->Locks.Lock();
	LV_ITEM ListItem;
    memset(&ListItem, 0, sizeof(ListItem));

	int iImage = pSSIFile->Enabled && this->Collection.Enabled;

    ListItem.pszText = (char *) pSSIFile->Extension;
    ListItem.mask = LVIF_TEXT|LVIF_IMAGE;
	ListItem.iImage = iImage;
    ListItem.iSubItem = SSIFILES_LIST_POS_EXT;
	ListItem.iItem = iInsertionIndex;
    int iItem = ListView_InsertItem(hListView, &ListItem);

    ListItem.pszText = (char *) pSSIFile->Description;
    ListItem.mask = LVIF_TEXT;
    ListItem.iSubItem = SSIFILES_LIST_POS_DESCR;
	ListItem.iItem = iInsertionIndex;
    ListView_SetItem(hListView, &ListItem);

	ListView_SetCheckState(hListView, iItem, pSSIFile->Enabled);
	this->Locks.UnLock();
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void CSSIFiles::PopConfigList(HWND hWnd, int iListView, int iEnabled)
{
	this->Locks.Lock();
	CheckDlgButton(hWnd, iEnabled, this->Collection.Enabled);
	this->PopInfoList(GetDlgItem(hWnd, iListView), false);
	this->Locks.UnLock();
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void CSSIFiles::PopInfoList(HWND hListView)
{
	this->Locks.Lock();
	this->PopInfoList(hListView, true);
	this->Locks.UnLock();
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void CSSIFiles::PopInfoList(HWND hListView, bool bUseIcons)
{
	this->Locks.Lock();
	SendMessage(hListView, (UINT)WM_SETREDRAW , (WPARAM)FALSE, (LPARAM)0);

	LV_COLUMN MyLVColumn;
    memset(&MyLVColumn, 0, sizeof(LV_COLUMN));

	MyLVColumn.cx        = 100;
    MyLVColumn.pszText   = "Ext";
    MyLVColumn.iSubItem  = SSIFILES_LIST_POS_EXT;
    MyLVColumn.mask      = LVCF_WIDTH | LVCF_TEXT | LVCF_SUBITEM | LVCF_FMT;
    MyLVColumn.fmt       = LVCFMT_LEFT;
    ListView_InsertColumn(hListView, SSIFILES_LIST_POS_EXT, &MyLVColumn);

	MyLVColumn.cx        = 300;
    MyLVColumn.pszText   = "Description";
    MyLVColumn.iSubItem  = SSIFILES_LIST_POS_DESCR;
    MyLVColumn.mask      = LVCF_WIDTH | LVCF_TEXT | LVCF_SUBITEM | LVCF_FMT;
    MyLVColumn.fmt       = LVCFMT_LEFT;
    ListView_InsertColumn(hListView, SSIFILES_LIST_POS_DESCR, &MyLVColumn);

	DWORD dwFlags = LVS_EX_FULLROWSELECT|LVS_EX_GRIDLINES|LVS_EX_HEADERDRAGDROP;

	if(!bUseIcons)
	{
		dwFlags |= LVS_EX_CHECKBOXES;
	}

    SendMessage(hListView, (UINT)LVM_SETEXTENDEDLISTVIEWSTYLE, (WPARAM)0, (LPARAM)dwFlags);

	if(bUseIcons)
	{
		ListView_SetImageList(hListView, hEnableDisableImageList, LVSIL_SMALL);
	}

	ListView_SetTextColor(hListView, RGB(0,0,100));

	//Pop the list-view.
	for(int x = 0; x < this->Collection.Count; x++)
	{
		this->AddConfigItem(hListView, &this->Collection.Items[x]);
	}
	LVAutoSizeColumns(hListView, 500);
	SendMessage(hListView, (UINT)WM_SETREDRAW , (WPARAM)TRUE, (LPARAM)0);
	this->Locks.UnLock();
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

CSSIFiles::~CSSIFiles(void)
{
	this->Destroy();
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

CSSIFiles::CSSIFiles(void *lpWebSites)
{
	this->Initialized = false;
	this->pWebSites = lpWebSites;
	strcpy_s(sFileName, sizeof(sFileName), "");
	memset(&this->Collection, 0, sizeof(this->Collection));
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

CSSIFiles::CSSIFiles(void *lpWebSites, CXMLReader *xmlConfig, CSSIFiles *pDefaults)
{
	this->Initialized = true;
	this->pWebSites = lpWebSites;
	strcpy_s(sFileName, sizeof(sFileName), "");
	memset(&this->Collection, 0, sizeof(this->Collection));
	this->Load(xmlConfig, pDefaults);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CSSIFiles::Save(void)
{
	this->Locks.Lock();

	CXMLReader xmlConfig;
	if(this->ToXML(&xmlConfig))
	{
		bool bResult = xmlConfig.ToFile(this->sFileName);
		if(!bResult)
		{
			char sErrorMsg[2048];
			GetLastError(sErrorMsg, sizeof(sErrorMsg),
				"Failed to save websites configuration file (", ")");
			strcat_s(sErrorMsg, sizeof(sErrorMsg), "\r\n\r\n\"");
			strcat_s(sErrorMsg, sizeof(sErrorMsg), this->sFileName);
			strcat_s(sErrorMsg, sizeof(sErrorMsg), "\"");
			((CWebSites *)this->pWebSites)->Trace(sErrorMsg);
		}
		return this->Locks.UnLock(bResult);
	}
	return this->Locks.UnLock(false);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CSSIFiles::ToXML(CXMLReader *lpXML)
{
	this->Locks.Lock();

	CXMLWriter xmlConfig("SSIFiles");

	xmlConfig.AddBool("Enable", this->Collection.Enabled);

	for(int iItem = 0; iItem < this->Collection.Count; iItem++)
	{
		CXMLWriter Item("File");
		Item.Add("Extension", this->Collection.Items[iItem].Extension);
		Item.Add("Description", this->Collection.Items[iItem].Description);
		Item.AddBool("Enable", this->Collection.Items[iItem].Enabled);
		xmlConfig.Add(&Item);
	}

	xmlConfig.ToReader(lpXML);

	xmlConfig.Destroy();

	return this->Locks.UnLock(true);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
	Loads a modified configuration from the configuration dialog (also saves to original file).
*/
bool CSSIFiles::LoadNewConfiguration(HWND hWnd, HWND hGrid)
{
	this->Locks.LockExclusive();

	if(this->Initialized)
	{
		this->Destroy();
	}

	int iGridItems = LVItemCount(hGrid);

	char sExt[SSIFILES_MAX_EXT_LENGTH];
	char sDescr[SSIFILES_MAX_EXT_LENGTH];

	CXMLWriter xmlConfig("SSIFiles");

	xmlConfig.AddBool("Enable", IsDlgButtonChecked(hWnd, IDC_ENABLED) == BST_CHECKED);

	for(int iItem = 0; iItem < iGridItems; iItem++)
	{
		CXMLWriter Item("File");

		LVGetText(hGrid, iItem, SSIFILES_LIST_POS_EXT, sExt, sizeof(sExt));
		Item.Add("Extension", sExt);

		LVGetText(hGrid, iItem, SSIFILES_LIST_POS_DESCR, sDescr, sizeof(sDescr));
		Item.Add("Description", sDescr);

		Item.AddBool("Enable", LVChecked(hGrid, iItem));
		xmlConfig.Add(&Item);
	}

	if(strlen(this->sFileName) > 0)
	{
		if(xmlConfig.ToFile(this->sFileName))
		{
			return this->Locks.UnLockExclusive(this->Reload());
		}
	}
	else{
		CXMLReader XMLRConfig;
		return this->Locks.UnLockExclusive(Load(xmlConfig.ToReader(&XMLRConfig), NULL));
	}

	return this->Locks.UnLockExclusive(false);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
	Reloads the configuration from the file it was originally loaded from.
*/

bool CSSIFiles::Reload(void)
{
	this->Locks.LockExclusive();

	if(this->Initialized)
	{
		this->Destroy();
	}

	this->Load(this->sFileName);

	return this->Locks.UnLockExclusive(this->Initialized);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CSSIFiles::Load(const char *sXMLFileName)
{
	this->Locks.LockExclusive();

	this->Initialized = false;

	strcpy_s(this->sFileName, sizeof(this->sFileName), sXMLFileName);

	CXMLReader xmlConfig;

	if(xmlConfig.FromFile(sXMLFileName))
	{
		CXMLReader xmlEntity;
		if(xmlConfig.ToReader("SSIFiles", &xmlEntity))
		{
			this->Initialized = this->Load(&xmlEntity, NULL);
		}
		xmlEntity.Destroy();
		xmlConfig.Destroy();
	}

	return this->Locks.UnLockExclusive(this->Initialized);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CSSIFiles::Load(CXMLReader *xmlConfig, CSSIFiles *pDefaults)
{
	this->Locks.LockExclusive();

	int iLength = 0;
	memset(&this->Collection, 0, sizeof(this->Collection));

	this->Collection.Enabled = xmlConfig->ToBoolean("Enable", false);

	xmlConfig->ProgressiveScan(true);
	CXMLReader XPSSIFile;

	while(xmlConfig->ToReader("File", &XPSSIFile))
	{
		char sExt[SSIFILES_MAX_EXT_LENGTH];
		char sDescr[SSIFILES_MAX_EXT_LENGTH];

		this->Collection.Items = (SSIFILE *)
			pMem->ReAllocate(this->Collection.Items, sizeof(SSIFILE), this->Collection.Count + 1);

		SSIFILE *p = &this->Collection.Items[this->Collection.Count++];

		XPSSIFile.ToString("Extension", sExt, sizeof(sExt), &iLength);
		p->Extension = (char *) pMem->Allocate(sizeof(char), iLength + 1);
		strcpy_s(p->Extension, iLength + 1, LCase(sExt, iLength));

		XPSSIFile.ToString("Description", sDescr, sizeof(sDescr), &iLength);
		p->Description = (char *) pMem->Allocate(sizeof(char), iLength + 1);
		strcpy_s(p->Description, iLength + 1, sDescr);

		p->Enabled = XPSSIFile.ToBoolean("Enable", false);

		XPSSIFile.Destroy();
	}

	return this->Locks.UnLockExclusive(this->Initialized = true);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CSSIFiles::Enabled(void)
{
	this->Locks.Lock();
	return this->Locks.UnLock(this->Collection.Enabled && (this->Collection.Count > 0));
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CSSIFiles::Destroy(void)
{
	this->Locks.LockExclusive();

	if(this->Initialized)
	{
		this->Initialized = false;
		if(this->Collection.Count > 0)
		{
			for(int iPos = 0; iPos < this->Collection.Count; iPos++)
			{
				pMem->Free(this->Collection.Items[iPos].Extension);
				pMem->Free(this->Collection.Items[iPos].Description);
			}

			pMem->Free(this->Collection.Items);

			this->Collection.Count = 0;
		}
	}
	return this->Locks.UnLockExclusive(true);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
	Returns true if the file should be process as an SSI file, otherwise returns false.
*/
bool CSSIFiles::IsSSIFile(const char *sFileName)
{
	this->Locks.Lock();
	if(this->Collection.Enabled && (this->Collection.Count > 0))
	{
		char sExt[MAX_PATH];
		GetFileExtension(sFileName, sExt, sizeof(sExt));

		for(int iPos = 0; iPos < this->Collection.Count; iPos++)
		{
			if(this->Collection.Items[iPos].Enabled)
			{
				if(_strcmpi(sExt, this->Collection.Items[iPos].Extension) == 0)
				{
					return this->Locks.UnLock(true);
				}
			}
		}
	}
	
	return this->Locks.UnLock(false);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CSSIFiles::ProcessServerSideInclude(VOID *pClient, const char *sFileName)
{
	this->Locks.Lock();
	CStringBuilder Buffer;
	bool bResult = ProcessServerSideInclude(pClient, sFileName, true, &Buffer);
	Buffer.Destroy();
	return this->Locks.UnLock(bResult);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CSSIFiles::ProcessServerSideInclude(VOID *pClient, const char *sFileName, bool bSend, CStringBuilder *pBuffer)
{
	this->Locks.Lock();
	CLIENT *pC = (CLIENT *)pClient;
	CWebSite *pWebSite = (CWebSite *)pC->pWebSite;
	CHTTP *pHTTP = (CHTTP *)pWebSite->pHttp;
	CWebSites *pWebSites = (CWebSites *)pWebSite->pWebSites;

	pC->IsDynamicContent = true;

	FILE *fSource = NULL;           // The handle of the RAW SSI file.
	char *sFileBuf = NULL;          // The buffer that contains the RAW SSI file.
	unsigned long ulFileBufSz = 0;         // The size of the buffer that contains the RAW SSI file.

	//Get the size of the RAW SSI file.
	if(!FileSize(sFileName, &ulFileBufSz))
	{
		pWebSite->pErrorPages->SendError(pC, "500");
		return this->Locks.UnLock(false);
	}

	if(ulFileBufSz <= 0)
	{
		return this->Locks.UnLock(false);
	}

	//Open the RAW SSI file.
	if(fopen_s(&fSource, sFileName, "rb") != 0)
	{
		pWebSite->pErrorPages->SendError(pC, "500");
		return this->Locks.UnLock(false);
	}

	//Allocate RAM for our RAW SSI file.
	sFileBuf = (char *) pMem->Allocate(sizeof(char), ulFileBufSz + 1);

	//Read the entire RAW SSI file into the buffer.
	ulFileBufSz = (int)fread(sFileBuf, sizeof(char), ulFileBufSz, fSource);

	fclose(fSource);

	bool bResult = ProcessServerSideInclude(pC, sFileBuf, ulFileBufSz, bSend, pBuffer);

	pMem->Free(sFileBuf);

	return this->Locks.UnLock(bResult);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CSSIFiles::ProcessServerSideInclude(VOID *pClient, char *sFileBuf, int iFileBufSz,
										 bool bSend, CStringBuilder *lpBuf)
{
	this->Locks.Lock();
	CLIENT *pC = (CLIENT *)pClient;
	CWebSite *pWebSite = (CWebSite *)pC->pWebSite;
	CHTTP *pHTTP = (CHTTP *)pWebSite->pHttp;
	CWebSites *pWebSites = (CWebSites *)pWebSite->pWebSites;

	int iRPos = 0;
	int iWPos = 0;

	//------------------------------------------------------------------------------------
	// Begin SSI Processing
	//------------------------------------------------------------------------------------

	int iSFlagLen = (int)strlen(SSI_BEGIN_FLAG);
	int iEFlagLen = (int)strlen(SSI_END_FLAG);
	
	int iFlagStartPos = 0;
	int iFlagEndPos = 0;
	int iSearchPos = 0;
	int iSSISz = 0;

	char sSSI[1024];
	char sSSIOth[1024];
	char sFileName[MAX_PATH];

	int iRPosA = 0;
	int iWPosA = 0;

	while(true)
	{
		if((iFlagStartPos = InStr(SSI_BEGIN_FLAG, sFileBuf, iFileBufSz, iSearchPos)) < 0)
		{
			iFlagStartPos = iFileBufSz;
			break;
			//No SSI start flags were found.
		}

		lpBuf->Append(sFileBuf + iRPos, iFlagStartPos - iRPos);

		if((iFlagEndPos = InStr(SSI_END_FLAG, sFileBuf, iFileBufSz, iFlagStartPos)) < 0)
		{
			//No SSI end flags were found.
			lpBuf->Append("<!--SSI ERROR(Expected: \"-->\")-->");
			break;
		}

		if(Mid(sFileBuf, iFileBufSz, iFlagStartPos + iSFlagLen, (iFlagEndPos - iFlagStartPos) - iSFlagLen, sSSI, sizeof(sSSI)) <= 0)
		{
			//No valid SSI tag was parsed.
		}

		iSSISz = (iFlagEndPos - iFlagStartPos) - iSFlagLen;

		if(_strnicmp(sSSI, "Echo ", 5) == 0) {

			iRPosA = 5;
			iWPosA = 0;

			//Skip White-Spaces.
			while(IsWhiteSpace(sSSI[iRPosA]) && iRPosA < iSSISz)
				iRPosA++;

			//Parse the second SSI command Word.
			while(!IsWhiteSpace(sSSI[iRPosA]) && sSSI[iRPosA] != '=' && iRPosA < iSSISz)
				sSSIOth[iWPosA++] = sSSI[iRPosA++];
			sSSIOth[iWPosA++] = '\0';

			if(sSSI[iRPosA] != '=')
			{
				lpBuf->Append("<!--SSI ERROR(Expected: \"=\")-->");
			}

			iRPosA++; //Skip the equal sign.

			//Skip White-Spaces.
			while(IsWhiteSpace(sSSI[iRPosA]) && iRPosA < iSSISz)
				iRPosA++;

			if(_strcmpi(sSSIOth, "VAR") == 0)
			{
				if(sSSI[iRPosA] != '\"')
				{
					lpBuf->Append("<!--SSI ERROR(Expected: \"\"\")-->");
				}
				else{
					iWPosA = 0;
					iRPosA++; //Skip the double quote.

					//Parse the second SSI command Word.
					while(!IsWhiteSpace(sSSI[iRPosA]) && sSSI[iRPosA] != '"' && iRPosA < iSSISz)
						sSSIOth[iWPosA++] = sSSI[iRPosA++];
					sSSIOth[iWPosA++] = '\0';

					EchoSSIVar(pC, lpBuf, sSSIOth);
				}
			}
			else if(_strcmpi(sSSIOth, "ENV") == 0)
			{
				if(sSSI[iRPosA] != '\"')
				{
					lpBuf->Append("<!--SSI ERROR(Expected: \"\"\")-->");
				}
				else{
					iWPosA = 0;
					iRPosA++; //Skip the double quote.

					//Parse the second SSI command Word.
					while(!IsWhiteSpace(sSSI[iRPosA]) && sSSI[iRPosA] != '"' && iRPosA < iSSISz)
						sSSIOth[iWPosA++] = sSSI[iRPosA++];
					sSSIOth[iWPosA++] = '\0';

					size_t iReqSz = 0;

					getenv_s(&iReqSz, NULL, 0, sSSIOth);

					if(iReqSz > 0)
					{
						char *sBuf = NULL;

						if((sBuf = (char *) pMem->Allocate(sizeof(char), iReqSz + 1)) != NULL)
						{
							if(getenv_s(&iReqSz, sBuf, iReqSz, sSSIOth) == 0)
							{
								lpBuf->Append(sBuf);
							}
							else{
								lpBuf->Append("");
							}

							pMem->Free(sBuf);
						}
						else{
							lpBuf->Append("");
						}
					}
					else{
						lpBuf->Append("");
					}
				}
			}
			else{
				lpBuf->Append("<!--SSI ERROR(Expected: \"VAR\")-->");
			}
			
		}
		else if(_strnicmp(sSSI, "FlastMod ", 9) == 0) {

			char sTime[255];
			
			GetVirtualOrFileName(pC, lpBuf, 9, sSSI, iSSISz, sFileName);
			if(strlen(sFileName) > 0)
			{
				GetLastFileModificationDate(sFileName, sTime, sizeof(sTime));
				lpBuf->Append(sTime);
			}
		}
		else if(_strnicmp(sSSI, "FSize ", 6) == 0) {

			GetVirtualOrFileName(pC, lpBuf, 6, sSSI, iSSISz, sFileName);
			if(strlen(sFileName) > 0)
			{
				__int64 i64FileSize = 0;
				if(!FileSize(sFileName, &i64FileSize))
				{
					i64FileSize = 0;
				}

				lpBuf->Append(i64FileSize);
			}
		}
		else if(_strnicmp(sSSI, "Include ", 8) == 0) {

			GetVirtualOrFileName(pC, lpBuf, 8, sSSI, iSSISz, sFileName);
			if(strlen(sFileName) > 0)
			{
				if(this->IsSSIFile(sFileName))
				{
					CStringBuilder RecursiveBuffer;
					if(this->ProcessServerSideInclude(pC, sFileName, false, &RecursiveBuffer))
					{
						lpBuf->Append(&RecursiveBuffer);
					}
					else{
						lpBuf->Append("<!--SSI ERROR(Error parsing recursive SSI)-->");
					}
					RecursiveBuffer.Destroy();
				}
				else{
					lpBuf->AppendFromFile(sFileName);
				}
			}
		}
		else if(_strnicmp(sSSI, "Exec ", 5) == 0) {
			GetCGIOrCMDName(pC, lpBuf, 5, sSSI, iSSISz, sFileName);
			if(strlen(sFileName) > 0)
			{
				int iResult = 0;
				CStringBuilder Str(1000);
				if((iResult = ((CWebSite*)pC->pWebSite)->pScriptingEngines->ProcessScript(pC, &Str, sFileName)) == EXEC_RESULT_OK)
				{
					lpBuf->Append(Str.Buffer, Str.Length);
				}
				else{
					lpBuf->Append("<!--SSI ERROR(");
					lpBuf->Append(((CWebSite*)pC->pWebSite)->pScriptingEngines->ErrorString(iResult));
					lpBuf->Append(")-->");
				}
				Str.Destroy();
			}
		}
		else {
			lpBuf->Append("<!--SSI ERROR(Unknown SSI tag)-->");
		}
		
		iSearchPos = (iFlagEndPos + iEFlagLen);
		iRPos = iSearchPos;
	}

	lpBuf->Append(sFileBuf + iRPos, iFlagStartPos - iRPos); //Append the file tail.

	//------------------------------------------------------------------------------------
	// End SSI Processing
	//------------------------------------------------------------------------------------

	if(bSend)
	{
		if(pHTTP->DoesValueContain(pC->Header.AcceptEncoding, "gzip"))
		{
			if(((CWebSite*)pC->pWebSite)->pCompression->CompressDynamicContent())
			{
				if(lpBuf->Length <= ((CWebSite*)pC->pWebSite)->pCompression->MaxCompressionSize()
					&& lpBuf->Length >= ((CWebSite*)pC->pWebSite)->pCompression->MinCompressionSize())
				{
					bool bResult = false;
					char sUncompressed[MAX_PATH];
					char sCompressed[MAX_PATH];

					if(!((CWebSite*)pC->pWebSite)->pCompression->GetUniqueFileName(
						sUncompressed, sizeof(sUncompressed), ((CLIENT*)pC)->Header.FullRequest))
					{
						pWebSite->pErrorPages->SendError(pC, "500");
						return this->Locks.UnLock(false);
					}
					if(!((CWebSite*)pC->pWebSite)->pCompression->GetUniqueFileName(
						sCompressed, sizeof(sCompressed)))
					{
						pWebSite->pErrorPages->SendError(pC, "500");
						return this->Locks.UnLock(false);
					}

					if(BufferDataToFile(sUncompressed, lpBuf->Buffer, lpBuf->Length))
					{
						if(((CWebSite*)pC->pWebSite)->pCompression->Deflate(sUncompressed, sCompressed))
						{
							unsigned long ulCompressedSize = 0;

							//Get the size of the compressed SSI file.
							if(!FileSize(sCompressed, &ulCompressedSize))
							{
								pWebSite->pErrorPages->SendError(pC, "500");
							}
							else{
								if(pWebSites->pHttp->SendOKHeader(pC, "text/html", "gzip", ulCompressedSize))
								{
									if(_strcmpi(pC->Header.Method, "HEAD") != 0)
									{
										if(pWebSites->pHttp->SendRawFile(pC, sCompressed))
										{
											bResult = true;
										}
									}
									else{
										bResult = true;
									}
								}
							}
						}
						else{
							pWebSite->pErrorPages->SendError(pC, "500");
						}
					}
					else{
						pWebSite->pErrorPages->SendError(pC, "500");
					}

					DeleteFile(sCompressed);
					DeleteFile(sUncompressed);
					return this->Locks.UnLock(bResult);
				}
				else{
					//No compression, continue below...
				}
			}
			else{
				//No compression, continue below...
			}
		}
		else{
			//No compression, continue below...
		}

		//Can't compress, just send the data.
		if(pWebSites->pHttp->SendOKHeader(pC, lpBuf->Length))
		{
			if(_strcmpi(pC->Header.Method, "HEAD") != 0)
			{
				if(pWebSites->pHttp->BufferDataToClient(pC, lpBuf->Buffer, lpBuf->Length))
				{
					return this->Locks.UnLock(true);
				}
				else{
					//We've already sent a header, so we cant go back on it now.
					//	All we can do is abandon the client...
					return this->Locks.UnLock(false);
				}
				return this->Locks.UnLock(true);
			}
		}
		else{
			pWebSite->pErrorPages->SendError(pC, "500");
			return this->Locks.UnLock(false);
		}
	}

	return this->Locks.UnLock(true);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CSSIFiles::GetSSITagEx(VOID *pClient, CStringBuilder *lpBuf, int iInitRPos, const char *sSSI,
				 const int iSSISz, char *sOutBuffer, int iOutBufMaxSz,
				 const char *sOpt1, const char *sOpt2)
{
	this->Locks.Lock();
	CLIENT *pC = (CLIENT *)pClient;

	char sSSIOth[1024];

	int iRPosA = iInitRPos;
	int iWPosA = 0;

	strcpy_s(sOutBuffer, iOutBufMaxSz, "");
	
	//Skip White-Spaces.
	while(IsWhiteSpace(sSSI[iRPosA]) && iRPosA < iSSISz)
		iRPosA++;

	//Parse the second SSI command Word.
	while(!IsWhiteSpace(sSSI[iRPosA]) && sSSI[iRPosA] != '=' && iRPosA < iSSISz)
		sSSIOth[iWPosA++] = sSSI[iRPosA++];
	sSSIOth[iWPosA++] = '\0';

	if(sSSI[iRPosA] == '=')
		iRPosA++;

	//Skip White-Spaces.
	while(IsWhiteSpace(sSSI[iRPosA]) && iRPosA < iSSISz)
		iRPosA++;

	if(sSSI[iRPosA] != '\"')
	{
		lpBuf->Append("<!--SSI ERROR(Expected: \"\"\")-->");
		return this->Locks.UnLock(false);
	}
	else{
		iRPosA++;

		iWPosA = 0;

		if(_strcmpi(sSSIOth, sOpt1) == 0)
		{
			//Parse the file name.
			while(sSSI[iRPosA] != '"' && iRPosA < iSSISz)
			{
				sOutBuffer[iWPosA++] = sSSI[iRPosA++];
			}
			sOutBuffer[iWPosA++] = '\0';
		}
		else if(_strcmpi(sSSIOth, sOpt2) == 0)
		{
			//Parse the file name.
			while(sSSI[iRPosA] != '"' && iRPosA < iSSISz)
			{
				sSSIOth[iWPosA++] = sSSI[iRPosA++];
			}
			sSSIOth[iWPosA++] = '\0';

			_makepath_s(sOutBuffer, iOutBufMaxSz, NULL,
				((CWebSite *)pC->pWebSite)->Root, sSSIOth, NULL);
			CorrectReversePath(sOutBuffer, iOutBufMaxSz, true);
		}
		else {
			lpBuf->Append("<!--SSI ERROR(Expected: \"");
			lpBuf->Append(sOpt1);
			lpBuf->Append("\" or \"");
			lpBuf->Append(sOpt2);
			lpBuf->Append("\")-->");
			return this->Locks.UnLock(false);
		}

		if(!FileAccess(sOutBuffer, FExist))
		{
			strcpy_s(sOutBuffer, iOutBufMaxSz, "");
			lpBuf->Append("<!--SSI ERROR(File not found)-->");
			return this->Locks.UnLock(false);
		}
	}

	return this->Locks.UnLock(true);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CSSIFiles::GetVirtualOrFileName(VOID *pClient, CStringBuilder *lpBuf, int iInitRPos,
						   const char *sSSI, const int iSSISz, char *sOutFileName)
{
	this->Locks.Lock();
	CLIENT *pC = (CLIENT *)pClient;
	return this->Locks.UnLock(GetSSITagEx(pC, lpBuf, iInitRPos, sSSI, iSSISz, sOutFileName, MAX_PATH, "File", "Virtual"));
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CSSIFiles::GetCGIOrCMDName(VOID *pClient, CStringBuilder *lpBuf, int iInitRPos,
					  const char *sSSI, const int iSSISz, char *sOutFileName)
{
	this->Locks.Lock();
	CLIENT *pC = (CLIENT *)pClient;
	return this->Locks.UnLock(GetSSITagEx(pC, lpBuf, iInitRPos, sSSI, iSSISz, sOutFileName, MAX_PATH, "CMD", "CGI"));
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CSSIFiles::EchoSSIVar(VOID *pClient, CStringBuilder *lpBuf, char *sVarName)
{
	this->Locks.Lock();
	CLIENT *pC = (CLIENT *)pClient;
	CWebSite *pWebSite = (CWebSite *)pC->pWebSite;
	CHTTP *pHTTP = (CHTTP *)pWebSite->pHttp;
	CWebSites *pWebSites = (CWebSites *)pWebSite->pWebSites;

	const char *sTrue = "True";
	const char *sFalse = "False";

	if(_strcmpi(sVarName, "DOCUMENT_NAME") == 0){
		lpBuf->Append(pC->Header.FullRequest);
	}
	else if(_strcmpi(sVarName, "DOCUMENT_URI") == 0){
		lpBuf->Append(pC->Header.Request);
	}
	else if(_strcmpi(sVarName, "QUERY_STRING_UNESCAPED") == 0){
		char sEncoded[MAX_QUERY_STRING];
		int iLength = URLEncode(pC->Header.Query, sEncoded, sizeof(sEncoded));
		if(iLength >= 0)
		{
			lpBuf->Append(sEncoded, iLength);
		}
		else{
			lpBuf->Append("<!--SSI ERROR(URLEncode Failed)-->");
		}
	}
	else if(_strcmpi(sVarName, "DATE_LOCAL") == 0){
		char sTime[255];
		GetLOCALTime(sTime, sizeof(sTime));
		lpBuf->Append(sTime);
	}
	else if(_strcmpi(sVarName, "DATE_GMT") == 0){
		char sGMTTime[255];
		GetGMTTime(sGMTTime, sizeof(sGMTTime));
		lpBuf->Append(sGMTTime);
	}
	else if(_strcmpi(sVarName, "LAST_MODIFIED") == 0){
		char sTime[255];
		GetLastFileModificationDate(pC->Header.FullRequest, sTime, sizeof(sTime));
		lpBuf->Append(sTime);
	}
	else if(_strcmpi(sVarName, "SERVER_SOFTWARE") == 0){
		char sSoftware[255];
		sprintf_s(sSoftware, sizeof(sSoftware), "NetworkDLS - %s", gsTitleCaption);
		lpBuf->Append(sSoftware);
	}
	else if(_strcmpi(sVarName, "SERVER_VERSION") == 0){
		lpBuf->Append(gsFileVersion);
	}
	else if(_strcmpi(sVarName, "SERVER_NAME") == 0){
		char sHostName[255];
		if(Get_MachineName(sHostName, sizeof(sHostName)))
		{
			lpBuf->Append(sHostName);
		}
	}
	else if(_strcmpi(sVarName, "GATEWAY_INTERFACE") == 0){
		lpBuf->Append(CGIINTERFACE);
	}
	else if(_strcmpi(sVarName, "SERVER_PROTOCOL") == 0){
		lpBuf->Append(pC->Header.Version);
	}
	else if(_strcmpi(sVarName, "SERVER_PORT") == 0){
		lpBuf->Append(pC->pServer->ListenPort());
	}
	else if(_strcmpi(sVarName, "REQUEST_METHOD") == 0){
		lpBuf->Append(pC->Header.Method);
	}
	else if(_strcmpi(sVarName, "PATH_INFO") == 0){
		lpBuf->Append(pC->Header.Request);
	}
	else if(_strcmpi(sVarName, "PATH_TRANSLATED") == 0){
		lpBuf->Append(pC->Header.FullRequest);
	}
	else if(_strcmpi(sVarName, "SCRIPT_NAME") == 0){
		lpBuf->Append(pC->Header.Request);
	}
	else if(_strcmpi(sVarName, "QUERY_STRING") == 0){
		lpBuf->Append(pC->Header.Query);
	}
	else if(_strcmpi(sVarName, "REMOTE_HOST") == 0){
		lpBuf->Append(pC->pClient->PeerAddress());
	}
	else if(_strcmpi(sVarName, "REMOTE_ADDR") == 0){
		lpBuf->Append(pC->pClient->PeerAddress());
	}
	else if(_strcmpi(sVarName, "AUTH_TYPE") == 0){
		lpBuf->Append("<!--SSI ERROR(Not yet implemented)-->");
	}
	else if(_strcmpi(sVarName, "REMOTE_USER") == 0){
		lpBuf->Append("<!--SSI ERROR(Not yet implemented)-->");
	}
	else if(_strcmpi(sVarName, "REMOTE_IDENT") == 0){
		lpBuf->Append("<!--SSI ERROR(Not yet implemented)-->");
	}
	else if(_strcmpi(sVarName, "CONTENT_TYPE") == 0){
		lpBuf->Append("*.*");
	}
	else if(_strcmpi(sVarName, "CONTENT_LENGTH") == 0){
		lpBuf->Append("<!--SSI ERROR(Not yet implemented)-->");
	}
	else if(_strcmpi(sVarName, "HTTP_USER_AGENT") == 0){
		lpBuf->Append(pC->Header.UserAgent);
	}
	else if(_strcmpi(sVarName, "ADMIN_EMAIL") == 0){
		lpBuf->Append(((CWebSite *)pC->pWebSite)->pSettings->ServerAdminEmail());
	}
	else if(_strcmpi(sVarName, "CONNECTION_ID") == 0){
		lpBuf->Append(pC->pClient->PeerID());
	}
	else if(_strcmpi(sVarName, "OS_VERSION") == 0){
	    char sTemp[255];
		Get_OsVersion(sTemp, sizeof(sTemp));
		lpBuf->Append(sTemp);
	}
	else if(_strcmpi(sVarName, "PROCESSOR_VENDOR") == 0){
		char sBuf[255];
		DWORD dwBufSz = sizeof(sBuf);
		memset(sBuf, 0, dwBufSz);
		if(Get_StringRegistryValue(HKEY_LOCAL_MACHINE, "HARDWARE\\DESCRIPTION\\System\\CentralProcessor\\0", "VendorIdentifier", sBuf, dwBufSz))
		{
			lpBuf->Append(sBuf);
		}
		else lpBuf->Append("n/a");
	}
	else if(_strcmpi(sVarName, "PROCESSOR_COUNT") == 0){
		SYSTEM_INFO SI;
		memset(&SI, 0, sizeof(SI));
	    GetSystemInfo(&SI);
		lpBuf->Append(SI.dwNumberOfProcessors);
	}
	else if(_strcmpi(sVarName, "PROCESSOR_NAME") == 0){
		char sBuf[255];
		DWORD dwBufSz = sizeof(sBuf);
		memset(sBuf, 0, dwBufSz);
		if(Get_StringRegistryValue(HKEY_LOCAL_MACHINE, "HARDWARE\\DESCRIPTION\\System\\CentralProcessor\\0", "ProcessorNameString", sBuf, dwBufSz))
		{
			dwBufSz--;

			ReverseString(sBuf, dwBufSz);

			while(IsWhiteSpace(sBuf[dwBufSz]) && dwBufSz > 0)
			{
				dwBufSz--;
			}

			if(!IsWhiteSpace(sBuf[dwBufSz]))
			{
				dwBufSz++;
			}

			sBuf[dwBufSz] = '\0';

			ReverseString(sBuf, dwBufSz);

			lpBuf->Append(sBuf);
		}
		else lpBuf->Append("n/a");
	}
	else if(_strcmpi(sVarName, "PROCESSOR_IDENTIFIER") == 0){
		char sBuf[255];
		DWORD dwBufSz = sizeof(sBuf);
		memset(sBuf, 0, dwBufSz);
		if(Get_StringRegistryValue(HKEY_LOCAL_MACHINE, "HARDWARE\\DESCRIPTION\\System\\CentralProcessor\\0", "Identifier", sBuf, dwBufSz))
		{
			lpBuf->Append(sBuf);
		}
		else lpBuf->Append("n/a");
	}
	else if(_strcmpi(sVarName, "PROCESSOR_MHZ") == 0){
		DWORD dwBuf = 0;
		if(Get_DWORDRegistryValue(HKEY_LOCAL_MACHINE, "HARDWARE\\DESCRIPTION\\System\\CentralProcessor\\0", "~MHz", dwBuf))
		{
			lpBuf->Append(dwBuf);
		}
		else lpBuf->Append("n/a");
	}
	else if(_strcmpi(sVarName, "SERVER_ADMIN_EMAIL") == 0){
		lpBuf->Append(((CWebSite*)pC->pWebSite)->pSettings->ServerAdminEmail());
	}
	else if(_strcmpi(sVarName, "SERVER_ALLOW_DIRECTORY_INDEXING") == 0){
		if(((CWebSite*)pC->pWebSite)->pSettings->AllowDirectoryIndexing())
		{
			lpBuf->Append(sTrue);
		}
		else{
			lpBuf->Append(sFalse);
		}
	}
	else if(_strcmpi(sVarName, "SERVER_BYTES_RECV") == 0){
		lpBuf->Append(((CWebSite *)pC->pWebSite)->Stats.BytesRecv);
	}
	else if(_strcmpi(sVarName, "SERVER_BYTES_SENT") == 0){
		lpBuf->Append(((CWebSite *)pC->pWebSite)->Stats.BytesSent);
	}
	else if(_strcmpi(sVarName, "SERVER_ACCEPTED_CONNECTIONS") == 0){
		lpBuf->Append(((CWebSite *)pC->pWebSite)->Stats.AcceptedConnections);
	}
	else if(_strcmpi(sVarName, "SERVER_HITS") == 0){
		lpBuf->Append(((CWebSite *)pC->pWebSite)->Stats.Hits);
	}
	else if(_strcmpi(sVarName, "SERVER_CACHE_NATIVE_SIZE") == 0){
		lpBuf->Append(((CWebSite*)pC->pWebSite)->pCompression->GetNativeCacheSize());
	}
	else if(_strcmpi(sVarName, "SERVER_CACHE_COMPRESSED_SIZE") == 0){
		lpBuf->Append(((CWebSite*)pC->pWebSite)->pCompression->GetCompressedCacheSize());
	}
	else if(_strcmpi(sVarName, "SERVER_CACHE_COMPRESSION_RATIO") == 0){
		lpBuf->Append(((CWebSite*)pC->pWebSite)->pCompression->GetCacheCompressionRatio(), 2);
	}
	else if(_strcmpi(sVarName, "SERVER_CACHE_CURRENT_FILES") == 0){
		lpBuf->Append(((CWebSite*)pC->pWebSite)->pCompression->CachedFiles());
	}
	else if(_strcmpi(sVarName, "SERVER_CACHE_MAX_FILES") == 0){
		lpBuf->Append(((CWebSite*)pC->pWebSite)->pCompression->MaxCachedFiles());
	}
	else if(_strcmpi(sVarName, "SERVER_CACHE_PATH") == 0){
		lpBuf->Append(((CWebSite*)pC->pWebSite)->pCompression->CachePath());
	}
	else if(_strcmpi(sVarName, "SERVER_CGI_TIMEOUT") == 0){
		lpBuf->Append(((CWebSite*)pC->pWebSite)->pSettings->ScriptTimeout());
	}
	else if(_strcmpi(sVarName, "SERVER_COMPRESS_DYNAMIC_CONTENT") == 0){
		if(((CWebSite*)pC->pWebSite)->pCompression->CompressDynamicContent())
		{
			lpBuf->Append(sTrue);
		}
		else{
			lpBuf->Append(sFalse);
		}
	}
	else if(_strcmpi(sVarName, "SERVER_CONNECTIONS") == 0){
		lpBuf->Append(((CWebSite*)pC->pWebSite)->pServer->CurrentConnections());
	}
	else if(_strcmpi(sVarName, "SERVER_GLOBAL_CONNECTIONS") == 0){
		lpBuf->Append(pWebSites->GetCurrentConenctions());
	}
	else if(_strcmpi(sVarName, "SERVER_WARNINGS") == 0){
		lpBuf->Append(pWebSite->pErrorPages->WarningCount());
	}
	else if(_strcmpi(sVarName, "SERVER_ERRORS") == 0){
		lpBuf->Append(pWebSite->pErrorPages->ErrorCount());
	}
	else if(_strcmpi(sVarName, "SERVER_KEEP_ALIVE_TIMEOUT") == 0){
		lpBuf->Append(((CWebSite*)pC->pWebSite)->pSettings->KeepAliveTimeout());
	}
	else if(_strcmpi(sVarName, "SERVER_LOG_PATH") == 0){
		lpBuf->Append(((CWebSite*)pC->pWebSite)->pSettings->LogPath());
	}
	else if(_strcmpi(sVarName, "SERVER_COMPRESS_MAX_SIZE") == 0){
		lpBuf->Append(((CWebSite*)pC->pWebSite)->pCompression->MaxCompressionSize());
	}
	else if(_strcmpi(sVarName, "SERVER_COMPRESS_MIN_SIZE") == 0){
		lpBuf->Append(((CWebSite*)pC->pWebSite)->pCompression->MinCompressionSize());
	}
	else if(_strcmpi(sVarName, "SERVER_PROCESS_EXTERNAL_CGI") == 0){
		if(((CWebSite*)pC->pWebSite)->pScriptingEngines->Enabled())
		{
			lpBuf->Append(sTrue);
		}
		else{
			lpBuf->Append(sFalse);
		}
	}
	else if(_strcmpi(sVarName, "SERVER_PROCESS_SSI_FILES") == 0){
		if(((CWebSite*)pC->pWebSite)->pSSIFiles->Enabled())
		{
			lpBuf->Append(sTrue);
		}
		else{
			lpBuf->Append(sFalse);
		}
	}
	else if(_strcmpi(sVarName, "SERVER_SCAN_EXTS") == 0){
		if(((CWebSite*)pC->pWebSite)->pExtensionFilters->Enabled())
		{
			lpBuf->Append(sTrue);
		}
		else{
			lpBuf->Append(sFalse);
		}	
	}
	else if(_strcmpi(sVarName, "SERVER_SCAN_IPS") == 0){
		if(((CWebSite*)pC->pWebSite)->pIPFilters->Enabled())
		{
			lpBuf->Append(sTrue);
		}
		else{
			lpBuf->Append(sFalse);
		}	
	}
	else if(_strcmpi(sVarName, "SERVER_SCAN_URLS") == 0){
		if(((CWebSite*)pC->pWebSite)->pURLFilters->Enabled())
		{
			lpBuf->Append(sTrue);
		}
		else{
			lpBuf->Append(sFalse);
		}	
	}
	else if(_strcmpi(sVarName, "SERVER_TOTAL_CONNECTIONS") == 0){
		lpBuf->Append(pWebSites->GetTotalConnections());
	}
	else if(_strcmpi(sVarName, "SERVER_USE_CGI_DIRECTORIES") == 0){
		if(((CWebSite*)pC->pWebSite)->pCGIFolders->Enabled())
		{
			lpBuf->Append(sTrue);
		}
		else{
			lpBuf->Append(sFalse);
		}	
	}
	else if(_strcmpi(sVarName, "SERVER_USE_HTTP_COMPRESSION") == 0){
		if(((CWebSite*)pC->pWebSite)->pCompression->Enabled())
		{
			lpBuf->Append(sTrue);
		}
		else{
			lpBuf->Append(sFalse);
		}	
	}
	else if(_strcmpi(sVarName, "SERVER_USE_KEEP_ALIVES") == 0){
		if(((CWebSite*)pC->pWebSite)->pSettings->UseKeepAlives())
		{
			lpBuf->Append(sTrue);
		}
		else{
			lpBuf->Append(sFalse);
		}	
	}
	else if(_strcmpi(sVarName, "SERVER_MEMORY_USED") == 0){
		PROCESS_MEMORY_COUNTERS MyPMC;
		memset(&MyPMC, 0, sizeof(MyPMC));

		if(GetProcessMemoryInfo(GetCurrentProcess(), &MyPMC, sizeof(MyPMC)))
		{
			lpBuf->Append(MyPMC.WorkingSetSize);
		}
	}
	else if(_strcmpi(sVarName, "SERVER_MEMORY_PAGEFILEUSAGE") == 0){
		PROCESS_MEMORY_COUNTERS MyPMC;
		memset(&MyPMC, 0, sizeof(MyPMC));

		if(GetProcessMemoryInfo(GetCurrentProcess(), &MyPMC, sizeof(MyPMC)))
		{
			lpBuf->Append(MyPMC.PagefileUsage);
		}
	}
	else if(_strcmpi(sVarName, "SERVER_MEMORY_PEAKMEMUSAGE") == 0){
		PROCESS_MEMORY_COUNTERS MyPMC;
		memset(&MyPMC, 0, sizeof(MyPMC));

		if(GetProcessMemoryInfo(GetCurrentProcess(), &MyPMC, sizeof(MyPMC)))
		{
			lpBuf->Append(MyPMC.PeakWorkingSetSize);
		}
	}
	else if(_strcmpi(sVarName, "SERVER_MEMORY_PEAKPAGEFILEUSAGE") == 0){
		PROCESS_MEMORY_COUNTERS MyPMC;
		memset(&MyPMC, 0, sizeof(MyPMC));

		if(GetProcessMemoryInfo(GetCurrentProcess(), &MyPMC, sizeof(MyPMC)))
		{
			lpBuf->Append(MyPMC.PeakPagefileUsage);
		}
	}
	else if(_strcmpi(sVarName, "SYSTEM_MEMORY_FREE") == 0){
		MEMORYSTATUSEX MS;
		memset(&MS, 0, sizeof(MS));
		MS.dwLength = sizeof(MS);
		GlobalMemoryStatusEx(&MS);

		lpBuf->Append(MS.ullAvailPhys);
	}
	else if(_strcmpi(sVarName, "SYSTEM_MEMORY_TOTAL") == 0){
		MEMORYSTATUSEX MS;
		memset(&MS, 0, sizeof(MS));
		MS.dwLength = sizeof(MS);
		GlobalMemoryStatusEx(&MS);

		lpBuf->Append(MS.ullTotalPhys);
	}
	else if(_strcmpi(sVarName, "SYSTEM_MEMORY_USED") == 0){
		MEMORYSTATUSEX MS;
		memset(&MS, 0, sizeof(MS));
		MS.dwLength = sizeof(MS);
		GlobalMemoryStatusEx(&MS);

		lpBuf->Append(MS.ullTotalPhys - MS.ullAvailPhys);
	}
	else{
		lpBuf->Append("<!--SSI ERROR(Unknown echo variable)-->");
	}

	return this->Locks.UnLock(true);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#endif
