///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//  Copyright  NetworkDLS 2002, All rights reserved
//
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF 
// ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO 
// THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A 
// PARTICULAR PURPOSE.
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#ifndef _CSSIFiles_H
#define _CSSIFiles_H
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include "../../@Classes/CXML/CXMLReader.H"
#include "../../@Classes/CStringBuilder/CStringBuilder.H"
#include "../../@Classes/CLocks/CLocks.H"
#include "../../@Classes/CStack/CStack.H"

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#define SSIFILES_LIST_POS_EXT	0
#define SSIFILES_LIST_POS_DESCR	1

#define SSIFILES_MAX_EXT_LENGTH		MAX_PATH
#define SSIFILES_MAX_DESCR_LENGTH	1024

#define SSI_BEGIN_FLAG			"<!--#"
#define SSI_END_FLAG			"-->"

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

typedef struct _tag_SSI_File{
	char *Extension;
	char *Description;
	bool Enabled;
} SSIFILE, *LPSSIFILE;

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

typedef struct _tag_SSI_Files{
	int Count;
	SSIFILE *Items;
	bool Enabled;
} SSIFILES, *LPSSIFILES;

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

class CSSIFiles{

private:
	bool Initialized;
	void *pWebSites;
	SSIFILES Collection;
	char sFileName[MAX_PATH];

	CLocks Locks;

	bool EchoSSIVar(VOID *pClient, CStringBuilder *lpBuf, char *sVarName);
	bool GetSSITagEx(VOID *pClient, CStringBuilder *lpBuf, int iInitRPos, const char *sSSI,
				 const int iSSISz, char *sOutBuffer, int iOutBufMaxSz,
				 const char *sOpt1, const char *sOpt2);
	bool GetVirtualOrFileName(VOID *pClient, CStringBuilder *lpBuf, int iInitRPos,
						   const char *sSSI, const int iSSISz, char *sOutFileName);
	bool GetCGIOrCMDName(VOID *pClient, CStringBuilder *lpBuf, int iInitRPos,
					  const char *sSSI, const int iSSISz, char *sOutFileName);

public:
	~CSSIFiles();
	CSSIFiles(void *lpWebSites);
	CSSIFiles(void *lpWebSites, CXMLReader *xmlConfig, CSSIFiles *pDefaults);
	bool Save(void);

	bool ToXML(CXMLReader *lpXML);

	void PopInfoList(HWND hListView, bool bUseIcons);
	void PopConfigList(HWND hWnd, int iListView, int iEnabled);
	void PopInfoList(HWND hListView);

	void AddConfigItem(HWND hListView, SSIFILE *pSSIFile);
	void AddConfigItem(HWND hListView, SSIFILE *pSSIFile, int iInsertionIndex);

	bool ProcessServerSideInclude(VOID *pClient, const char *sFileName);

	bool ProcessServerSideInclude(VOID *pClient, char *sFileBuf,
					int iFileBufSz, bool bSend, CStringBuilder *lpBuf);

	bool ProcessServerSideInclude(VOID *pClient,
		const char *sFileName, bool bSend, CStringBuilder *pBuffer);

	bool Load(const char *sXMLFileName);
	bool LoadNewConfiguration(HWND hWnd, HWND hGrid);
	bool Reload(void);
	bool Load(CXMLReader *xmlConfig, CSSIFiles *pDefaults);
	bool Destroy();
	bool IsSSIFile(const char *sFileName);
	bool Enabled(void);

};

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#endif
