///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//  Copyright  NetworkDLS 2002, All rights reserved
//
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF 
// ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO 
// THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A 
// PARTICULAR PURPOSE.
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#ifndef _CScriptingEngines_H
#define _CScriptingEngines_H
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include "../../@Classes/CXML/CXMLReader.H"
#include "../../@Classes/CStringBuilder/CStringBuilder.H"
#include "../../@Classes/CLocks/CLocks.H"
#include "../../@Classes/CStack/CStack.H"

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#define SCRIPTINGENGINES_MAX_EXT_LEGTH			MAX_PATH
#define SCRIPTINGENGINES_MAX_PARAMS_LEGTH		MAX_PATH
#define SCRIPTINGENGINES_MAX_ENGINE_LEGTH		MAX_PATH
#define SCRIPTINGENGINES_MAX_DESCR_LEGTH		1024
#define SCRIPTINGENGINES_MAX_STDIO_BUFFER_SIZE	1024

#define EXEC_RESULT_OK						1
#define EXEC_RESULT_ERROR					-1
#define EXEC_RESULT_TIMEOUT					-2
#define EXEC_RESULT_SCRIPT_NOT_FOUND		-3
#define EXEC_RESULT_ENGINE_NOT_FOUND		-4
#define EXEC_RESULT_EXTENSION_NOT_CONFIG	-5
#define EXEC_RESULT_INVALID_ISAPI			-6
#define EXEC_RESULT_NOT_IMPLEMENTED			-7
#define EXEC_RESULT_SSI_FAILED				-8

#define SCRIPTENGINES_LIST_POS_EXT			0
#define SCRIPTENGINES_LIST_POS_DESCR		1
#define SCRIPTENGINES_LIST_POS_PARAMS		2
#define SCRIPTENGINES_LIST_POS_ENGINE		3

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

typedef struct _tag_Script_Engine{
	char *Extension;
	char *Parameters;
	char *Engine;
	char *Description;
	bool Enabled;
} SCRIPTENGINE, *LPSCRIPTENGINE;

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

typedef struct _tag_Script_Engines{
	int Count;
	bool Enabled;
	SCRIPTENGINE *Items;
} SCRIPTENGINES, *LPSCRIPTENGINES;

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

class CScriptingEngines{

private:
	bool Initialized;
	void *pWebSites;
	SCRIPTENGINES Collection;
	char sFileName[MAX_PATH];

	CLocks Locks;

	bool AddEnvVar(CStringBuilder *lpBuf, const char *sVarName, const int iVarData);
	bool AddEnvVar(CStringBuilder *lpBuf, const char *sVarName, const char *sVarData);
	bool AddEnvVars(VOID *pClient, CStringBuilder *lpStr);
	bool CopyEnvVar(CStringBuilder *lpStr, const char *sVarName);

	int ProcessCGI(VOID *pClient, CStringBuilder *lpOutput, const char *sScriptFile,
		const char *sEngine, const char *sParams, bool bIsNativeExecutable);
	int ProcessFastCGI(VOID *pClient, CStringBuilder *lpOutput,
		const char *sScriptFile, const char *sEngine, const char *sParams);

public:
	CScriptingEngines(void *lpWebSites);
	CScriptingEngines(void *lpWebSites, CXMLReader *xmlConfig, CScriptingEngines *pDefaults);
	~CScriptingEngines(void);
	bool Save(void);

	bool ToXML(CXMLReader *lpXML);

	void AddConfigItem(HWND hListView, SCRIPTENGINE *pScriptEngine);
	void AddConfigItem(HWND hListView, SCRIPTENGINE *pScriptEngine, int iInsertionIndex);

	void PopInfoList(HWND hListView, bool bUseIcons);
	void PopConfigList(HWND hWnd, int iListView, int iEnabled);
	void PopInfoList(HWND hListView);

	bool GetScriptEngine(const char *sFileName, char *sEngine,
		int iMaxEngine, char *sParams, int iMaxParams, bool *bIsFastCGI);
	bool IsNativeExecutable(const char *sFileName);
	bool IsScriptFile(const char *sFileName);
	bool Load(const char *sXMLFileName);
	bool Load(CXMLReader *xmlConfig, CScriptingEngines *pDefaults);
	bool LoadNewConfiguration(HWND hWnd, HWND hGrid);
	bool Reload(void);

	bool Destroy(void);
	bool Enabled(void);
	int ProcessScript(VOID *pClient);
	int ProcessScript(VOID *pClient, CStringBuilder *lpOutput, const char *sScriptFile);

	const char *ErrorString(int iErrorCode);
};

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#endif
