///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//  Copyright  NetworkDLS 2002, All rights reserved
//
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF 
// ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO 
// THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A 
// PARTICULAR PURPOSE.
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#ifndef _CScriptingEngines_CPP
#define _CScriptingEngines_CPP
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include <Windows.H>
#include <ShlObj.H>
#include <Stdio.H>
#include <ShlOBJ.H>
#include <Stdlib.H>

extern HIMAGELIST hEnableDisableImageList; //Declared in MainDialog.cpp
extern HIMAGELIST hOnePixilImageList; //Declared in MainDialog.cpp

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include "CScriptingEngines.H"
#include "Entry.H"
#include "CWebSites.H"

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

using namespace NSWFL::Windows;
using namespace NSWFL::ListView;
using namespace NSWFL::String;
using namespace NSWFL::File;
using namespace NSWFL::System;

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void CScriptingEngines::AddConfigItem(HWND hListView, SCRIPTENGINE *pScriptEngine)
{
	this->Locks.Lock();
	this->AddConfigItem(hListView, pScriptEngine, ListView_GetItemCount(hListView));
	this->Locks.UnLock();
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void CScriptingEngines::AddConfigItem(HWND hListView, SCRIPTENGINE *pScriptEngine, int iInsertionIndex)
{
	this->Locks.Lock();
	LV_ITEM ListItem;
    memset(&ListItem, 0, sizeof(ListItem));

	int iImage = pScriptEngine->Enabled && this->Collection.Enabled;

    ListItem.pszText = (char *) pScriptEngine->Extension;
    ListItem.mask = LVIF_TEXT|LVIF_IMAGE;
	ListItem.iImage = iImage;
    ListItem.iSubItem = SCRIPTENGINES_LIST_POS_EXT;
	ListItem.iItem = iInsertionIndex;
    int iItem = ListView_InsertItem(hListView, &ListItem);

	ListItem.pszText = (char *) pScriptEngine->Description;
    ListItem.mask = LVIF_TEXT;
    ListItem.iSubItem = SCRIPTENGINES_LIST_POS_DESCR;
	ListItem.iItem = iInsertionIndex;
    ListView_SetItem(hListView, &ListItem);

    ListItem.pszText = (char *) pScriptEngine->Parameters;
    ListItem.mask = LVIF_TEXT;
    ListItem.iSubItem = SCRIPTENGINES_LIST_POS_PARAMS;
	ListItem.iItem = iInsertionIndex;
    ListView_SetItem(hListView, &ListItem);

    ListItem.pszText = (char *) pScriptEngine->Engine;
    ListItem.mask = LVIF_TEXT;
    ListItem.iSubItem = SCRIPTENGINES_LIST_POS_ENGINE;
	ListItem.iItem = iInsertionIndex;
    ListView_SetItem(hListView, &ListItem);

	ListView_SetCheckState(hListView, iItem, pScriptEngine->Enabled);
	this->Locks.UnLock();
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void CScriptingEngines::PopConfigList(HWND hWnd, int iListView, int iEnabled)
{
	this->Locks.Lock();
	CheckDlgButton(hWnd, iEnabled, this->Collection.Enabled);
	this->PopInfoList(GetDlgItem(hWnd, iListView), false);
	this->Locks.UnLock();
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void CScriptingEngines::PopInfoList(HWND hListView)
{
	this->Locks.Lock();
	this->PopInfoList(hListView, true);
	this->Locks.UnLock();
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void CScriptingEngines::PopInfoList(HWND hListView, bool bUseIcons)
{
	this->Locks.Lock();
	SendMessage(hListView, (UINT)WM_SETREDRAW , (WPARAM)FALSE, (LPARAM)0);

	LV_COLUMN MyLVColumn;
    memset(&MyLVColumn, 0, sizeof(LV_COLUMN));

	MyLVColumn.cx        = 100;
    MyLVColumn.pszText   = "Extension";
    MyLVColumn.iSubItem  = SCRIPTENGINES_LIST_POS_EXT;
    MyLVColumn.mask      = LVCF_WIDTH | LVCF_TEXT | LVCF_SUBITEM | LVCF_FMT;
    MyLVColumn.fmt       = LVCFMT_LEFT;
    ListView_InsertColumn(hListView, SCRIPTENGINES_LIST_POS_EXT, &MyLVColumn);

	MyLVColumn.cx        = 300;
    MyLVColumn.pszText   = "Description";
    MyLVColumn.iSubItem  = SCRIPTENGINES_LIST_POS_DESCR;
    MyLVColumn.mask      = LVCF_WIDTH | LVCF_TEXT | LVCF_SUBITEM | LVCF_FMT;
    MyLVColumn.fmt       = LVCFMT_LEFT;
    ListView_InsertColumn(hListView, SCRIPTENGINES_LIST_POS_DESCR, &MyLVColumn);

    MyLVColumn.cx        = 300;
    MyLVColumn.pszText   = "Parameters";
    MyLVColumn.iSubItem  = SCRIPTENGINES_LIST_POS_PARAMS;
    MyLVColumn.mask      = LVCF_WIDTH | LVCF_TEXT | LVCF_SUBITEM | LVCF_FMT;
    MyLVColumn.fmt       = LVCFMT_LEFT;
    ListView_InsertColumn(hListView, SCRIPTENGINES_LIST_POS_PARAMS, &MyLVColumn);

	MyLVColumn.cx        = 100;
    MyLVColumn.pszText   = "Engine";
    MyLVColumn.iSubItem  = SCRIPTENGINES_LIST_POS_ENGINE;
    MyLVColumn.mask      = LVCF_WIDTH | LVCF_TEXT | LVCF_SUBITEM | LVCF_FMT;
    MyLVColumn.fmt       = LVCFMT_LEFT;
    ListView_InsertColumn(hListView, SCRIPTENGINES_LIST_POS_ENGINE, &MyLVColumn);

	DWORD dwFlags = LVS_EX_FULLROWSELECT|LVS_EX_GRIDLINES|LVS_EX_HEADERDRAGDROP;

	if(!bUseIcons)
	{
		dwFlags |= LVS_EX_CHECKBOXES;
	}

    SendMessage(hListView, (UINT)LVM_SETEXTENDEDLISTVIEWSTYLE, (WPARAM)0, (LPARAM)dwFlags);

	if(bUseIcons)
	{
		ListView_SetImageList(hListView, hEnableDisableImageList, LVSIL_SMALL);
	}

	ListView_SetTextColor(hListView, RGB(0,0,100));

	//Pop the list-view.
	for(int x = 0; x < this->Collection.Count; x++)
	{
		this->AddConfigItem(hListView, &this->Collection.Items[x]);
	}
	LVAutoSizeColumns(hListView, 500);
	SendMessage(hListView, (UINT)WM_SETREDRAW , (WPARAM)TRUE, (LPARAM)0);
	this->Locks.UnLock();
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

CScriptingEngines::~CScriptingEngines(void)
{
	this->Destroy();
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

CScriptingEngines::CScriptingEngines(void *lpWebSites)
{
	this->Initialized = false;
	this->pWebSites = lpWebSites;
	strcpy_s(sFileName, sizeof(sFileName), "");
	memset(&this->Collection, 0, sizeof(this->Collection));
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

CScriptingEngines::CScriptingEngines(void *lpWebSites, CXMLReader *xmlConfig, CScriptingEngines *pDefaults)
{
	this->Initialized = true;
	this->pWebSites = lpWebSites;
	strcpy_s(sFileName, sizeof(sFileName), "");
	memset(&this->Collection, 0, sizeof(this->Collection));
	this->Load(xmlConfig, pDefaults);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CScriptingEngines::Save(void)
{
	this->Locks.Lock();

	CXMLReader xmlConfig;
	if(this->ToXML(&xmlConfig))
	{
		bool bResult = xmlConfig.ToFile(this->sFileName);
		if(!bResult)
		{
			char sErrorMsg[2048];
			GetLastError(sErrorMsg, sizeof(sErrorMsg),
				"Failed to save websites configuration file (", ")");
			strcat_s(sErrorMsg, sizeof(sErrorMsg), "\r\n\r\n\"");
			strcat_s(sErrorMsg, sizeof(sErrorMsg), this->sFileName);
			strcat_s(sErrorMsg, sizeof(sErrorMsg), "\"");
			((CWebSites *)this->pWebSites)->Trace(sErrorMsg);
		}
		return this->Locks.UnLock(bResult);
	}
	return this->Locks.UnLock(false);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CScriptingEngines::ToXML(CXMLReader *lpXML)
{
	this->Locks.Lock();

	CXMLWriter xmlConfig("ScriptingEngines");

	xmlConfig.AddBool("Enable", this->Collection.Enabled);

	for(int iItem = 0; iItem < this->Collection.Count; iItem++)
	{
		CXMLWriter Item("Engine");
		Item.Add("Extension", this->Collection.Items[iItem].Extension);
		Item.Add("Executable", this->Collection.Items[iItem].Engine);
		Item.Add("Parameters", this->Collection.Items[iItem].Parameters);
		Item.Add("Description", this->Collection.Items[iItem].Description);
		Item.AddBool("Enable", this->Collection.Items[iItem].Enabled);
		xmlConfig.Add(&Item);
	}

	xmlConfig.ToReader(lpXML);

	xmlConfig.Destroy();

	return this->Locks.UnLock(true);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

/*
	Loads a modified configuration from the configuration dialog (also saves to original file).
*/
bool CScriptingEngines::LoadNewConfiguration(HWND hWnd, HWND hGrid)
{
	this->Locks.LockExclusive();

	if(this->Initialized)
	{
		this->Destroy();
	}

	int iGridItems = LVItemCount(hGrid);

	char sExt[SCRIPTINGENGINES_MAX_EXT_LEGTH];
	char sEngine[SCRIPTINGENGINES_MAX_ENGINE_LEGTH];
	char sParams[SCRIPTINGENGINES_MAX_PARAMS_LEGTH];
	char sDescr[SCRIPTINGENGINES_MAX_DESCR_LEGTH];

	CXMLWriter xmlConfig("ScriptingEngines");

	xmlConfig.AddBool("Enable", IsDlgButtonChecked(hWnd, IDC_ENABLED) == BST_CHECKED);

	for(int iItem = 0; iItem < iGridItems; iItem++)
	{
		CXMLWriter Item("Engine");

		LVGetText(hGrid, iItem, SCRIPTENGINES_LIST_POS_EXT, sExt, sizeof(sExt));
		Item.Add("Extension", sExt);

		LVGetText(hGrid, iItem, SCRIPTENGINES_LIST_POS_ENGINE, sEngine, sizeof(sEngine));
		Item.Add("Executable", sEngine);

		LVGetText(hGrid, iItem, SCRIPTENGINES_LIST_POS_PARAMS, sParams, sizeof(sParams));
		Item.Add("Parameters", sParams);

		LVGetText(hGrid, iItem, SCRIPTENGINES_LIST_POS_DESCR, sDescr, sizeof(sDescr));
		Item.Add("Description", sDescr);

		Item.AddBool("Enable", LVChecked(hGrid, iItem));
		xmlConfig.Add(&Item);
	}

	if(strlen(this->sFileName) > 0)
	{
		if(xmlConfig.ToFile(this->sFileName))
		{
			return this->Locks.UnLockExclusive(this->Reload());
		}
	}
	else{
		CXMLReader XMLRConfig;
		return this->Locks.UnLockExclusive(Load(xmlConfig.ToReader(&XMLRConfig), NULL));
	}

	return this->Locks.UnLockExclusive(false);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
	Reloads the configuration from the file it was originally loaded from.
*/
bool CScriptingEngines::Reload(void)
{
	this->Locks.LockExclusive();

	if(this->Initialized)
	{
		this->Destroy();
	}

	this->Load(this->sFileName);

	return this->Locks.UnLockExclusive(this->Initialized);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CScriptingEngines::Load(const char *sXMLFileName)
{
	this->Locks.LockExclusive();

	this->Initialized = false;

	strcpy_s(this->sFileName, sizeof(this->sFileName), sXMLFileName);

	CXMLReader xmlConfig;

	if(xmlConfig.FromFile(sXMLFileName))
	{
		CXMLReader xmlEntity;
		if(xmlConfig.ToReader("ScriptingEngines", &xmlEntity))
		{
			this->Initialized = this->Load(&xmlEntity, NULL);
		}
		xmlEntity.Destroy();
		xmlConfig.Destroy();
	}

	return this->Locks.UnLockExclusive(this->Initialized);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CScriptingEngines::Load(CXMLReader *xmlConfig, CScriptingEngines *pDefaults)
{
	this->Locks.LockExclusive();

	int iLength = 0;
	memset(&this->Collection, 0, sizeof(this->Collection));

	this->Collection.Enabled = xmlConfig->ToBoolean("Enable", true);

	xmlConfig->ProgressiveScan(true);
	CXMLReader XPEngine;

	while(xmlConfig->ToReader("Engine", &XPEngine))
	{
		char sExt[SCRIPTINGENGINES_MAX_EXT_LEGTH];
		char sEngine[SCRIPTINGENGINES_MAX_ENGINE_LEGTH];
		char sParams[SCRIPTINGENGINES_MAX_PARAMS_LEGTH];
		char sDescr[SCRIPTINGENGINES_MAX_DESCR_LEGTH];

		this->Collection.Items = (SCRIPTENGINE *)
			pMem->ReAllocate(this->Collection.Items, sizeof(SCRIPTENGINE), this->Collection.Count + 1);

		SCRIPTENGINE *p = &this->Collection.Items[this->Collection.Count++];

		XPEngine.ToString("Extension", sExt, sizeof(sExt), &iLength);
		p->Extension = (char *) pMem->Allocate(sizeof(char), iLength + 1);
		strcpy_s(p->Extension, iLength + 1, LCase(sExt, iLength));

		XPEngine.ToString("Executable", sEngine, sizeof(sEngine), &iLength);
		p->Engine = (char *) pMem->Allocate(sizeof(char), iLength + 1);
		strcpy_s(p->Engine, iLength + 1, sEngine);

		XPEngine.ToString("Parameters", sParams, sizeof(sParams), &iLength);
		p->Parameters = (char *) pMem->Allocate(sizeof(char), iLength + 1);
		strcpy_s(p->Parameters, iLength + 1, sParams);

		XPEngine.ToString("Description", sDescr, sizeof(sDescr), &iLength);
		p->Description = (char *) pMem->Allocate(sizeof(char), iLength + 1);
		strcpy_s(p->Description, iLength + 1, sDescr);

		p->Enabled = XPEngine.ToBoolean("Enable", true);

		XPEngine.Destroy();
	}

	return this->Locks.UnLockExclusive(this->Initialized = true);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CScriptingEngines::Enabled(void)
{
	this->Locks.Lock();
	return this->Locks.UnLock(this->Collection.Enabled && (this->Collection.Count > 0));
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CScriptingEngines::Destroy(void)
{
	this->Locks.LockExclusive();

	if(this->Initialized)
	{
		this->Initialized = false;
		if(this->Collection.Count > 0)
		{
			for(int iPos = 0; iPos < this->Collection.Count; iPos++)
			{
				pMem->Free(this->Collection.Items[iPos].Extension);
				pMem->Free(this->Collection.Items[iPos].Engine);
				pMem->Free(this->Collection.Items[iPos].Parameters);
				pMem->Free(this->Collection.Items[iPos].Description);
			}

			pMem->Free(this->Collection.Items);

			this->Collection.Count = 0;
		}
	}
	return this->Locks.UnLockExclusive(true);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
	Returns true if the file is a scripted file, otherwise false.
*/
bool CScriptingEngines::GetScriptEngine(const char *sFileName,
										char *sEngine, int iMaxEngine,
										char *sParams, int iMaxParams, bool *bIsFastCGI)
{
	this->Locks.Lock();
	if(this->Collection.Enabled && (this->Collection.Count > 0))
	{
		char sExt[MAX_PATH];
		GetFileExtension(sFileName, sExt, sizeof(sExt));

		for(int iPos = 0; iPos < this->Collection.Count; iPos++)
		{
			if(this->Collection.Items[iPos].Enabled)
			{
				if(_strcmpi(this->Collection.Items[iPos].Extension, sExt) == 0)
				{
					if(bIsFastCGI)
					{
						char sEngineExt[MAX_PATH];
						if(GetFileExtension(this->Collection.Items[iPos].Engine, sEngineExt, sizeof(sEngineExt)))
						{
							*bIsFastCGI = (_strcmpi(".dll", sEngineExt) == 0);
						}
						else{
							*bIsFastCGI = false;
						}
					}
					strcpy_s(sEngine, iMaxEngine, this->Collection.Items[iPos].Engine);
					strcpy_s(sParams, iMaxParams, this->Collection.Items[iPos].Parameters);
					return this->Locks.UnLock(true);
				}
			}
		}
	}
	return this->Locks.UnLock(false);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
	Returns true if the file is a scripted file, otherwise false.
*/
bool CScriptingEngines::IsScriptFile(const char *sFileName)
{
	this->Locks.Lock();
	if(this->Collection.Enabled && (this->Collection.Count > 0))
	{
		char sExt[MAX_PATH];
		GetFileExtension(sFileName, sExt, sizeof(sExt));

		for(int iPos = 0; iPos < this->Collection.Count; iPos++)
		{
			if(this->Collection.Items[iPos].Enabled)
			{
				if(_strcmpi(this->Collection.Items[iPos].Extension, sExt) == 0)
				{
					return this->Locks.UnLock(true);
				}
			}
		}
	}
	return this->Locks.UnLock(false);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CScriptingEngines::AddEnvVar(CStringBuilder *lpBuf, const char *sVarName, const int iVarData)
{
	this->Locks.Lock();
	lpBuf->Append(sVarName);
	lpBuf->Append("=");
	lpBuf->Append(iVarData);
	lpBuf->Append("\0", 1);
	return this->Locks.UnLock(true);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CScriptingEngines::AddEnvVar(CStringBuilder *lpBuf, const char *sVarName, const char *sVarData)
{
	this->Locks.Lock();
	lpBuf->Append(sVarName);
	lpBuf->Append("=");
	lpBuf->Append(sVarData);
	lpBuf->Append("\0", 1);
	return this->Locks.UnLock(true);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CScriptingEngines::CopyEnvVar(CStringBuilder *lpStr, const char *sVarName)
{
	this->Locks.Lock();
	DWORD dwAlloc = GetEnvironmentVariable(sVarName, NULL, 0);
	char *sTemp = (char *)calloc(dwAlloc, sizeof(char));
	bool bResult = false;
	if(sTemp)
	{
		if(GetEnvironmentVariable(sVarName, sTemp, dwAlloc) == dwAlloc - 1)
		{
			bResult = this->AddEnvVar(lpStr, sVarName, sTemp);
		}
		free(sTemp);
	}
	return this->Locks.UnLock(bResult);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CScriptingEngines::AddEnvVars(VOID *pClient, CStringBuilder *lpStr)
{
	this->Locks.Lock();
	char sTemp[MAX_URI_LEN];

	CLIENT *pC = (CLIENT *)pClient;
	CHTTP *pHTTP = (CHTTP *)((CWebSite *)pC->pWebSite)->pHttp;
	CWebSites *pWebSites = (CWebSites *)((CWebSite *)pC->pWebSite)->pWebSites;

	if(pC->Header.PostDataSize > 0)
	{
		if(pC->Header.PostDataSize > 0)
		{
			this->AddEnvVar(lpStr, "CONTENT_LENGTH", pC->Header.PostDataSize);
			this->AddEnvVar(lpStr, "HTTP_CONTENT_LENGTH", pC->Header.PostDataSize);
		}
		if(strlen(pC->Header.ContentType) > 0)
		{
			this->AddEnvVar(lpStr, "CONTENT_TYPE", pC->Header.ContentType);
			this->AddEnvVar(lpStr, "HTTP_CONTENT_TYPE", pC->Header.ContentType);
		}
		else {
			this->AddEnvVar(lpStr, "CONTENT_TYPE", "application/x-www-form-urlencoded");
			this->AddEnvVar(lpStr, "HTTP_CONTENT_TYPE", "application/x-www-form-urlencoded");
		}
	}

	this->AddEnvVar(lpStr, "REMOTE_HOST", pC->pClient->PeerAddress());
    this->AddEnvVar(lpStr, "PATH_INFO", pC->Header.Request);
    this->AddEnvVar(lpStr, "SCRIPT_FILENAME", pC->Header.FullRequest);
	this->AddEnvVar(lpStr, "REQUEST_METHOD", pC->Header.Method);
    this->AddEnvVar(lpStr, "QUERY_STRING", pC->Header.Query);
	sprintf_s(sTemp, sizeof(sTemp), "NetworkDLS - %s", gsTitleCaption);

	if(pC->Header.Referer && pC->Header.Referer[0] != 0)
	{
		this->AddEnvVar(lpStr, "HTTP_REFERER", pC->Header.Referer);
	}

	if(pC->Header.Cookie && pC->Header.Cookie[0] != 0)
	{
		this->AddEnvVar(lpStr, "HTTP_COOKIE", pC->Header.Cookie);
	}

	if(_strcmpi(pC->Header.Connection, "Keep-Alive") == 0)
	{
	    this->AddEnvVar(lpStr, "HTTP_CONNECTION", "Keep-Alive");
	}
		
	this->AddEnvVar(lpStr, "SERVER_SOFTWARE", sTemp);
    this->AddEnvVar(lpStr, "SERVER_PROTOCOL", HTTP_VERSION);
    this->AddEnvVar(lpStr, "GATEWAY_INTERFACE", CGIINTERFACE);
    this->AddEnvVar(lpStr, "HTTP_HOST", pC->Header.Host);
    this->AddEnvVar(lpStr, "SERVER_PORT", ((CWebSite *)pC->pWebSite)->pServer->ListenPort());

	int iPortIndex = CharIndex(pC->Header.Host, ':');
	if(iPortIndex < 0)
	{
		iPortIndex = (int)strlen(pC->Header.Host);
	}
	if(iPortIndex > 0)
	{
		char sHost[MAX_HOST_STRING + 100];
		strncpy_s(sHost, sizeof(sHost), pC->Header.Host, iPortIndex);
		this->AddEnvVar(lpStr, "SERVER_NAME", sHost);
	}

	GetFilePath(pC->Header.FullRequest, sTemp, sizeof(sTemp));
    this->AddEnvVar(lpStr, "PATH_TRANSLATED", sTemp);
    //this->AddEnvVar(lpStr, "SERVER_NAME", "<!-- SERVER_NAME Not Implemented -->");
    this->AddEnvVar(lpStr, "REMOTE_ADDR", pC->pClient->PeerAddress());
    this->AddEnvVar(lpStr, "SCRIPT_NAME", pC->Header.Request);
    this->AddEnvVar(lpStr, "HTTP_ACCEPT", pC->Header.Accept);
    this->AddEnvVar(lpStr, "HTTP_ACCEPT_LANGUAGE", pC->Header.AcceptLanguage);
    this->AddEnvVar(lpStr, "HTTP_USER_AGENT", pC->Header.UserAgent);
	//this->AddEnvVar(lpStr, "CONTENT_TYPE", "*.*");
	this->AddEnvVar(lpStr, "DOCUMENT_ROOT", ((CWebSite *)pC->pWebSite)->Root);
	//this->AddEnvVar(lpStr, "REDIRECT_STATUS", ((CWebSite *)pC->pWebSite)->Root); //Non-Standard!

	this->CopyEnvVar(lpStr, "ComSpec");
	this->CopyEnvVar(lpStr, "PATH");
	this->CopyEnvVar(lpStr, "pathext");
	this->CopyEnvVar(lpStr, "systemroot");
	this->CopyEnvVar(lpStr, "TEMP");
	this->CopyEnvVar(lpStr, "TMP");
	this->CopyEnvVar(lpStr, "windir");
	this->CopyEnvVar(lpStr, "PATHEXT");
	this->CopyEnvVar(lpStr, "NUMBER_OF_PROCESSORS");
	this->CopyEnvVar(lpStr, "PROCESSOR_ARCHITECTURE");
	this->CopyEnvVar(lpStr, "PROCESSOR_IDENTIFIER");
	this->CopyEnvVar(lpStr, "PROCESSOR_LEVEL");
	this->CopyEnvVar(lpStr, "PROCESSOR_REVISION");
	this->CopyEnvVar(lpStr, "OS");

	lpStr->Append("\0", 1); //Terminate the environment.

	return this->Locks.UnLock(true);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CScriptingEngines::IsNativeExecutable(const char *sFileName)
{
	this->Locks.Lock();
	char sExt[MAX_PATH];

	if(GetFileExtension(sFileName, sExt, sizeof(sExt)))
	{
		if(_strcmpi(sExt, ".EXE") == 0 || _strcmpi(sExt, ".CMD") == 0
			||_strcmpi(sExt, ".BAT") == 0 || _strcmpi(sExt, ".CMD") == 0)
		{
			return this->Locks.UnLock(true);
		}
	}

	return this->Locks.UnLock(false);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
	Process script and send data directly to the client.
*/
int CScriptingEngines::ProcessScript(VOID *pClient)
{
	this->Locks.Lock();
	CLIENT *pC = (CLIENT *)pClient;
	CWebSite *pWebSite = (CWebSite *)pC->pWebSite;
	CHTTP *pHTTP = (CHTTP *)pWebSite->pHttp;
	CWebSites *pWebSites = (CWebSites *)pWebSite->pWebSites;

	CStringBuilder Content(SCRIPTINGENGINES_MAX_STDIO_BUFFER_SIZE);

	pC->IsDynamicContent = true;

	int iResult = ProcessScript(pC, &Content, pC->Header.FullRequest);
	if(iResult == EXEC_RESULT_OK)
	{
		iResult = EXEC_RESULT_ERROR;

		int iHeaderEndPos = 0;
		char *sMimeType = NULL;

		//A content type is REQUIRED, so if there is a content type, we
		//	will assume that the script also terminated the HTTP header.
		bool bHasContentType =
			pHTTP->DoesHeaderContainTag(Content.Buffer, Content.Length, "Content-type:", &iHeaderEndPos);

		//If no content type was specified, then we need to include one. Default to text/html.
		if(!bHasContentType)
		{
			sMimeType = "text/html";
		}

		if(pHTTP->DoesValueContain(pC->Header.AcceptEncoding, "gzip"))
		{
			//If the script didnt return a "Content-Encoding" then we are free to enforce our own.
			if(!pHTTP->DoesHeaderContainTag(Content.Buffer, iHeaderEndPos, "Content-Encoding:"))
			{
				if(((CWebSite*)pC->pWebSite)->pCompression->CompressDynamicContent() &&
					((CWebSite*)pC->pWebSite)->pCompression->IsCompressibleType(pC->Header.FullRequest))
				{
					if(Content.Length <= ((CWebSite*)pC->pWebSite)->pCompression->MaxCompressionSize()
						&& Content.Length >= ((CWebSite*)pC->pWebSite)->pCompression->MinCompressionSize())
					{
						char sUncompressed[MAX_PATH];
						char sCompressed[MAX_PATH];

						if(!((CWebSite*)pC->pWebSite)->pCompression->GetUniqueFileName(
							sUncompressed, sizeof(sUncompressed), ((CLIENT*)pC)->Header.FullRequest))
						{
							pWebSite->pErrorPages->SendError(pC, "500");
							Content.Destroy();
							return this->Locks.UnLock(EXEC_RESULT_ERROR);
						}
						if(!((CWebSite*)pC->pWebSite)->pCompression->GetUniqueFileName(
							sCompressed, sizeof(sCompressed)))
						{
							pWebSite->pErrorPages->SendError(pC, "500");
							Content.Destroy();
							return this->Locks.UnLock(EXEC_RESULT_ERROR);
						}

						//Compress the output from the scripting engine., excluding the header part (if any).
						if(BufferDataToFile(sUncompressed, Content.Buffer + iHeaderEndPos, Content.Length - iHeaderEndPos))
						{
							if(((CWebSite*)pC->pWebSite)->pCompression->Deflate(sUncompressed, sCompressed))
							{
								unsigned long ulCompressedSize = 0;

								//Get the size of the compressed file.
								if(!FileSize(sCompressed, &ulCompressedSize))
								{
									pWebSite->pErrorPages->SendError(pC, "500");
								}
								else{
									//Send the header.
									if(pWebSites->pHttp->SendOKHeader(pC, sMimeType, "gzip", ulCompressedSize, !bHasContentType))
									{
										//If the scripting engine supplied part of a header, send it.
										if(iHeaderEndPos > 0)
										{
											pWebSites->pHttp->BufferDataToClient(pC, Content.Buffer, iHeaderEndPos);
										}

										if(_strcmpi(pC->Header.Method, "HEAD") != 0)
										{
											//Send the compressed file.
											if(pWebSites->pHttp->SendRawFile(pC, sCompressed))
											{
												iResult = EXEC_RESULT_OK;
											}
										}
										else{
											iResult = EXEC_RESULT_OK;
										}
									}
								}
							}
							else{
								pWebSite->pErrorPages->SendError(pC, "500");
							}
						}
						else{
							pWebSite->pErrorPages->SendError(pC, "500");
						}

						DeleteFile(sCompressed);
						DeleteFile(sUncompressed);
						Content.Destroy();
						return this->Locks.UnLock(iResult);
					}
				}
				else{
					//No compression, continue below...
				}
			}
			else{
				//No compression, continue below...
			}
		}
		else{
			//No compression, continue below...
		}

		//Can't compress, just send the data.
		if(pWebSites->pHttp->SendOKHeader(pC, sMimeType, NULL, Content.Length - iHeaderEndPos, !bHasContentType))
		{
			if(_strcmpi(pC->Header.Method, "HEAD") != 0)
			{
				if(pWebSites->pHttp->BufferDataToClient(pC, Content.Buffer, Content.Length))
				{
					iResult = EXEC_RESULT_OK;
				}
			}
			else{
				iResult = EXEC_RESULT_OK;
			}
		}
	}

	Content.Destroy();
	return this->Locks.UnLock(iResult);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
	Process script and return all data through lpOutput.
*/
int CScriptingEngines::ProcessScript(VOID *pClient, CStringBuilder *lpOutput, const char *sScriptFile)
{
	this->Locks.Lock();
	if(!FileAccess(sScriptFile, FAccess))
	{
		return this->Locks.UnLock(EXEC_RESULT_SCRIPT_NOT_FOUND); //The script file was not found.
	}

	bool bIsNativeExecutable = this->IsNativeExecutable(sScriptFile);
	if(bIsNativeExecutable)
	{
		return this->Locks.UnLock(ProcessCGI(pClient, lpOutput, sScriptFile, NULL, NULL, true));
	}
	else {
		bool bIsFastCGI = false;
		char sEngine[MAX_PATH];
		char sParams[SCRIPTINGENGINES_MAX_PARAMS_LEGTH];

		if(!this->GetScriptEngine(sScriptFile, sEngine, sizeof(sEngine), sParams, sizeof(sParams), &bIsFastCGI))
		{
			return this->Locks.UnLock(EXEC_RESULT_EXTENSION_NOT_CONFIG); //As far as we are concerned, this is not an executable file.
		}

		if(!FileAccess(sEngine, FAccess))
		{
			return this->Locks.UnLock(EXEC_RESULT_ENGINE_NOT_FOUND); //The scripting engine executable was not found.
		}

		if(bIsFastCGI)
		{
			return this->Locks.UnLock(ProcessFastCGI(pClient, lpOutput, sScriptFile, sEngine, sParams));
		}
		else{
			return this->Locks.UnLock(ProcessCGI(pClient, lpOutput, sScriptFile, sEngine, sParams, false));
		}
	}

	return this->Locks.UnLock(EXEC_RESULT_ERROR); //Should never get here!
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

int CScriptingEngines::ProcessCGI(VOID *pClient, CStringBuilder *lpOutput,  const char *sScriptFile,
								  const char *sEngine, const char *sParams, bool bIsNativeExecutable)
{
	CLIENT *pC = (CLIENT *)pClient;
	CHTTP *pHTTP = (CHTTP *)((CWebSite *)pC->pWebSite)->pHttp;
	CWebSites *pWebSites = (CWebSites *)((CWebSite *)pC->pWebSite)->pWebSites;
	CWebSite *pWebSite = (CWebSite *)pC->pWebSite;

	char sSTDIO[SCRIPTINGENGINES_MAX_STDIO_BUFFER_SIZE];

	unsigned long ulExitCode     = 0; // Process Exit Code.
    unsigned long ulBytesRead    = 0; // Bytes Read.
    unsigned long ulBytesWritten = 0; // Bytes written.
    unsigned long ulBytesAvail   = 0; // Bytes Available.

	SECURITY_ATTRIBUTES SA;
    SECURITY_DESCRIPTOR SD;

	PROCESS_INFORMATION PI;
	memset(&PI, 0, sizeof(PI));

	STARTUPINFO SI;
	memset(&SI, 0, sizeof(SI));

	HANDLE hNewSTDIN = NULL;    //pipe handle.
	HANDLE hNewSTDOut = NULL;   //pipe handle.
	HANDLE hReadSTDOut = NULL;  //pipe handle.
	HANDLE hWriteSTDIn = NULL;  //pipe handle.

    // Initialize security descriptor (Windows NT).
	if(IsWinNT())
    {
        InitializeSecurityDescriptor(&SD, SECURITY_DESCRIPTOR_REVISION);
        SetSecurityDescriptorDacl(&SD, TRUE, NULL, FALSE);
        SA.lpSecurityDescriptor = &SD;
    }
    else SA.lpSecurityDescriptor = NULL;

    SA.nLength = sizeof(SECURITY_ATTRIBUTES);
    SA.bInheritHandle = TRUE; // Allow inheritable handles.

    //Create STD-IN pipe.
	if(!CreatePipe(&hNewSTDIN, &hWriteSTDIn, &SA, 0) )
    {
        return EXEC_RESULT_ERROR;
    }

	//Create STD-OUT pipe.
	if(!CreatePipe(&hReadSTDOut, &hNewSTDOut, &SA, 0))
    {
		CloseHandle(hNewSTDIN);
		CloseHandle(hWriteSTDIn);
        return EXEC_RESULT_ERROR;
    }

	//GetStartupInfo(&SI); //Default much of the startup info to the servers startup info.

    //Set StartupInfo for the spawned process.
	SI.cb			= sizeof(STARTUPINFO);
    SI.dwFlags		= STARTF_USESTDHANDLES | STARTF_USESHOWWINDOW;
    SI.wShowWindow	= SW_HIDE;
    SI.hStdOutput	= hNewSTDOut; //Set the new handles for the child process.
    SI.hStdError	= hNewSTDOut; //Set the new handles for the child process.
    SI.hStdInput	= hNewSTDIN;  //Set the new handles for the child process.

	/* --Why would this be up here???
	if( _strcmpi(pC->Header.Method, "POST") == 0 || _strcmpi(pC->Header.Method, "PUT") == 0)
	{
		//If we have post data, write it to the CGI's input.
		if(pC->Header.PostDataSize > 0)
	    {
			WriteFile(hWriteSTDIn, pC->Header.PostData, pC->Header.PostDataSize, &ulBytesWritten, NULL);
			WriteFile(hWriteSTDIn, "\n", 1, &ulBytesWritten, NULL);
		}
	}
	*/

	CStringBuilder Environment;
	this->AddEnvVars(pC, &Environment);

	int iResult = EXEC_RESULT_OK;
	char sCurrentDir[MAX_PATH];
	GetFilePath(sScriptFile, sCurrentDir, sizeof(sCurrentDir));

	if(bIsNativeExecutable)
	{
		iResult = (CreateProcess(sScriptFile, NULL, NULL, NULL, TRUE,
			CREATE_SUSPENDED, Environment.Buffer, sCurrentDir, &SI, &PI) > 0);
	}
	else{
		char sExecute[MAX_PATH * 2];
		char sParam[MAX_PATH * 2];

		if(strlen(sParams) > 0)
		{
			sprintf_s(sParam, sizeof(sParam), sParams, sScriptFile);
		}
		else{
			sprintf_s(sParam, sizeof(sParam), "\"%s\"", sScriptFile);
		}

		sprintf_s(sExecute, sizeof(sExecute), "\"%s\" %s", sEngine, sParam);

		iResult = (CreateProcess(NULL, sExecute, NULL, NULL, TRUE,
			DETACHED_PROCESS | CREATE_SUSPENDED, Environment.Buffer, sCurrentDir, &SI, &PI) > 0);
	}

	Environment.Destroy();

    if(iResult != EXEC_RESULT_OK)
    {
		CloseHandle(hNewSTDIN);
		CloseHandle(hNewSTDOut);
		CloseHandle(hReadSTDOut);
		CloseHandle(hWriteSTDIn);

        return iResult; //Failed to execute.
    }

	//If we have post/put/get data, write it to the CGI's input.
	if(pC->Header.PostDataSize > 0)
	{
		WriteFile(hWriteSTDIn, pC->Header.PostData, pC->Header.PostDataSize, &ulBytesWritten, NULL);
		WriteFile(hWriteSTDIn, "\n", 1, &ulBytesWritten, NULL);
	}

	ResumeThread(PI.hThread); //Start the process.

	//----------- Begin reading STDIO -------------------------------------

    DWORD dwStartTime = GetTickCount();
	DWORD dwTimeout = (DWORD)((CWebSite*)pC->pWebSite)->pSettings->ScriptTimeout() * 1000;
	DWORD dwTimeExpended = 0;

    while(true)
    {
		dwTimeExpended = GetTickCount() - dwStartTime;

		PeekNamedPipe(hReadSTDOut, NULL,
			SCRIPTINGENGINES_MAX_STDIO_BUFFER_SIZE, &ulBytesRead, &ulBytesAvail, NULL);

		//If there is no data pending, then sleep for awhile.
		if(ulBytesRead == 0 && ulBytesAvail == 0)
		{
			Sleep(10);
		}

        //If there is data pending, read it.
		if(ulBytesRead > 0)
        {
			//Get the data.
			ReadFile(hReadSTDOut, sSTDIO, SCRIPTINGENGINES_MAX_STDIO_BUFFER_SIZE, &ulBytesRead, NULL);
			lpOutput->Append(sSTDIO, ulBytesRead);
		}

        //Get the processes Exit Code.
		GetExitCodeProcess(PI.hProcess, &ulExitCode); 
        
		//If the application has exited, check for pending data,
		//	if there is none, break out of the loop.
		if(ulExitCode != STILL_ACTIVE)
        {
            // Check to see if there is any more pending data.
            PeekNamedPipe(hReadSTDOut, NULL,
				SCRIPTINGENGINES_MAX_STDIO_BUFFER_SIZE, &ulBytesRead, &ulBytesAvail, NULL);

			//If there is no more pending data, exit the loop.
			if(ulBytesRead == 0 && ulBytesAvail == 0)
			{
				break;
			}
        }

        if(dwTimeExpended > dwTimeout)
		{
			if(!TerminateProcess(PI.hProcess, 0))
			{
				//WriteLogEx("Failed to terminate the CGI process.", EVENT_WARN);
			}

			iResult = EXEC_RESULT_TIMEOUT;
			break;
		}
    }

	if(iResult == EXEC_RESULT_OK)
	{
		CStringBuilder SSIData(lpOutput);
		lpOutput->Clear();
		if(!pWebSite->pSSIFiles->ProcessServerSideInclude(pClient,
			SSIData.Buffer, SSIData.Length, false, lpOutput))
		{
			iResult = EXEC_RESULT_SSI_FAILED;
		}
		SSIData.Destroy();
	}

	CloseHandle(hNewSTDIN);
    CloseHandle(hWriteSTDIn);
    CloseHandle(hNewSTDOut);
    CloseHandle(hReadSTDOut);

	CloseHandle(PI.hProcess);
	CloseHandle(PI.hThread);

	return iResult;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

//#include "httpext.h"

//BOOL WINAPI GetExtensionVersion(HSE_VERSION_INFO* pVer);
//typedef BOOL (__stdcall *LPGetExtensionVersion)(HSE_VERSION_INFO* pVer);
//BOOL WINAPI TerminateExtension(DWORD dwFlags);
//typedef BOOL (__stdcall *LPTerminateExtension)(DWORD dwFlags);

int CScriptingEngines::ProcessFastCGI(VOID *pClient, CStringBuilder *lpOutput,
									  const char *sScriptFile, const char *sEngine, const char *sParams)
{
	//FIXFIX: Not implemented!
/*
	HMODULE hLibrary = LoadLibrary(sEngine);
	if(!hLibrary)
	{
		return EXEC_RESULT_INVALID_ISAPI;
	}

	LPGetExtensionVersion pGetExtensionVersion = (LPGetExtensionVersion)GetProcAddress(hLibrary, "GetExtensionVersion");
	if(pGetExtensionVersion)
	{
		HSE_VERSION_INFO VI;
		memset(&VI, 0, sizeof(VI));
		pGetExtensionVersion(&VI);
		printf("pGetExtensionVersion\n");
	}
	
	LPTerminateExtension pTerminateExtension = (LPTerminateExtension)GetProcAddress(hLibrary, "TerminateExtension");
	if(pTerminateExtension)
	{
		pTerminateExtension(HSE_TERM_MUST_UNLOAD //HSE_TERM_ADVISORY_UNLOAD);
		printf("pTerminateExtension\n");
	}

	FreeLibrary(hLibrary);
*/
	return EXEC_RESULT_NOT_IMPLEMENTED;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

const char *CScriptingEngines::ErrorString(int iErrorCode)
{
	switch(iErrorCode)
	{
		case EXEC_RESULT_ERROR:
			return "GCI processing failed with a generic error code";
		case EXEC_RESULT_TIMEOUT:
			return "Execution timed-out";
		case EXEC_RESULT_SCRIPT_NOT_FOUND:
			return "The specified script was not found";
		case EXEC_RESULT_ENGINE_NOT_FOUND:
			return "The required scripting engine was not found";
		case EXEC_RESULT_EXTENSION_NOT_CONFIG:
			return "The file extension is not configured as executable";
		case EXEC_RESULT_INVALID_ISAPI:
			return "The SAPI module is invalid";
		case EXEC_RESULT_NOT_IMPLEMENTED:
			return "GCI feature has not been implemented";
		case EXEC_RESULT_SSI_FAILED:
			return "Failed to post process SSI";
	}

	return "CGI Processing Error";
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#endif
