///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//  Copyright  NetworkDLS 2002, All rights reserved
//
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF 
// ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO 
// THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A 
// PARTICULAR PURPOSE.
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#ifndef _CSettings_H
#define _CSettings_H
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include "../../@Classes/CXML/CXMLReader.H"
#include "../../@Classes/CStringBuilder/CStringBuilder.H"
#include "../../@Classes/CLocks/CLocks.H"
#include "../../@Classes/CStack/CStack.H"

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#define SETTINGS_LIST_POS_ATTRIB	0
#define SETTINGS_LIST_POS_VALUE		1

#define MAX_ADMIN_EMAIL				255
#define MAX_USER_LENGTH				1024
#define MAX_DOMAIN_LENGTH			1024
#define MAX_PASS_LENGTH				1024

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

typedef struct _tag_Settings{
    int MaxConnections;
    int ScriptTimeout;
    bool CacheLogFile;
    bool AllowDirectoryIndexing;
    bool UseKeepAlives;
	bool AutoStart;
    int KeepAliveTimeout;
	int BufferSize;
	char LogPath[MAX_PATH];
	char ServerAdminEmail[MAX_ADMIN_EMAIL];

	char Username[MAX_USER_LENGTH];
	char Domain[MAX_DOMAIN_LENGTH];
	char Password[MAX_PASS_LENGTH];
} SETTINGS, *LPSETTINGS;

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

class CSettings{

private:
	bool Initialized;
	void *pWebSites;
	SETTINGS Collection;
	char sFileName[MAX_PATH];

	CLocks Locks;

	void AddConfigItem(HWND hListView, const char *sAttribute, int iValue);
	void AddConfigItem(HWND hListView, const char *sAttribute, double dValue);
	void AddConfigItem(HWND hListView, const char *sAttribute, bool bValue);
	void AddConfigItem(HWND hListView, const char *sAttribute, const char *sValue);
	void AddConfigItem(HWND hListView, const char *sAttribute, unsigned int iValue);

public:
	CSettings(void *lpWebSites);
	~CSettings(void);
	CSettings(void *lpWebSites, CXMLReader *xmlConfig, CSettings *pDefaults);
	bool Save(void);

	bool ToXML(CXMLReader *lpXML);

	void PopConfigList(HWND hWnd, int iListView);
	void PopInfoList(HWND hListView);

	bool AutoStart(void);
	int MaxConnections(void);
	int ScriptTimeout(void);
	bool CacheLogFile(void);
	bool AllowDirectoryIndexing(void);
	bool UseKeepAlives(void);
	int KeepAliveTimeout(void);
	int BufferSize(void);
	char *LogPath(void);
	char *ServerAdminEmail(void);
	char *Username(void);
	char *Domain(void);
	char *Password(void);

	bool LoadNewConfiguration(HWND hWnd, HWND hGrid);
	bool Reload(void);
	bool Load(const char *sXMLFileName);
	bool Load(CXMLReader *xmlConfig, CSettings *pDefaults);
	bool Destroy();
};

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#endif
