///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//  Copyright  NetworkDLS 2002, All rights reserved
//
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF 
// ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO 
// THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A 
// PARTICULAR PURPOSE.
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#ifndef _CSettings_CPP
#define _CSettings_CPP
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include <Windows.H>
#include <ShlObj.H>
#include <Stdio.H>
#include <ShlOBJ.H>
#include <Stdlib.H>

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

extern HIMAGELIST hEnableDisableImageList; //Declared in MainDialog.cpp
extern HIMAGELIST hOnePixilImageList; //Declared in MainDialog.cpp

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include "CSettings.H"
#include "Entry.H"
#include "CWebSites.H"

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

using namespace NSWFL::Windows;
using namespace NSWFL::ListView;
using namespace NSWFL::Conversion;
using namespace NSWFL::File;

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void CSettings::AddConfigItem(HWND hListView, const char *sAttribute, const char *sValue)
{
	this->Locks.Lock();
	LV_ITEM LVItem;
    memset(&LVItem, 0, sizeof(LVItem));

	int iIndex = ListView_GetItemCount(hListView);

    LVItem.pszText = (char *) sAttribute;
    LVItem.mask = LVIF_TEXT;
    LVItem.iSubItem = SETTINGS_LIST_POS_ATTRIB;
	LVItem.iItem = iIndex;
    ListView_InsertItem(hListView, &LVItem);

    LVItem.pszText = (char *) sValue;
    LVItem.mask = LVIF_TEXT;
    LVItem.iSubItem = SETTINGS_LIST_POS_VALUE;
	LVItem.iItem = iIndex;
    ListView_SetItem(hListView, &LVItem);
	this->Locks.UnLock();
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void CSettings::AddConfigItem(HWND hListView, const char *sAttribute, bool bValue)
{
	this->Locks.Lock();
	if(bValue)
	{
		this->AddConfigItem(hListView, sAttribute, "Yes");
	}
	else{
		this->AddConfigItem(hListView, sAttribute, "No");
	}
	this->Locks.UnLock();
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void CSettings::AddConfigItem(HWND hListView, const char *sAttribute, double dValue)
{
	this->Locks.Lock();
	char sValue[64];
	sprintf_s(sValue, sizeof(sValue), "%f", dValue);
	this->AddConfigItem(hListView, sAttribute, sValue);
	this->Locks.UnLock();
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void CSettings::AddConfigItem(HWND hListView, const char *sAttribute, unsigned int iValue)
{
	this->Locks.Lock();
	char sValue[64];
	sprintf_s(sValue, sizeof(sValue), "%d", iValue);
	this->AddConfigItem(hListView, sAttribute, sValue);
	this->Locks.UnLock();
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void CSettings::AddConfigItem(HWND hListView, const char *sAttribute, int iValue)
{
	this->Locks.Lock();
	char sValue[64];
	sprintf_s(sValue, sizeof(sValue), "%d", iValue);
	this->AddConfigItem(hListView, sAttribute, sValue);
	this->Locks.UnLock();
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void CSettings::PopConfigList(HWND hWnd, int iListView)
{
	this->Locks.Lock();
	this->PopInfoList(GetDlgItem(hWnd, iListView));
	this->Locks.UnLock();
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void CSettings::PopInfoList(HWND hListView)
{
	this->Locks.Lock();
	SendMessage(hListView, (UINT)WM_SETREDRAW , (WPARAM)FALSE, (LPARAM)0);

	LV_COLUMN MyLVColumn;
    memset(&MyLVColumn, 0, sizeof(LV_COLUMN));

	MyLVColumn.cx        = 100;
    MyLVColumn.pszText   = "Attribute";
    MyLVColumn.iSubItem  = SETTINGS_LIST_POS_ATTRIB;
    MyLVColumn.mask      = LVCF_WIDTH | LVCF_TEXT | LVCF_SUBITEM | LVCF_FMT;
    MyLVColumn.fmt       = LVCFMT_LEFT;
    ListView_InsertColumn(hListView, SETTINGS_LIST_POS_ATTRIB, &MyLVColumn);

	MyLVColumn.cx        = 100;
    MyLVColumn.pszText   = "Value";
    MyLVColumn.iSubItem  = SETTINGS_LIST_POS_VALUE;
    MyLVColumn.mask      = LVCF_WIDTH | LVCF_TEXT | LVCF_SUBITEM | LVCF_FMT;
    MyLVColumn.fmt       = LVCFMT_LEFT;
    ListView_InsertColumn(hListView, SETTINGS_LIST_POS_VALUE, &MyLVColumn);

	SendMessage(hListView, (UINT)LVM_SETEXTENDEDLISTVIEWSTYLE, (WPARAM)0,
		(LPARAM)LVS_EX_FULLROWSELECT|LVS_EX_GRIDLINES|LVS_EX_HEADERDRAGDROP);

	ListView_SetImageList(hListView, hOnePixilImageList, LVSIL_SMALL);

	ListView_SetTextColor(hListView, RGB(0,0,100));

	//Pop the list-view.

	char sInt[64];
	this->AddConfigItem(hListView, "Max Connections", FormatInteger(sInt, sizeof(sInt), this->Collection.MaxConnections));
	this->AddConfigItem(hListView, "Allow Directory Indexing", this->Collection.AllowDirectoryIndexing);
	this->AddConfigItem(hListView, "Auto Start", this->Collection.AutoStart);
	this->AddConfigItem(hListView, "Log Path", this->Collection.LogPath);
	this->AddConfigItem(hListView, "Buffer Size", FileSizeFriendly(this->Collection.BufferSize, 2, sInt, sizeof(sInt)));
	this->AddConfigItem(hListView, "Cache Log File", this->Collection.CacheLogFile);
	this->AddConfigItem(hListView, "Script Timeout",  FormatInteger(sInt, sizeof(sInt), this->Collection.ScriptTimeout));
	this->AddConfigItem(hListView, "Use Keep Alives", this->Collection.UseKeepAlives);
	this->AddConfigItem(hListView, "Keep Alive Timeout",  FormatInteger(sInt, sizeof(sInt), this->Collection.KeepAliveTimeout));
	this->AddConfigItem(hListView, "Server Admin Email",  this->Collection.ServerAdminEmail);
	this->AddConfigItem(hListView, "Root Username",  this->Collection.Username);
	this->AddConfigItem(hListView, "Root Domain",  this->Collection.Domain);
	//this->AddConfigItem(hListView, "Root Password",  this->Collection.Password);

	LVAutoSizeColumns(hListView, 500);
	SendMessage(hListView, (UINT)WM_SETREDRAW , (WPARAM)TRUE, (LPARAM)0);
	this->Locks.UnLock();
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

CSettings::~CSettings(void)
{
	this->Destroy();
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

CSettings::CSettings(void *lpWebSites)
{
	this->Initialized = false;
	this->pWebSites = lpWebSites;
	strcpy_s(sFileName, sizeof(sFileName), "");
	memset(&this->Collection, 0, sizeof(this->Collection));
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

CSettings::CSettings(void *lpWebSites, CXMLReader *xmlConfig, CSettings *pDefaults)
{
	this->Initialized = true;
	this->pWebSites = lpWebSites;
	strcpy_s(sFileName, sizeof(sFileName), "");
	memset(&this->Collection, 0, sizeof(this->Collection));
	this->Load(xmlConfig, pDefaults);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CSettings::Save(void)
{
	this->Locks.Lock();

	CXMLReader xmlConfig;
	if(this->ToXML(&xmlConfig))
	{
		bool bResult = xmlConfig.ToFile(this->sFileName);
		if(!bResult)
		{
			char sErrorMsg[2048];
			GetLastError(sErrorMsg, sizeof(sErrorMsg),
				"Failed to save websites configuration file (", ")");
			strcat_s(sErrorMsg, sizeof(sErrorMsg), "\r\n\r\n\"");
			strcat_s(sErrorMsg, sizeof(sErrorMsg), this->sFileName);
			strcat_s(sErrorMsg, sizeof(sErrorMsg), "\"");
			((CWebSites *)this->pWebSites)->Trace(sErrorMsg);
		}
		return this->Locks.UnLock(bResult);
	}
	return this->Locks.UnLock(false);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CSettings::ToXML(CXMLReader *lpXML)
{
	this->Locks.Lock();

	CXMLWriter xmlConfig("Settings");

	xmlConfig.AddBool("AllowDirectoryIndexing", this->Collection.AllowDirectoryIndexing);
	xmlConfig.AddBool("UseKeepAlives", this->Collection.UseKeepAlives);
	xmlConfig.AddBool("CacheLogFile", this->Collection.CacheLogFile);
	xmlConfig.AddBool("AutoStart", this->Collection.AutoStart);
	xmlConfig.Add("KeepAliveTimeout", this->Collection.KeepAliveTimeout);
	xmlConfig.Add("ScriptTimeout", this->Collection.ScriptTimeout);
	xmlConfig.Add("MaxConnections", this->Collection.MaxConnections);
	xmlConfig.Add("BufferSize", this->Collection.BufferSize);
	xmlConfig.Add("LogPath", this->Collection.LogPath);
	xmlConfig.Add("ServerAdminEmail", this->Collection.ServerAdminEmail);
	xmlConfig.Add("Username", this->Collection.Username);
	xmlConfig.Add("Domain", this->Collection.Domain);
	xmlConfig.Add("Password", this->Collection.Password);

	xmlConfig.ToReader(lpXML);

	xmlConfig.Destroy();

	return this->Locks.UnLock(true);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

/*
	Loads a modified configuration from the configuration dialog (also saves to original file).
*/
bool CSettings::LoadNewConfiguration(HWND hWnd, HWND hGrid)
{
	this->Locks.LockExclusive();

	if(this->Initialized)
	{
		this->Destroy();
	}

	int iGridItems = LVItemCount(hGrid);
	int iLength = 0;

	char sText[MAX_TEXT_SIZE];

	CXMLWriter xmlConfig("Settings");
	xmlConfig.AddBool("AllowDirectoryIndexing", IsDlgButtonChecked(hWnd, IDC_DIRINDEXING) == BST_CHECKED);
	xmlConfig.AddBool("UseKeepAlives", IsDlgButtonChecked(hWnd, IDC_USEKEEPALIVES) == BST_CHECKED);
	xmlConfig.AddBool("CacheLogFile", IsDlgButtonChecked(hWnd, IDC_CACHELOGFILE) == BST_CHECKED);
	xmlConfig.AddBool("AutoStart", IsDlgButtonChecked(hWnd, IDC_AUTOSTART) == BST_CHECKED);
	xmlConfig.Add("KeepAliveTimeout", Get_Long(GetDlgItem(hWnd, IDC_KEEPALIVETIMEOUT)));
	xmlConfig.Add("ScriptTimeout", Get_Long(GetDlgItem(hWnd, IDC_SCRIPTTIMEOUT)));
	xmlConfig.Add("MaxConnections", Get_Long(GetDlgItem(hWnd, IDC_MAXCONNECTIONS)));
	xmlConfig.Add("BufferSize", Get_Long(GetDlgItem(hWnd, IDC_BUFFERSIZE)));

	if(IsDlgButtonChecked(hWnd, IDC_NTAUTHENABLED))
	{
		if((iLength = Get_Text(GetDlgItem(hWnd, IDC_NTUSERNAME), sText, sizeof(sText))) > 0)
		{
			CXMLWriter xmlAuth("Login");
			xmlAuth.Add("Username", sText);
			Get_Text(GetDlgItem(hWnd, IDC_NTDOMAIN), sText, sizeof(sText));
			xmlAuth.Add("Domain", sText);
			Get_Text(GetDlgItem(hWnd, IDC_NTPASSWORD), sText, sizeof(sText));
			xmlAuth.Add("Password", sText);
			xmlConfig.Add(&xmlAuth);
			xmlAuth.Destroy();
		}
	}

	Get_Text(GetDlgItem(hWnd, IDC_LOGPATH), sText, sizeof(sText));
	xmlConfig.Add("LogPath", sText);

	Get_Text(GetDlgItem(hWnd, IDC_ADMINEMAIL), sText, sizeof(sText));
	xmlConfig.Add("ServerAdminEmail", sText);

	if(strlen(this->sFileName) > 0)
	{
		if(xmlConfig.ToFile(this->sFileName))
		{
			return this->Locks.UnLockExclusive(this->Reload());
		}
	}
	else{
		CXMLReader XMLRConfig;
		return this->Locks.UnLockExclusive(Load(xmlConfig.ToReader(&XMLRConfig), NULL));
	}

	return this->Locks.UnLockExclusive(false);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
	Reloads the configuration from the file it was originally loaded from.
*/
bool CSettings::Reload(void)
{
	this->Locks.LockExclusive();

	if(this->Initialized)
	{
		this->Destroy();
	}

	this->Load(this->sFileName);

	return this->Locks.UnLockExclusive(this->Initialized);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CSettings::Load(const char *sXMLFileName)
{
	this->Locks.LockExclusive();

	this->Initialized = false;

	strcpy_s(this->sFileName, sizeof(this->sFileName), sXMLFileName);

	CXMLReader xmlConfig;

	if(xmlConfig.FromFile(sXMLFileName))
	{
		CXMLReader xmlEntity;
		if(xmlConfig.ToReader("Settings", &xmlEntity))
		{
			this->Initialized = this->Load(&xmlEntity, NULL);
		}
		xmlEntity.Destroy();
		xmlConfig.Destroy();
	}

	return this->Locks.UnLockExclusive(this->Initialized);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CSettings::Load(CXMLReader *xmlConfig, CSettings *pDefaults)
{
	this->Locks.LockExclusive();

	memset(&this->Collection, 0, sizeof(this->Collection));

	if(pDefaults)
	{
		this->Collection.AllowDirectoryIndexing = xmlConfig->ToBoolean("AllowDirectoryIndexing", pDefaults->AllowDirectoryIndexing());
		this->Collection.UseKeepAlives = xmlConfig->ToBoolean("UseKeepAlives", pDefaults->UseKeepAlives());
		this->Collection.CacheLogFile = xmlConfig->ToBoolean("CacheLogFile", pDefaults->CacheLogFile());
		this->Collection.AutoStart = xmlConfig->ToBoolean("AutoStart", pDefaults->AutoStart());
		this->Collection.KeepAliveTimeout = xmlConfig->ToInteger("KeepAliveTimeout", pDefaults->KeepAliveTimeout());
		this->Collection.ScriptTimeout = xmlConfig->ToInteger("ScriptTimeout", pDefaults->ScriptTimeout());
		this->Collection.MaxConnections = xmlConfig->ToInteger("MaxConnections", pDefaults->MaxConnections());
		this->Collection.BufferSize = xmlConfig->ToInteger("BufferSize", pDefaults->BufferSize());
		xmlConfig->ToString("LogPath", this->Collection.LogPath, sizeof(this->Collection.LogPath), pDefaults->LogPath());
		xmlConfig->ToString("ServerAdminEmail", this->Collection.ServerAdminEmail, sizeof(this->Collection.ServerAdminEmail), pDefaults->ServerAdminEmail());
		xmlConfig->ToString("Username", this->Collection.Username, sizeof(this->Collection.Username), pDefaults->Username());
		xmlConfig->ToString("Domain", this->Collection.Domain, sizeof(this->Collection.Domain), pDefaults->Domain());
		xmlConfig->ToString("Password", this->Collection.Password, sizeof(this->Collection.Password), pDefaults->Password());
	}
	else{
		this->Collection.AllowDirectoryIndexing = xmlConfig->ToBoolean("AllowDirectoryIndexing", false);
		this->Collection.UseKeepAlives = xmlConfig->ToBoolean("UseKeepAlives", true);
		this->Collection.AutoStart = xmlConfig->ToBoolean("AutoStart", true);
		this->Collection.CacheLogFile = xmlConfig->ToBoolean("CacheLogFile", true);
		this->Collection.KeepAliveTimeout = xmlConfig->ToInteger("KeepAliveTimeout", 300);
		this->Collection.ScriptTimeout = xmlConfig->ToInteger("ScriptTimeout", 600);
		this->Collection.MaxConnections = xmlConfig->ToInteger("MaxConnections", 1000);
		this->Collection.BufferSize = xmlConfig->ToInteger("BufferSize", 1048576);
		xmlConfig->ToString("LogPath", this->Collection.LogPath, sizeof(this->Collection.LogPath));
		xmlConfig->ToString("ServerAdminEmail", this->Collection.ServerAdminEmail, sizeof(this->Collection.ServerAdminEmail));
		xmlConfig->ToString("Username", this->Collection.Username, sizeof(this->Collection.Username));
		xmlConfig->ToString("Domain", this->Collection.Domain, sizeof(this->Collection.Domain));
		xmlConfig->ToString("Password", this->Collection.Password, sizeof(this->Collection.Password));
	}

	return this->Locks.UnLockExclusive(this->Initialized = true);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

char *CSettings::LogPath(void)
{
	this->Locks.Lock();
	return this->Locks.UnLock(this->Collection.LogPath);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

int CSettings::BufferSize(void)
{
	this->Locks.Lock();
	return this->Locks.UnLock(this->Collection.BufferSize);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

int CSettings::MaxConnections(void)
{
	this->Locks.Lock();
	return this->Locks.UnLock(this->Collection.MaxConnections);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

int CSettings::ScriptTimeout(void)
{
	this->Locks.Lock();
	return this->Locks.UnLock(this->Collection.ScriptTimeout);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CSettings::CacheLogFile(void)
{
	this->Locks.Lock();
	return this->Locks.UnLock(this->Collection.CacheLogFile);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CSettings::AutoStart(void)
{
	this->Locks.Lock();
	return this->Locks.UnLock(this->Collection.AutoStart);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

char *CSettings::Username(void)
{
	this->Locks.Lock();
	return this->Locks.UnLock(this->Collection.Username);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

char *CSettings::Domain(void)
{
	this->Locks.Lock();
	return this->Locks.UnLock(this->Collection.Domain);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

char *CSettings::Password(void)
{
	this->Locks.Lock();
	return this->Locks.UnLock(this->Collection.Password);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

char *CSettings::ServerAdminEmail(void)
{
	this->Locks.Lock();
	return this->Locks.UnLock(this->Collection.ServerAdminEmail);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CSettings::AllowDirectoryIndexing(void)
{
	this->Locks.Lock();
	return this->Locks.UnLock(this->Collection.AllowDirectoryIndexing);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CSettings::UseKeepAlives(void)
{
	this->Locks.Lock();
	return this->Locks.UnLock(this->Collection.UseKeepAlives);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

int CSettings::KeepAliveTimeout(void)
{
	this->Locks.Lock();
	return this->Locks.UnLock(this->Collection.KeepAliveTimeout);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CSettings::Destroy(void)
{
	this->Locks.LockExclusive();

	if(this->Initialized)
	{
		this->Initialized = false;
		memset(&this->Collection, 0, sizeof(this->Collection));
	}
	return this->Locks.UnLockExclusive(true);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#endif
