///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//  Copyright  NetworkDLS 2002, All rights reserved
//
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF 
// ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO 
// THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A 
// PARTICULAR PURPOSE.
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#ifndef _CURLFilters_CPP
#define _CURLFilters_CPP
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include <Windows.H>
#include <ShlObj.H>
#include <Stdio.H>
#include <ShlOBJ.H>
#include <Stdlib.H>

extern HIMAGELIST hEnableDisableImageList; //Declared in MainDialog.cpp
extern HIMAGELIST hOnePixilImageList; //Declared in MainDialog.cpp

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include "CURLFilters.H"
#include "Entry.H"
#include "CWebSites.H"

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

using namespace NSWFL::Windows;
using namespace NSWFL::ListView;
using namespace NSWFL::Conversion;
using namespace NSWFL::String;

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void CURLFilters::AddConfigItem(HWND hListView, URLFILTER *pFilter)
{
	this->Locks.Lock();
	AddConfigItem(hListView, pFilter, ListView_GetItemCount(hListView));
	this->Locks.UnLock();
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void CURLFilters::AddConfigItem(HWND hListView, URLFILTER *pFilter, int iInsertionIndex)
{
	this->Locks.Lock();
	char sTemp[32];
	LV_ITEM ListItem;
    memset(&ListItem, 0, sizeof(ListItem));

	int iImage = pFilter->Enabled && this->Collection.Enabled;

    ListItem.pszText = (char *) pFilter->Sequence;
    ListItem.mask = LVIF_TEXT|LVIF_IMAGE;
	ListItem.iImage = iImage;
    ListItem.iSubItem = URLFILTERS_LIST_POS_SEQ;
	ListItem.iItem = iInsertionIndex;
    int iItem = ListView_InsertItem(hListView, &ListItem);

	sprintf_s(sTemp, sizeof(sTemp), "%s", pFilter->ScanRequest ? "Yes" : "No");
    ListItem.pszText = (char *) sTemp;
    ListItem.mask = LVIF_TEXT;
    ListItem.iSubItem = URLFILTERS_LIST_POS_SCANREQUEST;
	ListItem.iItem = iInsertionIndex;
    ListView_SetItem(hListView, &ListItem);

	sprintf_s(sTemp, sizeof(sTemp), "%s", pFilter->ScanQuery ? "Yes" : "No");
    ListItem.pszText = (char *) sTemp;
    ListItem.mask = LVIF_TEXT;
    ListItem.iSubItem = URLFILTERS_LIST_POS_SCANQUERY;
	ListItem.iItem = iInsertionIndex;
    ListView_SetItem(hListView, &ListItem);

    ListItem.pszText = (char *) pFilter->Description;
    ListItem.mask = LVIF_TEXT;
    ListItem.iSubItem = URLFILTERS_LIST_POS_DESCR;
	ListItem.iItem = iInsertionIndex;
    ListView_SetItem(hListView, &ListItem);

	ListView_SetCheckState(hListView, iItem, pFilter->Enabled);
	this->Locks.UnLock();
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void CURLFilters::PopConfigList(HWND hWnd, int iListView, int iEnabled)
{
	this->Locks.Lock();
	CheckDlgButton(hWnd, iEnabled, this->Collection.Enabled);
	this->PopInfoList(GetDlgItem(hWnd, iListView), false);
	this->Locks.UnLock();
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void CURLFilters::PopInfoList(HWND hListView)
{
	this->Locks.Lock();
	this->PopInfoList(hListView, true);
	this->Locks.UnLock();
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void CURLFilters::PopInfoList(HWND hListView, bool bUseIcons)
{
	this->Locks.Lock();
	SendMessage(hListView, (UINT)WM_SETREDRAW , (WPARAM)FALSE, (LPARAM)0);

	LV_COLUMN MyLVColumn;
    memset(&MyLVColumn, 0, sizeof(LV_COLUMN));

	MyLVColumn.cx        = 100;
    MyLVColumn.pszText   = "Sequence";
    MyLVColumn.iSubItem  = URLFILTERS_LIST_POS_SEQ;
    MyLVColumn.mask      = LVCF_WIDTH | LVCF_TEXT | LVCF_SUBITEM | LVCF_FMT;
    MyLVColumn.fmt       = LVCFMT_LEFT;
    ListView_InsertColumn(hListView, URLFILTERS_LIST_POS_SEQ, &MyLVColumn);

	MyLVColumn.cx        = 300;
    MyLVColumn.pszText   = "Scan Request";
    MyLVColumn.iSubItem  = URLFILTERS_LIST_POS_SCANREQUEST;
    MyLVColumn.mask      = LVCF_WIDTH | LVCF_TEXT | LVCF_SUBITEM | LVCF_FMT;
    MyLVColumn.fmt       = LVCFMT_LEFT;
    ListView_InsertColumn(hListView, URLFILTERS_LIST_POS_SCANREQUEST, &MyLVColumn);

	MyLVColumn.cx        = 300;
    MyLVColumn.pszText   = "Scan Query";
    MyLVColumn.iSubItem  = URLFILTERS_LIST_POS_SCANQUERY;
    MyLVColumn.mask      = LVCF_WIDTH | LVCF_TEXT | LVCF_SUBITEM | LVCF_FMT;
    MyLVColumn.fmt       = LVCFMT_LEFT;
    ListView_InsertColumn(hListView, URLFILTERS_LIST_POS_SCANQUERY, &MyLVColumn);

	MyLVColumn.cx        = 300;
    MyLVColumn.pszText   = "Description";
    MyLVColumn.iSubItem  = URLFILTERS_LIST_POS_DESCR;
    MyLVColumn.mask      = LVCF_WIDTH | LVCF_TEXT | LVCF_SUBITEM | LVCF_FMT;
    MyLVColumn.fmt       = LVCFMT_LEFT;
    ListView_InsertColumn(hListView, URLFILTERS_LIST_POS_DESCR, &MyLVColumn);

	DWORD dwFlags = LVS_EX_FULLROWSELECT|LVS_EX_GRIDLINES|LVS_EX_HEADERDRAGDROP;

	if(!bUseIcons)
	{
		dwFlags |= LVS_EX_CHECKBOXES;
	}

    SendMessage(hListView, (UINT)LVM_SETEXTENDEDLISTVIEWSTYLE, (WPARAM)0, (LPARAM)dwFlags);

	if(bUseIcons)
	{
		ListView_SetImageList(hListView, hEnableDisableImageList, LVSIL_SMALL);
	}

	ListView_SetTextColor(hListView, RGB(0,0,100));

	//Pop the list-view.
	for(int x = 0; x < this->Collection.Count; x++)
	{
		this->AddConfigItem(hListView, &this->Collection.Items[x]);
	}
	LVAutoSizeColumns(hListView, 500);
	SendMessage(hListView, (UINT)WM_SETREDRAW , (WPARAM)TRUE, (LPARAM)0);
	this->Locks.UnLock();
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

CURLFilters::~CURLFilters(void)
{
	this->Destroy();
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

CURLFilters::CURLFilters(void *lpWebSites)
{
	this->Initialized = false;
	this->pWebSites = lpWebSites;
	strcpy_s(sFileName, sizeof(sFileName), "");
	memset(&this->Collection, 0, sizeof(this->Collection));
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

CURLFilters::CURLFilters(void *lpWebSites, CXMLReader *xmlConfig, CURLFilters *pDefaults)
{
	this->Initialized = true;
	this->pWebSites = lpWebSites;
	strcpy_s(sFileName, sizeof(sFileName), "");
	memset(&this->Collection, 0, sizeof(this->Collection));
	this->Load(xmlConfig, pDefaults);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CURLFilters::Save(void)
{
	this->Locks.Lock();

	CXMLReader xmlConfig;
	if(this->ToXML(&xmlConfig))
	{
		bool bResult = xmlConfig.ToFile(this->sFileName);
		if(!bResult)
		{
			char sErrorMsg[2048];
			GetLastError(sErrorMsg, sizeof(sErrorMsg),
				"Failed to save websites configuration file (", ")");
			strcat_s(sErrorMsg, sizeof(sErrorMsg), "\r\n\r\n\"");
			strcat_s(sErrorMsg, sizeof(sErrorMsg), this->sFileName);
			strcat_s(sErrorMsg, sizeof(sErrorMsg), "\"");
			((CWebSites *)this->pWebSites)->Trace(sErrorMsg);
		}
		return this->Locks.UnLock(bResult);
	}
	return this->Locks.UnLock(false);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CURLFilters::ToXML(CXMLReader *lpXML)
{
	this->Locks.Lock();

	CXMLWriter xmlConfig("URLFilters");

	xmlConfig.AddBool("Enable", this->Collection.Enabled);

	for(int iItem = 0; iItem < this->Collection.Count; iItem++)
	{
		CXMLWriter Item("Filter");
		Item.Add("Sequence", this->Collection.Items[iItem].Sequence);
		Item.Add("Description", this->Collection.Items[iItem].Description);
		Item.Add("ScanRequest", this->Collection.Items[iItem].ScanQuery);
		Item.Add("ScanQuery", this->Collection.Items[iItem].ScanRequest);
		Item.AddBool("Enable", this->Collection.Items[iItem].Enabled);
		xmlConfig.Add(&Item);
	}

	xmlConfig.ToReader(lpXML);

	xmlConfig.Destroy();

	return this->Locks.UnLock(true);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
	Loads a modified configuration from the configuration dialog (also saves to original file).
*/
bool CURLFilters::LoadNewConfiguration(HWND hWnd, HWND hGrid)
{
	this->Locks.LockExclusive();

	if(this->Initialized)
	{
		this->Destroy();
	}

	int iGridItems = LVItemCount(hGrid);

	char sSequence[URLFILTERS_MAX_SEQ_LENGTH];
	char sDescr[URLFILTERS_MAX_DESCR_LENGTH];

	CXMLWriter xmlConfig("URLFilters");

	xmlConfig.AddBool("Enable", IsDlgButtonChecked(hWnd, IDC_ENABLED) == BST_CHECKED);

	for(int iItem = 0; iItem < iGridItems; iItem++)
	{
		CXMLWriter Item("Filter");

		LVGetText(hGrid, iItem, URLFILTERS_LIST_POS_SEQ, sSequence, sizeof(sSequence));
		Item.Add("Sequence", sSequence);

		LVGetText(hGrid, iItem, URLFILTERS_LIST_POS_DESCR, sDescr, sizeof(sDescr));
		Item.Add("Description", sDescr);

		LVGetText(hGrid, iItem, URLFILTERS_LIST_POS_SCANREQUEST, sDescr, sizeof(sDescr));
		Item.AddBool("ScanRequest", ParseBoolean(sDescr));

		LVGetText(hGrid, iItem, URLFILTERS_LIST_POS_SCANQUERY, sDescr, sizeof(sDescr));
		Item.AddBool("ScanQuery", ParseBoolean(sDescr));

		Item.AddBool("Enable", LVChecked(hGrid, iItem));
		xmlConfig.Add(&Item);
	}

	if(strlen(this->sFileName) > 0)
	{
		if(xmlConfig.ToFile(this->sFileName))
		{
			return this->Locks.UnLockExclusive(this->Reload());
		}
	}
	else{
		CXMLReader XMLRConfig;
		return this->Locks.UnLockExclusive(Load(xmlConfig.ToReader(&XMLRConfig), NULL));
	}

	return this->Locks.UnLockExclusive(false);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
	Reloads the configuration from the file it was originally loaded from.
*/
bool CURLFilters::Reload(void)
{
	this->Locks.LockExclusive();

	if(this->Initialized)
	{
		this->Destroy();
	}

	this->Load(this->sFileName);

	return this->Locks.UnLockExclusive(this->Initialized);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CURLFilters::Load(const char *sXMLFileName)
{
	this->Locks.LockExclusive();

	this->Initialized = false;

	strcpy_s(this->sFileName, sizeof(this->sFileName), sXMLFileName);

	CXMLReader xmlConfig;

	if(xmlConfig.FromFile(sXMLFileName))
	{
		CXMLReader xmlEntity;
		if(xmlConfig.ToReader("URLFilters", &xmlEntity))
		{
			this->Initialized = this->Load(&xmlEntity, NULL);
		}
		xmlEntity.Destroy();
		xmlConfig.Destroy();
	}

	return this->Locks.UnLockExclusive(this->Initialized);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CURLFilters::Load(CXMLReader *xmlConfig, CURLFilters *pDefaults)
{
	this->Locks.LockExclusive();

	int iLength = 0;
	memset(&this->Collection, 0, sizeof(this->Collection));

	xmlConfig->ProgressiveScan(true);
	CXMLReader XPURLFilter;
	
	this->Collection.Enabled = xmlConfig->ToBoolean("Enable", true);

	while(xmlConfig->ToReader("Filter", &XPURLFilter))
	{
		char sSequence[URLFILTERS_MAX_SEQ_LENGTH];
		char sDescr[URLFILTERS_MAX_DESCR_LENGTH];

		this->Collection.Items = (URLFILTER *)
			pMem->ReAllocate(this->Collection.Items, sizeof(URLFILTER), this->Collection.Count + 1);

		URLFILTER *p = &this->Collection.Items[this->Collection.Count++];

		XPURLFilter.ToString("Sequence", sSequence, sizeof(sSequence), &iLength);
		p->Sequence = (char *) pMem->Allocate(sizeof(char), iLength + 1);
		strcpy_s(p->Sequence, iLength + 1, sSequence);
		p->SequenceLength = iLength;

		XPURLFilter.ToString("Description", sDescr, sizeof(sDescr), &iLength);
		p->Description = (char *) pMem->Allocate(sizeof(char), iLength + 1);
		strcpy_s(p->Description, iLength + 1, sDescr);

		p->ScanQuery = XPURLFilter.ToBoolean("ScanQuery", true);
		p->ScanRequest = XPURLFilter.ToBoolean("ScanRequest", true);
		p->Enabled = XPURLFilter.ToBoolean("Enable", true);

		XPURLFilter.Destroy();
	}

	return this->Locks.UnLockExclusive((this->Initialized = true));
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CURLFilters::Enabled(void)
{
	this->Locks.Lock();
	return this->Locks.UnLock(this->Collection.Enabled && (this->Collection.Count > 0));
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CURLFilters::Destroy(void)
{
	this->Locks.LockExclusive();

	if(this->Initialized)
	{
		this->Initialized = false;
		if(this->Collection.Count > 0)
		{
			for(int iPos = 0; iPos < this->Collection.Count; iPos++)
			{
				pMem->Free(this->Collection.Items[iPos].Sequence);
				pMem->Free(this->Collection.Items[iPos].Description);
			}

			pMem->Free(this->Collection.Items);

			this->Collection.Count = 0;
		}
	}

	return this->Locks.UnLockExclusive(true);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
	Returns true if the header passes, otherwise false.
*/
bool CURLFilters::ScanHeader(VOID *pHeader)
{
	this->Locks.Lock();

	HTTPHEADER *pH = (HTTPHEADER *)(pHeader);
	if(this->Collection.Enabled && (this->Collection.Count > 0))
	{
		for(int iPos = 0; iPos < this->Collection.Count; iPos++)
		{
			if(this->Collection.Items[iPos].Enabled)
			{
				if(this->Collection.Items[iPos].ScanQuery)
				{
					if(InStrI(this->Collection.Items[iPos].Sequence,
						this->Collection.Items[iPos].SequenceLength, pH->Query, (int)strlen(pH->Query), 0) >= 0)
					{
						return this->Locks.UnLock(false);
					}
				}
				if(this->Collection.Items[iPos].ScanRequest)
				{
					if(InStrI(this->Collection.Items[iPos].Sequence,
						this->Collection.Items[iPos].SequenceLength, pH->Request, (int)strlen(pH->Request), 0) >= 0)
					{
						return this->Locks.UnLock(false);
					}
				}
			}
		}
	}

	return this->Locks.UnLock(true);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#endif
