///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//  Copyright  NetworkDLS 2002, All rights reserved
//
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF 
// ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO 
// THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A 
// PARTICULAR PURPOSE.
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#ifndef _CVirtualRoots_CPP
#define _CVirtualRoots_CPP
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include <Windows.H>
#include <ShlObj.H>
#include <Stdio.H>
#include <ShlOBJ.H>
#include <Stdlib.H>

extern HIMAGELIST hEnableDisableImageList; //Declared in MainDialog.cpp
extern HIMAGELIST hOnePixilImageList; //Declared in MainDialog.cpp

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include "CVirtualRoots.H"
#include "Entry.H"
#include "CWebSites.H"
#include "CWebSite.H"

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

using namespace NSWFL::Windows;
using namespace NSWFL::ListView;
using namespace NSWFL::String;
using namespace NSWFL::File;

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void CVirtualRoots::AddConfigItem(HWND hListView, ROOT *pRoot)
{
	this->Locks.Lock();
	this->AddConfigItem(hListView, pRoot, ListView_GetItemCount(hListView));
	this->Locks.UnLock();
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void CVirtualRoots::AddConfigItem(HWND hListView, ROOT *pRoot, int iInsertionIndex)
{
	this->Locks.Lock();
	LV_ITEM ListItem;
    memset(&ListItem, 0, sizeof(ListItem));

	int iImage = pRoot->Enabled && this->Collection.Enabled;

    ListItem.pszText = (char *) pRoot->Root;
    ListItem.mask = LVIF_TEXT|LVIF_IMAGE;
	ListItem.iImage = iImage;
    ListItem.iSubItem = VIRTUALROOTS_LIST_POS_ROOT;
	ListItem.iItem = iInsertionIndex;
    int iItem = ListView_InsertItem(hListView, &ListItem);

    ListItem.pszText = (char *) pRoot->Path;
    ListItem.mask = LVIF_TEXT;
    ListItem.iSubItem = VIRTUALROOTS_LIST_POS_PATH;
	ListItem.iItem = iInsertionIndex;
    ListView_SetItem(hListView, &ListItem);

    ListItem.pszText = (char *) pRoot->Description;
    ListItem.mask = LVIF_TEXT;
    ListItem.iSubItem = VIRTUALROOTS_LIST_POS_DESCR;
	ListItem.iItem = iInsertionIndex;
    ListView_SetItem(hListView, &ListItem);

	if(pRoot->Username)
	{
		ListItem.pszText = (char *) pRoot->Username;
		ListItem.mask = LVIF_TEXT;
		ListItem.iSubItem = VIRTUALROOTS_LIST_POS_USER;
		ListItem.iItem = iInsertionIndex;
		ListView_SetItem(hListView, &ListItem);

		ListItem.pszText = (char *) pRoot->Domain;
		ListItem.mask = LVIF_TEXT;
		ListItem.iSubItem = VIRTUALROOTS_LIST_POS_DOMAIN;
		ListItem.iItem = iInsertionIndex;
		ListView_SetItem(hListView, &ListItem);

		int iPwdLength = (int)strlen(pRoot->Password);
		char *sPwd = Space(iPwdLength);
		ReplaceCharacter(sPwd, ' ', '*');
		
		ListItem.pszText = (char *) pRoot->Password; //sPwd;
		ListItem.mask = LVIF_TEXT;
		ListItem.iSubItem = VIRTUALROOTS_LIST_POS_PASS;
		ListItem.iItem = iInsertionIndex;
		ListView_SetItem(hListView, &ListItem);

		free(sPwd);
	}

	ListView_SetCheckState(hListView, iItem, pRoot->Enabled);
	this->Locks.UnLock();
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void CVirtualRoots::PopConfigList(HWND hWnd, int iListView, int iEnabled)
{
	this->Locks.Lock();
	CheckDlgButton(hWnd, iEnabled, this->Collection.Enabled);
	this->PopInfoList(GetDlgItem(hWnd, iListView), false);
	this->Locks.UnLock();
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void CVirtualRoots::PopInfoList(HWND hListView)
{
	this->Locks.Lock();
	this->PopInfoList(hListView, true);
	this->Locks.UnLock();
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void CVirtualRoots::PopInfoList(HWND hListView, bool bUseIcons)
{
	this->Locks.Lock();
	SendMessage(hListView, (UINT)WM_SETREDRAW , (WPARAM)FALSE, (LPARAM)0);

	LV_COLUMN MyLVColumn;
    memset(&MyLVColumn, 0, sizeof(LV_COLUMN));

	MyLVColumn.cx        = 100;
    MyLVColumn.pszText   = "Root";
    MyLVColumn.iSubItem  = VIRTUALROOTS_LIST_POS_ROOT;
    MyLVColumn.mask      = LVCF_WIDTH | LVCF_TEXT | LVCF_SUBITEM | LVCF_FMT;
    MyLVColumn.fmt       = LVCFMT_LEFT;
    ListView_InsertColumn(hListView, VIRTUALROOTS_LIST_POS_ROOT, &MyLVColumn);

	MyLVColumn.cx        = 300;
    MyLVColumn.pszText   = "Path";
    MyLVColumn.iSubItem  = VIRTUALROOTS_LIST_POS_PATH;
    MyLVColumn.mask      = LVCF_WIDTH | LVCF_TEXT | LVCF_SUBITEM | LVCF_FMT;
    MyLVColumn.fmt       = LVCFMT_LEFT;
    ListView_InsertColumn(hListView, VIRTUALROOTS_LIST_POS_PATH, &MyLVColumn);

	MyLVColumn.cx        = 300;
    MyLVColumn.pszText   = "Description";
    MyLVColumn.iSubItem  = VIRTUALROOTS_LIST_POS_DESCR;
    MyLVColumn.mask      = LVCF_WIDTH | LVCF_TEXT | LVCF_SUBITEM | LVCF_FMT;
    MyLVColumn.fmt       = LVCFMT_LEFT;
    ListView_InsertColumn(hListView, VIRTUALROOTS_LIST_POS_DESCR, &MyLVColumn);

	MyLVColumn.cx        = 300;
    MyLVColumn.pszText   = "Username";
    MyLVColumn.iSubItem  = VIRTUALROOTS_LIST_POS_USER;
    MyLVColumn.mask      = LVCF_WIDTH | LVCF_TEXT | LVCF_SUBITEM | LVCF_FMT;
    MyLVColumn.fmt       = LVCFMT_LEFT;
    ListView_InsertColumn(hListView, VIRTUALROOTS_LIST_POS_USER, &MyLVColumn);

	MyLVColumn.cx        = 300;
    MyLVColumn.pszText   = "Domain";
    MyLVColumn.iSubItem  = VIRTUALROOTS_LIST_POS_DOMAIN;
    MyLVColumn.mask      = LVCF_WIDTH | LVCF_TEXT | LVCF_SUBITEM | LVCF_FMT;
    MyLVColumn.fmt       = LVCFMT_LEFT;
    ListView_InsertColumn(hListView, VIRTUALROOTS_LIST_POS_DOMAIN, &MyLVColumn);

	MyLVColumn.cx        = 300;
    MyLVColumn.pszText   = "Password";
    MyLVColumn.iSubItem  = VIRTUALROOTS_LIST_POS_PASS;
    MyLVColumn.mask      = LVCF_WIDTH | LVCF_TEXT | LVCF_SUBITEM | LVCF_FMT;
    MyLVColumn.fmt       = LVCFMT_LEFT;
    ListView_InsertColumn(hListView, VIRTUALROOTS_LIST_POS_PASS, &MyLVColumn);

	DWORD dwFlags = LVS_EX_FULLROWSELECT|LVS_EX_GRIDLINES|LVS_EX_HEADERDRAGDROP;

	if(!bUseIcons)
	{
		dwFlags |= LVS_EX_CHECKBOXES;
	}

    SendMessage(hListView, (UINT)LVM_SETEXTENDEDLISTVIEWSTYLE, (WPARAM)0, (LPARAM)dwFlags);

	if(bUseIcons)
	{
		ListView_SetImageList(hListView, hEnableDisableImageList, LVSIL_SMALL);
	}

	ListView_SetTextColor(hListView, RGB(0,0,100));

	//Pop the list-view.
	for(int x = 0; x < this->Collection.Count; x++)
	{
		this->AddConfigItem(hListView, &this->Collection.Items[x]);
	}
	LVAutoSizeColumns(hListView, 500);
	SendMessage(hListView, (UINT)WM_SETREDRAW , (WPARAM)TRUE, (LPARAM)0);
	this->Locks.UnLock();
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

CVirtualRoots::~CVirtualRoots(void)
{
	this->Destroy();
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

CVirtualRoots::CVirtualRoots(void *lpWebSites)
{
	this->Initialized = false;
	this->pWebSites = lpWebSites;
	strcpy_s(sFileName, sizeof(sFileName), "");
	memset(&this->Collection, 0, sizeof(this->Collection));
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

CVirtualRoots::CVirtualRoots(void *lpWebSites, CXMLReader *xmlConfig, CVirtualRoots *pDefaults)
{
	this->Initialized = true;
	this->pWebSites = lpWebSites;
	strcpy_s(sFileName, sizeof(sFileName), "");
	memset(&this->Collection, 0, sizeof(this->Collection));
	this->Load(xmlConfig, pDefaults);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CVirtualRoots::Save(void)
{
	this->Locks.Lock();

	CXMLReader xmlConfig;
	if(this->ToXML(&xmlConfig))
	{
		bool bResult = xmlConfig.ToFile(this->sFileName);
		if(!bResult)
		{
			char sErrorMsg[2048];
			GetLastError(sErrorMsg, sizeof(sErrorMsg),
				"Failed to save websites configuration file (", ")");
			strcat_s(sErrorMsg, sizeof(sErrorMsg), "\r\n\r\n\"");
			strcat_s(sErrorMsg, sizeof(sErrorMsg), this->sFileName);
			strcat_s(sErrorMsg, sizeof(sErrorMsg), "\"");
			((CWebSites *)this->pWebSites)->Trace(sErrorMsg);
		}
		return this->Locks.UnLock(bResult);
	}
	return this->Locks.UnLock(false);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CVirtualRoots::ToXML(CXMLReader *lpXML)
{
	this->Locks.Lock();

	CXMLWriter xmlConfig("VirtualRoots");

	xmlConfig.AddBool("Enable", this->Collection.Enabled);

	for(int iItem = 0; iItem < this->Collection.Count; iItem++)
	{
		CXMLWriter Item("Root");
		Item.Add("URI", this->Collection.Items[iItem].Root);
		Item.Add("Path", this->Collection.Items[iItem].Path);
		Item.Add("Description", this->Collection.Items[iItem].Description);
		Item.AddBool("Enable", this->Collection.Items[iItem].Enabled);

		if(this->Collection.Items[iItem].Username && strlen(this->Collection.Items[iItem].Username) > 0)
		{
			CXMLWriter xmlAuth("Login");
			xmlAuth.Add("Username", this->Collection.Items[iItem].Username);
			xmlAuth.Add("Domain", this->Collection.Items[iItem].Domain);
			xmlAuth.Add("Password", this->Collection.Items[iItem].Password);
			Item.Add(&xmlAuth);
		}

		xmlConfig.Add(&Item);
	}

	xmlConfig.ToReader(lpXML);

	xmlConfig.Destroy();

	return this->Locks.UnLock(true);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
	Loads a modified configuration from the configuration dialog (also saves to original file).
*/
bool CVirtualRoots::LoadNewConfiguration(HWND hWnd, HWND hGrid)
{
	this->Locks.LockExclusive();

	if(this->Initialized)
	{
		this->Destroy();
	}

	int iGridItems = LVItemCount(hGrid);

	char sURI[VIRTUALROOTS_MAX_ROOT_LENGTH];
	char sPath[VIRTUALROOTS_MAX_PATH_LENGTH];
	char sDescr[VIRTUALROOTS_MAX_DESCR_LENGTH];
	char sUser[VIRTUALROOTS_MAX_USER_LENGTH];
	char sDomain[VIRTUALROOTS_MAX_DOMAIN_LENGTH];
	char sPass[VIRTUALROOTS_MAX_PASS_LENGTH];

	CXMLWriter xmlConfig("VirtualRoots");

	xmlConfig.AddBool("Enable", IsDlgButtonChecked(hWnd, IDC_ENABLED) == BST_CHECKED);

	for(int iItem = 0; iItem < iGridItems; iItem++)
	{
		CXMLWriter Item("Root");

		LVGetText(hGrid, iItem, VIRTUALROOTS_LIST_POS_ROOT, sURI, sizeof(sURI));
		Item.Add("URI", sURI);

		LVGetText(hGrid, iItem, VIRTUALROOTS_LIST_POS_PATH, sPath, sizeof(sPath));
		Item.Add("Path", sPath);

		LVGetText(hGrid, iItem, VIRTUALROOTS_LIST_POS_DESCR, sDescr, sizeof(sDescr));
		Item.Add("Description", sDescr);

		Item.AddBool("Enable", LVChecked(hGrid, iItem));

		LVGetText(hGrid, iItem, VIRTUALROOTS_LIST_POS_USER, sUser, sizeof(sUser));
		LVGetText(hGrid, iItem, VIRTUALROOTS_LIST_POS_DOMAIN, sDomain, sizeof(sDomain));
		LVGetText(hGrid, iItem, VIRTUALROOTS_LIST_POS_PASS, sPass, sizeof(sPass));

		if(strlen(sUser) > 0)
		{
			CXMLWriter xmlAuth("Login");
			xmlAuth.Add("Username", sUser);
			xmlAuth.Add("Domain", sDomain);
			xmlAuth.Add("Password", sPass);
			Item.Add(&xmlAuth);
			xmlAuth.Destroy();
		}

		xmlConfig.Add(&Item);
	}

	if(strlen(this->sFileName) > 0)
	{
		if(xmlConfig.ToFile(this->sFileName))
		{
			return this->Locks.UnLockExclusive(this->Reload());
		}
	}
	else{
		CXMLReader XMLRConfig;
		return this->Locks.UnLockExclusive(Load(xmlConfig.ToReader(&XMLRConfig), NULL));
	}

	return this->Locks.UnLockExclusive(false);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
	Reloads the configuration from the file it was originally loaded from.
*/
bool CVirtualRoots::Reload(void)
{
	this->Locks.LockExclusive();

	if(this->Initialized)
	{
		this->Destroy();
	}

	this->Load(this->sFileName);

	return this->Locks.UnLockExclusive(this->Initialized);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CVirtualRoots::Load(const char *sXMLFileName)
{
	this->Locks.LockExclusive();
	this->Initialized = false;

	strcpy_s(this->sFileName, sizeof(this->sFileName), sXMLFileName);

	CXMLReader xmlConfig;

	if(xmlConfig.FromFile(sXMLFileName))
	{
		CXMLReader xmlEntity;
		if(xmlConfig.ToReader("VirtualRoots", &xmlEntity))
		{
			this->Initialized = this->Load(&xmlEntity, NULL);
		}
		xmlEntity.Destroy();
		xmlConfig.Destroy();
	}

	return this->Locks.UnLockExclusive(this->Initialized);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CVirtualRoots::Load(CXMLReader *xmlConfig, CVirtualRoots *pDefaults)
{
	this->Locks.LockExclusive();

	int iLength = 0;
	memset(&this->Collection, 0, sizeof(this->Collection));

	this->Collection.Enabled = xmlConfig->ToBoolean("Enable", true);

	xmlConfig->ProgressiveScan(true);
	CXMLReader XPVirtualRoot;
	CXMLReader xmlAuth;

	while(xmlConfig->ToReader("Root", &XPVirtualRoot))
	{
		char sURI[VIRTUALROOTS_MAX_ROOT_LENGTH];
		char sPath[VIRTUALROOTS_MAX_PATH_LENGTH];
		char sDescr[VIRTUALROOTS_MAX_DESCR_LENGTH];
		char sUser[VIRTUALROOTS_MAX_USER_LENGTH];
		char sDomain[VIRTUALROOTS_MAX_DOMAIN_LENGTH];
		char sPass[VIRTUALROOTS_MAX_PASS_LENGTH];

		this->Collection.Items = (ROOT *) pMem->ReAllocate(
			this->Collection.Items, sizeof(ROOT), this->Collection.Count + 1);

		ROOT *p = &this->Collection.Items[this->Collection.Count++];

		XPVirtualRoot.ToString("URI", sURI, sizeof(sURI), &iLength);
		p->Root = (char *) pMem->Allocate(sizeof(char), iLength + 1);
		strcpy_s(p->Root, iLength + 1, sURI);
		p->RootLength = iLength;

		XPVirtualRoot.ToString("Path", sPath, sizeof(sPath), &iLength);
		p->Path = (char *) pMem->Allocate(sizeof(char), iLength + 1);
		strcpy_s(p->Path, iLength + 1, sPath);
		p->PathLength = iLength;

		XPVirtualRoot.ToString("Description", sDescr, sizeof(sDescr), &iLength);
		p->Description = (char *) pMem->Allocate(sizeof(char), iLength + 1);
		strcpy_s(p->Description, iLength + 1, sDescr);

		p->Enabled = XPVirtualRoot.ToBoolean("Enable", false);
		
		if(XPVirtualRoot.ToReader("Login", &xmlAuth))
		{
			xmlAuth.ToString("Username", sUser, sizeof(sUser), &iLength);
			p->Username = (char *) pMem->Allocate(sizeof(char), iLength + 1);
			strcpy_s(p->Username, iLength + 1, sUser);

			xmlAuth.ToString("Domain", sDomain, sizeof(sDomain), &iLength);
			p->Domain = (char *) pMem->Allocate(sizeof(char), iLength + 1);
			strcpy_s(p->Domain, iLength + 1, sDomain);
	
			xmlAuth.ToString("Password", sPass, sizeof(sPass), &iLength);
			p->Password = (char *) pMem->Allocate(sizeof(char), iLength + 1);
			strcpy_s(p->Password, iLength + 1, sPass);
		}
		else{
			p->Username = NULL;
			p->Domain = NULL;
			p->Password = NULL;
		}

		xmlAuth.Destroy();

		XPVirtualRoot.Destroy();
	}

	return this->Locks.UnLockExclusive(this->Initialized = true);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CVirtualRoots::Destroy(void)
{
	this->Locks.LockExclusive();

	if(this->Initialized)
	{
		this->Initialized = false;
		if(this->Collection.Count > 0)
		{
			for(int iPos = 0; iPos < this->Collection.Count; iPos++)
			{
				pMem->Free(this->Collection.Items[iPos].Path);
				pMem->Free(this->Collection.Items[iPos].Root);
				pMem->Free(this->Collection.Items[iPos].Description);
				if(this->Collection.Items[iPos].Username)
				{
					pMem->Free(this->Collection.Items[iPos].Username);
				}
				if(this->Collection.Items[iPos].Domain)
				{
					pMem->Free(this->Collection.Items[iPos].Domain);
				}
				if(this->Collection.Items[iPos].Password)
				{
					pMem->Free(this->Collection.Items[iPos].Password);
				}
				this->Collection.Items[iPos].RootLength = 0;
			}

			pMem->Free(this->Collection.Items);

			this->Collection.Count = 0;
		}
	}
	return this->Locks.UnLockExclusive(true);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
	Returns a pointer to a virtual root if the request is a virtual root otherwise returns NULL.
*/
ROOT *CVirtualRoots::IsVirtualRoot(const char *sRequest, char *&sFullRequest)
{
	this->Locks.Lock();

	if(this->Collection.Enabled && (this->Collection.Count > 0))
	{
		int isRequestSize = (int)strlen(sRequest);

		for(int iPos = 0; iPos < this->Collection.Count; iPos++)
		{
			if(this->Collection.Items[iPos].Enabled)
			{
				if(InStrI(this->Collection.Items[iPos].Root,
					this->Collection.Items[iPos].RootLength, sRequest, isRequestSize, 0) == 0)
				{
					int iAlloc = this->Collection.Items[iPos].PathLength + 1;
					if(isRequestSize > this->Collection.Items[iPos].RootLength)
					{
						iAlloc += isRequestSize - this->Collection.Items[iPos].RootLength;
					}

					sFullRequest = (char *) pMem->ReAllocate(sFullRequest, sizeof(char), iAlloc);

					strcpy_s(sFullRequest, iAlloc, this->Collection.Items[iPos].Path);
					if(isRequestSize > this->Collection.Items[iPos].RootLength)
					{
						strcat_s(sFullRequest, iAlloc, sRequest + this->Collection.Items[iPos].RootLength);
					}

					CorrectReversePath(sFullRequest, iAlloc, true);

					return (ROOT *)this->Locks.UnLock((void *)&this->Collection.Items[iPos]);
				}
			}
		}
	}
	return (ROOT *)this->Locks.UnLock((void *)NULL);
}


///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#endif
