///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//  Copyright  NetworkDLS 2002, All rights reserved
//
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF 
// ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO 
// THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A 
// PARTICULAR PURPOSE.
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#ifndef _CWEBSITE_H
#define _CWEBSITE_H
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include "../../@Classes/CSocketServer/CSocketServer.H"
#include "../../@Classes/CXML/CXMLReader.H"
#include "../../@Classes/CStringBuilder/CStringBuilder.H"
#include "../../@Classes/CStack/CStack.H"

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#define MAX_HTTP_HEADER					1024 * 2
#define MAX_FILE_BUFFER					1048576 //One megabyte
#define MAX_QUERY_STRING				2048
#define MAX_URI_LEN						1024 + MAX_QUERY_STRING
//#define MAX_HTTP_METHOD_NAME			10
//#define MAX_HTTP_VERSION_NAME			15
#define MAX_IP_ADDRESS					16
#define MAX_POST_DATA					10240
#define MAX_RESPONSE_CODE				10

//#define MAX_ACCEPT_STRING				1024
//#define MAX_LANGUAGE_STRING			1024
//#define MAX_ACCEPT_ENCODING_STRING	1024
//#define MAX_USER_AGENT_STRING			1024
#define MAX_HOST_STRING					255
//#define MAX_CONNECTION_STRING			100

#define	MAX_SITE_NAME					512
#define MAX_SITE_DESCR					1024
#define HTTP_VERSION					"HTTP/1.1"
#define CGIINTERFACE					"CGI/1.1"
#define MIMEVERSION						"1.1"

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

typedef struct _tag_Last_Response {
	char ResponseCode[MAX_RESPONSE_CODE];
	__int64 ContentLength;
} LASTRESPONSE, *LPLASTRESPONSE;

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

typedef struct _tag_HTTP_Header {
	char *Method;
	char *Request;
	char *Version;
	char *Query;
	char *ContentType;

	char *Accept;
	char *AcceptLanguage;
	char *AcceptEncoding;
	char *UserAgent;
	char *Host;
	char *Connection;
	char *CacheControl;
	char *Referer;
	char *Cookie;
	char *ContentRange;

	char *FullRequest;

	char *AuthenticationRealm;

	__int64 ContentBegin;
	__int64 ContentEnd;

	char *PostData;
	int PostDataSize;
} HTTPHEADER, *LPHTTPHEADER;

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

typedef struct _tag_Client {
	HTTPHEADER Header;
	LASTRESPONSE LastResponse;

	CSocketClient *pClient;
	bool KeepAlive;
	bool IsDynamicContent;

	VOID *pWebSite; //CWebSite*
	CSocketServer *pServer; //CSocketServer*.

} CLIENT, *LPCLIENT;

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

typedef struct _tag_Site_Stats {
	__int64 AcceptedConnections;
	__int64 Hits;
	__int64 BytesRecv;
	__int64 BytesSent;
} SITESTATS, *LPSITESTATS;

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

typedef struct _tag_LOG_FILE_ENTRY {
	char sDate[64];
	char sTime[64];
	char *sSiteName;
	char sSourceIP[20];
	//char sTargetIP[20]; //Not yet implemented.
	char *sMethod; //GET, PUT, POST
	char *sAgent;
	char *sStatus;
	char *sAccept;
	char *sAcceptEncoding;
	char *sAcceptLanguage;
	char *sCacheControl;
	char *sConnection;
	char *sHost;
	char *sVersion;
	unsigned int iTimeTaken; //Miliseconds.
	__int64 iContentLength; //Bytes
	char *sURI;
	char *sQuery;
	int iPort;
	//char sUserName[...]; //Not yet implemented.
} LOGFILEENTRY, *LPLOGFILEENTRY;

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include "CMimeTypes.h"
#include "CURLFilters.h"
#include "CExtensionFilters.h"
#include "CDefaultPages.H"
#include "CSettings.H"
#include "CVirtualRoots.H"
#include "CSSIFiles.H"
#include "CIPFilters.H"
#include "CCGIFolders.H"
#include "CCompression.H"
#include "CCustomFilters.H"
#include "CScriptingEngines.H"
#include "CSSIFiles.H"
#include "CURLFilters.H"
#include "CCGIFolders.H"
#include "CScriptingEngines.H"
#include "CErrorPages.H"
#include "CAuthentication.H"
#include "CRealms.H"

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

class CWebSite{

private:
	FILE *hLogFile;
	char sLogCycleDate[64];
	CRITICAL_SECTION csLogFile;
	CLIENT *Clients;		//CLIENT[]

public:
	CWebSite();
	CWebSite(void *pWebSites, CXMLReader *pXMLSite);
	~CWebSite();

	friend void Socket_ClientHandler(CSocketServer *pSock, CSocketClient *pClient, LPBASICHUNK pChunk);

	void PopInfoList(HWND hListView);

	void AddConfigItem(HWND hListView, const char *sAttribute, const char *sValue);
	void AddConfigItem(HWND hListView, const char *sAttribute, bool bValue);
	void AddConfigItem(HWND hListView, const char *sAttribute, double dValue);
	void AddConfigItem(HWND hListView, const char *sAttribute, int iValue);
	void AddConfigItem(HWND hListView, const char *sAttribute, unsigned int iValue);
	void AddConfigItem(HWND hListView);
	int Index(void);

	bool IsActive(void);
	int SubItemsToXML(CXMLWriter *xmlConfig);
	bool ToXML(CXMLWriter *pXMLSite);
	bool Start(int iPort);
	bool Start(void);
	bool Stop(void);

	bool CycleLog(void);
	void CloseLog(void);
	void WriteLog(const char *sText);
	void WriteLog(LOGFILEENTRY *pLFE);

	char Name[MAX_SITE_NAME];
	char Description[MAX_SITE_DESCR];
	char Root[MAX_PATH];
	unsigned int Port;

	SITESTATS Stats;

	VOID *pWebSites;		//CWebSites*
	VOID *pHttp;			//CHTTP*
	CSocketServer *pServer;	//CSocketServer*

	CMimeTypes *pMimeTypes;
	CExtensionFilters *pExtensionFilters;
	CURLFilters *pURLFilters;
	CDefaultPages *pDefaultPages;
	CVirtualRoots *pVirtualRoots;
	CSSIFiles *pSSIFiles;
	CIPFilters *pIPFilters;
	CCGIFolders *pCGIFolders;
	CCompression *pCompression;
	CCustomFilters *pCustomFilters;
	CScriptingEngines *pScriptingEngines;
	CErrorPages *pErrorPages;
	CSettings *pSettings;
	CAuthentication *pAuthentication;
	CRealms *pRealms;
};

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#endif
