///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//  Copyright  NetworkDLS 2002, All rights reserved
//
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF 
// ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO 
// THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A 
// PARTICULAR PURPOSE.
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#ifndef _CWebSites_CPP
#define _CWebSites_CPP
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include <Windows.H>
#include <ShlObj.H>
#include <Stdio.H>
#include <ShlOBJ.H>
#include <Stdlib.H>

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

extern HIMAGELIST hEnableDisableImageList; //Declared in MainDialog.cpp
extern HIMAGELIST hOnePixilImageList; //Declared in MainDialog.cpp

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include "Entry.H"
#include "CWebSites.H"
#include "SocketHandlers.H"
#include "ConfigTree.H"

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

using namespace NSWFL::ListView;
using namespace NSWFL::Conversion;
using namespace NSWFL::File;
using namespace NSWFL::DateTime;

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

CWebSites *gWebSites;

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

CWebSites::CWebSites()
{
	memset(&this->InternalTrace, 0, sizeof(this->InternalTrace));
	memset(&this->Collection, 0, sizeof(this->Collection));
	memset(&this->sFileName, 0, sizeof(this->sFileName));

	this->dwTotalConnections = 0;

	this->pHttp = NULL;
	this->DfltSettings = NULL;
	this->DfltMimeTypes = NULL;
	this->DfltExtensionFilters = NULL;
	this->DfltURLFilters = NULL;
	this->DfltDefaultPages = NULL;
	this->DfltVirtualRoots = NULL;
	this->DfltErrorPages = NULL;
	this->DfltSSIFiles = NULL;
	this->DfltIPFilters = NULL;
	this->DfltCGIFolders = NULL;
	this->DfltCompression = NULL;
	this->DfltCustomFilters = NULL;
	this->DfltScriptingEngines = NULL;
	this->DfltAuthentication = NULL;
	this->DfltRealms = NULL;

	memset(&this->InternalTrace.CS, 0, sizeof(this->InternalTrace.CS));
	InitializeCriticalSection(&this->InternalTrace.CS);

	//Trace is "on" to begin with.
	//-------------------------------------------------------------------
	this->StartTrace(); //Must stop the trace after construction.
	//-------------------------------------------------------------------

	this->Initialize();
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

CWebSites::~CWebSites()
{
	this->Destroy();

	DeleteCriticalSection(&this->InternalTrace.CS);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void CWebSites::AddConfigItem(HWND hListView, CWebSite *pWebSite, int iSiteIndex)
{
	AddConfigItem(hListView, pWebSite, iSiteIndex, ListView_GetItemCount(hListView));
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void CWebSites::AddConfigItem(HWND hListView, CWebSite *pWebSite, int iSiteIndex, int iInsertionIndex)
{
	LV_ITEM LVItem;
    memset(&LVItem, 0, sizeof(LVItem));

	char sInt[64];

	int iImage = 0;
	
	if(iSiteIndex != INVALID_SITE_INDEX)
	{
		iImage = this->Site(iSiteIndex)->pServer->IsActive();
	}
	else{
		//Newly added site.
	}

    LVItem.pszText = (char *) pWebSite->Name;
    LVItem.mask = LVIF_TEXT|LVIF_IMAGE;
	LVItem.iImage = iImage;
    LVItem.iSubItem = WEBSITES_LIST_POS_NAME;
	LVItem.iItem = iInsertionIndex;
    ListView_InsertItem(hListView, &LVItem);

    LVItem.pszText = (char *) FormatInteger(sInt, sizeof(sInt), pWebSite->Port);
    LVItem.mask = LVIF_TEXT;
    LVItem.iSubItem = WEBSITES_LIST_POS_PORT;
	LVItem.iItem = iInsertionIndex;
    ListView_SetItem(hListView, &LVItem);

    LVItem.pszText = (char *) pWebSite->Description;
    LVItem.mask = LVIF_TEXT;
    LVItem.iSubItem = WEBSITES_LIST_POS_DESCR;
	LVItem.iItem = iInsertionIndex;
    ListView_SetItem(hListView, &LVItem);

    LVItem.pszText = (char *) pWebSite->Root;
    LVItem.mask = LVIF_TEXT;
    LVItem.iSubItem = WEBSITES_LIST_POS_ROOT;
	LVItem.iItem = iInsertionIndex;
    ListView_SetItem(hListView, &LVItem);

	sprintf_s(sInt, sizeof(sInt), "%d", iSiteIndex);
    LVItem.pszText = (char *) sInt;
    LVItem.mask = LVIF_TEXT;
    LVItem.iSubItem = WEBSITES_LIST_POS_INDEX;
	LVItem.iItem = iInsertionIndex;
    ListView_SetItem(hListView, &LVItem);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void CWebSites::PopConfigList(HWND hWnd, int iListView)
{
	this->PopInfoList(GetDlgItem(hWnd, iListView), false);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void CWebSites::PopInfoList(HWND hListView)
{
	this->PopInfoList(hListView, true);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void CWebSites::PopInfoList(HWND hListView, bool bUseIcons)
{
	SendMessage(hListView, (UINT)WM_SETREDRAW , (WPARAM)FALSE, (LPARAM)0);

	LV_COLUMN MyLVColumn;
    memset(&MyLVColumn, 0, sizeof(LV_COLUMN));

	MyLVColumn.cx        = 100;
    MyLVColumn.pszText   = "Name";
    MyLVColumn.iSubItem  = WEBSITES_LIST_POS_ATTRIB;
    MyLVColumn.mask      = LVCF_WIDTH | LVCF_TEXT | LVCF_SUBITEM | LVCF_FMT;
    MyLVColumn.fmt       = LVCFMT_LEFT;
    ListView_InsertColumn(hListView, WEBSITES_LIST_POS_NAME, &MyLVColumn);

	MyLVColumn.cx        = 500;
    MyLVColumn.pszText   = "Port";
    MyLVColumn.iSubItem  = WEBSITES_LIST_POS_VALUE;
    MyLVColumn.mask      = LVCF_WIDTH | LVCF_TEXT | LVCF_SUBITEM | LVCF_FMT;
    MyLVColumn.fmt       = LVCFMT_LEFT;
    ListView_InsertColumn(hListView, WEBSITES_LIST_POS_PORT, &MyLVColumn);

	MyLVColumn.cx        = 500;
    MyLVColumn.pszText   = "Description";
    MyLVColumn.iSubItem  = WEBSITES_LIST_POS_VALUE;
    MyLVColumn.mask      = LVCF_WIDTH | LVCF_TEXT | LVCF_SUBITEM | LVCF_FMT;
    MyLVColumn.fmt       = LVCFMT_LEFT;
    ListView_InsertColumn(hListView, WEBSITES_LIST_POS_DESCR, &MyLVColumn);

	MyLVColumn.cx        = 500;
    MyLVColumn.pszText   = "Root";
    MyLVColumn.iSubItem  = WEBSITES_LIST_POS_VALUE;
    MyLVColumn.mask      = LVCF_WIDTH | LVCF_TEXT | LVCF_SUBITEM | LVCF_FMT;
    MyLVColumn.fmt       = LVCFMT_LEFT;
    ListView_InsertColumn(hListView, WEBSITES_LIST_POS_ROOT, &MyLVColumn);

	MyLVColumn.cx        = 0;
    MyLVColumn.pszText   = "Index";
    MyLVColumn.iSubItem  = WEBSITES_LIST_POS_INDEX;
    MyLVColumn.mask      = LVCF_WIDTH | LVCF_TEXT | LVCF_SUBITEM | LVCF_FMT;
    MyLVColumn.fmt       = LVCFMT_LEFT;
    ListView_InsertColumn(hListView, WEBSITES_LIST_POS_INDEX, &MyLVColumn);

	DWORD dwFlags = LVS_EX_FULLROWSELECT|LVS_EX_GRIDLINES|LVS_EX_HEADERDRAGDROP;
	SendMessage(hListView, (UINT)LVM_SETEXTENDEDLISTVIEWSTYLE, (WPARAM)0, (LPARAM)dwFlags);

	if(bUseIcons)
	{
		ListView_SetImageList(hListView, hEnableDisableImageList, LVSIL_SMALL);
	}

	ListView_SetTextColor(hListView, RGB(0,0,100));

	//Pop the list-view.
	for(int iSiteIndex = 0; iSiteIndex < this->Collection.Count; iSiteIndex++)
	{
		this->Site(iSiteIndex)->AddConfigItem(hListView);
	}
	LVAutoSizeColumns(hListView, 500);
	SendMessage(hListView, (UINT)WM_SETREDRAW , (WPARAM)TRUE, (LPARAM)0);
}


///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void CWebSites::RefreshPerformanceList(HWND hListView)
{
	char sText[255];
	char sInt[64];

	//Pop the list-view.
	SendMessage(hListView, (UINT)WM_SETREDRAW , (WPARAM)FALSE, (LPARAM)0);

	int iItemCount = ListView_GetItemCount(hListView);

	for(int iSiteIndex = 0; iSiteIndex < this->Collection.Count; iSiteIndex++)
	{
		LV_ITEM LVItem;
		memset(&LVItem, 0, sizeof(LVItem));

		int iImage = 0;
		
		if(iSiteIndex != INVALID_SITE_INDEX)
		{
			iImage = this->Site(iSiteIndex)->pServer->IsActive();
		}
		else{
			iImage = 0;
		}

		LVItem.pszText = (char *)
			this->Collection.WebSite[iSiteIndex]->Name;
		LVItem.mask = LVIF_TEXT|LVIF_PARAM|LVIF_IMAGE;
		LVItem.iSubItem = WEBSITES_LIST_POS_NAME;
		LVItem.iItem = iSiteIndex;
		LVItem.lParam = iSiteIndex;
		LVItem.iImage = iImage;

		if(iItemCount > 0)
		{
			ListView_SetItem(hListView, &LVItem);
		}
		else{
			ListView_InsertItem(hListView, &LVItem);
		}

		LVItem.pszText = (char *)
			FileSizeFriendly(this->Collection.WebSite[iSiteIndex]->Stats.BytesSent, 2, sText, sizeof(sText));
		LVItem.mask = LVIF_TEXT;
		LVItem.iSubItem = WEBSITES_LIST_POS_BSENT;
		LVItem.iItem = iSiteIndex;
		ListView_SetItem(hListView, &LVItem);

		LVItem.pszText = (char *)
			FileSizeFriendly(this->Collection.WebSite[iSiteIndex]->Stats.BytesRecv, 2, sText, sizeof(sText));
		LVItem.mask = LVIF_TEXT;
		LVItem.iSubItem = WEBSITES_LIST_POS_BRECV;
		LVItem.iItem = iSiteIndex;
		ListView_SetItem(hListView, &LVItem);

		LVItem.pszText = (char *)
			I64FormatInteger(sInt, sizeof(sInt), this->Collection.WebSite[iSiteIndex]->Stats.Hits);
		LVItem.mask = LVIF_TEXT;
		LVItem.iSubItem = WEBSITES_LIST_POS_HITS;
		LVItem.iItem = iSiteIndex;
		ListView_SetItem(hListView, &LVItem);

		strcpy_s(sText, sizeof(sText),
			FormatInteger(sInt, sizeof(sInt), this->Collection.WebSite[iSiteIndex]->pServer->CurrentConnections()));
		strcat_s(sText, sizeof(sText), " of ");
		strcat_s(sText, sizeof(sText),
			FormatInteger(sInt, sizeof(sInt), this->Collection.WebSite[iSiteIndex]->pServer->MaxConnections()));
		LVItem.pszText = (char *) sText;
		LVItem.mask = LVIF_TEXT;
		LVItem.iSubItem = WEBSITES_LIST_POS_CONNECTS;
		LVItem.iItem = iSiteIndex;
		ListView_SetItem(hListView, &LVItem);

	}

	SendMessage(hListView, (UINT)WM_SETREDRAW , (WPARAM)TRUE, (LPARAM)0);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void CWebSites::PopPerformanceList(HWND hListView)
{
	LV_COLUMN MyLVColumn;
    memset(&MyLVColumn, 0, sizeof(LV_COLUMN));

	MyLVColumn.cx        = 100;
    MyLVColumn.pszText   = "Name";
    MyLVColumn.iSubItem  = WEBSITES_LIST_POS_NAME;
    MyLVColumn.mask      = LVCF_WIDTH | LVCF_TEXT | LVCF_SUBITEM | LVCF_FMT;
    MyLVColumn.fmt       = LVCFMT_LEFT;
    ListView_InsertColumn(hListView, WEBSITES_LIST_POS_NAME, &MyLVColumn);

	MyLVColumn.cx        = 500;
    MyLVColumn.pszText   = "Bytes Sent";
    MyLVColumn.iSubItem  = WEBSITES_LIST_POS_BSENT;
    MyLVColumn.mask      = LVCF_WIDTH | LVCF_TEXT | LVCF_SUBITEM | LVCF_FMT;
    MyLVColumn.fmt       = LVCFMT_RIGHT;
    ListView_InsertColumn(hListView, WEBSITES_LIST_POS_BSENT, &MyLVColumn);

	MyLVColumn.cx        = 500;
    MyLVColumn.pszText   = "Bytes Recv";
    MyLVColumn.iSubItem  = WEBSITES_LIST_POS_BRECV;
    MyLVColumn.mask      = LVCF_WIDTH | LVCF_TEXT | LVCF_SUBITEM | LVCF_FMT;
    MyLVColumn.fmt       = LVCFMT_RIGHT;
    ListView_InsertColumn(hListView, WEBSITES_LIST_POS_BRECV, &MyLVColumn);

	MyLVColumn.cx        = 500;
    MyLVColumn.pszText   = "Hits";
    MyLVColumn.iSubItem  = WEBSITES_LIST_POS_HITS;
    MyLVColumn.mask      = LVCF_WIDTH | LVCF_TEXT | LVCF_SUBITEM | LVCF_FMT;
    MyLVColumn.fmt       = LVCFMT_RIGHT;
    ListView_InsertColumn(hListView, WEBSITES_LIST_POS_HITS, &MyLVColumn);

	MyLVColumn.cx        = 500;
    MyLVColumn.pszText   = "Connections";
    MyLVColumn.iSubItem  = WEBSITES_LIST_POS_CONNECTS;
    MyLVColumn.mask      = LVCF_WIDTH | LVCF_TEXT | LVCF_SUBITEM | LVCF_FMT;
    MyLVColumn.fmt       = LVCFMT_RIGHT;
    ListView_InsertColumn(hListView, WEBSITES_LIST_POS_CONNECTS, &MyLVColumn);

	SendMessage(hListView, (UINT)LVM_SETEXTENDEDLISTVIEWSTYLE, (WPARAM)0,
		(LPARAM)LVS_EX_FULLROWSELECT|LVS_EX_GRIDLINES|LVS_EX_HEADERDRAGDROP);

	ListView_SetImageList(hListView, hEnableDisableImageList, LVSIL_SMALL);
	
	ListView_SetTextColor(hListView, RGB(0,0,100));

	this->RefreshPerformanceList(hListView);

	LVAutoSizeColumns(hListView, 500);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CWebSites::SaveAll(void)
{
	if(this->Save())
	{
		if(!this->DfltCGIFolders->Save()) {
			return false;
		}
		if(!this->DfltCompression->Save()) {
			return false;
		}
		if(!this->DfltCustomFilters->Save()) {
			return false;
		}
		if(!this->DfltDefaultPages->Save()) {
			return false;
		}
		if(!this->DfltExtensionFilters->Save()) {
			return false;
		}
		if(!this->DfltIPFilters->Save()) {
			return false;
		}
		if(!this->DfltMimeTypes->Save()) {
			return false;
		}
		if(!this->DfltScriptingEngines->Save()) {
			return false;
		}
		if(!this->DfltSettings->Save()) {
			return false;
		}
		if(!this->DfltSSIFiles->Save()) {
			return false;
		}
		if(!this->DfltURLFilters->Save()) {
			return false;
		}
		if(!this->DfltVirtualRoots->Save()) {
			return false;
		}
		if(!this->DfltErrorPages->Save()) {
			return false;
		}
		return this->Save();
	}
	return false;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CWebSites::Save(void)
{
	CXMLWriter xmlConfig("WebSites");
	
	for(int iItem = 0; iItem < this->Collection.Count; iItem++)
	{
		CXMLWriter Item;
		this->Collection.WebSite[iItem]->ToXML(&Item);
		xmlConfig.Add(&Item);
		Item.Destroy();
	}

	if(strlen(this->sFileName) > 0)
	{
		if(xmlConfig.ToFile(this->sFileName))
		{
			return true;
		}

		char sErrorMsg[2048];
		GetLastError(sErrorMsg, sizeof(sErrorMsg),
			"Failed to save websites configuration file (", ")");
		strcat_s(sErrorMsg, sizeof(sErrorMsg), "\r\n\r\n\"");
		strcat_s(sErrorMsg, sizeof(sErrorMsg), this->sFileName);
		strcat_s(sErrorMsg, sizeof(sErrorMsg), "\"");
		this->Trace(sErrorMsg);
	}

	return false;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CWebSites::SaveNewConfiguration(HWND hWnd, HWND hGrid)
{
	int iGridItems = LVItemCount(hGrid);

	char sName[MAX_SITE_NAME];
	char sDescription[MAX_SITE_DESCR];
	char sRoot[MAX_PATH];

	CXMLWriter xmlConfig("WebSites");
	
	for(int iItem = 0; iItem < iGridItems; iItem++)
	{
		CXMLWriter Item("Site");

		LVGetText(hGrid, iItem, WEBSITES_LIST_POS_NAME, sName, sizeof(sName));
		Item.Add("Name", sName);
		LVGetText(hGrid, iItem, WEBSITES_LIST_POS_DESCR, sDescription, sizeof(sDescription));
		Item.Add("Description", sDescription);
		LVGetText(hGrid, iItem, WEBSITES_LIST_POS_ROOT, sRoot, sizeof(sRoot));
		Item.Add("Root", sRoot);
		LVGetText(hGrid, iItem, WEBSITES_LIST_POS_PORT, sDescription, sizeof(sDescription));
		Item.Add("Port", iFormattedValue(sDescription));

		LVGetText(hGrid, iItem, WEBSITES_LIST_POS_INDEX, sDescription, sizeof(sDescription));
		int iSiteIndex = atol(sDescription);

		if(iSiteIndex != INVALID_SITE_INDEX)
		{
			this->Site(iSiteIndex)->SubItemsToXML(&Item);
		}

		xmlConfig.Add(&Item);
	}

	if(strlen(this->sFileName) > 0)
	{
		if(xmlConfig.ToFile(this->sFileName))
		{
			return true;
		}
	}

	return false;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CWebSites::LoadSites(char *sFile)
{
	CXMLReader xmlWebSites;
	strcpy_s(this->sFileName, sizeof(this->sFileName), sFile);

	if(xmlWebSites.FromFile(this->sFileName))
	{
		return this->LoadSites(&xmlWebSites);
	}
	return false;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CWebSites::LoadSites(CXMLReader *pWebSites)
{
	pWebSites->ProgressiveScan(true);
	CXMLReader xmlSite;

	while(pWebSites->ToReader("Site", &xmlSite))
	{
		this->Add(&xmlSite);
		xmlSite.Destroy();
	}
	
	return true;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CWebSites::ReloadSites(CXMLReader *pWebSites)
{
	this->DestroySites();
	return this->LoadSites(pWebSites);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
	Reloads all server settings and websites.
*/
bool CWebSites::Reload(void)
{
	if(this->Destroy())
	{
		return this->Initialize();
	}
	return false;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
	Reloads all server websites.
*/
bool CWebSites::ReloadSites(void)
{
	this->DestroySites();
	return this->LoadSites(this->sFileName);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

CWebSite *CWebSites::Site(int iIndex)
{
	return this->Collection.WebSite[iIndex];
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CWebSites::Initialize(void)
{
	char sMissingFiles[2048];
	char sConfigFile[MAX_PATH];
	const char *sErrorMsg = "Could not load \"%s\"";

	memset(&this->Collection, 0, sizeof(this->Collection));
	this->dwTotalConnections = 0;

	//this->pHttp = new zzzzzzzzzzzzzzzzzzz(this);
	this->DfltSettings = new CSettings(this);
	this->DfltMimeTypes = new CMimeTypes(this);
	this->DfltExtensionFilters = new CExtensionFilters(this);
	this->DfltURLFilters = new CURLFilters(this);
	this->DfltDefaultPages = new CDefaultPages(this);
	this->DfltVirtualRoots = new CVirtualRoots(this);
	this->DfltErrorPages = new CErrorPages(this);
	this->DfltSSIFiles = new CSSIFiles(this);
	this->DfltIPFilters = new CIPFilters(this);
	this->DfltCGIFolders = new CCGIFolders(this);
	this->DfltCompression = new CCompression(this);
	this->DfltCustomFilters = new CCustomFilters(this);
	this->DfltScriptingEngines = new CScriptingEngines(this);
	this->DfltAuthentication = new CAuthentication(this);
	this->DfltRealms = new CRealms(this);

	sprintf_s(sConfigFile, sizeof(sConfigFile), "%s\\Settings.xml", gsDataPath);
	if(!this->DfltSettings->Load(sConfigFile))
	{
		sprintf_s(sMissingFiles, sizeof(sMissingFiles), sErrorMsg, sConfigFile);
		this->Trace(sMissingFiles);
		return false;
	}
	sprintf_s(sConfigFile, sizeof(sConfigFile), "%s\\MimeTypes.xml", gsDataPath);
	if(!this->DfltMimeTypes->Load(sConfigFile))
	{
		sprintf_s(sMissingFiles, sizeof(sMissingFiles), sErrorMsg, sConfigFile);
		this->Trace(sMissingFiles);
		return false;
	}
	sprintf_s(sConfigFile, sizeof(sConfigFile), "%s\\URLFilters.xml", gsDataPath);
	if(!this->DfltURLFilters->Load(sConfigFile))
	{
		sprintf_s(sMissingFiles, sizeof(sMissingFiles), sErrorMsg, sConfigFile);
		this->Trace(sMissingFiles);
		return false;
	}
	sprintf_s(sConfigFile, sizeof(sConfigFile), "%s\\ExtensionFilters.xml", gsDataPath);
	if(!this->DfltExtensionFilters->Load(sConfigFile))
	{
		sprintf_s(sMissingFiles, sizeof(sMissingFiles), sErrorMsg, sConfigFile);
		this->Trace(sMissingFiles);
		return false;
	}
	sprintf_s(sConfigFile, sizeof(sConfigFile), "%s\\DefaultPages.xml", gsDataPath);
	if(!this->DfltDefaultPages->Load(sConfigFile))
	{
		sprintf_s(sMissingFiles, sizeof(sMissingFiles), sErrorMsg, sConfigFile);
		this->Trace(sMissingFiles);
		return false;
	}
	sprintf_s(sConfigFile, sizeof(sConfigFile), "%s\\VirtualRoots.xml", gsDataPath);
	if(!this->DfltVirtualRoots->Load(sConfigFile))
	{
		sprintf_s(sMissingFiles, sizeof(sMissingFiles), sErrorMsg, sConfigFile);
		this->Trace(sMissingFiles);
		return false;
	}
	sprintf_s(sConfigFile, sizeof(sConfigFile), "%s\\SSIFiles.xml", gsDataPath);
	if(!this->DfltSSIFiles->Load(sConfigFile))
	{
		sprintf_s(sMissingFiles, sizeof(sMissingFiles), sErrorMsg, sConfigFile);
		this->Trace(sMissingFiles);
		return false;
	}
	sprintf_s(sConfigFile, sizeof(sConfigFile), "%s\\IPFilters.xml", gsDataPath);
	if(!this->DfltIPFilters->Load(sConfigFile))
	{
		sprintf_s(sMissingFiles, sizeof(sMissingFiles), sErrorMsg, sConfigFile);
		this->Trace(sMissingFiles);
		return false;
	}
	sprintf_s(sConfigFile, sizeof(sConfigFile), "%s\\CGIFolders.xml", gsDataPath);
	if(!this->DfltCGIFolders->Load(sConfigFile))
	{
		sprintf_s(sMissingFiles, sizeof(sMissingFiles), sErrorMsg, sConfigFile);
		this->Trace(sMissingFiles);
		return false;
	}
	sprintf_s(sConfigFile, sizeof(sConfigFile), "%s\\Compression.xml", gsDataPath);
	if(!this->DfltCompression->Load(sConfigFile))
	{
		sprintf_s(sMissingFiles, sizeof(sMissingFiles), sErrorMsg, sConfigFile);
		this->Trace(sMissingFiles);
		return false;
	}
	sprintf_s(sConfigFile, sizeof(sConfigFile), "%s\\CustomFilters.xml", gsDataPath);
	if(!this->DfltCustomFilters->Load(sConfigFile))
	{
		sprintf_s(sMissingFiles, sizeof(sMissingFiles), sErrorMsg, sConfigFile);
		this->Trace(sMissingFiles);
		return false;
	}
	sprintf_s(sConfigFile, sizeof(sConfigFile), "%s\\ScriptingEngines.xml", gsDataPath);
	if(!this->DfltScriptingEngines->Load(sConfigFile))
	{
		sprintf_s(sMissingFiles, sizeof(sMissingFiles), sErrorMsg, sConfigFile);
		this->Trace(sMissingFiles);
		return false;
	}
	sprintf_s(sConfigFile, sizeof(sConfigFile), "%s\\ErrorPages.xml", gsDataPath);
	if(!this->DfltErrorPages->Load(sConfigFile))
	{
		sprintf_s(sMissingFiles, sizeof(sMissingFiles), sErrorMsg, sConfigFile);
		this->Trace(sMissingFiles);
		return false;
	}
	sprintf_s(sConfigFile, sizeof(sConfigFile), "%s\\WebSites.xml", gsDataPath);
	if(!this->LoadSites(sConfigFile))
	{
		sprintf_s(sMissingFiles, sizeof(sMissingFiles), sErrorMsg, sConfigFile);
		this->Trace(sMissingFiles);
		return false;
	}

	pHttp = new CHTTP(this);

	return true;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

int CWebSites::GetCurrentConenctions(void)
{
	int iConnections = 0;

	for(int iSite = 0; iSite < this->Collection.Count; iSite++)
	{
		if(this->Collection.WebSite[iSite]->pServer->IsActive())
		{
			iConnections += this->Collection.WebSite[iSite]->pServer->CurrentConnections();
		}
	}

	return iConnections;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void CWebSites::IncrementConnectionCount(void)
{
	this->dwTotalConnections++;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

int CWebSites::GetTotalConnections(void)
{
	return this->dwTotalConnections;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void CWebSites::DestroySites(void)
{
	if(this->Collection.Count > 0)
	{
		for(int iIndex = 0; iIndex < this->Collection.Count; iIndex++)
		{
			delete this->Collection.WebSite[iIndex];
		}
		pMem->Free(this->Collection.WebSite);
	}

	this->Collection.Count = 0;
	this->Collection.WebSite = NULL;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CWebSites::Destroy(void)
{
	this->DestroySites();

	if(this->DfltMimeTypes) delete this->DfltMimeTypes;
	if(this->DfltURLFilters) delete this->DfltURLFilters;
	if(this->DfltExtensionFilters) delete this->DfltExtensionFilters;
	if(this->DfltDefaultPages) delete this->DfltDefaultPages;
	if(this->DfltSettings) delete this->DfltSettings;
	if(this->DfltVirtualRoots) delete this->DfltVirtualRoots;
	if(this->DfltSSIFiles) delete this->DfltSSIFiles;
	if(this->DfltIPFilters) delete this->DfltIPFilters;
	if(this->DfltCGIFolders) delete this->DfltCGIFolders;
	if(this->DfltCompression) delete this->DfltCompression;
	if(this->DfltCustomFilters) delete this->DfltCustomFilters;
	if(this->DfltScriptingEngines) delete this->DfltScriptingEngines;
	if(this->DfltErrorPages) delete this->DfltErrorPages;
	if(pHttp) delete pHttp;

	this->StopTrace();

	return true;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CWebSites::Add(CXMLReader *xmlSite)
{
	this->Collection.WebSite = (CWebSite **)
		pMem->ReAllocate(this->Collection.WebSite, sizeof(CWebSite), (this->Collection.Count + 1));

	this->Collection.WebSite[this->Collection.Count] = new CWebSite(this, xmlSite);

	this->Collection.Count++;

	return true;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void CWebSites::StartTrace(void)
{
	EnterCriticalSection(&this->InternalTrace.CS);
	if(InternalTrace.bTracingEnabled)
	{
		InternalTrace.Stack->Clear();
	}
	else{
		InternalTrace.Stack = new CStack();
		InternalTrace.bTracingEnabled = true;
	}
	LeaveCriticalSection(&this->InternalTrace.CS);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void CWebSites::Trace(const char *sText)
{
	EnterCriticalSection(&this->InternalTrace.CS);
	if(InternalTrace.bTracingEnabled)
	{
		char sDate[255];
		char sTime[255];

		if(Get_ShortDate(sDate, sizeof(sDate)) && Get_Time(sTime, sizeof(sTime), 0))
		{
			int iAllocate = (int)(strlen(sText) + strlen(sDate) + strlen(sTime) + 10);
			char *sMessage = (char *) calloc(iAllocate, sizeof(char));
			if(sMessage)
			{
				sprintf_s(sMessage, iAllocate, "(%s %s) - %s", sDate, sTime, sText);
				this->InternalTrace.Stack->Push(sMessage);
				free(sMessage);
			}
			else{
				//Low on memory or the heap is corrupt, either way, I have money on things going badly!
				this->InternalTrace.Stack->Push(sText);
			}
		}
	}
	LeaveCriticalSection(&this->InternalTrace.CS);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

int CWebSites::StopTrace(bool bVerbose, HWND hWindow)
{
	CStack *pVerboseStack = NULL;

	int iErrorCount = 0;
	EnterCriticalSection(&this->InternalTrace.CS);
	if(InternalTrace.bTracingEnabled)
	{
		if((iErrorCount = (int)this->InternalTrace.Stack->StackSize()) > 0)
		{
			if(bVerbose)
			{
				pVerboseStack = new CStack(this->InternalTrace.Stack);
			}

			char sDate[255];
			if(Get_ShortDate(sDate, sizeof(sDate)))
			{
				char sMyDocs[MAX_PATH];
				if(Get_DocumentsDirectory(sMyDocs, sizeof(sMyDocs)))
				{
					char sFileName[MAX_PATH];
					sprintf_s(sFileName, sizeof(sFileName), "%s\\%s", sMyDocs, gsTitleCaption);
					CorrectReversePath(sFileName, sizeof(sFileName), true);
					CreateFolderStructure(sFileName);

					sprintf_s(sFileName, sizeof(sFileName), "%s\\%s\\Errors %s.txt",
						sMyDocs, gsTitleCaption, sDate);
					CorrectReversePath(sFileName, sizeof(sFileName), true);

					FILE *hLogFile = NULL;
					if(fopen_s(&hLogFile, sFileName, "ab") == 0)
					{
						for(unsigned int iExp = 0; iExp < this->InternalTrace.Stack->StackSize(); iExp++)
						{
							fprintf_s(hLogFile, "%s\r\n", (char *)this->InternalTrace.Stack->Peek(iExp)->Value);
						}
						fclose(hLogFile);
					}
				}
			}
		}
		InternalTrace.bTracingEnabled = false;
		delete this->InternalTrace.Stack;
	}
	LeaveCriticalSection(&this->InternalTrace.CS);

	//Verbose messages must be called out of the "Critical Section" lock.
	if(pVerboseStack)
	{
		for(unsigned int iExp = 0; iExp < pVerboseStack->StackSize(); iExp++)
		{
			MessageBox(hWindow, (char *)pVerboseStack->Peek(iExp)->Value, gsTitleCaption, MB_ICONERROR);
		}

		delete pVerboseStack;
	}

	return iErrorCount;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#endif
