///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//  Copyright  NetworkDLS 2002, All rights reserved
//
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF 
// ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO 
// THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A 
// PARTICULAR PURPOSE.
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#ifndef _CWebSites_H
#define _CWebSites_H
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include "../../@Classes/CSocketServer/CSocketServer.H"
#include "../../@Classes/CXML/CXMLReader.H"
#include "../../@Classes/CStringBuilder/CStringBuilder.H"
#include "../../@Classes/CStack/CStack.H"

#include "CWebSite.H"
#include "CHTTP.H"

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#define MAX_TEXT_SIZE				2048

#define WEBSITES_LIST_POS_ATTRIB	0
#define WEBSITES_LIST_POS_VALUE		1

#define WEBSITES_LIST_POS_NAME		0
#define WEBSITES_LIST_POS_BSENT		1
#define WEBSITES_LIST_POS_BRECV		2
#define WEBSITES_LIST_POS_HITS		3
#define WEBSITES_LIST_POS_CONNECTS	4

#define WEBSITES_LIST_POS_NAME		0
#define WEBSITES_LIST_POS_PORT		1
#define WEBSITES_LIST_POS_DESCR		2
#define WEBSITES_LIST_POS_ROOT		3
#define WEBSITES_LIST_POS_INDEX		4

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

typedef struct _tag_WSTrace { //FIXFIX: This needs to be a class with its own critical section.
	CStack *Stack;
	bool bTracingEnabled;
	CRITICAL_SECTION CS;
} ERRORTRACING, LPERRORTRACING;

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

class CWebSites{

private:
	DWORD dwTotalConnections;
	char sFileName[MAX_PATH];

	void PopInfoList(HWND hListView, bool bUseIcons);
	int StopTrace(bool bVerbose, HWND hWindow);
	ERRORTRACING InternalTrace;

	bool Initialize(void);
	bool Destroy(void);

public:
	CWebSites();
	~CWebSites();

	void StartTrace(void);
	void Trace(const char *sText);
	int StopTrace(void) { return this->StopTrace(false, NULL); } //Not-Verbose.
	int StopTrace(HWND hWindow) { return this->StopTrace(true, hWindow); } //Verbose.

	typedef struct _tag_Web_Site_Collection{
		int Count;
		CWebSite **WebSite;
	} WEBSITECOLLECTION;

	WEBSITECOLLECTION Collection;

	CHTTP *pHttp;

	CSettings *DfltSettings;
	CMimeTypes *DfltMimeTypes;
	CExtensionFilters *DfltExtensionFilters;
	CURLFilters *DfltURLFilters;
	CDefaultPages *DfltDefaultPages;
	CVirtualRoots *DfltVirtualRoots;
	CErrorPages *DfltErrorPages;
	CSSIFiles *DfltSSIFiles;
	CIPFilters *DfltIPFilters;
	CCGIFolders *DfltCGIFolders;
	CCompression *DfltCompression;
	CCustomFilters *DfltCustomFilters;
	CScriptingEngines *DfltScriptingEngines;
	CAuthentication *DfltAuthentication;
	CRealms *DfltRealms;

	void PopInfoList(HWND hListView);
	void PopConfigList(HWND hWnd, int iListView);
	void PopPerformanceList(HWND hListView);
	void RefreshPerformanceList(HWND hListView);

	int GetCurrentConenctions(void);
	int GetTotalConnections(void);
	void IncrementConnectionCount(void);
	//bool Initialize(void);
	//bool Destroy(void);
	void DestroySites(void);
	bool Add(CXMLReader *xmlSite);

	void AddConfigItem(HWND hListView, CWebSite *pWebSite, int iSiteIndex);
	void AddConfigItem(HWND hListView, CWebSite *pWebSite, int iSiteIndex, int iInsertionIndex);
	bool SaveNewConfiguration(HWND hWnd, HWND hGrid);
	bool Save(void);
	bool SaveAll(void);
	bool Reload(void);
	bool ReloadSites(void);
	bool ReloadSites(CXMLReader *pWebSites);
	bool LoadSites(CXMLReader *pWebSites);
	bool LoadSites(char *sFile);

	CWebSite *Site(int iIndex);
};

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

extern CWebSites *gWebSites;

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#endif

