///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//  Copyright  NetworkDLS 2002, All rights reserved
//
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF 
// ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO 
// THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A 
// PARTICULAR PURPOSE.
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#ifndef _ConfigTree_CPP
#define _ConfigTree_CPP
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include <Windows.H>
#include <ShlObj.H>
#include <Stdio.H>
#include <Stdlib.H>

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include "../Source/Entry.H"
#include "../Source/CWebSites.H"
#include "../Source/ConfigTree.H"

#include "../Dialogs/MainDlg.H"
#include "../Dialogs/NetLogoDlg.H"

#include "../Dialogs/Config_DefaultPages.H"
#include "../Dialogs/Config_CGIFolders.H"
#include "../Dialogs/Config_URLFilters.H"
#include "../Dialogs/Config_IPFilters.H"
#include "../Dialogs/Config_ExtensionFilters.H"
#include "../Dialogs/Config_SSIFiles.H"
#include "../Dialogs/Config_CustomFilters.H"
#include "../Dialogs/Config_MimeTypes.H"
#include "../Dialogs/Config_VirtualRoots.H"
#include "../Dialogs/Config_ScriptingEngines.H"
#include "../Dialogs/Config_CompressibleFiles.H"
#include "../Dialogs/Config_ErrorPages.H"
#include "../Dialogs/Config_Settings.H"
#include "../Dialogs/Config_WebSites.H"
#include "../Dialogs/Config_WebSites_Item.H"

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

HWND ghConfigTree = NULL;
HIMAGELIST ghImageList = NULL;
WNDPROC gpOldConfigTreeProc = NULL;
CONFIGITEM gCI;

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

HTREEITEM ATI(HTREEITEM hParent, char *sINText, int iImage, int lParam);
bool PopulateConfigTree(void);
LRESULT CALLBACK ConfigTreeProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void SetConfigItem(void)
{
	gCI.iType = PAGE_TYPE_INVALID;
	gCI.iSite = INVALID_SITE_INDEX;
	gCI.iListIndex = INVALID_LIST_INDEX;
	gCI.hList = NULL;
	gCI.pClass = NULL;
	gCI.hDialog = NULL;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void SetConfigItem(int iType, void *pClass)
{
	gCI.iType = iType;
	gCI.pClass = pClass;
	gCI.iSite = INVALID_SITE_INDEX;
	gCI.hList = NULL;
	gCI.iListIndex = INVALID_LIST_INDEX;
	gCI.hDialog = NULL;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void SetConfigItem(int iType, int iSite, void *pClass)
{
	gCI.iType = iType;
	gCI.pClass = pClass;
	gCI.iSite = iSite;
	gCI.hList = NULL;
	gCI.iListIndex = INVALID_LIST_INDEX;
	gCI.hDialog = NULL;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void SetConfigItem(int iType, int iSite, void *pClass, HWND hList, int iListIndex)
{
	gCI.iType = iType;
	gCI.pClass = pClass;
	gCI.iSite = iSite;
	gCI.hList = hList;
	gCI.iListIndex = iListIndex;
	gCI.hDialog = NULL;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void ConfigureTreeItem(HWND hList, int iSiteIndex, int iPageType)
{
	bool bRefresh = false;
	if(iSiteIndex == INVALID_SITE_INDEX)
	{
		//Global Settings...
		if(iPageType == PAGE_TYPE_WEBSITES)
		{
			SetConfigItem(iPageType, gWebSites);
			bRefresh = DialogBox(ghAppInstance, MAKEINTRESOURCE(IDD_CONFIG_GENERICLIST), hMainDialog, Config_WebSites) > 0;
			if(bRefresh)
			{
				SendMessage(GetParent(hList), WM_REFRESH_CONFIG_TREE_ITEM, 0, 0);
			}
		}
		else if(iPageType == PAGE_TYPE_SETTINGS)
		{
			SetConfigItem(iPageType, gWebSites->DfltSettings);
			bRefresh = DialogBox(ghAppInstance, MAKEINTRESOURCE(IDD_CONFIG_SETTINGSLIST), hMainDialog, Config_Settings) > 0;
		}
		else if(iPageType == PAGE_TYPE_DEFAULTPAGES)
		{
			SetConfigItem(iPageType, gWebSites->DfltDefaultPages);
			bRefresh = DialogBox(ghAppInstance, MAKEINTRESOURCE(IDD_CONFIG_GENERICLIST), hMainDialog, Config_DefaultPages) > 0;
		}
		else if(iPageType == PAGE_TYPE_VIRTUAL_ROOTS)
		{
			SetConfigItem(iPageType, gWebSites->DfltVirtualRoots);
			bRefresh = DialogBox(ghAppInstance, MAKEINTRESOURCE(IDD_CONFIG_GENERICLIST), hMainDialog, Config_VirtualRoots) > 0;
		}
		else if(iPageType == PAGE_TYPE_COMPRESSION)
		{
			SetConfigItem(iPageType, gWebSites->DfltCompression);
			bRefresh = DialogBox(ghAppInstance, MAKEINTRESOURCE(IDD_CONFIG_COMPRESSIBLEFILESLIST), hMainDialog, Config_CompressibleFiles) > 0;
		}
		else if(iPageType == PAGE_TYPE_MIMETYPES)
		{
			SetConfigItem(iPageType, gWebSites->DfltMimeTypes);
			bRefresh = DialogBox(ghAppInstance, MAKEINTRESOURCE(IDD_CONFIG_GENERICLIST), hMainDialog, Config_MimeTypes) > 0;
		}
		else if(iPageType == PAGE_TYPE_IPFILTERS)
		{
			SetConfigItem(iPageType, gWebSites->DfltIPFilters);
			bRefresh = DialogBox(ghAppInstance, MAKEINTRESOURCE(IDD_CONFIG_GENERICLIST), hMainDialog, Config_IPFilters) > 0;
		}
		else if(iPageType == PAGE_TYPE_URLFILTERS)
		{
			SetConfigItem(iPageType, gWebSites->DfltURLFilters);
			bRefresh = DialogBox(ghAppInstance, MAKEINTRESOURCE(IDD_CONFIG_GENERICLIST), hMainDialog, Config_URLFilters) > 0;
		}
		else if(iPageType == PAGE_TYPE_EXTFILTERS)
		{
			SetConfigItem(iPageType, gWebSites->DfltExtensionFilters);
			bRefresh = DialogBox(ghAppInstance, MAKEINTRESOURCE(IDD_CONFIG_GENERICLIST), hMainDialog, Config_ExtensionFilters) > 0;
		}
		else if(iPageType == PAGE_TYPE_CUSTOMFILTERS)
		{
			SetConfigItem(iPageType, gWebSites->DfltCustomFilters);
			bRefresh = DialogBox(ghAppInstance, MAKEINTRESOURCE(IDD_CONFIG_GENERICLIST), hMainDialog, Config_CustomFilters) > 0;
		}
		else if(iPageType == PAGE_TYPE_SCRIPTINGENGINES)
		{
			SetConfigItem(iPageType, gWebSites->DfltScriptingEngines);
			bRefresh = DialogBox(ghAppInstance, MAKEINTRESOURCE(IDD_CONFIG_GENERICLIST), hMainDialog, Config_ScriptingEngines) > 0;
		}
		else if(iPageType == PAGE_TYPE_REALMS)
		{
			//SetConfigItem(iPageType, gWebSites->DfltRealms);
			//bRefresh = DialogBox(ghAppInstance, MAKEINTRESOURCE(IDD_CONFIG_GENERICLIST), hMainDialog, Config_Realms) > 0;
		}
		else if(iPageType == PAGE_TYPE_AUTHENTICATION)
		{
			//SetConfigItem(iPageType, gWebSites->DfltAuthentication);
			//bRefresh = DialogBox(ghAppInstance, MAKEINTRESOURCE(IDD_CONFIG_GENERICLIST), hMainDialog, Config_Authentication) > 0;
		}
		else if(iPageType == PAGE_TYPE_SSIFILES)
		{
			SetConfigItem(iPageType, gWebSites->DfltSSIFiles);
			bRefresh = DialogBox(ghAppInstance, MAKEINTRESOURCE(IDD_CONFIG_GENERICLIST), hMainDialog, Config_SSIFiles) > 0;
		}
		else if(iPageType == PAGE_TYPE_CGIFOLDERS)
		{
			SetConfigItem(iPageType, gWebSites->DfltCGIFolders);
			bRefresh = DialogBox(ghAppInstance, MAKEINTRESOURCE(IDD_CONFIG_GENERICLIST), hMainDialog, Config_CGIFolders) > 0;
		}
		else if(iPageType == PAGE_TYPE_ERRORPAGES)
		{
			SetConfigItem(iPageType, gWebSites->DfltErrorPages);
			bRefresh = DialogBox(ghAppInstance, MAKEINTRESOURCE(IDD_CONFIG_ERRORPAGESLIST), hMainDialog, Config_ErrorPages) > 0;
		}
		else{
			//MsgBox("Invalid Config Item!");
		}
	}
	else{
		//Site Settings...
		if(iPageType == PAGE_TYPE_WEBSITE)
		{
			SetConfigItem(iPageType, iSiteIndex, gWebSites);

			bRefresh = DialogBox(ghAppInstance,
				MAKEINTRESOURCE(IDD_CONFIG_WEBSITES_ITEM),
				hList, Config_WebSites_Item) > 0;

			if(bRefresh)
			{
				SendMessage(GetParent(hList), WM_REFRESH_CONFIG_TREE_ITEM, 0, 0);
			}
		}
		else if(iPageType == PAGE_TYPE_SETTINGS)
		{
			SetConfigItem(iPageType, iSiteIndex, gWebSites->Collection.WebSite[iSiteIndex]->pSettings);
			bRefresh = DialogBox(ghAppInstance, MAKEINTRESOURCE(IDD_CONFIG_SETTINGSLIST), hMainDialog, Config_Settings) > 0;
		}
		else if(iPageType == PAGE_TYPE_DEFAULTPAGES)
		{
			SetConfigItem(iPageType, iSiteIndex, gWebSites->Collection.WebSite[iSiteIndex]->pDefaultPages);
			bRefresh = DialogBox(ghAppInstance, MAKEINTRESOURCE(IDD_CONFIG_GENERICLIST), hMainDialog, Config_DefaultPages) > 0;
		}
		else if(iPageType == PAGE_TYPE_VIRTUAL_ROOTS)
		{
			SetConfigItem(iPageType, iSiteIndex, gWebSites->Collection.WebSite[iSiteIndex]->pVirtualRoots);
			bRefresh = DialogBox(ghAppInstance, MAKEINTRESOURCE(IDD_CONFIG_GENERICLIST), hMainDialog, Config_VirtualRoots) > 0;
		}
		else if(iPageType == PAGE_TYPE_COMPRESSION)
		{
			SetConfigItem(iPageType, iSiteIndex, gWebSites->Collection.WebSite[iSiteIndex]->pCompression);
			bRefresh = DialogBox(ghAppInstance, MAKEINTRESOURCE(IDD_CONFIG_COMPRESSIBLEFILESLIST), hMainDialog, Config_CompressibleFiles) > 0;
		}
		else if(iPageType == PAGE_TYPE_MIMETYPES)
		{
			SetConfigItem(iPageType, iSiteIndex, gWebSites->Collection.WebSite[iSiteIndex]->pMimeTypes);
			bRefresh = DialogBox(ghAppInstance, MAKEINTRESOURCE(IDD_CONFIG_GENERICLIST), hMainDialog, Config_MimeTypes) > 0;
		}
		else if(iPageType == PAGE_TYPE_IPFILTERS)
		{
			SetConfigItem(iPageType, iSiteIndex, gWebSites->Collection.WebSite[iSiteIndex]->pIPFilters);
			bRefresh = DialogBox(ghAppInstance, MAKEINTRESOURCE(IDD_CONFIG_GENERICLIST), hMainDialog, Config_IPFilters) > 0;
		}
		else if(iPageType == PAGE_TYPE_URLFILTERS)
		{
			SetConfigItem(iPageType, iSiteIndex, gWebSites->Collection.WebSite[iSiteIndex]->pURLFilters);
			bRefresh = DialogBox(ghAppInstance, MAKEINTRESOURCE(IDD_CONFIG_GENERICLIST), hMainDialog, Config_URLFilters) > 0;
		}
		else if(iPageType == PAGE_TYPE_EXTFILTERS)
		{
			SetConfigItem(iPageType, iSiteIndex, gWebSites->Collection.WebSite[iSiteIndex]->pExtensionFilters);
			bRefresh = DialogBox(ghAppInstance, MAKEINTRESOURCE(IDD_CONFIG_GENERICLIST), hMainDialog, Config_ExtensionFilters) > 0;
		}
		else if(iPageType == PAGE_TYPE_CUSTOMFILTERS)
		{
			SetConfigItem(iPageType, iSiteIndex, gWebSites->Collection.WebSite[iSiteIndex]->pCustomFilters);
			bRefresh = DialogBox(ghAppInstance, MAKEINTRESOURCE(IDD_CONFIG_GENERICLIST), hMainDialog, Config_CustomFilters) > 0;
		}
		else if(iPageType == PAGE_TYPE_SCRIPTINGENGINES)
		{
			SetConfigItem(iPageType, iSiteIndex, gWebSites->Collection.WebSite[iSiteIndex]->pScriptingEngines);
			bRefresh = DialogBox(ghAppInstance, MAKEINTRESOURCE(IDD_CONFIG_GENERICLIST), hMainDialog, Config_ScriptingEngines) > 0;
		}
		else if(iPageType == PAGE_TYPE_REALMS)
		{
			//SetConfigItem(iPageType, iSiteIndex, gWebSites->Collection.WebSite[iSiteIndex]->pRealms);
			//bRefresh = DialogBox(ghAppInstance, MAKEINTRESOURCE(IDD_CONFIG_GENERICLIST), hMainDialog, Config_Realms) > 0;
		}
		else if(iPageType == PAGE_TYPE_AUTHENTICATION)
		{
			//SetConfigItem(iPageType, iSiteIndex, gWebSites->Collection.WebSite[iSiteIndex]->pAuthentication);
			//bRefresh = DialogBox(ghAppInstance, MAKEINTRESOURCE(IDD_CONFIG_GENERICLIST), hMainDialog, Config_Authentication) > 0;
		}
		else if(iPageType == PAGE_TYPE_SSIFILES)
		{
			SetConfigItem(iPageType, iSiteIndex, gWebSites->Collection.WebSite[iSiteIndex]->pSSIFiles);
			bRefresh = DialogBox(ghAppInstance, MAKEINTRESOURCE(IDD_CONFIG_GENERICLIST), hMainDialog, Config_SSIFiles) > 0;
		}
		else if(iPageType == PAGE_TYPE_CGIFOLDERS)
		{
			SetConfigItem(iPageType, iSiteIndex, gWebSites->Collection.WebSite[iSiteIndex]->pCGIFolders);
			bRefresh = DialogBox(ghAppInstance, MAKEINTRESOURCE(IDD_CONFIG_GENERICLIST), hMainDialog, Config_CGIFolders) > 0;
		}
		else if(iPageType == PAGE_TYPE_ERRORPAGES)
		{
			SetConfigItem(iPageType, iSiteIndex, gWebSites->Collection.WebSite[iSiteIndex]->pErrorPages);
			bRefresh = DialogBox(ghAppInstance, MAKEINTRESOURCE(IDD_CONFIG_ERRORPAGESLIST), hMainDialog, Config_ErrorPages) > 0;
		}
		else{
			//MsgBox("Invalid Config Item!");
		}
	}

	if(bRefresh)
	{
		gWebSites->Save(); //Rewrite all web sites.
		SendMessage(GetParent(hList), WM_REFRESH_SELECTED_CONFIG_ITEM, 0, 0);
	}
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

LRESULT CALLBACK ConfigTreeProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	if(uMsg == WM_LBUTTONDBLCLK)
	{
		SendMessage(hWnd, WM_COMMAND, TREE_CMD_CONFIGURE, 0);
		return 0; //Do not expand the tree items on double click.
    }

	LRESULT lResult = CallWindowProc(gpOldConfigTreeProc, hWnd, uMsg, wParam, lParam);

	static int iPageType = -1;
	static int iSiteIndex = -1;

	if(uMsg == WM_COMMAND)
	{
		if(GetSelectedConfigItem(&iPageType, &iSiteIndex))
		{
			bool bResult = false;

			if(LOWORD(wParam) == TREE_CMD_CONFIGURE)
			{
				ConfigureTreeItem(hWnd, iSiteIndex, iPageType);
			}
			else if(LOWORD(wParam) == TREE_CMD_RESTARTSITES)
			{
				if(MessageBox(hWnd,
					"Are you sure you want to reload all of the webSites?",
					gsTitleCaption, MB_ICONQUESTION | MB_YESNO | MB_DEFBUTTON2) == IDYES)
				{
					gWebSites->StartTrace();
					bResult = gWebSites->ReloadSites();
					bResult = bResult && !gWebSites->StopTrace(hWnd);
					SendMessage(GetParent(hWnd), WM_REFRESH_CONFIG_TREE_ITEM, 0, 0);
					if(bResult)
					{
						MessageBox(hWnd, "The sites were successfully reloaded!", gsTitleCaption, MB_ICONINFORMATION);
					}
				}
			}
			else if(LOWORD(wParam) == TREE_CMD_RESTARTSERVER)
			{
				if(MessageBox(hWnd,
					"Are you sure you want to reload the webserver?",
					gsTitleCaption, MB_ICONQUESTION | MB_YESNO | MB_DEFBUTTON2) == IDYES)
				{
					gWebSites->StartTrace();
					bResult = gWebSites->Reload();
					bResult = bResult && !gWebSites->StopTrace(hWnd);
					SendMessage(GetParent(hWnd), WM_REFRESH_CONFIG_TREE_ITEM, 0, 0);
					if(bResult)
					{
						MessageBox(hWnd, "The webserver was successfully reloaded!", gsTitleCaption, MB_ICONINFORMATION);
					}
				}
			}
			else if(LOWORD(wParam) == TREE_CMD_START_SITE)
			{
				gWebSites->StartTrace();
				if(gWebSites->Site(iSiteIndex)->Start())
				{
					TV_ITEM TVItem;
					memset(&TVItem, 0, sizeof(TVItem));

					if((TVItem.hItem = TreeView_GetSelection(hWnd)))
					{
						TVItem.mask = TVIF_IMAGE|TVIF_SELECTEDIMAGE;
						TVItem.iImage = TREE_IMAGE_SITE_ONLINE;
						TVItem.iSelectedImage = TREE_IMAGE_SITE_ONLINE;
						TreeView_SetItem(hWnd, &TVItem);
					}
				}
				if(!gWebSites->StopTrace(hWnd))
				{
					MessageBox(hWnd, "The site was successfully started!", gsTitleCaption, MB_ICONINFORMATION);
				}
			}
			else if(LOWORD(wParam) == TREE_CMD_STOP_SITE)
			{
				if(MessageBox(hWnd,
					"Are you sure you want to stop this site?",
					gsTitleCaption, MB_ICONQUESTION | MB_YESNO | MB_DEFBUTTON2) == IDYES)
				{
					gWebSites->StartTrace();

					if(gWebSites->Site(iSiteIndex)->Stop())
					{
						TV_ITEM TVItem;
						memset(&TVItem, 0, sizeof(TVItem));

						if((TVItem.hItem = TreeView_GetSelection(hWnd)))
						{
							TVItem.mask = TVIF_IMAGE|TVIF_SELECTEDIMAGE;
							TVItem.iImage = TREE_IMAGE_SITE_OFFLINE;
							TVItem.iSelectedImage = TREE_IMAGE_SITE_OFFLINE;
							TreeView_SetItem(hWnd, &TVItem);
						}
					}
					if(!gWebSites->StopTrace(hWnd))
					{
						MessageBox(hWnd, "The site was successfully stopped!", gsTitleCaption, MB_ICONINFORMATION);
					}
				}
			}
			else if(LOWORD(wParam) == TREE_CMD_RESTART_SITE)
			{
				if(MessageBox(hWnd,
					"Are you sure you want to restart this site?",
					gsTitleCaption, MB_ICONQUESTION | MB_YESNO | MB_DEFBUTTON2) == IDYES)
				{
					gWebSites->StartTrace();

					if(gWebSites->Site(iSiteIndex)->Stop())
					{
						TV_ITEM TVItem;
						memset(&TVItem, 0, sizeof(TVItem));

						if((TVItem.hItem = TreeView_GetSelection(hWnd)))
						{
							TVItem.mask = TVIF_IMAGE|TVIF_SELECTEDIMAGE;
							TVItem.iImage = TREE_IMAGE_SITE_OFFLINE;
							TVItem.iSelectedImage = TREE_IMAGE_SITE_OFFLINE;
							TreeView_SetItem(hWnd, &TVItem);
						}

						if(gWebSites->Site(iSiteIndex)->Start())
						{
							TV_ITEM TVItem;
							memset(&TVItem, 0, sizeof(TVItem));

							if((TVItem.hItem = TreeView_GetSelection(hWnd)))
							{
								TVItem.mask = TVIF_IMAGE|TVIF_SELECTEDIMAGE;
								TVItem.iImage = TREE_IMAGE_SITE_ONLINE;
								TVItem.iSelectedImage = TREE_IMAGE_SITE_ONLINE;
								TreeView_SetItem(hWnd, &TVItem);
							}
						}
					}
					if(!gWebSites->StopTrace(hWnd))
					{
						MessageBox(hWnd, "The site was successfully restarted!", gsTitleCaption, MB_ICONINFORMATION);
					}
				}
			}
			else if(LOWORD(wParam) == TREE_CMD_VIEW_SITE)
			{
				char sURL[100];
				sprintf_s(sURL, sizeof(sURL), "http://localhost");

				if(gWebSites->Site(iSiteIndex)->Port != 80)
				{
					char sPort[64];
					sprintf_s(sPort, sizeof(sPort), ":%d", gWebSites->Site(iSiteIndex)->Port);
					strcat_s(sURL, sizeof(sURL), sPort);
				}

				strcat_s(sURL, sizeof(sURL), "/");

				ShellExecute(hWnd, "open", sURL, NULL, NULL, SW_NORMAL);
			}
		}
	}
	else if(uMsg == WM_RBUTTONDOWN)
	{
		//Perform a hit-test to find the item that was clicked.
		TVHITTESTINFO tvHit;
		memset(&tvHit, 0, sizeof(tvHit));
		tvHit.pt.x=LOWORD(GetMessagePos());
		tvHit.pt.y=HIWORD(GetMessagePos());
		ScreenToClient(hWnd, &tvHit.pt);
		HTREEITEM hSelectedItem = TreeView_HitTest(hWnd, &tvHit);

		//Select the item that was clicked.
		TreeView_SelectItem(hWnd, hSelectedItem);
		TreeView_EnsureVisible(hWnd, hSelectedItem);

		if(GetSelectedConfigItem(&iPageType, &iSiteIndex))
		{
			if((iSiteIndex != INVALID_SITE_INDEX && iPageType == PAGE_TYPE_WEBSITE)
				|| (iSiteIndex == INVALID_SITE_INDEX &&
				(iPageType == PAGE_TYPE_WEBSITES || iPageType == PAGE_TYPE_SERVER)))
			{
				HMENU hParent = CreateMenu();
				HMENU hChild = CreateMenu();
				HMENU hSelect = CreateMenu();

				if(iPageType == PAGE_TYPE_WEBSITE)
				{
					if(gWebSites->Site(iSiteIndex)->IsActive())
					{
						AppendMenu(hChild, MF_STRING, TREE_CMD_STOP_SITE, "Stop");
						AppendMenu(hChild, MF_STRING, TREE_CMD_RESTART_SITE, "Restart");
						AppendMenu(hChild, MF_SEPARATOR, TREE_CMD_SEPERATOR, "");
						AppendMenu(hChild, MF_STRING, TREE_CMD_VIEW_SITE, "View Site");
					}
					else{
						AppendMenu(hChild, MF_STRING, TREE_CMD_START_SITE, "Start");
						AppendMenu(hChild, MF_SEPARATOR, TREE_CMD_SEPERATOR, "");
					}

					AppendMenu(hChild, MF_STRING, TREE_CMD_CONFIGURE, "Configure");
				}
				else if(iPageType == PAGE_TYPE_SERVER)
				{
					AppendMenu(hChild, MF_STRING, TREE_CMD_RESTARTSERVER, "Reload Server");
				}
				else if(iPageType == PAGE_TYPE_WEBSITES)
				{
					AppendMenu(hChild, MF_STRING, TREE_CMD_RESTARTSITES, "Reload Sites");
					AppendMenu(hChild, MF_STRING, TREE_CMD_CONFIGURE, "Configure");
				}

				InsertMenu(hParent, 0, MF_POPUP|MF_BYPOSITION, (UINT)hChild, "HTTP WebSites");

				POINT xy;
				GetCursorPos(&xy);
				TrackPopupMenuEx(hChild, TPM_LEFTALIGN|TPM_RIGHTBUTTON, (xy.x) + 1, xy.y, hWnd, NULL);
				DestroyMenu(hChild);
				DestroyMenu(hParent);
			}
			else{
				if(iPageType == PAGE_TYPE_SETTINGS
					|| iPageType == PAGE_TYPE_DEFAULTPAGES
					|| iPageType == PAGE_TYPE_VIRTUAL_ROOTS
					|| iPageType == PAGE_TYPE_COMPRESSION
					|| iPageType == PAGE_TYPE_MIMETYPES
					|| iPageType == PAGE_TYPE_IPFILTERS
					|| iPageType == PAGE_TYPE_URLFILTERS
					|| iPageType == PAGE_TYPE_EXTFILTERS
					|| iPageType == PAGE_TYPE_CUSTOMFILTERS
					|| iPageType == PAGE_TYPE_SCRIPTINGENGINES
					|| iPageType == PAGE_TYPE_SSIFILES
					|| iPageType == PAGE_TYPE_CGIFOLDERS
					|| iPageType == PAGE_TYPE_ERRORPAGES
					|| iPageType == PAGE_TYPE_WEBSITES
					|| iPageType == PAGE_TYPE_WEBSITE)
				{
					HMENU hParent = CreateMenu();
					HMENU hChild = CreateMenu();
					HMENU hSelect = CreateMenu();

					AppendMenu(hChild, MF_STRING, TREE_CMD_CONFIGURE, "Configure");
					InsertMenu(hParent, 0, MF_POPUP|MF_BYPOSITION, (UINT)hChild, "Attribute");

					POINT xy;
					GetCursorPos(&xy);
					TrackPopupMenuEx(hChild, TPM_LEFTALIGN|TPM_RIGHTBUTTON, (xy.x) + 1, xy.y, hWnd, NULL);
					DestroyMenu(hChild);
					DestroyMenu(hParent);
				}
			}
		}
	}

	//--------------------------------------------------------------------------

	return lResult;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool InitializeConfigTree(HWND hTV)
{
	ghConfigTree = hTV;

	TreeView_DeleteAllItems(ghConfigTree);
	if(!ghImageList)
	{
		ghImageList = ImageList_Create(16, 16, ILC_COLOR24, 10, 1);

		HBITMAP hBitMap = NULL;

		hBitMap = LoadBitmap(ghAppInstance, MAKEINTRESOURCE(IDB_TREE_SERVER));
		ImageList_Add(ghImageList, hBitMap, hBitMap);
		DeleteObject(hBitMap);
		hBitMap = LoadBitmap(ghAppInstance, MAKEINTRESOURCE(IDB_TREE_SITESTARTED));
		ImageList_Add(ghImageList, hBitMap, hBitMap);
		DeleteObject(hBitMap);
		hBitMap = LoadBitmap(ghAppInstance, MAKEINTRESOURCE(IDB_TREE_SITESTOPPED));
		ImageList_Add(ghImageList, hBitMap, hBitMap);
		DeleteObject(hBitMap);
		hBitMap = LoadBitmap(ghAppInstance, MAKEINTRESOURCE(IDB_TREE_CONFIG_SETTINGS));
		ImageList_Add(ghImageList, hBitMap, hBitMap);
		DeleteObject(hBitMap);
		hBitMap = LoadBitmap(ghAppInstance, MAKEINTRESOURCE(IDB_TREE_CONFIG_FILTERS));
		ImageList_Add(ghImageList, hBitMap, hBitMap);
		DeleteObject(hBitMap);
		hBitMap = LoadBitmap(ghAppInstance, MAKEINTRESOURCE(IDB_TREE_CONFIG_SCRIPTING));
		ImageList_Add(ghImageList, hBitMap, hBitMap);
		DeleteObject(hBitMap);
		hBitMap = LoadBitmap(ghAppInstance, MAKEINTRESOURCE(IDB_TREE_CONFIG_SCRIPTINGENGINES));
		ImageList_Add(ghImageList, hBitMap, hBitMap);
		DeleteObject(hBitMap);
		hBitMap = LoadBitmap(ghAppInstance, MAKEINTRESOURCE(IDB_TREE_CONFIG_WEBSITES));
		ImageList_Add(ghImageList, hBitMap, hBitMap);
		DeleteObject(hBitMap);
		hBitMap = LoadBitmap(ghAppInstance, MAKEINTRESOURCE(IDB_TREE_CONFIG_CACHE));
		ImageList_Add(ghImageList, hBitMap, hBitMap);
		DeleteObject(hBitMap);
		hBitMap = LoadBitmap(ghAppInstance, MAKEINTRESOURCE(IDB_TREE_CONFIG_CGIFOLDERS));
		ImageList_Add(ghImageList, hBitMap, hBitMap);
		DeleteObject(hBitMap);
		hBitMap = LoadBitmap(ghAppInstance, MAKEINTRESOURCE(IDB_TREE_CONFIG_CUSTOMFILTERS));
		ImageList_Add(ghImageList, hBitMap, hBitMap);
		DeleteObject(hBitMap);
		hBitMap = LoadBitmap(ghAppInstance, MAKEINTRESOURCE(IDB_TREE_CONFIG_DEFAULTPAGES));
		ImageList_Add(ghImageList, hBitMap, hBitMap);
		DeleteObject(hBitMap);
		hBitMap = LoadBitmap(ghAppInstance, MAKEINTRESOURCE(IDB_TREE_CONFIG_EXTENSIONFILTERS));
		ImageList_Add(ghImageList, hBitMap, hBitMap);
		DeleteObject(hBitMap);
		hBitMap = LoadBitmap(ghAppInstance, MAKEINTRESOURCE(IDB_TREE_CONFIG_IPFILTERS));
		ImageList_Add(ghImageList, hBitMap, hBitMap);
		DeleteObject(hBitMap);
		hBitMap = LoadBitmap(ghAppInstance, MAKEINTRESOURCE(IDB_TREE_CONFIG_MIMETYPES));
		ImageList_Add(ghImageList, hBitMap, hBitMap);
		DeleteObject(hBitMap);
		hBitMap = LoadBitmap(ghAppInstance, MAKEINTRESOURCE(IDB_TREE_CONFIG_SSIFILES));
		ImageList_Add(ghImageList, hBitMap, hBitMap);
		DeleteObject(hBitMap);
		hBitMap = LoadBitmap(ghAppInstance, MAKEINTRESOURCE(IDB_TREE_CONFIG_URLFILTERS));
		ImageList_Add(ghImageList, hBitMap, hBitMap);
		DeleteObject(hBitMap);
		hBitMap = LoadBitmap(ghAppInstance, MAKEINTRESOURCE(IDB_TREE_CONFIG_VIRTUALROOTS));
		ImageList_Add(ghImageList, hBitMap, hBitMap);
		DeleteObject(hBitMap);
		hBitMap = LoadBitmap(ghAppInstance, MAKEINTRESOURCE(IDB_TREE_CONFIG_COMPRESSION));
		ImageList_Add(ghImageList, hBitMap, hBitMap);
		DeleteObject(hBitMap);
		hBitMap = LoadBitmap(ghAppInstance, MAKEINTRESOURCE(IDB_TREE_CONFIG_ERRORPAGES));
		ImageList_Add(ghImageList, hBitMap, hBitMap);
		DeleteObject(hBitMap);
		hBitMap = LoadBitmap(ghAppInstance, MAKEINTRESOURCE(IDB_TREE_CONFIG_REALMS));
		ImageList_Add(ghImageList, hBitMap, hBitMap);
		DeleteObject(hBitMap);
		hBitMap = LoadBitmap(ghAppInstance, MAKEINTRESOURCE(IDB_TREE_CONFIG_AUTHENTICATION));
		ImageList_Add(ghImageList, hBitMap, hBitMap);
		DeleteObject(hBitMap);
	}

	TreeView_SetImageList(ghConfigTree, ghImageList, TVSIL_NORMAL);

	bool bResult = PopulateConfigTree();

	gpOldConfigTreeProc = (WNDPROC) SetWindowLongPtr(ghConfigTree, GWLP_WNDPROC, (LRESULT) &ConfigTreeProc);

	return bResult;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool RefreshConfigTree(void)
{
	SetWindowLongPtr(ghConfigTree, GWLP_WNDPROC, (LRESULT) gpOldConfigTreeProc);

	bool bResult = InitializeConfigTree(ghConfigTree);

	return bResult;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//ATI: AddTreeItem
HTREEITEM ATI(HTREEITEM hParent, char *sINText, int iImage, int lParam)
{
    TV_INSERTSTRUCT MyTVIS;
	memset(&MyTVIS, 0, sizeof(MyTVIS));
    MyTVIS.item.hItem          = hParent;
    MyTVIS.item.pszText        = sINText;
    MyTVIS.item.iImage         = iImage;
    MyTVIS.item.iSelectedImage = iImage;
    MyTVIS.item.mask           = TVIF_TEXT|TVIF_SELECTEDIMAGE|TVIF_IMAGE|TVIF_PARAM;

    MyTVIS.hParent             = hParent;
	MyTVIS.item.lParam		   = lParam;
    MyTVIS.item.cchTextMax     = (int)strlen(sINText);

    return TreeView_InsertItem(ghConfigTree, &MyTVIS);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool PopulateConfigTree(void)
{
	TreeView_DeleteAllItems(ghConfigTree);

	HTREEITEM hRoot = ATI(TVI_ROOT, "(Local Server)", TREE_IMAGE_SERVER, MAKELONG(INVALID_SITE_INDEX, PAGE_TYPE_SERVER));
	HTREEITEM hSites = ATI(hRoot, "Web Sites", TREE_IMAGE_WEBSITES, MAKELONG(INVALID_SITE_INDEX, PAGE_TYPE_WEBSITES));
	HTREEITEM hTI = NULL;

	//MAKELONG("SITE INDEX", "PAGE TYPE")

	for(int iSite = 0; iSite < gWebSites->Collection.Count; iSite++)
	{
		HTREEITEM hSite = NULL;

		if(gWebSites->Site(iSite)->IsActive())
		{
			hSite = ATI(hSites, gWebSites->Site(iSite)->Name, TREE_IMAGE_SITE_ONLINE, MAKELONG(iSite, PAGE_TYPE_WEBSITE));
		}
		else{
			hSite = ATI(hSites, gWebSites->Site(iSite)->Name, TREE_IMAGE_SITE_OFFLINE, MAKELONG(iSite, PAGE_TYPE_WEBSITE));
		}

		//Per site settings.
		hTI = ATI(hSite, "Advanced", TREE_IMAGE_SETTINGS, MAKELONG(iSite, PAGE_TYPE_SETTINGS));
		ATI(hTI, "Default Pages", TREE_IMAGE_DEFAULTPAGES, MAKELONG(iSite, PAGE_TYPE_DEFAULTPAGES));
		ATI(hTI, "Virtual Roots", TREE_IMAGE_VIRTUALROOTS, MAKELONG(iSite, PAGE_TYPE_VIRTUAL_ROOTS));
		ATI(hTI, "Compression", TREE_IMAGE_COMPRESSION, MAKELONG(iSite, PAGE_TYPE_COMPRESSION));
		ATI(hTI, "Mime Types", TREE_IMAGE_MIMETYPES, MAKELONG(iSite, PAGE_TYPE_MIMETYPES));
		//ATI(hTI, "Realms", TREE_IMAGE_REALMS, MAKELONG(iSite, PAGE_TYPE_REALMS));
		//ATI(hTI, "Authentication", TREE_IMAGE_AUTHENTICATION, MAKELONG(iSite, PAGE_TYPE_AUTHENTICATION));

		//Per site filters.
		hTI = ATI(hSite, "Filters", TREE_IMAGE_FILTERS, MAKELONG(iSite, PAGE_TYPE_INVALID));
		ATI(hTI, "IP Filters", TREE_IMAGE_IPFILTERS, MAKELONG(iSite, PAGE_TYPE_IPFILTERS));
		ATI(hTI, "URL Filters", TREE_IMAGE_URLFILTERS, MAKELONG(iSite, PAGE_TYPE_URLFILTERS));
		ATI(hTI, "Extension Filters", TREE_IMAGE_EXTENSIONFILTERS, MAKELONG(iSite, PAGE_TYPE_EXTFILTERS));
		ATI(hTI, "Custom Filters", TREE_IMAGE_CUSTOMFILTERS, MAKELONG(iSite, PAGE_TYPE_CUSTOMFILTERS));

		//Per site scripting.
		hTI = ATI(hSite, "Scripting", TREE_IMAGE_SCRIPTING, MAKELONG(iSite, PAGE_TYPE_INVALID));
		ATI(hTI, "Scripting Engines", TREE_IMAGE_SCRIPTINGENGINES, MAKELONG(iSite, PAGE_TYPE_SCRIPTINGENGINES));
		ATI(hTI, "SSI Files", TREE_IMAGE_SSIFILES, MAKELONG(iSite, PAGE_TYPE_SSIFILES));
		ATI(hTI, "CGI Folders", TREE_IMAGE_CGIFOLDERS, MAKELONG(iSite, PAGE_TYPE_CGIFOLDERS));

		//Per site cache.
		hTI = ATI(hSite, "Cache", TREE_IMAGE_CACHE, MAKELONG(iSite, PAGE_TYPE_CACHE));
	}

	//Global settings.
	hTI = ATI(hRoot, "Global Settings", TREE_IMAGE_SETTINGS, MAKELONG(INVALID_SITE_INDEX, PAGE_TYPE_SETTINGS));
	ATI(hTI, "Default Pages", TREE_IMAGE_DEFAULTPAGES, MAKELONG(INVALID_SITE_INDEX, PAGE_TYPE_DEFAULTPAGES));
	ATI(hTI, "Virtual Roots", TREE_IMAGE_VIRTUALROOTS, MAKELONG(INVALID_SITE_INDEX, PAGE_TYPE_VIRTUAL_ROOTS));
	ATI(hTI, "Compression", TREE_IMAGE_COMPRESSION, MAKELONG(INVALID_SITE_INDEX, PAGE_TYPE_COMPRESSION));
	ATI(hTI, "Mime Types", TREE_IMAGE_MIMETYPES, MAKELONG(INVALID_SITE_INDEX, PAGE_TYPE_MIMETYPES));
	ATI(hTI, "Error Pages", TREE_IMAGE_ERRORPAGES, MAKELONG(INVALID_SITE_INDEX, PAGE_TYPE_ERRORPAGES));
	//ATI(hTI, "Realms", TREE_IMAGE_REALMS, MAKELONG(INVALID_SITE_INDEX, PAGE_TYPE_REALMS));
	//ATI(hTI, "Authentication", TREE_IMAGE_AUTHENTICATION, MAKELONG(INVALID_SITE_INDEX, PAGE_TYPE_AUTHENTICATION));

	//Global filters.
	hTI = ATI(hRoot, "Global Filters", TREE_IMAGE_FILTERS, MAKELONG(INVALID_SITE_INDEX, PAGE_TYPE_INVALID));
	ATI(hTI, "IP Filters", TREE_IMAGE_IPFILTERS, MAKELONG(INVALID_SITE_INDEX, PAGE_TYPE_IPFILTERS));
	ATI(hTI, "URL Filters", TREE_IMAGE_URLFILTERS, MAKELONG(INVALID_SITE_INDEX, PAGE_TYPE_URLFILTERS));
	ATI(hTI, "Extension Filters", TREE_IMAGE_EXTENSIONFILTERS, MAKELONG(INVALID_SITE_INDEX, PAGE_TYPE_EXTFILTERS));
	ATI(hTI, "Custom Filters", TREE_IMAGE_CUSTOMFILTERS, MAKELONG(INVALID_SITE_INDEX, PAGE_TYPE_CUSTOMFILTERS));

	//Global scripting.
	hTI = ATI(hRoot, "Global Scripting", TREE_IMAGE_SCRIPTING, MAKELONG(INVALID_SITE_INDEX, PAGE_TYPE_INVALID));
	ATI(hTI, "Scripting Engines", TREE_IMAGE_SCRIPTINGENGINES, MAKELONG(INVALID_SITE_INDEX, PAGE_TYPE_SCRIPTINGENGINES));
	ATI(hTI, "SSI Files", TREE_IMAGE_SSIFILES, MAKELONG(INVALID_SITE_INDEX, PAGE_TYPE_SSIFILES));
	ATI(hTI, "CGI Folders", TREE_IMAGE_CGIFOLDERS, MAKELONG(INVALID_SITE_INDEX, PAGE_TYPE_CGIFOLDERS));

	SendMessage(ghConfigTree, TVM_EXPAND, TVE_EXPAND, (LPARAM)hRoot);
	SendMessage(ghConfigTree, TVM_EXPAND, TVE_EXPAND, (LPARAM)hSites);

	TreeView_Select(ghConfigTree, hRoot, TVGN_CARET);

	return true;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool GetSelectedConfigItem(int *iPageType, int *iSiteIndex)
{
	*iPageType = -1;
	*iSiteIndex = -1;

	TV_ITEM SelectedItem;
	memset(&SelectedItem, 0, sizeof(SelectedItem));

	if((SelectedItem.hItem = TreeView_GetSelection(ghConfigTree)))
	{
		SelectedItem.mask = TVIF_PARAM|TVIF_IMAGE;
		if(TreeView_GetItem(ghConfigTree, &SelectedItem))
		{
			*iPageType = HIWORD(SelectedItem.lParam);
			*iSiteIndex = LOWORD(SelectedItem.lParam);
			return true;
		}
	}
	return false;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#endif
