///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//  Copyright  NetworkDLS 2002, All rights reserved
//
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF 
// ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO 
// THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A 
// PARTICULAR PURPOSE.
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#ifndef _ENTRY_CPP
#define _ENTRY_CPP
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include <Windows.H>
#include <ShlObj.H>
#include <Stdio.H>
#include <ShlOBJ.H>
#include <Stdlib.H>

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include "Entry.H"
#include "Init.H"
#include "CWebSites.H"

#include "../Dialogs/MainDlg.H"

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

HINSTANCE ghAppInstance = NULL;

char gsAppCopyRight[64];
char gsFileVersion[10];
char gsTitleCaption[64];
char gsRegistryKey[255];

char gsPath[MAX_PATH];
char gsDataPath[MAX_PATH];

CMemPool *pMem; //The global memory object.
CService gService;	//The global service.

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool IsApplicationRunning(void)
{
	bool bResult = false;
	HANDLE hMutex = NULL;
	SECURITY_DESCRIPTOR SD;
	SECURITY_ATTRIBUTES SA;

	if(InitializeSecurityDescriptor(&SD, SECURITY_DESCRIPTOR_REVISION) == FALSE)
	{
		return false;
	}

	if(SetSecurityDescriptorDacl(&SD, TRUE, 0, FALSE) == FALSE)
	{
		return false;
	}

	SA.nLength				= sizeof(SA);
	SA.lpSecurityDescriptor = &SD;
	SA.bInheritHandle		= FALSE;

	char sMutex[255];
	sprintf_s(sMutex, "Global\\%s", gsTitleCaption);
	if((hMutex = CreateMutex(&SA, FALSE, sMutex)) != NULL)
	{
		if(GetLastError() == ERROR_ALREADY_EXISTS)
		{
			bResult = true;
		}

		ReleaseMutex(hMutex);
		CloseHandle(hMutex);
	}

	return bResult;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

int ProcessCommandLine(CCmdLine *lpCmd)
{
	for(int iArg = 0; iArg < lpCmd->iCount; iArg++)
	{
		//------------------------------------------------------------------------------------------
		if(lpCmd->IsCommand(iArg, "/NonService"))
		{
			if(IsApplicationRunning())
			{
				return 0;
			}
			return gService.RunAsStandAlone();
		}
		//------------------------------------------------------------------------------------------
		else if(lpCmd->IsCommand(iArg, "/RunService"))
		{
			if(IsApplicationRunning())
			{
				return 0;
			}
			return gService.RunAsService();
		}
		//------------------------------------------------------------------------------------------
		else if(lpCmd->IsCommand(iArg, "/Config"))
		{
			//IsServerConfigured(true);
			return 0;
		}
		//------------------------------------------------------------------------------------------
		else if(lpCmd->IsCommand(iArg, "/OpenManager")
			|| lpCmd->IsCommand(iArg, "/FlushSettings")
			|| lpCmd->IsCommand(iArg, "/ReloadSites")
			|| lpCmd->IsCommand(iArg, "/ReloadSettings"))
		{
			if(gService.IsInstalled() == _SVC_NO_)
   			{
       			MessageBox(NULL, "The service is not installed.", gsTitleCaption, MB_ICONINFORMATION);
   				return 0;
			}

			if(gService.IsActive() == _SVC_NO_)
			{
				if(IsApplicationRunning())
				{
					return 0;
				}

				if(MessageBox(NULL, "The service is not started. Would you like to start it now?",
					gsTitleCaption, MB_YESNO|MB_APPLMODAL|MB_ICONQUESTION) == IDNO)
				{
					return 0;
				}

    			if(gService.Start() != _SVC_SUCCESS_)
				{
      				return 0;
				}
				Sleep(1000);
			}

			char sEvent[1024];

			if(lpCmd->IsCommand(iArg, "/OpenManager")) {
				sprintf_s(sEvent, "Global\\%s_Manager", gsTitleCaption);
			}
			else if(lpCmd->IsCommand(iArg, "/FlushSettings")) {
				sprintf_s(sEvent, "Global\\%s_FlushSettings", gsTitleCaption);
			}
			else if(lpCmd->IsCommand(iArg, "/ReloadSites")) {
				sprintf_s(sEvent, "Global\\%s_ReloadSites", gsTitleCaption);
			}
			else if(lpCmd->IsCommand(iArg, "/ReloadSettings")) {
				sprintf_s(sEvent, "Global\\%s_ReloadSettings", gsTitleCaption);
			}

			if(gService.TriggerEvent(sEvent) != _SVC_SUCCESS_)
			{
				MessageBox(NULL, "Failed to modify service event state.", gsTitleCaption, MB_ICONINFORMATION);
			}
			return 0;
		}
		//------------------------------------------------------------------------------------------
		else if(lpCmd->IsCommand(iArg, "/Install"))
		{
			if(gService.IsInstalled() == _SVC_YES_)
			{
    			//MessageBox(NULL, "The service is already installed.", gsTitleCaption, MB_ICONINFORMATION);
				return 0;
			}

			char sApplication[257];
			sprintf_s(sApplication, sizeof(sApplication), "NetworkDLS %s", gsTitleCaption);
			if(gService.Install(sApplication) == _SVC_SUCCESS_ )
			{
				gService.SetDescription("Provides HTTP web services on NT based Operating Systems.");
				return 0;
			}
			else{
				MessageBox(NULL, "Failed to install the service.", gsTitleCaption, MB_ICONINFORMATION);
				return 0;
			}

			return 0;
		}
		//------------------------------------------------------------------------------------------
		else if(lpCmd->IsCommand(iArg, "/Start"))
		{
			if(IsApplicationRunning())
			{
				return 0;
			}
			else if(gService.IsInstalled() == _SVC_NO_)
			{
    			MessageBox(NULL, "The service is not installed.", gsTitleCaption, MB_ICONINFORMATION);
				return 0;
			}
			else if(gService.IsActive() == _SVC_YES_)
			{
				return 0;
			}
			else if(gService.Start() != _SVC_SUCCESS_)
			{
    			MessageBox(NULL, "Failed to start the service.", gsTitleCaption, MB_ICONINFORMATION);
   				return 0;
			}

			return 0;
		}
		//------------------------------------------------------------------------------------------
		else if(lpCmd->IsCommand(iArg, "/Delete"))
		{
			if(gService.IsInstalled() == _SVC_NO_)
			{
				return 0;
			}
			else if(gService.IsActive() == _SVC_YES_)
			{
				gService.Stop(); // Failure would not be an error.
				return 0;
			}
    		else if(gService.Uninstall() != _SVC_SUCCESS_)
    		{
    			MessageBox(NULL, "Failed to mark the service for deletion.", gsTitleCaption, MB_ICONINFORMATION);
				return 0;
			}

			return 0;
		}
		//------------------------------------------------------------------------------------------
		else if(lpCmd->IsCommand(iArg, "/Stop"))
		{
			if(gService.IsInstalled() == _SVC_NO_)
			{
    			MessageBox(NULL, "The service is not installed.", gsTitleCaption, MB_ICONINFORMATION);
				return 0;
			}
			else if(gService.IsActive() == _SVC_NO_)
			{
				return 0; //Already stopped.
			}
    		else if(gService.Stop() != _SVC_SUCCESS_)
    		{
    			MessageBox(NULL, "Failed to stop the service.", gsTitleCaption, MB_ICONINFORMATION);
				return 0;
			}
			return 0;
		}
		//------------------------------------------------------------------------------------------
	}

	return 0;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void Service_OnShutDown(void)
{
	CloseMainDialog();
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool Service_Thread(LPVOID lpVoid)
{
	HANDLE MutexHandle = NULL;
	char sEvent[255];

	//-------------------------------------------------------------------------------
	SECURITY_DESCRIPTOR SD;
	SECURITY_ATTRIBUTES SA;

	if(InitializeSecurityDescriptor(&SD, SECURITY_DESCRIPTOR_REVISION) == FALSE)
		return false;

	if(SetSecurityDescriptorDacl(&SD, TRUE, 0, FALSE) == FALSE)
		return false;

	SA.nLength				= sizeof(SA);
	SA.lpSecurityDescriptor = &SD;
	SA.bInheritHandle		= FALSE;

	if((MutexHandle = CreateMutex(&SA, FALSE, gsTitleCaption)) != NULL)
	{
		if(GetLastError() == ERROR_ALREADY_EXISTS)
		{
			MessageBox(NULL, "Application already running.",
				gsTitleCaption, MB_SYSTEMMODAL + MB_TASKMODAL);
			return false;
		}
	}

	HANDLE hOpenManagerEvent = NULL;
	HANDLE hFlushSettingsEvent = NULL;
	HANDLE hReloadSitesEvent = NULL;
	HANDLE hReloadSettings = NULL;

	gWebSites = new CWebSites;
	if(gService.bRunningAsService)
	{
		gWebSites->StopTrace();
	}
	else{
		gWebSites->StopTrace(NULL);
	}

	if(gService.bRunningAsService)
	{
		sprintf_s(sEvent, sizeof(sEvent), "Global\\%s_Manager", gsTitleCaption);
		if((hOpenManagerEvent = CreateEvent(&SA, FALSE, FALSE, sEvent)) == NULL)
		{
			MessageBox(NULL, "Manager event failed!", gsTitleCaption, MB_TASKMODAL|MB_ICONERROR);
			return false;
		}
		sprintf_s(sEvent, sizeof(sEvent), "Global\\%s_FlushSettings", gsTitleCaption);
		if((hFlushSettingsEvent = CreateEvent(&SA, FALSE, FALSE, sEvent)) == NULL)
		{
			MessageBox(NULL, "FlushSettings event failed!", gsTitleCaption, MB_TASKMODAL|MB_ICONERROR);
			CloseHandle( hOpenManagerEvent );
			return false;
		}
		sprintf_s(sEvent, sizeof(sEvent), "Global\\%s_ReloadSites", gsTitleCaption);
		if((hReloadSitesEvent = CreateEvent(&SA, FALSE, FALSE, sEvent)) == NULL)
		{
			MessageBox(NULL, "ReloadSites event failed!", gsTitleCaption, MB_TASKMODAL|MB_ICONERROR);
			CloseHandle( hOpenManagerEvent );
			CloseHandle( hFlushSettingsEvent );
			return false;
		}
		sprintf_s(sEvent, sizeof(sEvent), "Global\\%s_ReloadSettings", gsTitleCaption);
		if((hReloadSettings = CreateEvent(&SA, FALSE, FALSE, sEvent)) == NULL)
		{
			MessageBox(NULL, "Reload event failed!", gsTitleCaption, MB_TASKMODAL|MB_ICONERROR);
			CloseHandle( hOpenManagerEvent );
			CloseHandle( hFlushSettingsEvent );
			CloseHandle( hReloadSitesEvent );
			return false;
		}

		while(gService.bAlive)
		{
			if(WaitForSingleObject(hOpenManagerEvent, 1) == WAIT_OBJECT_0)
			{
				LoadMainDialog();
			}

			if(!IsWindowVisible(hMainDialog)) //Don't want to risk interfering with the config dialogs.
			{
				if(WaitForSingleObject(hFlushSettingsEvent, 1) == WAIT_OBJECT_0)
				{
					gWebSites->SaveAll();
				}

				if(WaitForSingleObject(hReloadSitesEvent, 1) == WAIT_OBJECT_0)
				{
					gWebSites->ReloadSites();
				}

				if(WaitForSingleObject(hReloadSettings, 1) == WAIT_OBJECT_0)
				{
					gWebSites->Reload();
				}
			}

			Sleep(10);
		}

		CloseHandle(hOpenManagerEvent);
	}
	else{
		LoadMainDialog();
	}

	return true;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevious, LPSTR sCmdLine, int iCmdShow)
{
	ghAppInstance = GetModuleHandle(NULL);

    int iReturnVal = 0;

	if(!InitializeApp())
    {
        return 1;
    }

	gService.Initialize(gsTitleCaption, &Service_Thread, &Service_OnShutDown);

	if(strlen(sCmdLine) > 0)
	{
		CCmdLine *pCmd = new CCmdLine;
		iReturnVal = ProcessCommandLine(pCmd);
		delete pCmd;
	}
	else{
		if(IsApplicationRunning())
		{
			if(gService.IsActive())
			{
				char sEvent[1024];
				sprintf_s(sEvent, "Global\\%s_Manager", gsTitleCaption);
				if(gService.TriggerEvent(sEvent) != _SVC_SUCCESS_)
				{
					MessageBox(NULL, "The server is already running", gsTitleCaption, MB_ICONINFORMATION);
				}
			}
			return 0;
		}

		iReturnVal =  gService.RunAsStandAlone();
	}

	UninitializeApp();

	return iReturnVal;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#endif

