#ifndef HTTPTypes_H
#define HTTPTypes_H
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include "../../@Classes/CSocketServer/CSocketServer.H"
#include "../../@Classes/CStringBuilder/CStringBuilder.H"

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#define MAX_HTTP_HEADER				1024 * 2
#define MAX_FILE_BUFFER				1048576 //One megabyte
#define MAX_QUERY_STRING			2048
#define MAX_URI_LEN					1024 + MAX_QUERY_STRING
//#define MAX_HTTP_METHOD_NAME		10
//#define MAX_HTTP_VERSION_NAME		15
#define MAX_IP_ADDRESS				16
#define MAX_POST_DATA				10240
#define MAX_RESPONSE_CODE			10

//#define MAX_ACCEPT_STRING			1024
//#define MAX_LANGUAGE_STRING			1024
//#define MAX_ACCEPT_ENCODING_STRING	1024
//#define MAX_USER_AGENT_STRING		1024
#define MAX_HOST_STRING				255
//#define MAX_CONNECTION_STRING		100

#define MAX_SITE_NAME				512
#define MAX_SITE_DESCR				1024
#define HTTP_VERSION				"HTTP/1.1"
#define CGIINTERFACE				"CGI/1.1"
#define MIMEVERSION					"1.1"

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

typedef struct _tag_Last_Response {
	char ResponseCode[MAX_RESPONSE_CODE];
	__int64 ContentLength;
} LASTRESPONSE, *LPLASTRESPONSE;

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

typedef struct _tag_HTTP_Header {
	char *Method;
	char *Request;
	char *Version;
	char *Query;

	char *Accept;
	char *AcceptLanguage;
	char *AcceptEncoding;
	char *UserAgent;
	char *Host;
	char *Connection;
	char *CacheControl;
	char *Referer;
	char *Cookie;
	char *ContentRange;

	char *FullRequest;

	__int64 ContentBegin;
	__int64 ContentEnd;

	char *PostData;
	int PostDataSize;
} HTTPHEADER, *LPHTTPHEADER;

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

typedef struct _tag_Client {
	LPVOID pWebSite; //WEBSITE*

	HTTPHEADER Header;
	LASTRESPONSE LastResponse;
	bool KeepAlive;
	bool IsDynamicContent;

	CSocketServer *Server;
	int ClientID;

} CLIENT, *LPCLIENT;

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include "CMimeTypes.h"
#include "CURLFilters.h"
#include "CExtensionFilters.h"
#include "CDefaultPages.H"
#include "CSettings.H"
#include "CVirtualRoots.H"
#include "CSSIFiles.H"
#include "CIPFilters.H"
#include "CCGIFolders.H"
#include "CCompression.H"
#include "CCustomFilters.H"
#include "CScriptingEngines.H"
#include "CSSIFiles.H"
#include "CURLFilters.H"
#include "CCGIFolders.H"
#include "CScriptingEngines.H"

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

typedef struct _tag_Site_Stats {
	unsigned __int64 AcceptedConnections;
	unsigned __int64 Hits;
	unsigned __int64 BytesRecv;
	unsigned __int64 BytesSent;
} SITESTATS, *LPSITESTATS;

typedef struct _tag_Web_Site{
	LPVOID pWebSites; //WEBSITES*

	char Name[MAX_SITE_NAME];
	char Description[MAX_SITE_DESCR];
	char Root[MAX_PATH];
	unsigned int Port;
	CSocketServer *Server;

	SITESTATS Stats;

	CMimeTypes *pMimeTypes;
	CExtensionFilters *pExtensionFilters;
	CURLFilters *pURLFilters;
	CDefaultPages *pDefaultPages;
	CVirtualRoots *pVirtualRoots;
	CSSIFiles *pSSIFiles;
	CIPFilters *pIPFilters;
	CCGIFolders *pCGIFolders;
	CCompression *pCompression;
	CCustomFilters *pCustomFilters;
	CScriptingEngines *pScriptingEngines;
	CSettings *pSettings;

	CLIENT *Clients;
} WEBSITE, *LPWEBSITE;

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

typedef struct _tag_Web_Sites{
	int Count;
	WEBSITE *WebSite;
	VOID *pWebSites; //CWebSites*
	VOID *pHTTP; //CHTTP*
} WEBSITES, *LPWEBSITES;

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#endif
