///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//  Copyright  NetworkDLS 2002, All rights reserved
//
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF 
// ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO 
// THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A 
// PARTICULAR PURPOSE.
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#ifndef _SOCKETS_CPP
#define _SOCKETS_CPP
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include <Windows.H>
#include <WindowsX.H>
#include <ShellAPI.H>
#include <Stdio.H>
#include <Stdlib.H>

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include "Entry.H"
#include "CWebSites.H"
#include "SocketHandlers.H"

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

using namespace NSWFL::DateTime;

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool Socket_SendHandler(CSocketServer *pSock, CSocketClient *pClient, LPBASICHUNK pChunk)
{
	SERVERTHREADINFO *pSTI = (SERVERTHREADINFO *)pSock->pUserData;
	CWebSites *pWebSites = pSTI->pWebSites;
	CWebSite *pWebSite = pWebSites->Site(pSTI->iSiteIndex);

	pWebSite->Stats.BytesSent += pChunk->iLength;

	return pWebSite->pCustomFilters->ProcessSendFilters(pClient, (const char *)pChunk->pBuffer, pChunk->iLength);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool Socket_RecvHandler(CSocketServer *pSock, CSocketClient *pClient, LPBASICHUNK pChunk)
{
	SERVERTHREADINFO *pSTI = (SERVERTHREADINFO *)pSock->pUserData;
	CWebSites *pWebSites = pSTI->pWebSites;
	CWebSite *pWebSite = pWebSites->Site(pSTI->iSiteIndex);

	pWebSite->Stats.BytesRecv += pChunk->iLength;

	return pWebSite->pCustomFilters->ProcessRecvFilters(pClient, (const char *)pChunk->pBuffer, pChunk->iLength);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void Socket_DisconnectHandler(CSocketServer *pSock, CSocketClient *pClient)
{
	//Pointer to our Server Thread info structure.
	SERVERTHREADINFO *pSTI = (SERVERTHREADINFO *)pSock->pUserData;
	CWebSites *pWebSites = pSTI->pWebSites; //The CWebSites class pointer.
	CWebSite *pWebSite = pWebSites->Site(pSTI->iSiteIndex); //The web-site structure pointer.

	pWebSite->pCustomFilters->ProcessDisconnectFilters(pClient);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool Socket_AcceptHandler(CSocketServer *pSock, CSocketClient *pClient)
{
	//Pointer to our Server Thread info structure.
	SERVERTHREADINFO *pSTI = (SERVERTHREADINFO *)pSock->pUserData;
	CWebSites *pWebSites = pSTI->pWebSites; //The CWebSites class pointer.
	CWebSite *pWebSite = pWebSites->Site(pSTI->iSiteIndex); //The web-site structure pointer.

	pWebSites->Site(pSTI->iSiteIndex)->Stats.AcceptedConnections++;

	if(!pWebSite->pIPFilters->IsIPAllowed(pClient->PeerAddress()))
	{
		//pWebSite->pErrorPages->SendError(??, "403", IP Address Rejected);
		return false; //Just disconnect the client.
	}

	return pWebSite->pCustomFilters->ProcessConnectFilters(pClient, pClient->PeerAddress());
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void Socket_ClientHandler(CSocketServer *pSock, CSocketClient *pClient, LPBASICHUNK pChunk)
{
	//Pointer to our Server Thread info structure.
	SERVERTHREADINFO *pSTI = (SERVERTHREADINFO *)pSock->pUserData;
	CWebSites *pWebSites = pSTI->pWebSites; //The CWebSites class pointer.
	CWebSite *pWebSite = pWebSites->Site(pSTI->iSiteIndex); //The web-site structure pointer.

	pWebSites->IncrementConnectionCount();

	int iSlot = pClient->Slot();

	//Clear the client info.

	memset(&pWebSite->Clients[iSlot], 0, sizeof(CLIENT));
	pWebSite->Clients[iSlot].pWebSite = pWebSite;

	pWebSite->Clients[iSlot].pServer = pSock;
	pWebSite->Clients[iSlot].pClient = pClient;

	CLIENT *pC = &pWebSite->Clients[iSlot];

	//Default timeout, amount of time to wait for data before disconnecting (even if not using keep alive).
	int iKeepAliveTimeout = pWebSite->pSettings->KeepAliveTimeout() * 1000;
	int iTimeoutCount = 0;

	//---BEGIN---------------(Get the Login Information that will be used to access resources)-------------------
	HANDLE hSiteToken = NULL;

	if(strlen(pWebSite->pSettings->Username()) > 0)
	{
		HANDLE hToken = NULL;
		char *sDomain = NULL;
		char sErrorMessage[1024];

		if(strlen(pWebSite->pSettings->Domain()) > 0)
		{
			sDomain = pWebSite->pSettings->Domain();
		}

		if(!(hSiteToken = NTLogin(pWebSite->pSettings->Username(),
			sDomain, pWebSite->pSettings->Password(), sErrorMessage, sizeof(sErrorMessage))))
		{
			pWebSite->pErrorPages->SendError(pC, "500", sErrorMessage);
			pClient->WaitOnDataToBeSent();
			return;
		}
	}
	else {
		HANDLE hProcess = GetCurrentProcess();

		if(!OpenProcessToken(hProcess, TOKEN_ALL_ACCESS, &hSiteToken))
		{
			char sErrorMessage[1024];
			if(GetLastError(sErrorMessage, sizeof(sErrorMessage)))
			{
				pWebSite->pErrorPages->SendError(pC, "500", sErrorMessage);
			}
			else{
				pWebSite->pErrorPages->SendError(pC, "500", "Unknown original logon error.");
			}

			pClient->WaitOnDataToBeSent();
			return;
		}
	}
	//---END---------------(Get the Login Information that will be used to access resources)-------------------

    while(pClient->IsConnected())
    {
		if(pClient->GetClientData(pChunk))
		{
			//Reset the threads impersonation token, as it may have been changed by a previous request.
			if(hSiteToken)
			{
				if(!ImpersonateLoggedOnUser(hSiteToken))
				{
					char sErrorMessage[1024];
					if(GetLastError(sErrorMessage, sizeof(sErrorMessage)))
					{
						pWebSite->pErrorPages->SendError(pC, "500", sErrorMessage);
					}
					else{
						pWebSite->pErrorPages->SendError(pC, "500", "Unknown original logon error.");
					}

					CloseHandle(hSiteToken);
					pClient->WaitOnDataToBeSent();
					return;
				}
			}

			unsigned long ulStartTime = GetTickCount();
			LOGFILEENTRY LFE;
			memset(&LFE, 0, sizeof(LFE));

			Get_ShortDate(LFE.sDate, sizeof(LFE.sDate));
			Get_Time(LFE.sTime, sizeof(LFE.sTime), 0);

			if(pWebSites->pHttp->ParseHTTPHeader(pWebSite, pC, (const char *)pChunk->pBuffer, pChunk->iLength))
			{
				iKeepAliveTimeout = pWebSite->pSettings->KeepAliveTimeout() * 1000; //Convert timeout to seconds.

				if(!pWebSites->pHttp->ProcessRequest(pWebSite, pC, (const char *)pChunk->pBuffer, pChunk->iLength))
				{
					pClient->Disconnect();
				}
				else{
					if(!pC->KeepAlive)
					{
						pClient->Disconnect();
					}
					else{
						iTimeoutCount = 0; //Reset the keep-alive timeout.
					}
				}

				//-------------------------------------------------------------------------------------------
				//-( Begin Log File Write )------------------------------------------------------------------
				//-------------------------------------------------------------------------------------------
				LFE.iPort = pWebSite->Port;
				LFE.iTimeTaken = GetTickCount() - ulStartTime;
				LFE.sAgent = pC->Header.UserAgent;
				LFE.sMethod = pC->Header.Method;
				LFE.sQuery = pC->Header.Query;
				LFE.sSiteName = pWebSite->Name;
				strcpy_s(LFE.sSourceIP, sizeof(LFE.sSourceIP), pClient->PeerAddress());
				LFE.sStatus = pC->LastResponse.ResponseCode;
				LFE.iContentLength = pC->LastResponse.ContentLength;
				//strcpy_s(LFE.sTargetIP, sizeof(LFE.sTargetIP), "*.*.*.*");
				LFE.sURI = pC->Header.Request;
				LFE.sAccept = pC->Header.Accept;
				LFE.sAcceptEncoding = pC->Header.AcceptEncoding;
				LFE.sAcceptLanguage = pC->Header.AcceptLanguage;
				LFE.sCacheControl = pC->Header.CacheControl;
				LFE.sConnection = pC->Header.Connection;
				LFE.sHost = pC->Header.Host;
				LFE.sVersion = pC->Header.Version;

				FILTERLOGENTRY FLE;
				memset(&FLE, 0, sizeof(FLE));
				FLE.sDate           = LFE.sDate;
				FLE.sTime           = LFE.sTime;
				FLE.sSiteName       = LFE.sSiteName;
				FLE.sSourceIP       = LFE.sSourceIP;
				FLE.sMethod         = LFE.sMethod;
				FLE.sAgent          = LFE.sAgent;
				FLE.sStatus         = LFE.sStatus;
				FLE.sAccept         = LFE.sAccept;
				FLE.sAcceptEncoding = LFE.sAcceptEncoding;
				FLE.sAcceptLanguage = LFE.sAcceptLanguage;
				FLE.sCacheControl   = LFE.sCacheControl;
				FLE.sConnection     = LFE.sConnection;
				FLE.sHost           = LFE.sHost;
				FLE.sVersion        = LFE.sVersion;
				FLE.iTimeTaken      = LFE.iTimeTaken;
				FLE.iContentLength  = LFE.iContentLength;
				FLE.sURI            = LFE.sURI;
				FLE.sQuery          = LFE.sQuery;
				FLE.iPort           = LFE.iPort;

				if(pWebSite->pCustomFilters->ProcessLogFilters(pC, &FLE))
				{
					pWebSite->WriteLog(&LFE);
				}
				//-------------------------------------------------------------------------------------------
				//-( End Log File Write )--------------------------------------------------------------------
				//-------------------------------------------------------------------------------------------
			}

			pWebSites->pHttp->FreeClientHeader(pC);
		}

		if(iTimeoutCount++ == iKeepAliveTimeout)
		{
			pClient->Disconnect();
		}

		Sleep(1);
    }

	if(hSiteToken)
	{
		CloseHandle(hSiteToken);
	}

	//Clear the client info.
	memset(pC, 0, sizeof(CLIENT));
	pC->pWebSite = pWebSite;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void Socket_ErrorHandler(CSocketServer *pSock, CSocketClient *pClient, int iErrorNumber, const char *sErrorMsg)
{
	SERVERTHREADINFO *pSTI = (SERVERTHREADINFO *)pSock->pUserData;
	if(pSTI)
	{
		CWebSites *pWebSites = pSTI->pWebSites;
		if(pWebSites)
		{
			//CWebSite *pWebSite = pWebSites->Site(pSTI->iSiteIndex);
			pWebSites->Trace(sErrorMsg);
		}
	}
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#endif
