#ifndef _FILTER_CPP
#define _FILTER_CPP
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include <Windows.H>
#include <STDLib.H>
#include <STDIO.H>

#include "Entry.H"
#include "Filter.H"

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////

//Fully Implemented!
//Notifies the filter that the server has preprocessed the client headers.
DllExport unsigned int RegisterEvents(unsigned int iReserved)
{
	return FILTER_EVENT_ONURLMAP |
		FILTER_EVENT_ONSENDRAWDATA |
		FILTER_EVENT_ONRECVRAWDATA |
		FILTER_EVENT_ONLOG |
		FILTER_EVENT_ONDISCONNECT |
		FILTER_EVENT_ONPREPROCHEADERS;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
	Note: Not all of the attributes of HTTPHEADER are valid when this event is called.
		For Example: [FullRequest] is not filled in untill well after the pre-process.
*/
//Notifies the filter that the server has preprocessed the client headers.
DllExport unsigned int OnPreProcHeaders(unsigned int iClient, HTTPHEADER *pHead)
{
	char sText[255];
	sprintf_s(sText, sizeof(sText), "%d, Method: %s, Host: %s, Request: %s",
		iClient, pHead->Method, pHead->Host, pHead->Request);
	MessageBox(NULL, sText, "OnPreProcHeaders", 0);

	return FILTER_EVENT_RESULT_OK;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////

//Notifies the filter when the server is mapping a logical URL to a physical path.
DllExport unsigned int OnUrlMap(unsigned int iClient, const char *sURIStem, const char *sFileName)
{
	char sText[255];
	sprintf_s(sText, sizeof(sText), "%s\r\n%s", sURIStem, sFileName);
	MessageBox(NULL, sText, "OnUrlMap", 0);

	return FILTER_EVENT_RESULT_OK;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////

//Notifies the filter before raw data is sent from the server to the client.
DllExport unsigned int OnSendRawData(unsigned int iClient, const char *sBuf, unsigned int uiBufSz)
{
	char sText[255];
	sprintf_s(sText, sizeof(sText), "%d Bytes Written", uiBufSz);
	MessageBox(NULL, sText, "OnSendRawData", 0);

	return FILTER_EVENT_RESULT_OK;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////

//Notifies the filter after raw data is sent from the client to the server, but before the server processes it.
DllExport unsigned int OnRecvRawData(unsigned int iClient, const char *sBuf, unsigned int uiBufSz)
{
	char sText[255];
	sprintf_s(sText, sizeof(sText), "%d Bytes Read", uiBufSz);
	MessageBox(NULL, sText, "OnReadRawData", 0);

	return FILTER_EVENT_RESULT_OK;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////

//Logs information to a server file.
DllExport unsigned int OnLog(unsigned int iClient, LOGFILEENTRY *pLFE)
{

	char sText[255];
	sprintf_s(sText, sizeof(sText), "%d, Method: %s, Time: %s, IP: %s",
		iClient, pLFE->sMethod, pLFE->sTime, pLFE->sSourceIP);

	MessageBox(NULL, sText, "OnLog", 0);

	return FILTER_EVENT_RESULT_OK;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////

//Notifies the filter that the session is beginning.
DllExport unsigned int OnConnect(unsigned int iClient)
{
	char sText[255];
	sprintf_s(sText, sizeof(sText), "Connecting client: %d", iClient);
	MessageBox(NULL, sText, "OnDisconnect", 0);

	return FILTER_EVENT_RESULT_OK;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////

//Notifies the filter that the session is ending.
DllExport unsigned int OnDisconnect(unsigned int iClient)
{
	char sText[255];
	sprintf_s(sText, sizeof(sText), "Disconnecting client: %d", iClient);
	MessageBox(NULL, sText, "OnDisconnect", 0);

	return FILTER_EVENT_RESULT_OK;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#endif


